/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_define_macros.h                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* global macros                                                *
*                                                              *
***************************************************************/

#ifndef _SNARL_DEFINE_MACROS_H
#define _SNARL_DEFINE_MACROS_H

       /* ------------------------------------ */
       /* MAXSNARL must not be greater than    */
       /* maximum number of threads per        */
       /* process the system can handle        */
       /* This number limits the maximum       */
       /* number of tasks this program will    */
       /* accept. Any task configured beyond   */
       /* will simply be ignored               */
       /* ------------------------------------ */

#define MAXSNARL                                            1000
#define SNARL_INITIAL_SHORT_ARRAY_SIZE                        50

#define SNARL_TASK_DEFAULT_INTERVAL                          300
#define SNARL_TASK_DEFAULT_ALERT_INTERVAL                    150
#define SNARL_TASK_DEFAULT_WARNINGS_BEFORE_CRITICAL            1
#define SNARL_TASK_DEFAULT_TIMEOUT                             5
#define SNARL_TASK_DEFAULT_PROXY_INDEX_ADD                   100
#define SNARL_TASK_DEFAULT_PERFORMANCE_WARNING_THRESHOLD    3000
#define SNARL_TASK_DEFAULT_PERFORMANCE_CRITICAL_THRESHOLD  10000
#define SNARL_TASK_DEFAULT_TRAP_ATTACK                         1
#define SNARL_TASK_MAX_HTTP_REDIRECTIONS                       8


/* ---------------------------------------------------------- */
/* ----- default storage sizes ------------------------------ */
/* ---------------------------------------------------------- */

#define SNARL_TASK_URL_MAX_LENGTH                           1024
#define SNARL_TASK_NUMERICAL_STRING_MAX_LENGTH                64
#define SNARL_TASK_MAX_DATE_STRING_LENGTH                    128
#define SNARL_SITE_REGEX_DATE_AGE_STRING_MAX_LENGTH          128
#define SNARL_TASK_MAX_URL_APPENDIX_LENGTH                    32
#define SNARL_TASK_PROT_MAX_LENGTH                            64
#define SNARL_TASK_HOST_MAX_LENGTH                           256
#define SNARL_TASK_PATH_MAX_LENGTH                           256
#define SNARL_TASK_PROXY_MAX_LENGTH                          256
#define SNARL_TASK_REGEX_MAX_LENGTH                          256
#define SNARL_TASK_MAX_SITENAME                               64
#define SNARL_TASK_PASSPHRASE_MAX_LENGTH                     255
#define SNARL_TASK_MAX_FILENAME_LENGTH                      1024
#define SNARL_TASK_MAX_STATISTICS_FILE_LINE_LENGTH           255
#define SNARL_TASK_MAX_COOKIEFILE_LINE_LENGTH               2048
#define SNARL_TASK_REGEX_MAX_ERROR_STRING_LENGTH             256
#define SNARL_TASK_FORM_MAX_LENGTH                           256
#define SNARL_TASK_FORMDATA_MAX_LENGTH                       512
#define SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH                 1024
#define SNARL_TASK_MAX_REGEX_RESULT_LENGTH                  1024


       /* ------------------------------------ */
       /* SNARL_TASK_MAX_REGEX_MATCHES_MASK    */
       /* must correspond with                 */
       /* SNARL_TASK_MAX_REGEX_MATCHES - 1     */
       /* ------------------------------------ */

#define SNARL_TASK_MAX_REGEX_MATCHES                           8
#define SNARL_TASK_MAX_REGEX_MATCHES_MASK                   0x07


#define SNARL_TASK_MAX_LENGTH_TOKEN                         1024
#define SNARL_TASK_MAX_LENGTH_SUBCONFIG_LINE                1024


/* ---------------------------------------------------------- */
/* -- default names etc ------------------------------------- */
/* ---------------------------------------------------------- */

#define SNARL_APP_NAME                         "SNARLsnmp 2.0.1"
#define SNARL_WORLDWIDEWEB_HOME          "http://snarl-snmp.org"

#define SNARL_TASK_PROFILING_FILE_SUFFIX   "downloadprofile.txt"
#define SNARL_TASK_DUMP_FILE_SUFFIX                   "dump.dat"

#define SNARL_TASK_URL_CRAWLERTIME_TOKEN        "MONITORINGTIME"

/* -- scheduling parameters used if SNARLsnmp has root rights --*/
#define SNARL_TASK_REAL_TIME_PRIORITY                        90
#define SNARL_TASK_SCHEDULING_POLICY                 SCHED_FIFO

#define SNARL_SITE_DEFAULT_SITENAME                    "default"
#define SNARL_SITE_DEFAULT_URLNAME                     "default"


/* ---------------------------------------------------------- */
/* ----- snarl internal events ------------------------------ */
/* ---------------------------------------------------------- */

#define SNARL_SITE_DEFAULT_WAKEUP_SEC                          1
#define SNARL_SITE_DEFAULT_WAKEUP_USEC                         0


/* ---------------------------------------------------------- */
/* ----- snarl internal events ------------------------------ */
/* ----- start event is currently unused -------------------- */
/* ---------------------------------------------------------- */

#define SNARL_NULL_EVENT                                       0
#define SNARL_UPDATE_TASK_EVENT                                1
#define SNARL_START_SCHEDULE_EVENTS                            2
#define SNARL_STOP_SCHEDULE_EVENTS                             3
#define SNARL_TRAP_EVENT                                       4
#define SNARL_HEART_BEAT_EVENT                                 5


/* ---------------------------------------------------------- */
/* ----- snarl snmp api ------------------------------------- */
/* ---------------------------------------------------------- */

#define NEAR                                                   0
#define EXACT                                                  1


/* ---------------------------------------------------------- */
/* ----- MIB definitions - must match ----------------------- */
/* ----- SNARLSNMP-MIB.txt, the MIB definition -------------- */
/* ----- file ----------------------------------------------- */
/* ---------------------------------------------------------- */

       /* ------------------------------------ */
       /* OID of snarlsnmp traps               */
       /* ------------------------------------ */

#define SNARLSNMP_TRAP                                       100

#define SNARLSNMP_TRAP_INFO                                    1
#define SNARLSNMP_TRAP_NORMAL                                  2
#define SNARLSNMP_TRAP_WARNING                                 3
#define SNARLSNMP_TRAP_CRITICAL                                4

#define SNARLSNMP_TRAP_INHIBITED                               5
#define SNARLSNMP_TRAP_RESUMED                                 6

#define SNARLSNMP_TRAP_GENERIC_INFO                            7
#define SNARLSNMP_TRAP_STATE_UNKNOWN                           8

#define SNARLSNMP_TRAP_SUBTASK_ID_GENERAL                      1
#define SNARLSNMP_TRAP_SUBTASK_ID_PERFORMANCE                  2
#define SNARLSNMP_TRAP_SUBTASK_ID_HTTPCODE                     3
#define SNARLSNMP_TRAP_SUBTASK_ID_REGEX                        4
#define SNARLSNMP_TRAP_SUBTASK_ID_CONTENTSIZE                  5
#define SNARLSNMP_TRAP_SUBTASK_ID_SIZEMATCH                    6
#define SNARLSNMP_TRAP_SUBTASK_ID_CONTENTMATCH                 7
#define SNARLSNMP_TRAP_SUBTASK_ID_NUMICMATCH                   8
#define SNARLSNMP_TRAP_SUBTASK_ID_DATEAGEMATCH                 9
#define SNARLSNMP_TRAP_SUBTASK_ID_NOUPDATE                    10
#define SNARLSNMP_TRAP_SUBTASK_ID_WARMSTART                   11
#define SNARLSNMP_TRAP_SUBTASK_ID_COLDSTART                   12
#define SNARLSNMP_TRAP_SUBTASK_ID_HEARTBEAT                   13
#define SNARLSNMP_TRAP_SUBTASK_ID_SYNCHRONIZE                 14


       /* ------------------------------------ */
       /* OID of snarl tables                  */
       /* three and four digit numbers will be */
       /* ignored in a get request             */
       /* ------------------------------------ */

#define SNARL_MAGIC_NUMBER_BOUNDARY                       100
#define SNARL_NUM_OF_TASKS                                1
#define SNARL_NUM_OF_TASKS_MAGIC_NUMBER                   SNARL_NUM_OF_TASKS     + SNARL_MAGIC_NUMBER_BOUNDARY
#define SNARL_NUM_OF_TRANSFERS                            2
#define SNARL_NUM_OF_TRANSFERS_MAGIC_NUMBER               SNARL_NUM_OF_TRANSFERS + SNARL_MAGIC_NUMBER_BOUNDARY
#define SNARL_TRANSFER_VOLUME                             3
#define SNARL_TRANSFER_VOLUME_MAGIC_NUMBER                SNARL_TRANSFER_VOLUME  + SNARL_MAGIC_NUMBER_BOUNDARY

#define SNARL_SITE_INDEX                                  1
#define SNARL_SITE_TRAP_ATTACK                            10001
#define SNARL_SITE_RANDOM_SOON_START                      10002
#define SNARL_SITE_HEARTBEAT_INTERVAL                     10003
#define SNARL_SITE_CONFIG_ID                              10004
#define SNARL_SITE_TRIGGER_TASK                           10005
#define SNARL_SITE_DUMP_CONTENT                           10006
#define SNARL_SITE_INHIBIT_GENERAL                        10007
#define SNARL_SITE_RESET_DEFAULTS                         10008
#define SNARL_SITE_SYNCH_TRAPS                            10009
#define SNARL_SITE_KEEP_CONTENT                           10010
#define SNARL_SITE_QUIT_ON_DATASIZE                       10011

#define SNARL_SITE_NAME                                   2
#define SNARL_SITE_ID                                     3
#define SNARL_SITE_INTERVAL                               4
#define SNARL_SITE_ALERT_INTERVAL                         40001
#define SNARL_SITE_WARNINGS_BEFORE_CRITICAL               40002

#define SNARL_SITE_URL                                    5
#define SNARL_SITE_URL_PASSPHRASE                         50001
#define SNARL_SITE_WITH_COOKIES                           50002
#define SNARL_SITE_END_COOKIE_SESSION                     50003
#define SNARL_SITE_WITH_HEADER                            50004
#define SNARL_SITE_NO_HTTP_CODE_TEST                      50005

#define SNARL_SITE_DYNAMIC_URL_PATTERN                    6
#define SNARL_SITE_DYNAMIC_URL_PATTERN_COMP               7
#define SNARL_SITE_DYNAMIC_URL_PATTERN_ERR                8

#define SNARL_SITE_DYNAMIC_URL                            9
#define SNARL_SITE_APPEND_DYNAMIC_URL_COMPONENT           90001
#define SNARL_SITE_APPEND_DYNAMIC_FORM_COMPONENT          90002
#define SNARL_SITE_SUBTOKEN_DYN_URL_COMP_MTIME            90003
#define SNARL_SITE_SUBTOKEN_DYN_URL_COMP_TIMESTAMP        90004
#define SNARL_SITE_SUBTOKEN_DYN_URL_COMP_CTIME            90005
#define SNARL_SITE_SUBTOKEN_DYN_URL_COMP_REGEX            90006
#define SNARL_SITE_SUBTOKEN_DYN_URL_STATIC_TEXT           90007
#define SNARL_SITE_SUBTOKEN_DYN_URL_STATIC_AMPERSAND      90008

#define SNARL_SITE_GET_DYNAMIC_URL_FROM                   10
#define SNARL_SITE_IGNORE_PROXY_GROUP                     10001

#define SNARL_SITE_PROXY                                  11
#define SNARL_SITE_PROXY_ADD_INDEX                        110001
#define SNARL_SITE_CONNECT_VIA_PROXY_ONLY                 110002
#define SNARL_SITE_PROXY_KEEPALIVE                        110003

#define SNARL_SITE_MP_FORM_DATA                           12
#define SNARL_SITE_FORM_DATA                              13
#define SNARL_SITE_EFFECTIVE_URL                          14
#define SNARL_SITE_URL_ERROR                              15

#define SNARL_SITE_1PERF                                  16
#define SNARL_SITE_2PERF                                  17
#define SNARL_SITE_3PERF                                  18
#define SNARL_SITE_4PERF                                  19
#define SNARL_SITE_PERFORMANCE_THRESHOLD_WARN             20
#define SNARL_SITE_PERFORMANCE_THRESHOLD_TOKEN            200001
#define SNARL_SITE_PERFORMANCE_THRESHOLD_CRIT             21
#define SNARL_SITE_PERFORMANCE_TIMEOUT_MAIN               210001
#define SNARL_SITE_PERFORMANCE_TIMEOUT_SMURL              210002
#define SNARL_SITE_PERFORMANCE_TIMEOUT_CMURL              210003
#define SNARL_SITE_ALL_AGENTS                             210004
#define SNARL_SITE_MAIN_AGENT                             210005
#define SNARL_SITE_SIZEMATCH_AGENT                        210006
#define SNARL_SITE_CONTENTMATCH_AGENT                     210007
#define SNARL_SITE_REFERER                                210008
#define SNARL_SITE_LOG_TIMING                             210009

#define SNARL_SITE_REGEX                                  22
#define SNARL_SITE_REGEX_COMP                             23
#define SNARL_SITE_REGEX_MATCH                            24
#define SNARL_SITE_REGEX_TRAP_ON_MATCH                    25

#define SNARL_SITE_CONTENT_SIZE                           26
#define SNARL_SITE_CONTENT_ACTUAL_SIZE                    27
#define SNARL_SITE_CONTENT_SIZE_MATCH                     28
#define SNARL_SITE_CONTENT_SIZE_TRAP_ON_MATCH             29

#define SNARL_SITE_CONTENT_URL                            30
#define SNARL_SITE_CONTENT_URL_MATCH                      31
#define SNARL_SITE_CONTENT_URL_TRAP_ON_MATCH              32

#define SNARL_SITE_CONTENT_URL_MATCH_PROXY                33
#define SNARL_SITE_CONTENT_URL_MATCH_PASSPHRASE           330001

#define SNARL_SITE_CONTENT_URL_MATCH_ERROR                34

#define SNARL_SITE_SIZE_URL                               35
#define SNARL_SITE_SIZE_URL_MATCH                         36
#define SNARL_SITE_SIZE_URL_TRAP_ON_MATCH                 37

#define SNARL_SITE_SIZE_URL_MATCH_PROXY                   38
#define SNARL_SITE_SIZE_URL_MATCH_PASSPHRASE              380001

#define SNARL_SITE_SIZE_URL_MATCH_ERROR                   39

#define SNARL_SITE_FETCH_REGEX                            40
#define SNARL_SITE_FETCH_REGEX_COMP                       41
#define SNARL_SITE_FETCH_REGEX_ERR                        42
#define SNARL_SITE_REGEX_VAR0                             43
#define SNARL_SITE_REGEX_VAR1                             44
#define SNARL_SITE_REGEX_VAR2                             45
#define SNARL_SITE_REGEX_VAR3                             46
#define SNARL_SITE_REGEX_VAR4                             47
#define SNARL_SITE_REGEX_VAR5                             48
#define SNARL_SITE_REGEX_VAR6                             49
#define SNARL_SITE_REGEX_VAR7                             50

#define SNARL_SITE_APPEND_REGEX_VAR_FROM                  51
#define SNARL_SITE_APPEND_REGEX_VAR_FROM_INDEX            52

#define SNARL_SITE_NUMERICAL_REGEX_INDEX                  53
#define SNARL_SITE_NUMERICAL_COMPARE_STRING               54
#define SNARL_SITE_NUMERICAL_ALERT_TRIGGER                55

#define SNARL_SITE_FILE_TIME                              56

#define SNARL_SITE_DATE_AGE_REGEX_CONSTRUCT               57
#define SNARL_SITE_DATE_AGE_COMPARE_STRING                58
#define SNARL_SITE_DATE_AGE_ALERT_TRIGGER                 59
#define SNARL_SITE_PROPAGATE_ERROR                        590001

#define SNARL_SITE_DATE_SUSPEND_START                     60
#define SNARL_SITE_DATE_SUSPEND_END                       61
#define SNARL_SITE_INDEX_NO_UPDATE_BETWEEN                610001

#define SNARL_SITE_STATUS_NORMAL                          62
#define SNARL_SITE_STATUS_WARNING                         63
#define SNARL_SITE_STATUS_CRITICAL                        64

#define SNARL_SITE_GROUP_STATUS_NORMAL                    65
#define SNARL_SITE_GROUP_STATUS_WARNING                   66
#define SNARL_SITE_GROUP_STATUS_CRITICAL                  67

#define SNARL_SITE_RELIABILITY                            68
#define SNARL_SITE_MTBF_W                                 69
#define SNARL_SITE_MTTR_W                                 70
#define SNARL_SITE_WARNINGS                               71
#define SNARL_SITE_MTBF_C                                 72
#define SNARL_SITE_MTTR_C                                 73
#define SNARL_SITE_CRITICALS                              74

#define SNARL_SITE_GROUP_RELIABILITY                      75
#define SNARL_SITE_GROUP_MTBF_W                           76
#define SNARL_SITE_GROUP_MTTR_W                           77
#define SNARL_SITE_GROUP_WARNINGS                         78
#define SNARL_SITE_GROUP_MTBF_C                           79
#define SNARL_SITE_GROUP_MTTR_C                           80
#define SNARL_SITE_GROUP_CRITICALS                        81

#define SNARL_SITE_LAST_RUN                               82
#define SNARL_SITE_LAST_RUN_ABS_TIME_STRING               83

#define SNARL_SITE_NEXT_RUN                               84
#define SNARL_SITE_NEXT_RUN_ABS_TIME_STRING               85

#define SNARL_SITE_FIRST_RUN_ABS_TIME_STRING              86

#define SNARL_SITE_TOTAL_TRANSFERS                        87
#define SNARL_SITE_TOTAL_TRANSFER_VOLUME                  88

#define SNARL_SITE_GROUPSTATUS_ID                         89
#define SNARL_SITE_SUBTASK_ID                             90
#define SNARL_SITE_EVENT_TIME                             91
#define SNARL_SITE_TRAP_MESSAGE                           92
#define SNARL_SITE_SITESTATUS_ID                          93
#define SNARL_SITE_PROXYSTATUS_ID                         94

#define SNARL_SITE_SITE_EVENT_NUM                         95
#define SNARL_SITE_PROXY_EVENT_NUM                        96
#define SNARL_SITE_GROUP_EVENT_NUM                        97


       /* ------------------------------------ */
       /* location of the snarlsnmp tree         */
       /* ------------------------------------ */

#define SNARLSNMP_MIB                                     1, 3, 6, 1, 4, 1, 15553
#define SNARL_MIB                                         SNARLSNMP_MIB, 1
#define SNARL_SITE_ENTRY_MIB                              SNARL_MIB, 2, 1

       /* --- varbind 1: index --------------- */
#define SNARL_SITE_INDEX_MIB                              SNARL_SITE_ENTRY_MIB, SNARL_SITE_INDEX
       /* --- varbind 2: subTaskID ----------- */
#define SNARL_SITE_SUBTASK_ID_MIB                         SNARL_SITE_ENTRY_MIB, SNARL_SITE_SUBTASK_ID
       /* --- varbind 3: siteMessage --------- */
#define SNARL_SITE_EVENT_MESSAGE                          SNARL_SITE_ENTRY_MIB, SNARL_SITE_TRAP_MESSAGE
       /* --- varbind 4: group status -------- */
#define SNARL_SITE_GROUPSTATUS_MIB                        SNARL_SITE_ENTRY_MIB, SNARL_SITE_GROUPSTATUS_ID
       /* --- varbind 5: group name ---------- */
#define SNARL_SITE_GROUPNAME_MIB                          SNARL_SITE_ENTRY_MIB, SNARL_SITE_NAME
       /* --- varbind 6: site name ----------- */
#define SNARL_SITE_NAME_MIB                               SNARL_SITE_ENTRY_MIB, SNARL_SITE_ID
       /* --- varbind 7: proxy --------------- */
#define SNARL_PROXY_MIB                                   SNARL_SITE_ENTRY_MIB, SNARL_SITE_PROXY
       /* --- varbind 8: event time ---------- */
#define SNARL_SITE_EVENT_TIME_MIB                         SNARL_SITE_ENTRY_MIB, SNARL_SITE_EVENT_TIME
       /* --- varbind 9: site status --------- */
#define SNARL_SITE_SITESTATUS_MIB                         SNARL_SITE_ENTRY_MIB, SNARL_SITE_SITESTATUS_ID
       /* --- varbind 10: proxy status--- ---- */
#define SNARL_SITE_PROXYSTATUS_MIB                        SNARL_SITE_ENTRY_MIB, SNARL_SITE_PROXYSTATUS_ID
       /* --- varbind 11: site event num ----- */
#define SNARL_SITE_SITE_EVENT_NUM_MIB                     SNARL_SITE_ENTRY_MIB, SNARL_SITE_SITE_EVENT_NUM
       /* --- varbind 12: proxy event num ---- */
#define SNARL_SITE_PROXY_EVENT_NUM_MIB                    SNARL_SITE_ENTRY_MIB, SNARL_SITE_PROXY_EVENT_NUM
       /* --- varbind 13: group event num ---- */
#define SNARL_SITE_GROUP_EVENT_NUM_MIB                    SNARL_SITE_ENTRY_MIB, SNARL_SITE_GROUP_EVENT_NUM
       /* --- varbind 14: group event num ---- */
#define SNARL_SITE_EFFECTIVE_URL_MIB                      SNARL_SITE_ENTRY_MIB, SNARL_SITE_EFFECTIVE_URL

/* ---------------------------------------------------------- */
/* ----- snarl thread --------------------------------------- */
/* ---------------------------------------------------------- */

#define SNARL_TASK_URL_MALFORMAT                               1
#define SNARL_TASK_DYN_URL_MALFORMAT                           2

#define SNARL_TASK_BAD_COOKIEFILE_SUFFIX_TYPE                  3
#define SNARL_TASK_NO_CURL_HANDLE                              4

#define SNARL_TASK_MAIN_URL                                    5
#define SNARL_TASK_CONTENTMATCH_URL                            7
#define SNARL_TASK_SIZEMATCH_URL                               6


/* ---------------------------------------------------------- */
/* ----- snarl macros --------------------------------------- */
/* ---------------------------------------------------------- */

#ifndef __STREQUAL_H
#define __STREQUAL_H
#define strequal(a,b) curl_strequal(a,b)
#define strnequal(a,b,c) curl_strnequal(a,b,c)
#endif

#ifndef __max_and_min
#define __max_and_min
#define max( a, b) (a)>=(b)?(a):(b)
#define min( a, b) (a)<=(b)?(a):(b)
#endif


#endif /* _SNARL_DEFINE_MACROS_H */
/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_define_curlfn.h                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* The GNU Public License may be overriden by the statement of  *
* Daniel Stenberg, the author of this module                   *
*                                                              *
***************************************************************/

/***************************************************************
 *                         _   _ ____  _					   *
 *  Project            ___| | | |  _ \| |					   *
 *                    / __| | | | |_) | |					   *
 *                   | (__| |_| |  _ <| |___				   *
 *                    \___|\___/|_| \_\_____|				   *
 *															   *
 * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>,	   *
 * et al.													   *
 *															   *
 * In order to be useful for every potential user, curl and	   *
 * libcurl are dual-licensed under the MPL and the MIT/X-      *
 * derivate licenses.										   *
 *															   *
 * You may opt to use, copy, modify, merge, publish,		   *
 * distribute and/or sell copies of the Software, and permit   *
 * persons to whom the Software is furnished to do so, under   *
 * the terms of the MPL or the MIT/X-derivate licenses.        **
 * You may pick one of these licenses.						   *
 *															   *
 * This software is distributed on an "AS IS" basis,		   *
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.	   *
 *															   *
 * $Id: setup.h,v 1.11 2001/05/31 07:01:08 bagder Exp $		   *
 **************************************************************/

#ifndef _SNARL_DEFINE_CURLFN_H
#define _SNARL_DEFINE_CURLFN_H

#if !defined(WIN32) && defined(_WIN32)
	/* This _might_ be a good Borland fix. Please report whether this works or
	   not! */
	#define WIN32
#endif

#ifdef HAVE_CONFIG_H
	#include "config.h" /* the configure script results */
#else
	#ifdef WIN32
		/* include the hand-modified win32 adjusted config.h! */
		#include "../config-win32.h"
	#endif
#endif

#ifndef __cplusplus        /* (rabe) */
	typedef char bool;
#endif                     /* (rabe) */

#ifdef NEED_REENTRANT
	/* Solaris machines needs _REENTRANT set for a few function prototypes and
	   things to appear in the #include files. We need to #define it before all
	   #include files */
	#define _REENTRANT
#endif


#include <stdio.h>

#ifndef OS
	#ifdef WIN32
		#define OS "win32"
	#else
		#define OS "unknown"
	#endif
#endif

#if defined(HAVE_X509_H) && defined(HAVE_SSL_H) && defined(HAVE_RSA_H) && \
	defined(HAVE_PEM_H) && defined(HAVE_ERR_H) && defined(HAVE_CRYPTO_H) && \
	defined(HAVE_LIBSSL) && defined(HAVE_LIBCRYPTO)
	  /* the six important includes files all exist and so do both libs,
		 defined SSLeay usage */
	#define USE_SSLEAY 1
#endif

#if defined(HAVE_OPENSSL_X509_H) && defined(HAVE_OPENSSL_SSL_H) && \
	defined(HAVE_OPENSSL_RSA_H) && defined(HAVE_OPENSSL_PEM_H) && \
	defined(HAVE_OPENSSL_ERR_H) && defined(HAVE_OPENSSL_CRYPTO_H) && \
	defined(HAVE_LIBSSL) && defined(HAVE_LIBCRYPTO)
	  /* the six important includes files all exist and so do both libs,
		 defined SSLeay usage */
	#define USE_SSLEAY 1
	#define USE_OPENSSL 1
#endif

#ifndef STDC_HEADERS /* no standard C headers! */
	#include "curl/stdcheaders.h"
#else
	#ifdef _AIX
		#include "curl/stdcheaders.h"
	#endif
#endif

#if 0 /* zlib experiments are halted 17th october, 1999 (Daniel) */
	#if defined(HAVE_ZLIB_H) && defined(HAVE_LIBZ)
			 /* Both lib and header file exist, we have libz! */
		#define USE_ZLIB
	#endif
#endif

#if 0
	#ifdef HAVE_STRCASECMP
		#define strnequal(x,y,z) !(strncasecmp)(x,y,z)
		#define strequal(x,y) !(strcasecmp)(x,y)

	#else
		#define strnequal(x,y,z) !strnicmp(x,y,z)
		#define strequal(x,y) !stricmp(x,y)
	#endif
#endif

/* Below we define four functions. They should
   1. close a socket
   2. read from a socket
   3. write to a socket

   4. set the SIGALRM signal timeout
   5. set dir/file naming defines
   */

#ifdef WIN32
	#if !defined(__GNUC__) || defined(__MINGW32__)
		#define sclose(x) closesocket(x)
		#define sread(x,y,z) recv(x,y,z,0)
		#define swrite(x,y,z) (size_t)send(x,y,z,0)
		#define myalarm(x) /* win32 is a silly system */
	#else
		 /* gcc-for-win is still good :) */
		#define sclose(x) close(x)
		#define sread(x,y,z) recv(x,y,z,0)
		#define swrite(x,y,z) send(x,y,z,0)
		#define myalarm(x) alarm(x)
	#endif

	#define PATH_CHAR     ";"
	#define DIR_CHAR      "\\"
	#define DOT_CHAR      "_"

#else
	#define sclose(x) close(x)
	#define sread(x,y,z) read(x,y,z)
	#define swrite(x,y,z) write(x,y,z)
	#define myalarm(x) alarm(x)

	#define PATH_CHAR     ":"
	#define DIR_CHAR      "/"
	#define DOT_CHAR      "."

	#ifdef HAVE_STRCASECMP
		/* this is for "-ansi -Wall -pedantic" to stop complaining! */
		extern int (strcasecmp)(const char *s1, const char *s2);
		extern int (strncasecmp)(const char *s1, const char *s2, size_t n);
		#ifndef fileno /* sunos 4 have this as a macro! */
			int fileno( FILE *stream);
		#endif
	#endif

#endif

#endif /* _SNARL_DEFINE_CURLFN_H */
/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_define_global.h                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/


/***************************************************************
*                                                              *
* struct templates                                             *
*                                                              *
***************************************************************/

#ifndef _SNARL_DEFINE_GLOBAL_H
#define _SNARL_DEFINE_GLOBAL_H

/***************************************************************
*                                                              *
* foreign includes                                             *
*                                                              *
***************************************************************/

#include "mibdefs.h"


/* ----- foreign libraries ---------------------------------- */

#include <pthread.h>
#include <curl/curl.h>


/* ----- ANSI / standard ------------------------------------ */

#include <time.h>
#include <regex.h>
#include <sys/time.h>
#include <signal.h>
#include <unistd.h>


/* ---------------------------------------------------------- */
/* ----- snarl snmp api ------------------------------------- */
/* ---------------------------------------------------------- */

// none

/* ---------------------------------------------------------- */
/* ----- snarl thread and queue ----------------------------- */
/* ---------------------------------------------------------- */

#ifndef _KERNEL_SYSTEM_H
#define _KERNEL_SYSTEM_H

#define _BSD_SOURCE             1 /* enables vsnprintf */

typedef unsigned char           Bool;

#ifndef __cplusplus             /* only for C, in C++ true and false are language keywords */
#define true                    1
#define false                   0
#endif

typedef char                    Char;
typedef char*                   String;

typedef signed char             Int8;
typedef signed short            Int16;
typedef signed long             Int32;
typedef signed long             Int;        /* will always hold a pointer */

typedef unsigned char           UInt8;
typedef unsigned short          UInt16;
typedef unsigned long           UInt32;
typedef unsigned long           UInt;

typedef void                    Void;
typedef void *                  VPtr;
typedef const void *            CVPtr;

typedef double                  Float;
                                                        /* example signatures:                  */
typedef Void    (*VoidVoid_fpt)(Void);                  /*  void  fun(void)                     */
typedef Void    (*VoidVPtr_fpt)(VPtr);                  /*  void  fun(void*)                    */
typedef Void    (*VoidCVPtr_fpt)(CVPtr);                /*  void  fun(const void*)              */
typedef Void*   (*VPtrUInt_fpt)(UInt);                  /*  void* fun(UInt)                     */
typedef Void*   (*VPtrVPtr_fpt)(VPtr);                  /*  void* fun(void*)                    */
typedef Void*   (*VPtrCVPtr_fpt)(CVPtr);                /*  void* fun(const void*)              */
typedef Void    (*VoidVPtrVPtr_fpt)(VPtr, VPtr);        /*  void  fun(void*,void*               */
typedef Void    (*VoidCVPtrCVPtr_fpt)(CVPtr, CVPtr);    /*  void  fun(const void*,const void*)  */

typedef Bool    (*BoolVPtrVPtr_fpt)(VPtr, VPtr);        /*  bool  fun(void*, void*)             */
typedef Bool    (*BoolCVPtrCVPtr_fpt)(CVPtr, CVPtr);    /*  bool  fun(const void*, const void*) */
typedef UInt    (*UIntVPtr_fpt)(VPtr);                  /*  Uint  fun(void*)                    */
typedef UInt    (*UIntCVPtr_fpt)(CVPtr);                /*  Uint  fun(const void*)              */
typedef Void    (*VoidVPtrUInt_fpt)(VPtr, UInt);        /*  void  fun(void*, Uint)              */
typedef Void    (*VoidCVPtrUInt_fpt)(CVPtr, UInt);      /*  void  fun(const void*, Uint)        */


extern String strlower (const String string);

#define TIME_UNIT   2L

typedef UInt16          Date;
typedef UInt16          Time;
typedef UInt32          DateTime;

typedef float           Price;
typedef UInt32          Volume;
typedef float           Performance;

typedef UInt32          Addr_t;
typedef UInt16          Port_t;


typedef UInt8*          UInt24;


#endif /* _KERNEL_SYSTEM_H */



/* ---------------------------------------------------------- */
/* ----- snarl thread manager ------------------------------- */
/* ---------------------------------------------------------- */

typedef struct event {

    int         what;
    time_t      when;
    void*       ptr;
    UInt        pos;

} event;

typedef struct event* eventPtr;


/* ---------------------------------------------------------- */
/* ----- snarl thread --------------------------------------- */
/* ---------------------------------------------------------- */

typedef struct fakeStream {

    void*           start;
    long            currentSize;
    long            leftSize;
    void*           currentPtr;
    char*           statisticsTxtPtr;
    struct timeval  firstInvocation;
    long            maxSize;

} fakeStream;


typedef struct snarlWebData {

    void*           data;
    long            size;
    double          DNSTime;
    double          connectTime;
    double          transferStartTime;
    double          totalTime;
    long            httpCode;

} snarlWebData;


typedef struct regexdata {

    char        regex[SNARL_TASK_REGEX_MAX_LENGTH];
    regex_t     regexBuffer;
    int         regexError;
    int         regexMatchingError;
    char        regexError_string[SNARL_TASK_REGEX_MAX_ERROR_STRING_LENGTH];
    char        regexMatchingError_string[SNARL_TASK_REGEX_MAX_ERROR_STRING_LENGTH];

} regexdata;

typedef struct regexdata* regexdataPtr;


typedef struct statusVariables {
/* variables that hold data to compute */
/* SSL fulfilments                     */

    pthread_mutex_t   *statusLock;

    struct snarltask  *parentTask;

    time_t             first_run;
    char               first_run_string[SNARL_TASK_MAX_DATE_STRING_LENGTH];

    time_t             last_run;
    char               last_run_string[SNARL_TASK_MAX_DATE_STRING_LENGTH];

    time_t             next_run;
    char               next_run_string[SNARL_TASK_MAX_DATE_STRING_LENGTH];

    time_t             last_update;
    time_t             current_run;

    int                currentStatus;
    int                lastStatus;

    int                statusNormal;
    int                statusWarning;
    int                statusCritical;

    int                accumulatedWarnings;
    time_t             lastTimeWarningStart;
    time_t             accumulatedTimeWarning;
    time_t             accumulatedTimeBetweenWarning;

    int                accumulatedCriticals;
    time_t             lastTimeCriticalStart;
    time_t             accumulatedTimeCritical;
    time_t             accumulatedTimeBetweenCritical;

    long               statusChangeCount;

} statusVariables;

typedef struct statusVariables* statusVarsPtr;


typedef struct siteVariables {

    char              sitename[SNARL_TASK_MAX_SITENAME];
    char              cookiefilename[SNARL_TASK_MAX_FILENAME_LENGTH];
    FILE             *cookiefile;
    pthread_mutex_t  *cookielock;
    statusVariables   statusVars;
    struct snarltask *parentTask;

} siteVariables;

typedef struct siteVariables* siteVarsPtr;


typedef struct proxyVariables {

    statusVariables   statusVars;
    int               keepAlive;
    CURL*             curl_handle;
    struct snarltask *parentTask;

} proxyVariables;

typedef struct proxyVariables* proxyVarsPtr;


typedef struct urldata {

/* url specific */

    char              url[SNARL_TASK_URL_MAX_LENGTH];
    char              currenturl[SNARL_TASK_URL_MAX_LENGTH];
    char              effectiveURL[SNARL_TASK_URL_MAX_LENGTH];
    char              proxy[SNARL_TASK_PROXY_MAX_LENGTH];
    char              agent[SNARL_TASK_URL_MAX_LENGTH];
    char              referer[SNARL_TASK_URL_MAX_LENGTH];
    char              passphrase[SNARL_TASK_PASSPHRASE_MAX_LENGTH];
    char              formData[SNARL_TASK_FORMDATA_MAX_LENGTH];
    char              currentformData[SNARL_TASK_FORMDATA_MAX_LENGTH];
    long              fileTime;
    long              HTTPCode;
    long              timeout;
    long              connect_timeout;

/* CURL specific */
    char              CurlError[CURL_ERROR_SIZE];
    int               noCookies;
    int               task_killCookies;
    int               dumpContent;
    char              dumpFileName[SNARL_TASK_MAX_FILENAME_LENGTH];
    int               keepContent;
    char              keepContentFileName[SNARL_TASK_MAX_FILENAME_LENGTH];
    int               withHeader;
    CURL*             curl_handle;
    int               isForm;
    int               isMultiForm;
    struct HttpPost  *firstFormItem;
    struct HttpPost  *lastFormItem;

/* task specific configurations */

    int              viaProxyOnly;
    int              logTiming;

/* pointer to parent task */

    void            *parentTask;

} urldata;

typedef struct urldata* urldataPtr;


typedef struct trapdata {

    netsnmp_variable_list            var_siteIndex;
//  long                             siteIndex;

    netsnmp_variable_list            var_subTaskID;
//  long                             subTaskID;

    netsnmp_variable_list            var_siteMessage;
//  char                             siteMessage[SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH];

    netsnmp_variable_list            var_siteGroupStatus;
//  long                             siteGroupStatus;

    netsnmp_variable_list            var_siteGroupName;
//  char                             siteGroupName[SNARL_TASK_MAX_SITENAME];

    netsnmp_variable_list            var_siteName;
//  char                             siteName[SNARL_TASK_MAX_SITENAME];

    netsnmp_variable_list            var_proxy;
//  char                             proxy[SNARL_TASK_PROXY_MAX_LENGTH];

    netsnmp_variable_list            var_eventTime;
//  long                             eventTime;

    netsnmp_variable_list            var_siteStatus;
//  long                             siteStatus;

    netsnmp_variable_list            var_proxyStatus;
//  long                             proxyStatus;

    netsnmp_variable_list            var_siteEventNum;
//  long                             siteEventNum;

    netsnmp_variable_list            var_proxyEventNum;
//  long                             proxyEventNum;

    netsnmp_variable_list            var_groupEventNum;
//  long                             groupEventNum;

    netsnmp_variable_list            var_effectiveURL;
//  char                             effectiveURL[SNARL_TASK_URL_MAX_LENGTH];

} trapdata;

typedef struct trapdata* trapdataPtr;


typedef struct tasktrapdata {

    struct snarltask       *parentTask;
    int                     currentWarnings;
    int                     specificTrap;
    long                    siteIndex;
    long                    subTaskID;
    char                    siteMessage[SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH];
    long                    siteGroupStatus;
    char                    siteGroupName[SNARL_TASK_MAX_SITENAME];
    char                    siteName[SNARL_TASK_MAX_SITENAME];
    char                    proxy[SNARL_TASK_PROXY_MAX_LENGTH];
    long                    eventTime;
    long                    siteStatus;
    long                    proxyStatus;
    long                    siteEventNum;
    long                    proxyEventNum;
    long                    groupEventNum;
    char                    effectiveURL[SNARL_TASK_URL_MAX_LENGTH];

} tasktrapdata;

typedef struct tasktrapdata* tasktrapdataPtr;


typedef struct snarltask
{
/* task management     */
    void              *last_task;
    void              *next_task;
/* index used for snmp instances */
    int                index;

/* index used for internal purposes */
    int                ordinal_index;

    char               sitename[SNARL_TASK_MAX_SITENAME];
    char               urlname[SNARL_TASK_MAX_SITENAME];

    int                interval;

    int                alertInterval;
    int                useAlertInterval;
    int                warningsBeforeCritical;

    int                randomSoonStart;

    int                configError;

/* this is the maximum number of bytes */
/* downloaded per transaction          */
/* if this number is exceeded the trans*/
/* action is considered as failed      */
    long               maxDownloadSize;

/* status keeps data to compue SSL's */
    statusVariables    statusVars;
    long               totalTransfers;
    long               totalTransferVolume;

/* variables that keep track of the validity */
/* of this tas data                          */
    int                isVirgin;
    int                isOnlyTemplate;
    int                isRunning;

    int                initialized;
    int                hasData;
    int                aboutToCease;

/* should the proxy be ignored for mutual */
/* addressing in                          */
/*  - GetDynamicURLFrom                   */
/*  - TriggerTask                         */
/*  - AppendDynmaic...                    */
    int                ignoreProxyGroup;

/* variables that point to shared data of */
/* one sitewatch group                    */
    int                hasSiteVarsPtr;
    int                holdsRecycableSiteVars;
    siteVarsPtr        siteVars;
    siteVariables      siteVarsGroup;


/* variables that point to shared data of */
/* one proxy group                        */
    int                hasProxyVarsPtr;
    int                holdsRecycableProxyVars;
    proxyVarsPtr       proxyVars;
    proxyVariables     proxyVarsGroup;

    int                force_status_unknown;

/* wait for external trigger task */
    int                task_waitForExternalTrigger;
    char               triggerTaskName[SNARL_TASK_MAX_SITENAME];
    int                propagateError;

/* send synchronize traps to external program */
    int                task_synchronize;
    int                synchronize_trap_available;
    tasktrapdata       synchronize_trap;

/*  no update between */

    int                task_noUpdateBetween;
    time_t             noUpdateBetweenStart;
    char               noUpdateBetweenStart_string[SNARL_SITE_REGEX_DATE_AGE_STRING_MAX_LENGTH];
    time_t             noUpdateBetweenEnd;
    char               noUpdateBetweenEnd_string[SNARL_SITE_REGEX_DATE_AGE_STRING_MAX_LENGTH];
    int                noUpdateBetween_trap_available;
    tasktrapdata       noUpdateBetween_trap;


/* run time configuration directive line */
    int                task_appendDynamicURLComponent;
    char               dynurlsubconfigline[SNARL_TASK_MAX_LENGTH_SUBCONFIG_LINE];

    int                task_appendDynamicFORMComponent;
    char               dynformsubconfigline[SNARL_TASK_MAX_LENGTH_SUBCONFIG_LINE];

/* pthread management  */

    pthread_t          task_registry;
    pthread_mutex_t   *lock;
    pthread_cond_t    *init_done;
    struct sched_param schedulingParameters;

/* actual url data     */
    urldata            mainURL;

/* general trap data     */
    int                general_trap_inhibit;
    int                general_trap_available;
    tasktrapdata       general_trap;


/* dynamic URL         */
    int               task_fetchDynamicURL;
    char              dynamicURL[SNARL_TASK_URL_MAX_LENGTH];
    regexdata         dynamicURLRegex;

    int               task_getDynamicURLFrom;
    char              getDynamicURLFrom[SNARL_TASK_MAX_SITENAME];

/* dynamic time */
    int               task_appendTimeToURL;

/* dynamic regex result */
    int               task_appendRegexToURL;
    char              getRegexResultFrom[SNARL_TASK_MAX_SITENAME];
    int               getRegexResultFromIndex;

/* fetch regular expression */
    int               task_fetchRegex;
    regexdata         varRegex;
    char              regexMatchContent[SNARL_TASK_MAX_REGEX_MATCHES][SNARL_TASK_MAX_REGEX_RESULT_LENGTH];

/* performance indicators */
    int               task_performance;

    double            time_DNS;
    double            time_connect;
    double            time_firstByte;
    double            time_total;

    int               performance_DNS;
    int               performance_connect;
    int               performance_firstByte;
    int               performance_total;

    int               performance_warning_threshold_total;
    int               performance_critical_threshold_total;
    int               performance_trap_available;
    tasktrapdata      performance_trap;

/* httpcode indicators */
    int               task_httpcode;
    int               httperror_trap_available;
    tasktrapdata      httperror_trap;

/* regex pattern       */
    int               task_matchRegex;
    int               matchRegex_flag;
    regexdata         matchRegex_data;

    int               matchRegex_flag_trap;
    int               matchRegex_trap_available;
    tasktrapdata      matchRegex_trap;

/* numerical pattern       */
    int               task_matchNumic;
    int               matchNumic_flag;
    char              matchNumic_string[SNARL_TASK_NUMERICAL_STRING_MAX_LENGTH];
    int               matchNumic_withRegexIndex;

    int               matchNumic_flag_trap;
    int               matchNumic_trap_available;
    tasktrapdata      matchNumic_trap;

/* date age pattern    */
    int               task_matchDateAge;
    int               matchDateAge_flag;

    char              matchDateAge_tokenline[SNARL_SITE_REGEX_DATE_AGE_STRING_MAX_LENGTH];
    char              matchDateAge_construct[SNARL_SITE_REGEX_DATE_AGE_STRING_MAX_LENGTH];

    char              matchDateAge_string[SNARL_SITE_REGEX_DATE_AGE_STRING_MAX_LENGTH];
    int               matchDateAge_withRegexIndex;

    int               matchDateAge_flag_trap;
    int               matchDateAge_trap_available;
    tasktrapdata      matchDateAge_trap;

/* content size        */
    int               task_matchContentSize;
    int               matchContentSize_flag;
    int               matchContentSize_actual;
    int               matchContentSize;

    int               matchContentSize_flag_trap;
    int               matchContentSize_trap_available;
    tasktrapdata      matchContentSize_trap;

/* url content pattern */
    int               task_contentMatchURL;
    int               contentMatchURL_flag;
    urldata           contentMatchURL;

    int               contentMatchURL_flag_trap;
    int               contentMatchURL_trap_available;
    tasktrapdata      contentMatchURL_trap;

/* url size pattern    */
    int               task_sizeMatchURL;
    int               sizeMatchURL_flag;
    urldata           sizeMatchURL;

    int               sizeMatchURL_flag_trap;
    int               sizeMatchURL_trap_available;
    tasktrapdata      sizeMatchURL_trap;

} snarltask;

typedef snarltask* snarltaskPtr;


typedef struct snarltaskArray {

    snarltaskPtr             firstsnarltask;
    snarltaskPtr             lastsnarltask;
    int                      taskCount;

} snarltaskArray;

typedef snarltaskArray* snarltaskArrayPtr;

#endif /* _SNARL_DEFINE_GLOBAL_H */

/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/


/***************************************************************
*                                                              *
* snarl_module_mibapi.h                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/

#ifndef _SNARL_MODULE_MIBAPI_H
#define _SNARL_MODULE_MIBAPI_H


void                    init_snarl(                 void                     );
void                    snarl_free_config(          void                     );

extern FindVarMethod    var_snarl_table;
int                     header_snarl_table(         struct  variable  *vp,
                                                    oid               *name,
                                                    size_t            *length,
                                                    int                exact,
                                                    size_t            *var_len,
                                                    WriteMethod      **write_method,
                                                    int max                  );

#endif /* _SNARL_MODULE_MIBAPI_H */
/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_manage_config.h                                         *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/


#ifndef _SNARL_MANAGE_CONFIG_H
#define _SNARL_MANAGE_CONFIG_H

void snarl_parse_config(const char *token, char* cptr);

int matchToken(               char* aToken );


#endif /* _SNARL_MANAGE_CONFIG_H */

/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/


/***************************************************************
*                                                              *
* snarl_manage_switch.h                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/


#ifndef _SNARL_MANAGE_SWITCH_H
#define _SNARL_MANAGE_SWITCH_H

void  startScheduler(             void );
void  stopThreadManager(          void );

void  checkInTask(                void );
void  checkOutTask(               void );
void  waitForAllRunningTasks(     void );

#endif /* _SNARL_MANAGE_SWITCH_H */
/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_manage_equeue.h                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/


#ifndef _SNARL_MANAGE_EQUEUE_H
#define _SNARL_MANAGE_EQUEUE_H


#ifdef __cplusplus
extern "C" {
#endif


typedef struct PriQueue_s {
    Void *                  first;
    UInt                    count;
    Void * *                items;
    UInt                    alloc;
    BoolCVPtrCVPtr_fpt      is_less_item;
    VoidVPtrUInt_fpt        set_pos_item;
}
PriQueue_t;

eventPtr   getNextEvent(       eventPtr yourEventPtr      );

void       queueEvent(         int      what,
						       time_t   when,
						       void*    ptr               );

void       flushEvents(        void                       );

Bool       earlierEvent( const void* ptr1,
                         const void* ptr2 );

void       setEventPosition(   void* ptr, UInt   position );


extern Void InitPriQueue (
    PriQueue_t *            priq,
    UInt                    init_count,
    BoolCVPtrCVPtr_fpt      is_less_item,
    VoidVPtrUInt_fpt        set_pos_item);

extern Void DestroyPriQueue (PriQueue_t* priq);


extern Void FixPriQueue (
    PriQueue_t *            priq,
    UInt                    pos );


extern Void AddPriQueue (
    PriQueue_t *            priq,
    Void *                  item );

extern Void RemovePriQueue (
    PriQueue_t *            priq,
    UInt                    pos );


#ifdef __cplusplus
}
#endif


#endif /* _SNARL_MANAGE_EQUEUE_H */
/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_manage_memory.h                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/


#ifndef _SNARL_MANAGE_MEMORY_H
#define _SNARL_MANAGE_MEMORY_H

void                mask_snarltask_defaults(        snarltaskPtr       );

void                copy_masked_data(               void* dest,
                                                    void* src,
                                                    void* mask,
                                                    long len           );


snarltaskPtr        create_snarltask(               char*,
                                                    char*,
                                                    snarltaskPtr,
                                                    snarltaskArrayPtr  );

void                invalidate_curlhandles(         snarltaskPtr       );

void                invalidate_snarltask(           snarltaskPtr,
                                                    snarltaskArrayPtr  );

void                clone_proxy_task(               char*,
                                                    char*,
                                                    int                );

siteVarsPtr         create_siteVars(                snarltaskPtr,
                                                    siteVarsPtr        );

proxyVarsPtr        create_proxyVars(               snarltaskPtr ,
                                                    proxyVarsPtr ,
                                                    int                );

void                fixPointers(                    snarltaskPtr       );
void                setTrapIndex(                   snarltaskPtr       );


       /* ------------------------------------ */
       /* works with                           */
       /*                 snarltaskArray       */
       /* references                           */
       /*            shortsnarlTaskArray       */
       /* ------------------------------------ */

snarltaskPtr        find_snarltask(                 char* ,
                                                    char* ,
                                                    int*,
                                                    snarltaskArrayPtr  );


       /* ------------------------------------ */
       /* works with                           */
       /*                 snarltaskArray       */
       /*            shortsnarlTaskArray       */
       /* ------------------------------------ */

void                renumber_ordinalIndex(          snarltaskArrayPtr  );

int                 find_maxOrdinalIndex(           snarltaskArrayPtr  );

int                 find_maxIndex(                  snarltaskArrayPtr  );

int                 find_minIndex(                  snarltaskArrayPtr  );

int                 find_nextIndex(                 int,
                                                    snarltaskArrayPtr  );

int                 find_lastIndex(                 int,
                                                    snarltaskArrayPtr  );

snarltaskPtr        find_IndexedTask(               int,
                                                    int,
                                                    snarltaskArrayPtr  );


       /* ------------------------------------ */
       /* works with                           */
       /*                 snarltaskArray       */
       /* handles both                         */
       /*                 snarltaskArray       */
       /*                 shortsnarltaskArray  */
       /* ------------------------------------ */

void                attach_snarltask(               snarltaskPtr,
                                                    snarltaskArrayPtr  );

void                insert_snarltask(               snarltaskPtr,
                                                    snarltaskPtr,
                                                    snarltaskArrayPtr  );

void                detach_snarltask(               snarltaskPtr,
                                                    snarltaskArrayPtr  );
#endif
/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_manage_thread.h                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/


#ifndef _SNARL_MANAGE_THREAD_H
#define _SNARL_MANAGE_THREAD_H

int    initializeThreadManager( void        );
void   startThreadManager(      void* null  );

#endif
/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_manage_v1trap.h                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/


#ifndef _SNARL_MANAGE_V1TRAP_H
#define _SNARL_MANAGE_V1TRAP_H

void                    snarl_sendtrap(             int                specificTrap,
                                                    char*              agent,
                                                    tasktrapdataPtr    theTaskTrapdata  );

void snarl_send_enterprise_trap_vars (              int                    trap,
                                                    int                    specific,
                                                    char*                  agent,
                                                    oid                   *enterprise,
                                                    int                    enterprise_length,
                                                    netsnmp_variable_list *vars         );

#endif
/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_agent_getdat.h                                         *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* The GNU Public License may be overriden by the statement of  *
* Steven M. Bellovin, the author of this module                *
*                                                              *
***************************************************************/

/**************************************************************
*														      *
* Originally written by Steven M. Bellovin             	      *
* <smb@research.att.com> while at the University of North     *
* Carolina at Chapel Hill.  Later tweaked by a couple of      *
* people on Usenet.  Completely overhauled by Rich $alz	      *
* <rsalz@bbn.com> and Jim Berets <jberets@bbn.com> in	      *
* August, 1990.											      *
*														      *
* This code is in the public domain and has no copyright.     *
*														      *
**************************************************************/


#ifndef _SNARL_MANAGE_GETDAT_H
#define _SNARL_MANAGE_GETDAT_H

#if HAVE_CONFIG_H
# include <config.h>
#endif

#ifndef PARAMS
# if defined PROTOTYPES || (defined __STDC__ && __STDC__)
#  define PARAMS(Args) Args
# else
#  define PARAMS(Args) ()
# endif
#endif

# include <sys/types.h>

# if TIME_WITH_SYS_TIME
#  include <sys/time.h>
#  include <time.h>
# else
#  if HAVE_SYS_TIME_H
#   include <sys/time.h>
#  else
#   include <time.h>
#  endif
# endif

void     snarl_time(  time_t* now );
void     snarl_ctime( char* buffer, time_t* time, int maxlen );

void     snarl_getdate PARAMS ((const char *p, const time_t *now, time_t *result));

#endif

/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_manage_dotask.h                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/

#ifndef _SNARL_MANAGE_DOTASKS_H
#define _SNARL_MANAGE_DOTASKS_H

int     initSnarlTask(                  snarltaskPtr       aTask      );
void    updateSnarlTask(                snarltaskPtr       aTask      );

int     snarl_setup_connection_data(    urldataPtr         urldata,
                                        char*              sitename,
                                        char*              urlname,
                                        int                urltype    );


size_t  snarlDataAcquisition(           void*              ptr,
                                        size_t             size,
                                        size_t             nmemb,
                                        void*              data        );


size_t  snarl_filter_cookie_from_header(void*              ptr,
                                        size_t             size,
                                        size_t             nmemb,
                                        void*              stream      );

void    snarl_log_statistics(           snarlWebData      *webData,
                                        snarltaskPtr       aTask       );

void    invalidate_traps(               snarltaskPtr       aTask       );

void    recordStatus(                   int                status,
                                        statusVarsPtr      statusVars  );

int     snarl_getContentOfURL(          urldataPtr         urldata,
                                        snarlWebData      *snarlWebData);

int     snarl_make_absolute_url(        snarltaskPtr       aTask       );


int     new_cookie(                     FILE*              cookiefile,
                                        char*              cookie      );
void    cleanup_cookiefile(             char*              cookiefilename);

int     prioritizeTrap(                 int                a,
                                        int                b           );

int     createDynamicURLData( 			snarltaskPtr 	   aTask,
										char* 			   urldata,
										int 			   urldatamaxlength,
										char* 			   subconfigline );


#endif

