/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl.c                                                      *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* this file is the main code module                            *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* net-snmp-5.0.3 include files                                 *
* (these must be included before any snarl related code)       *
*                                                              *
***************************************************************/

#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>
#include <net-snmp/agent/agent_callbacks.h>

/***************************************************************
*                                                              *
* include individually for this module                         *
*                                                              *
***************************************************************/

#include "snarl.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/mman.h>
#include <netdb.h>


/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_define_global.c                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* this code defines the global variables and definitions for   *
* the SNARLsnmp MIB module                                     *
*                                                              *
* in addition this file points to global variables of the      *
* net-snmp snmpd agent                                         *
*                                                              *
***************************************************************/


struct trap_sink {
    netsnmp_session    *sesp;
    struct trap_sink   *next;
    int                 pdutype;
    int                 version;
};

extern struct trap_sink *sinks;

extern struct timeval    starttime;

extern oid               objid_enterprisetrap;
extern oid               trap_version_id;
extern int               enterprisetrap_len;
extern int               trap_version_id_len;


extern oid               cold_start_oid;
extern oid               warm_start_oid;
extern oid               link_down_oid;
extern oid               link_up_oid;
extern oid               auth_fail_oid;
extern oid               egp_xxx_oid;

extern oid               snmptrap_oid;
extern oid               snmptrapenterprise_oid;
extern oid               sysuptime_oid;

extern size_t            snmptrap_oid_len;
extern size_t            snmptrapenterprise_oid_len;
extern size_t            sysuptime_oid_len;

extern int               snmp_enableauthentraps;
extern int               snmp_enableauthentrapsset;
extern char             *snmp_trapcommunity;

#define SNMPV2_TRAPS_PREFIX	SNMP_OID_SNMPMODULES,1,1,5
#define SNMPV2_TRAP_OBJS_PREFIX	SNMP_OID_SNMPMODULES,1,1,4

#define SNMP_AUTHENTICATED_TRAPS_ENABLED	1
#define SNMP_AUTHENTICATED_TRAPS_DISABLED	2


/***************************************************************
*                                                              *
* SNARLsnmp global variables                                   *
*                                                              *
***************************************************************/

       /* ------------------------------------ */
       /* globalStartUpType contains           */
       /* information concerning if the        */
       /* application was cold- or warmstarted */
       /* ------------------------------------ */
int                 globalStartUpCount;
int                 globalStartUpType;

pid_t               globalPID;
char                globalID[SNARL_TASK_MAX_SITENAME];


       /* ------------------------------------ */
       /* globalFirstRun holds the time when   */
       /* this daemon went through a cold      */
       /* start                                */
       /* globalHeartBeatInterval holds the    */
       /* interval time in seconds when to     */
       /* send a heart beat trap               */
       /* ------------------------------------ */
time_t              globalFirstRun;
time_t              globalHeartBeatInterval;
long                globalHeartBeatCount;


       /* ------------------------------------ */
       /* globalTransfers holds total number   */
       /* of transfers as an indicator for     */
       /* activity                             */
       /* ------------------------------------ */
long                globalTotalTransfers;
long                globalTotalTransferVolume;
pthread_mutex_t     globalTotalTransferLock;


       /* ------------------------------------ */
       /* global task array is a pointer       */
       /* struct holding first and last        */
       /* address of the linked list of tasks  */
       /* ------------------------------------ */
snarltaskArray      globalTaskArrayStruct;
snarltaskArray      globalRecycledTaskArrayStruct;

snarltaskArrayPtr   globalTaskArray;
snarltaskArrayPtr   globalRecycledTaskArray;


       /* ------------------------------------ */
       /* global running tasks count is used   */
       /* for accounting currently running     */
       /* crawler instances (snarlThreads)     */
       /* ------------------------------------ */
int                 globalRunningTaskCount;
pthread_mutex_t     globalRunningTaskCountLock;


       /* ------------------------------------ */
       /* curl inited is a flag signaling the  */
       /* current status of the curl library   */
       /* ------------------------------------ */
int                 globalCURLinited;


       /* ------------------------------------ */
       /* scheduler ... is used to control     */
       /* the task scheduler                   */
       /* (snarlThreadManager)                 */
       /* ------------------------------------ */
int                 globalSchedulerInited;
pthread_mutex_t     globalSchedulerInitLock;
pthread_cond_t      globalSchedulerInitDone;
int                 globalSchedulerRegistry;


       /* ------------------------------------ */
       /* schedule events is used to start     */
       /* and stop the scheduler               */
       /* ------------------------------------ */
int                 globalScheduleEvents;
pthread_mutex_t     globalScheduleEventsLock;
pthread_cond_t      globalScheduleEventsAck;


       /* ------------------------------------ */
       /* the getdate function                 */
       /* is not thread safe - use a lock      */
       /* ------------------------------------ */
pthread_mutex_t     globalGetDateLock;
pthread_mutex_t     globalSnarlTimeLock;
pthread_mutex_t     globalSnarlCtimeLock;


       /* ------------------------------------ */
       /* the priority queue is used to        */
       /* schedule events such as              */
       /* task invocation, trap submission     */
       /* ------------------------------------ */
PriQueue_t          globalEventQueue;
int                 globalEventQueueInited;
pthread_mutex_t     globalEventLock;
pthread_cond_t      globalEventAvailable;


       /* ------------------------------------ */
       /* global add index is an integer that  */
       /* added onto the index of related task */
       /* structures                           */
       /* when creating a task clone with a    */
       /* different proxy                      */
       /* this value can be altered by the     */
       /* addIndexOnProxy directive            */
       /* ------------------------------------ */
int                 globalAddIndex;


       /* ------------------------------------ */
       /* globalTrapAttack set's the maximum   */
       /* number of traps sent for each        */
       /* found condition tra                  */
       /* each trap is sent once on one task   */
       /* run                                  */
       /* if there is a status change after    */
       /* less than globalTrapAttck traps have */
       /* been sent - the individual trap      */
       /* counters are reset                   */
       /* ------------------------------------ */
int                 globalTrapAttack;


       /* ------------------------------------ */
       /* global task template struct is used  */
       /* to keep default values for task      */
       /* creation upon config file parsing    */
       /* ------------------------------------ */
snarltask           globalTaskTemplateStruct;
snarltaskPtr        globalTaskTemplate;

snarltask           globalDefaultsMaskStruct;
snarltaskPtr        globalDefaultsMask;

snarltask           globalTaskCopyMaskStruct;
snarltaskPtr        globalTaskCopyMask;
siteVarsPtr         globalSiteCopyMask;
proxyVarsPtr        globalProxyCopyMask;


oid                 globalOID_snarlsnmp_enterprise[]               = {  SNARLSNMP_MIB                                };
int                 globalOID_snarlsnmp_enterprise_length          = OID_LENGTH( globalOID_snarlsnmp_enterprise      );

oid                 globalOID_snarlsnmp_siteIndex[]                = {  SNARL_SITE_INDEX_MIB                         };
int                 globalOID_snarlsnmp_siteIndex_length           = OID_LENGTH( globalOID_snarlsnmp_siteIndex       );

oid                 globalOID_snarlsnmp_subTaskID[]                = {  SNARL_SITE_SUBTASK_ID_MIB                    };
int                 globalOID_snarlsnmp_subTaskID_length           = OID_LENGTH( globalOID_snarlsnmp_subTaskID       );

oid                 globalOID_snarlsnmp_siteTrapMessage[]          = {  SNARL_SITE_EVENT_MESSAGE                     };
int                 globalOID_snarlsnmp_siteTrapMessage_length     = OID_LENGTH( globalOID_snarlsnmp_siteTrapMessage );

oid                 globalOID_snarlsnmp_siteGroupStatus[]          = {  SNARL_SITE_GROUPSTATUS_MIB                   };
int                 globalOID_snarlsnmp_siteGroupStatus_length     = OID_LENGTH( globalOID_snarlsnmp_siteGroupStatus );

oid                 globalOID_snarlsnmp_siteGroupName[]            = {  SNARL_SITE_GROUPNAME_MIB                     };
int                 globalOID_snarlsnmp_siteGroupName_length       = OID_LENGTH( globalOID_snarlsnmp_siteGroupName   );

oid                 globalOID_snarlsnmp_siteName[]                 = {  SNARL_SITE_NAME_MIB                          };
int                 globalOID_snarlsnmp_siteName_length            = OID_LENGTH( globalOID_snarlsnmp_siteName        );

oid                 globalOID_snarlsnmp_proxy[]                    = {  SNARL_PROXY_MIB                              };
int                 globalOID_snarlsnmp_proxy_length               = OID_LENGTH( globalOID_snarlsnmp_proxy           );

oid                 globalOID_snarlsnmp_siteEventTime[]            = {  SNARL_SITE_EVENT_TIME_MIB                    };
int                 globalOID_snarlsnmp_siteEventTime_length       = OID_LENGTH( globalOID_snarlsnmp_siteEventTime   );

oid                 globalOID_snarlsnmp_siteSiteStatus[]           = {  SNARL_SITE_SITESTATUS_MIB                    };
int                 globalOID_snarlsnmp_siteSiteStatus_length      = OID_LENGTH( globalOID_snarlsnmp_siteSiteStatus  );

oid                 globalOID_snarlsnmp_siteProxyStatus[]          = {  SNARL_SITE_PROXYSTATUS_MIB                   };
int                 globalOID_snarlsnmp_siteProxyStatus_length     = OID_LENGTH( globalOID_snarlsnmp_siteProxyStatus );

oid                 globalOID_snarlsnmp_siteEventNum[]             = {  SNARL_SITE_SITE_EVENT_NUM_MIB                };
int                 globalOID_snarlsnmp_siteEventNum_length        = OID_LENGTH( globalOID_snarlsnmp_siteEventNum    );

oid                 globalOID_snarlsnmp_proxyEventNum[]            = {  SNARL_SITE_PROXY_EVENT_NUM_MIB               };
int                 globalOID_snarlsnmp_proxyEventNum_length       = OID_LENGTH( globalOID_snarlsnmp_proxyEventNum   );

oid                 globalOID_snarlsnmp_groupEventNum[]            = {  SNARL_SITE_GROUP_EVENT_NUM_MIB               };
int                 globalOID_snarlsnmp_groupEventNum_length       = OID_LENGTH( globalOID_snarlsnmp_groupEventNum   );

oid                 globalOID_snarlsnmp_effectiveURL[]             = {  SNARL_SITE_EFFECTIVE_URL_MIB                 };
int                 globalOID_snarlsnmp_effectiveURL_length        = OID_LENGTH( globalOID_snarlsnmp_effectiveURL    );

/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/


/***************************************************************
*                                                              *
* snarl_module_mibapi.c                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* the start of SNARLsnmp MIB module for net-snmp's snmpd       *
*                                                              *
* this program does not contain 'main()'                       *
* because the following functions are called by the snmp       *
* framework in the following order                             *
*                                                              *
* init_snarl()            once at startup                      *
*                                                              *
* snarl_free_config()     once at startup and prior to any     *
*                         reconfiguration event of the snmp    *
*                         framework                            *
*                                                              *
* snarl_parse_config()    prior to any reconfiguration event   *
*                         (this function has been moved to     *
*                         snarl_agent_config.c)                *
*                                                              *
* var_snarl_table()       on every snmp read/write event       *
*                         of the snmp framework whenever       *
*                         an snarlsnmp OID is queried          *
*                                                              *
***************************************************************/


/***************************************************************
*                                                              *
* snmp module initialization                                   *
*                                                              *
***************************************************************/

void init_snarl(void)
{

     int    len;

     struct variable2 snarl_info[] = {
       {SNARL_NUM_OF_TASKS_MAGIC_NUMBER,        ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_NUM_OF_TASKS}},
       {SNARL_NUM_OF_TRANSFERS_MAGIC_NUMBER,    ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_NUM_OF_TRANSFERS}},
       {SNARL_TRANSFER_VOLUME_MAGIC_NUMBER,     ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_TRANSFER_VOLUME}}
     };

     struct variable2 snarl_table[] = {
       {SNARL_SITE_INDEX,                       ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_INDEX}},

       {SNARL_SITE_NAME,                        ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_NAME}},
       {SNARL_SITE_ID,                          ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_ID}},
       {SNARL_SITE_INTERVAL,                    ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_INTERVAL}},

       {SNARL_SITE_URL,                         ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_URL}},
       {SNARL_SITE_DYNAMIC_URL_PATTERN,         ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_DYNAMIC_URL_PATTERN}},
       {SNARL_SITE_DYNAMIC_URL_PATTERN_COMP,    ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_DYNAMIC_URL_PATTERN_COMP}},
       {SNARL_SITE_DYNAMIC_URL_PATTERN_ERR,     ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_DYNAMIC_URL_PATTERN_ERR}},
       {SNARL_SITE_DYNAMIC_URL,                 ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_DYNAMIC_URL}},
       {SNARL_SITE_GET_DYNAMIC_URL_FROM,        ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_GET_DYNAMIC_URL_FROM}},

       {SNARL_SITE_PROXY,                       ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_PROXY}},
       {SNARL_SITE_MP_FORM_DATA,                ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_MP_FORM_DATA}},
       {SNARL_SITE_FORM_DATA,                   ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_FORM_DATA}},
       {SNARL_SITE_EFFECTIVE_URL,               ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_EFFECTIVE_URL}},
       {SNARL_SITE_URL_ERROR,                   ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_URL_ERROR}},

       {SNARL_SITE_1PERF,                       ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_1PERF}},
       {SNARL_SITE_2PERF,                       ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_2PERF}},
       {SNARL_SITE_3PERF,                       ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_3PERF}},
       {SNARL_SITE_4PERF,                       ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_4PERF}},
       {SNARL_SITE_PERFORMANCE_THRESHOLD_WARN,  ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_PERFORMANCE_THRESHOLD_WARN}},
       {SNARL_SITE_PERFORMANCE_THRESHOLD_CRIT,  ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_PERFORMANCE_THRESHOLD_CRIT}},

       {SNARL_SITE_REGEX,                       ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_REGEX}},
       {SNARL_SITE_REGEX_COMP,                  ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_REGEX_COMP}},
       {SNARL_SITE_REGEX_MATCH,                 ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_REGEX_MATCH}},
       {SNARL_SITE_REGEX_TRAP_ON_MATCH,         ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_REGEX_TRAP_ON_MATCH}},

       {SNARL_SITE_CONTENT_SIZE,                ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_CONTENT_SIZE}},
       {SNARL_SITE_CONTENT_ACTUAL_SIZE,         ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_CONTENT_ACTUAL_SIZE}},
       {SNARL_SITE_CONTENT_SIZE_MATCH,          ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_CONTENT_SIZE_MATCH}},
       {SNARL_SITE_CONTENT_SIZE_TRAP_ON_MATCH,  ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_CONTENT_SIZE_TRAP_ON_MATCH}},

       {SNARL_SITE_CONTENT_URL,                 ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_CONTENT_URL}},
       {SNARL_SITE_CONTENT_URL_MATCH,           ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_CONTENT_URL_MATCH}},
       {SNARL_SITE_CONTENT_URL_TRAP_ON_MATCH,   ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_CONTENT_URL_TRAP_ON_MATCH}},
       {SNARL_SITE_CONTENT_URL_MATCH_PROXY,     ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_CONTENT_URL_MATCH_PROXY}},
       {SNARL_SITE_CONTENT_URL_MATCH_ERROR,     ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_CONTENT_URL_MATCH_ERROR}},

       {SNARL_SITE_SIZE_URL,                    ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_SIZE_URL}},
       {SNARL_SITE_SIZE_URL_MATCH,              ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_SIZE_URL_MATCH}},
       {SNARL_SITE_SIZE_URL_TRAP_ON_MATCH,      ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_SIZE_URL_TRAP_ON_MATCH}},
       {SNARL_SITE_SIZE_URL_MATCH_PROXY,        ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_SIZE_URL_MATCH_PROXY}},
       {SNARL_SITE_SIZE_URL_MATCH_ERROR,        ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_SIZE_URL_MATCH_ERROR}},

       {SNARL_SITE_FETCH_REGEX,                 ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_FETCH_REGEX}},
       {SNARL_SITE_FETCH_REGEX_COMP,            ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_FETCH_REGEX_COMP}},
       {SNARL_SITE_FETCH_REGEX_ERR,             ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_FETCH_REGEX_ERR}},
       {SNARL_SITE_REGEX_VAR0,                  ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_REGEX_VAR0}},
       {SNARL_SITE_REGEX_VAR1,                  ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_REGEX_VAR1}},
       {SNARL_SITE_REGEX_VAR2,                  ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_REGEX_VAR2}},
       {SNARL_SITE_REGEX_VAR3,                  ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_REGEX_VAR3}},
       {SNARL_SITE_REGEX_VAR4,                  ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_REGEX_VAR4}},
       {SNARL_SITE_REGEX_VAR5,                  ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_REGEX_VAR5}},
       {SNARL_SITE_REGEX_VAR6,                  ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_REGEX_VAR6}},
       {SNARL_SITE_REGEX_VAR7,                  ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_REGEX_VAR7}},

       {SNARL_SITE_APPEND_REGEX_VAR_FROM,       ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_APPEND_REGEX_VAR_FROM}},
       {SNARL_SITE_APPEND_REGEX_VAR_FROM_INDEX, ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_APPEND_REGEX_VAR_FROM_INDEX}},

       {SNARL_SITE_NUMERICAL_REGEX_INDEX,       ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_NUMERICAL_REGEX_INDEX}},
       {SNARL_SITE_NUMERICAL_COMPARE_STRING,    ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_NUMERICAL_COMPARE_STRING}},
       {SNARL_SITE_NUMERICAL_ALERT_TRIGGER,     ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_NUMERICAL_ALERT_TRIGGER}},

       {SNARL_SITE_FILE_TIME,                   ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_FILE_TIME}},

       {SNARL_SITE_DATE_AGE_REGEX_CONSTRUCT,    ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_DATE_AGE_REGEX_CONSTRUCT}},
       {SNARL_SITE_DATE_AGE_COMPARE_STRING,     ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_DATE_AGE_COMPARE_STRING}},
       {SNARL_SITE_DATE_AGE_ALERT_TRIGGER,      ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_DATE_AGE_ALERT_TRIGGER}},

       {SNARL_SITE_STATUS_NORMAL,               ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_STATUS_NORMAL}},
       {SNARL_SITE_STATUS_WARNING,              ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_STATUS_WARNING}},
       {SNARL_SITE_STATUS_CRITICAL,             ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_STATUS_CRITICAL}},

       {SNARL_SITE_GROUP_STATUS_NORMAL,         ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_GROUP_STATUS_NORMAL}},
       {SNARL_SITE_GROUP_STATUS_WARNING,        ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_GROUP_STATUS_WARNING}},
       {SNARL_SITE_GROUP_STATUS_CRITICAL,       ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_GROUP_STATUS_CRITICAL}},

       {SNARL_SITE_DATE_SUSPEND_START,          ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_DATE_SUSPEND_START}},
       {SNARL_SITE_DATE_SUSPEND_END,            ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_DATE_SUSPEND_END}},

       {SNARL_SITE_RELIABILITY,                 ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_RELIABILITY}},
       {SNARL_SITE_MTBF_W,                      ASN_TIMETICKS,     RONLY, var_snarl_table, 1, {SNARL_SITE_MTBF_W}},
       {SNARL_SITE_MTTR_W,                      ASN_TIMETICKS,     RONLY, var_snarl_table, 1, {SNARL_SITE_MTTR_W}},
       {SNARL_SITE_WARNINGS,                    ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_WARNINGS}},
       {SNARL_SITE_MTBF_C,                      ASN_TIMETICKS,     RONLY, var_snarl_table, 1, {SNARL_SITE_MTBF_C}},
       {SNARL_SITE_MTTR_C,                      ASN_TIMETICKS,     RONLY, var_snarl_table, 1, {SNARL_SITE_MTTR_C}},
       {SNARL_SITE_CRITICALS,                   ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_CRITICALS}},

       {SNARL_SITE_GROUP_RELIABILITY,           ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_GROUP_RELIABILITY}},
       {SNARL_SITE_GROUP_MTBF_W,                ASN_TIMETICKS,     RONLY, var_snarl_table, 1, {SNARL_SITE_GROUP_MTBF_W}},
       {SNARL_SITE_GROUP_MTTR_W,                ASN_TIMETICKS,     RONLY, var_snarl_table, 1, {SNARL_SITE_GROUP_MTTR_W}},
       {SNARL_SITE_GROUP_WARNINGS,              ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_GROUP_WARNINGS}},
       {SNARL_SITE_GROUP_MTBF_C,                ASN_TIMETICKS,     RONLY, var_snarl_table, 1, {SNARL_SITE_GROUP_MTBF_C}},
       {SNARL_SITE_GROUP_MTTR_C,                ASN_TIMETICKS,     RONLY, var_snarl_table, 1, {SNARL_SITE_GROUP_MTTR_C}},
       {SNARL_SITE_GROUP_CRITICALS,             ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_GROUP_CRITICALS}},

       {SNARL_SITE_LAST_RUN,                    ASN_TIMETICKS,     RONLY, var_snarl_table, 1, {SNARL_SITE_LAST_RUN}},
       {SNARL_SITE_LAST_RUN_ABS_TIME_STRING,    ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_LAST_RUN_ABS_TIME_STRING}},

       {SNARL_SITE_NEXT_RUN,                    ASN_TIMETICKS,     RONLY, var_snarl_table, 1, {SNARL_SITE_NEXT_RUN}},
       {SNARL_SITE_NEXT_RUN_ABS_TIME_STRING,    ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_NEXT_RUN_ABS_TIME_STRING}},

       {SNARL_SITE_FIRST_RUN_ABS_TIME_STRING,   ASN_OCTET_STR,     RONLY, var_snarl_table, 1, {SNARL_SITE_FIRST_RUN_ABS_TIME_STRING}},

       {SNARL_SITE_TOTAL_TRANSFERS,             ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_TOTAL_TRANSFERS}},
       {SNARL_SITE_TOTAL_TRANSFER_VOLUME,       ASN_INTEGER,       RONLY, var_snarl_table, 1, {SNARL_SITE_TOTAL_TRANSFER_VOLUME}}
    };

       /* ------------------------------------ */
       /* Define the OID pointer to the top    */
       /* of the mib tree that we're           */
       /* registering underneath               */
       /* ------------------------------------ */

       oid snarl_info_oid[]      = { SNARLSNMP_MIB,1,1 };
       oid snarl_variables_oid[] = { SNARLSNMP_MIB,1,2,1 };



       /* ------------------------------------ */
       /* register ourselves with the agent    */
       /* to handle our mib tree               */
       /* ------------------------------------ */

       REGISTER_MIB("snarlsnmp/snarl", snarl_info,  variable2, snarl_info_oid);
       REGISTER_MIB("snarlsnmp/snarl", snarl_table, variable2, snarl_variables_oid);


       /* ------------------------------------ */
       /* register ourselves with the agent    */
       /* to handle config token               */
       /* ------------------------------------ */

       snmpd_register_config_handler("sitewatch", snarl_parse_config, snarl_free_config,
                                     "sitewatch [name | token] [id <id> snarltoken arg1 arg2]");

       /* ------------------------------------ */
       /* setup globals                        */
       /* ------------------------------------ */


       /* ------------------------------------ */
       /* globalStartUpType contains           */
       /* information concerning if the        */
       /* application was cold- or warmstarted */
       /* ------------------------------------ */
       globalStartUpCount                      = 0;
       globalStartUpType                       = SNARLSNMP_TRAP_SUBTASK_ID_COLDSTART;

       globalPID                               = getpid();

       snprintf( globalID, SNARL_TASK_MAX_SITENAME, "%s",       SNARL_APP_NAME  );
       len = min(          SNARL_TASK_MAX_SITENAME - 1, strlen( SNARL_APP_NAME ));

       globalID[len]                           = '\0';

       /* ------------------------------------ */
       /* the global first run time            */
       /* this value cannot be reset except    */
       /* for restarting the daemon with       */
       /* kill -9                              */
       /* ------------------------------------ */
       snarl_time(&globalFirstRun);
       globalHeartBeatInterval                 = 900;
       globalHeartBeatCount                    = 0;

       globalTotalTransfers                    = 0;
       globalTotalTransferVolume               = 0;

       pthread_mutex_init(         &globalTotalTransferLock, NULL );


       /* ------------------------------------ */
       /* the global pointers to task data     */
       /* structure - as a linked list         */
       /* ------------------------------------ */
       globalTaskArray                         = &globalTaskArrayStruct;
       globalTaskArray->firstsnarltask         = NULL;
       globalTaskArray->lastsnarltask          = NULL;
       globalTaskArray->taskCount              = 0;

       globalRecycledTaskArray                 = &globalRecycledTaskArrayStruct;
       globalRecycledTaskArray->firstsnarltask = NULL;
       globalRecycledTaskArray->lastsnarltask  = NULL;
       globalRecycledTaskArray->taskCount      = 0;


       /* ------------------------------------ */
       /* the global task counter              */
       /* this variable holds the number of    */
       /* currently active threads             */
       /* ------------------------------------ */
       globalRunningTaskCount      = 0;
       pthread_mutex_init(         &globalRunningTaskCountLock, NULL);


       /* ------------------------------------ */
       /* the global curl inited flag          */
       /* this flag demonstrates the state     */
       /* of the curl library                  */
       /* ------------------------------------ */
       globalCURLinited          = FALSE;


       /* ------------------------------------ */
       /* the global scheduler inited flag     */
       /* this variable group demonstrates     */
       /* the status of the event scheduler    */
       /* ------------------------------------ */
       globalSchedulerInited     = FALSE;
       pthread_mutex_init(         &globalSchedulerInitLock,  NULL);
       pthread_cond_init(          &globalSchedulerInitDone,  NULL);

       globalSchedulerRegistry   = 0;

       globalScheduleEvents       = FALSE;
       pthread_mutex_init(         &globalScheduleEventsLock, NULL);
       pthread_cond_init(          &globalScheduleEventsAck,  NULL);


       /* ------------------------------------ */
       /* the getdate function                 */
       /* is not thread safe - use a lock      */
       /* ------------------------------------ */
       pthread_mutex_init(         &globalGetDateLock,    NULL);
       pthread_mutex_init(         &globalSnarlTimeLock,  NULL);
       pthread_mutex_init(         &globalSnarlCtimeLock, NULL);


       /* ------------------------------------ */
       /* the global event queue               */
       /* this variable group demonstrates the */
       /* state of the event queue             */
       /* ------------------------------------ */
       globalEventQueueInited    = FALSE;
       pthread_mutex_init(         &globalEventLock,      NULL);
       pthread_cond_init(          &globalEventAvailable, NULL);


       /* ------------------------------------ */
       /* the global add index variable        */
       /* this variable holds the integer that */
       /* should be added to the index no for  */
       /* snmp tables for proxies - that means */
       /* that one task for instance should    */
       /* have index 1 and another sibling     */
       /* task that connects via a proxy       */
       /* should have index 1001               */
       /* ------------------------------------ */
       globalAddIndex            = SNARL_TASK_DEFAULT_PROXY_INDEX_ADD;


       /* ------------------------------------ */
       /* globalTrapAttack sets the maximum    */
       /* number of traps sent for each        */
       /* found conditional trap               */
       /* each trap is sent once on one task   */
       /* run                                  */
       /* if there is a status change after    */
       /* less than globalTrapAttck traps have */
       /* been sent - the individual trap      */
       /* counters are reset                   */
       /* ------------------------------------ */
       globalTrapAttack          = SNARL_TASK_DEFAULT_TRAP_ATTACK;


       /* ------------------------------------ */
       /* now create a globalTaskTemplate      */
       /* filled with all defaults             */
       /* ------------------------------------ */
       globalTaskTemplate = &globalTaskTemplateStruct;
       globalDefaultsMask = &globalDefaultsMaskStruct;

       memset( globalTaskTemplate, 0,        sizeof(snarltask) );
       memset( globalDefaultsMask, UINT_MAX, sizeof(snarltask) );

       globalTaskTemplate->index                               = 0;


       /* ------------------------------------ */
       /* don't mask index it is set automat.  */
       /* ------------------------------------ */
     /*globalDefaultsMask->index                               = 0;*/

       globalTaskTemplate->interval                            = SNARL_TASK_DEFAULT_INTERVAL;
       globalDefaultsMask->interval                            = 0;

       globalTaskTemplate->alertInterval                       = SNARL_TASK_DEFAULT_ALERT_INTERVAL;
       globalDefaultsMask->alertInterval                       = 0;

       globalTaskTemplate->warningsBeforeCritical              = SNARL_TASK_DEFAULT_WARNINGS_BEFORE_CRITICAL;
       globalDefaultsMask->warningsBeforeCritical              = 0;


       /* ------------------------------------ */
       /* maxDownloadSize 0 means unlimited    */
       /* ------------------------------------ */

       globalTaskTemplate->maxDownloadSize                     = 0;
       globalDefaultsMask->maxDownloadSize                     = 0;

       globalTaskTemplate->mainURL.noCookies                   = TRUE;
       globalDefaultsMask->mainURL.noCookies                   = 0;

       globalTaskTemplate->mainURL.timeout                     = SNARL_TASK_DEFAULT_TIMEOUT;
       globalDefaultsMask->mainURL.timeout                     = 0;
       globalTaskTemplate->mainURL.connect_timeout             = SNARL_TASK_DEFAULT_TIMEOUT;

       globalDefaultsMask->mainURL.connect_timeout             = 0;

       snprintf( globalTaskTemplate->mainURL.agent,         SNARL_TASK_URL_MAX_LENGTH, "%s",       SNARL_APP_NAME  );
       len = min(                                           SNARL_TASK_URL_MAX_LENGTH - 1, strlen( SNARL_APP_NAME ));
       globalTaskTemplate->mainURL.agent[len] = '\0';
       memset(  &globalDefaultsMask->mainURL.agent,              0, SNARL_TASK_URL_MAX_LENGTH );

       globalTaskTemplate->contentMatchURL.noCookies           = TRUE;
       globalDefaultsMask->contentMatchURL.noCookies           = 0;

       globalTaskTemplate->contentMatchURL.timeout             = SNARL_TASK_DEFAULT_TIMEOUT;
       globalDefaultsMask->contentMatchURL.timeout             = 0;

       globalTaskTemplate->contentMatchURL.connect_timeout     = SNARL_TASK_DEFAULT_TIMEOUT;
       globalDefaultsMask->contentMatchURL.connect_timeout     = 0;

       snprintf( globalTaskTemplate->contentMatchURL.agent, SNARL_TASK_URL_MAX_LENGTH, "%s",       SNARL_APP_NAME  );
       len = min(                                           SNARL_TASK_URL_MAX_LENGTH - 1, strlen( SNARL_APP_NAME ));
       globalTaskTemplate->contentMatchURL.agent[len] = '\0';
       memset(  &globalDefaultsMask->contentMatchURL.agent,      0, SNARL_TASK_URL_MAX_LENGTH );

       globalTaskTemplate->sizeMatchURL.noCookies              = TRUE;
       globalDefaultsMask->sizeMatchURL.noCookies              = 0;

       globalTaskTemplate->sizeMatchURL.timeout                = SNARL_TASK_DEFAULT_TIMEOUT;
       globalDefaultsMask->sizeMatchURL.timeout                = 0;

       globalTaskTemplate->sizeMatchURL.connect_timeout        = SNARL_TASK_DEFAULT_TIMEOUT;
       globalDefaultsMask->sizeMatchURL.connect_timeout        = 0;

       snprintf( globalTaskTemplate->sizeMatchURL.agent,    SNARL_TASK_URL_MAX_LENGTH, "%s",       SNARL_APP_NAME  );
       len = min(                                           SNARL_TASK_URL_MAX_LENGTH - 1, strlen( SNARL_APP_NAME ));
       globalTaskTemplate->sizeMatchURL.agent[len] = '\0';
       memset(  &globalDefaultsMask->sizeMatchURL.agent,         0, SNARL_TASK_URL_MAX_LENGTH );

       globalTaskTemplate->performance_warning_threshold_total = SNARL_TASK_DEFAULT_PERFORMANCE_WARNING_THRESHOLD;
       globalDefaultsMask->performance_warning_threshold_total = 0;

       globalTaskTemplate->performance_critical_threshold_total= SNARL_TASK_DEFAULT_PERFORMANCE_CRITICAL_THRESHOLD;
       globalDefaultsMask->performance_critical_threshold_total= 0;

       /* ------------------------------------ */
       /* internal stuff - don't tamper with   */
       /* ------------------------------------ */
       globalTaskTemplate->isVirgin                            = TRUE;
       globalDefaultsMask->isVirgin                            = 0;


       /* ------------------------------------ */
       /* now check which priorities we've     */
       /* got                                  */
       /* ------------------------------------ */
       if( getuid() == 0 ) {

       /* ------------------------------------ */
       /* we are running as root use high      */
       /* priority thread scheduling           */
       /* ------------------------------------ */

           globalTaskTemplate->schedulingParameters.sched_priority = SNARL_TASK_REAL_TIME_PRIORITY;
       } else {
	   globalTaskTemplate->schedulingParameters.sched_priority = 0;
       }

       globalTaskTemplate->general_trap.subTaskID              = SNARLSNMP_TRAP_SUBTASK_ID_GENERAL;
       globalTaskTemplate->synchronize_trap.subTaskID          = SNARLSNMP_TRAP_SUBTASK_ID_SYNCHRONIZE;
       globalTaskTemplate->performance_trap.subTaskID          = SNARLSNMP_TRAP_SUBTASK_ID_PERFORMANCE;
       globalTaskTemplate->httperror_trap.subTaskID            = SNARLSNMP_TRAP_SUBTASK_ID_HTTPCODE;
       globalTaskTemplate->matchRegex_trap.subTaskID           = SNARLSNMP_TRAP_SUBTASK_ID_REGEX;
       globalTaskTemplate->matchContentSize_trap.subTaskID     = SNARLSNMP_TRAP_SUBTASK_ID_CONTENTSIZE;
       globalTaskTemplate->contentMatchURL_trap.subTaskID      = SNARLSNMP_TRAP_SUBTASK_ID_SIZEMATCH;
       globalTaskTemplate->sizeMatchURL_trap.subTaskID         = SNARLSNMP_TRAP_SUBTASK_ID_CONTENTMATCH;
       globalTaskTemplate->matchNumic_trap.subTaskID           = SNARLSNMP_TRAP_SUBTASK_ID_NUMICMATCH;
       globalTaskTemplate->matchDateAge_trap.subTaskID         = SNARLSNMP_TRAP_SUBTASK_ID_DATEAGEMATCH;
       globalTaskTemplate->noUpdateBetween_trap.subTaskID      = SNARLSNMP_TRAP_SUBTASK_ID_NOUPDATE;


       /* ------------------------------------ */
       /* now create a globalTaskCopyMask      */
       /* to copy data that should be saved    */
       /* from one configuration to another    */
       /* this mask is first filled with       */
       /* UINT_MAX - meaning do not copy any   */
       /* thing - then it is filled with zeros */
       /* for those data that absolutely       */
       /* should be copied                     */
       /* ------------------------------------ */
       globalTaskCopyMask = &globalTaskCopyMaskStruct;
       memset(  globalTaskCopyMask,                                UINT_MAX, sizeof(snarltask) );
       globalTaskCopyMask->totalTransfers                        = 0;
       globalTaskCopyMask->totalTransferVolume                   = 0;


       /* ------------------------------------ */
       /* copy status data, but                */
       /* internal pointers and locks          */
       /* ------------------------------------ */
       memset( &globalTaskCopyMask->statusVars,                    0, sizeof(statusVariables) );

       globalTaskCopyMask->statusVars.next_run                   = UINT_MAX;
       memset( &globalTaskCopyMask->statusVars.next_run_string,    UINT_MAX, SNARL_TASK_MAX_DATE_STRING_LENGTH );
       globalTaskCopyMask->statusVars.statusLock                 = (void*)UINT_MAX;
       globalTaskCopyMask->statusVars.parentTask                 = (void*)UINT_MAX;


       /* ------------------------------------ */
       /* now create a globalSiteCopyMask      */
       /* to copy data that should be saved    */
       /* from one configuration to another    */
       /* ------------------------------------ */
       globalSiteCopyMask = &globalTaskCopyMask->siteVarsGroup;

       memset( &globalTaskCopyMask->siteVarsGroup.statusVars,           0, sizeof(statusVariables) );

       globalTaskCopyMask->siteVarsGroup.statusVars.next_run          = UINT_MAX;
       memset( &globalTaskCopyMask->siteVarsGroup.statusVars.next_run_string,    UINT_MAX, SNARL_TASK_MAX_DATE_STRING_LENGTH );
       globalTaskCopyMask->siteVarsGroup.statusVars.statusLock        = (void*)UINT_MAX;
       globalTaskCopyMask->siteVarsGroup.statusVars.parentTask        = (void*)UINT_MAX;


       /* ------------------------------------ */
       /* now create a globalProxyCopyMask     */
       /* to copy data that should be saved    */
       /* from one configuration to another    */
       /* ------------------------------------ */
       globalProxyCopyMask = &globalTaskCopyMask->proxyVarsGroup;

       memset( &globalTaskCopyMask->proxyVarsGroup.statusVars,           0, sizeof(statusVariables) );

       globalTaskCopyMask->proxyVarsGroup.statusVars.next_run          = UINT_MAX;
       memset( &globalTaskCopyMask->proxyVarsGroup.statusVars.next_run_string,    UINT_MAX, SNARL_TASK_MAX_DATE_STRING_LENGTH );
       globalTaskCopyMask->proxyVarsGroup.statusVars.statusLock        = (void*)UINT_MAX;
       globalTaskCopyMask->proxyVarsGroup.statusVars.parentTask        = (void*)UINT_MAX;

       globalTaskCopyMask->isOnlyTemplate                              = UINT_MAX;

}



/***************************************************************
*                                                              *
* module configuration cleanup                                 *
*                                                              *
***************************************************************/

void   snarl_free_config(void)
{
       int             index;
       snarltaskPtr    aTask;


       /* ------------------------------------ */
       /* everytime this daemon starts this    */
       /* function is called no matter if warm */
       /* or cold start                        */
       /* ------------------------------------ */
       globalStartUpCount++;

       if( globalStartUpCount > 1 ) {
           globalStartUpType = SNARLSNMP_TRAP_SUBTASK_ID_WARMSTART;
       }

       /* ------------------------------------ */
       /* first stop the thread manager        */
       /* ------------------------------------ */
        stopThreadManager();


       /* ------------------------------------ */
       /* kill the stash                       */
       /* ------------------------------------ */
       index     = 0;
       while( aTask = find_snarltask( NULL, NULL, &index, globalRecycledTaskArray )) {

           invalidate_snarltask( aTask, globalRecycledTaskArray );
           free( aTask );
       }

       /* ------------------------------------ */
       /* mark all memory as unitialized       */
       /* ------------------------------------ */
       index     = 0;
       while( aTask = find_snarltask( NULL, NULL, &index, globalTaskArray )) {

           aTask->initialized    = FALSE;
           aTask->aboutToCease   = TRUE;
           /*
           aTask->hasData        = FALSE;
           */
       /* ------------------------------------ */
       /* empty the task but keep current data */
       /* we might use it for later recycling  */
       /* ------------------------------------ */
           invalidate_snarltask( aTask, globalTaskArray );

       /* ------------------------------------ */
       /* we are left with an empty trunk      */
       /* of the task - now attach it to the   */
       /* recycled list                        */
       /* ------------------------------------ */
          attach_snarltask( aTask, globalRecycledTaskArray );
       }

       /* ------------------------------------ */
       /* clean up curl                        */
       /* ------------------------------------ */

       /* ------------------------------------ */
       /* in tests curl did not seem to appre  */
       /* ciate being constantly reinitialized */
       /* so it is uncommented here            */
       /* ------------------------------------ */
//       if( globalCURLinited  ) {
//           curl_global_cleanup();
//           globalCURLinited = FALSE;
//       }

       /* ------------------------------------ */
       /* and last but not least the globals   */
       /* ------------------------------------ */
       globalAddIndex               = SNARL_TASK_DEFAULT_PROXY_INDEX_ADD;

       /* ------------------------------------ */
       /* now we create the global task        */
       /* template as allocated memory struct  */
       /* from now on the original global      */
       /* task template is orphaned            */
       /* ------------------------------------ */
       if( aTask = create_snarltask( SNARL_SITE_DEFAULT_SITENAME, SNARL_SITE_DEFAULT_URLNAME, &globalTaskTemplateStruct, globalTaskArray ) ) {

          globalTaskTemplate = aTask;
          globalTaskTemplate->index = 0;

      } else {

          globalTaskTemplate = NULL;

      }
}


/***************************************************************
*                                                              *
* snmp incoming request handling                               *
*                                                              *
***************************************************************/

u_char *var_snarl_table(struct variable   *vp,
                        oid               *name,
                        size_t            *length,
                        int                exact,
                        size_t            *var_len,
                        WriteMethod      **write_method)
{
       static long           long_ret = 0;
       static double         double_ret;

       static snarltask      localTaskCopy;
       int                   iindex, maxIndex, i;
       int                   myIndex;

       snarltaskPtr          aTask;
       siteVarsPtr           siteVars;
       struct HttpPost*      nextForm;

       char                  inbuf[256];
       static char           message[256];
       static char          *empty = "n/a";
       time_t                now, then;


       /* ------------------------------------ */
       /* snmpd does not call a function after */
       /* the configuration file has been read */
       /* completely, the only chance to start */
       /* all threads is here when asked to    */
       /* respond to an incoming request for   */
       /* the first time                       */
       /* ------------------------------------ */

       if( ! globalSchedulerInited ) {

           startScheduler();
       }


       /* ------------------------------------ */
       /* idenify which OID to supply          */
       /* ------------------------------------ */

       if( vp->magic >= SNARL_MAGIC_NUMBER_BOUNDARY ) {

           if (    header_generic(vp, name, length, exact, var_len, write_method) == MATCH_FAILED) {
               return(NULL);
           }

       } else {

           maxIndex = find_maxIndex(globalTaskArray);

           if (header_snarl_table(vp, name, length, exact, var_len, write_method, maxIndex) == MATCH_FAILED) {
               return(NULL);
           } else {

               iindex = name[*length-1];
                myIndex = iindex;

               if( NULL == (aTask = find_snarltask(NULL, NULL, &myIndex, globalTaskArray))) {
                   return(NULL);
               } else {

       /* ------------------------------------ */
       /* everything is o.k. we've found the   */
       /* task in question                     */
       /* ------------------------------------ */
               }
           }
       }



       if( vp->magic >= SNARL_MAGIC_NUMBER_BOUNDARY    ) {

       /* ------------------------------------ */
       /* do specific chechks                  */
       /* ------------------------------------ */

       } else {

       /* ------------------------------------ */
       /* do task specific checks              */
       /* ------------------------------------ */

       /* ------------------------------------ */
       /* don't touch a perishing task         */
       /* ------------------------------------ */

           if( aTask->aboutToCease ) {
               return( NULL );
           }

       /* ------------------------------------ */
       /* only answer if task has any data     */
       /* to report and prevent to give        */
       /* 0-answers that will be interpreted   */
       /* as such                              */
       /* ------------------------------------ */


       /* ------------------------------------ */
       /* strategy to cope with invalid data   */
       /* not yet implemented                  */
       /* ------------------------------------ */

           if( aTask->hasData || ! aTask->hasData ) {

           }

       /* ------------------------------------ */
       /* make a copy of aTask                 */
       /* ------------------------------------ */

           recordStatus( -1, &aTask->statusVars     );
           siteVars = aTask->siteVars;
           recordStatus( -1, &siteVars->statusVars );

           pthread_mutex_lock(     aTask->lock );
               memcpy( (void*)&localTaskCopy, (void*)aTask, sizeof(snarltask) );
               fixPointers( &localTaskCopy );
           pthread_mutex_unlock(   aTask->lock );

           aTask = &localTaskCopy;

       }


       switch (vp->magic)
       {

           case SNARL_NUM_OF_TASKS_MAGIC_NUMBER:
               long_ret = globalTaskArray->taskCount;
               return( u_char *)&long_ret;

           case SNARL_NUM_OF_TRANSFERS_MAGIC_NUMBER:
               long_ret = globalTotalTransfers;
               return( u_char *)&long_ret;

           case SNARL_TRANSFER_VOLUME_MAGIC_NUMBER:
               long_ret = globalTotalTransferVolume;
               return( u_char *)&long_ret;


           case SNARL_SITE_NAME:
               *var_len = strlen(aTask->sitename);
               return (u_char *)(aTask->sitename);

           case SNARL_SITE_ID:
               *var_len = strlen(aTask->urlname);
               return (u_char *)(aTask->urlname);

           case SNARL_SITE_INDEX:
               long_ret = aTask->index;
               return (u_char *)&long_ret;

           case SNARL_SITE_INTERVAL:
               long_ret = aTask->interval;
               return (u_char *)&long_ret;


           case SNARL_SITE_URL:
               *var_len = strlen( aTask->mainURL.url);
                return (u_char *)(aTask->mainURL.url);

           case SNARL_SITE_DYNAMIC_URL_PATTERN:
               *var_len = strlen( aTask->dynamicURLRegex.regex);
                return (u_char *)(aTask->dynamicURLRegex.regex);

           case SNARL_SITE_DYNAMIC_URL_PATTERN_COMP:
               *var_len = strlen( aTask->dynamicURLRegex.regexError_string);
                return (u_char *)(aTask->dynamicURLRegex.regexError_string);

           case SNARL_SITE_DYNAMIC_URL_PATTERN_ERR:
               *var_len = strlen( aTask->dynamicURLRegex.regexMatchingError_string);
                return (u_char *)(aTask->dynamicURLRegex.regexMatchingError_string);

           case SNARL_SITE_DYNAMIC_URL:
               *var_len = strlen( aTask->dynamicURL);
                return (u_char *)(aTask->dynamicURL);

           case SNARL_SITE_GET_DYNAMIC_URL_FROM:
               *var_len = strlen( aTask->getDynamicURLFrom);
                return (u_char *)(aTask->getDynamicURLFrom);


           case SNARL_SITE_PROXY:
               *var_len = strlen( aTask->mainURL.proxy);
                return (u_char *)(aTask->mainURL.proxy);

           case SNARL_SITE_MP_FORM_DATA:
               *var_len = 0;
               nextForm = aTask->mainURL.firstFormItem;

               while( nextForm && (*var_len <= SNARL_TASK_FORM_MAX_LENGTH) ) {
                   sprintf( inbuf, "%s=%s", nextForm->name, nextForm->contents );
                   if( (*var_len + strlen(inbuf) + 2 ) <= sizeof(message)) {
                       strcpy( &message[*var_len], inbuf );
                       (*var_len) += strlen(inbuf);

                       if( nextForm = (nextForm->next) ) {
                           strcpy( &message[*var_len], ", " );
                           *var_len += 2;
                       }
                   }
               }

               message[*var_len] = '\0';
               return (u_char *)(message);

           case SNARL_SITE_FORM_DATA:
               *var_len = strlen( aTask->mainURL.currentformData);
                return (u_char *)(aTask->mainURL.currentformData);

           case SNARL_SITE_EFFECTIVE_URL:
               *var_len = strlen( aTask->mainURL.effectiveURL?aTask->mainURL.effectiveURL:empty);
                return (u_char *)(aTask->mainURL.effectiveURL?aTask->mainURL.effectiveURL:empty);

           case SNARL_SITE_URL_ERROR:
                long_ret = (aTask->mainURL.HTTPCode);
                return (u_char *)&long_ret;



           case SNARL_SITE_1PERF:
               long_ret = (aTask->performance_DNS);
               return (u_char *)&long_ret;

           case SNARL_SITE_2PERF:
               long_ret = (aTask->performance_connect);
               return (u_char *)&long_ret;

           case SNARL_SITE_3PERF:
               long_ret = (aTask->performance_firstByte);
               return (u_char *)&long_ret;

           case SNARL_SITE_4PERF:
               long_ret = (aTask->performance_total);
               return (u_char *)&long_ret;

           case SNARL_SITE_PERFORMANCE_THRESHOLD_WARN:
               long_ret = (aTask->performance_warning_threshold_total);
               return (u_char *)&long_ret;

           case SNARL_SITE_PERFORMANCE_THRESHOLD_CRIT:
               long_ret = (aTask->performance_critical_threshold_total);
               return (u_char *)&long_ret;


           case SNARL_SITE_REGEX:
               *var_len = strlen(aTask->matchRegex_data.regex);
               return (u_char *)(aTask->matchRegex_data.regex);

           case SNARL_SITE_REGEX_COMP:
               *var_len = strlen(aTask->matchRegex_data.regexError_string);
               return (u_char *)(aTask->matchRegex_data.regexError_string);

           case SNARL_SITE_REGEX_MATCH:
               long_ret = (aTask->matchRegex_flag);
               return (u_char *)&long_ret;

           case SNARL_SITE_REGEX_TRAP_ON_MATCH:
               long_ret = (aTask->matchRegex_flag_trap);
               return (u_char *)&long_ret;


           case SNARL_SITE_CONTENT_SIZE:
               long_ret = aTask->matchContentSize;
               return (u_char *)&long_ret;

           case SNARL_SITE_CONTENT_ACTUAL_SIZE:
               long_ret = aTask->matchContentSize_actual;
               return (u_char *)&long_ret;

           case SNARL_SITE_CONTENT_SIZE_MATCH:
               long_ret = (aTask->matchContentSize_flag);
               return (u_char *)&long_ret;

           case SNARL_SITE_CONTENT_SIZE_TRAP_ON_MATCH:
               long_ret = (aTask->matchContentSize_flag_trap);
               return (u_char *)&long_ret;


           case SNARL_SITE_CONTENT_URL:
               *var_len = strlen(aTask->contentMatchURL.url);
               return (u_char *)(aTask->contentMatchURL.url);

           case SNARL_SITE_CONTENT_URL_MATCH:
               long_ret = (aTask->contentMatchURL_flag);
               return (u_char *)&long_ret;

           case SNARL_SITE_CONTENT_URL_TRAP_ON_MATCH:
               long_ret = (aTask->contentMatchURL_flag_trap);
               return (u_char *)&long_ret;

           case SNARL_SITE_CONTENT_URL_MATCH_PROXY:
               *var_len = strlen(aTask->contentMatchURL.proxy);
               return (u_char *)(aTask->contentMatchURL.proxy);

           case SNARL_SITE_CONTENT_URL_MATCH_ERROR:
                long_ret = (aTask->contentMatchURL.HTTPCode);
                return (u_char *)&long_ret;



           case SNARL_SITE_SIZE_URL:
               *var_len = strlen(aTask->sizeMatchURL.url);
               return (u_char *)(aTask->sizeMatchURL.url);

           case SNARL_SITE_SIZE_URL_MATCH:
               long_ret = (aTask->sizeMatchURL_flag);
               return (u_char *)&long_ret;

           case SNARL_SITE_SIZE_URL_TRAP_ON_MATCH:
               long_ret = (aTask->sizeMatchURL_flag_trap);
               return (u_char *)&long_ret;

           case SNARL_SITE_SIZE_URL_MATCH_PROXY:
               *var_len = strlen(aTask->sizeMatchURL.proxy);
               return (u_char *)(aTask->sizeMatchURL.proxy);

           case SNARL_SITE_SIZE_URL_MATCH_ERROR:
                long_ret = (aTask->sizeMatchURL.HTTPCode);
                return (u_char *)&long_ret;


           case SNARL_SITE_FETCH_REGEX:
               *var_len = strlen(aTask->varRegex.regex);
               return (u_char *)(aTask->varRegex.regex);

           case SNARL_SITE_FETCH_REGEX_COMP:
               *var_len = strlen(aTask->varRegex.regexError_string);
               return (u_char *)(aTask->varRegex.regexError_string);

           case SNARL_SITE_FETCH_REGEX_ERR:
               *var_len = strlen(aTask->varRegex.regexMatchingError_string);
               return (u_char *)(aTask->varRegex.regexMatchingError_string);

           case SNARL_SITE_REGEX_VAR0:
               *var_len = strlen(aTask->regexMatchContent[0]);
               return (u_char *)(aTask->regexMatchContent[0]);

           case SNARL_SITE_REGEX_VAR1:
               *var_len = strlen(aTask->regexMatchContent[1]);
               return (u_char *)(aTask->regexMatchContent[1]);

           case SNARL_SITE_REGEX_VAR2:
               *var_len = strlen(aTask->regexMatchContent[2]);
               return (u_char *)(aTask->regexMatchContent[2]);

           case SNARL_SITE_REGEX_VAR3:
               *var_len = strlen(aTask->regexMatchContent[3]);
               return (u_char *)(aTask->regexMatchContent[3]);

           case SNARL_SITE_REGEX_VAR4:
               *var_len = strlen(aTask->regexMatchContent[4]);
               return (u_char *)(aTask->regexMatchContent[4]);

           case SNARL_SITE_REGEX_VAR5:
               *var_len = strlen(aTask->regexMatchContent[5]);
               return (u_char *)(aTask->regexMatchContent[5]);

           case SNARL_SITE_REGEX_VAR6:
               *var_len = strlen(aTask->regexMatchContent[6]);
               return (u_char *)(aTask->regexMatchContent[6]);

           case SNARL_SITE_REGEX_VAR7:
               *var_len = strlen(aTask->regexMatchContent[7]);
               return (u_char *)(aTask->regexMatchContent[7]);


           case SNARL_SITE_APPEND_REGEX_VAR_FROM:
               *var_len = strlen(aTask->getRegexResultFrom);
               return (u_char *)(aTask->getRegexResultFrom);

           case SNARL_SITE_APPEND_REGEX_VAR_FROM_INDEX:
                long_ret = (aTask->getRegexResultFromIndex);
                return (u_char *)&long_ret;


           case SNARL_SITE_NUMERICAL_REGEX_INDEX:
                long_ret = (aTask->matchNumic_withRegexIndex);
                return (u_char *)&long_ret;

           case SNARL_SITE_NUMERICAL_COMPARE_STRING:
               *var_len = strlen(aTask->matchNumic_string);
               return (u_char *)(aTask->matchNumic_string);

           case SNARL_SITE_NUMERICAL_ALERT_TRIGGER:
                long_ret = (aTask->matchNumic_flag_trap);
                return (u_char *)&long_ret;


           case SNARL_SITE_FILE_TIME:
                long_ret = (aTask->mainURL.fileTime);
                return (u_char *)&long_ret;


           case SNARL_SITE_DATE_AGE_REGEX_CONSTRUCT:
               *var_len = strlen(aTask->matchDateAge_construct);
               return (u_char *)(aTask->matchDateAge_construct);

           case SNARL_SITE_DATE_AGE_COMPARE_STRING:
               *var_len = strlen(aTask->matchDateAge_string);
               return (u_char *)(aTask->matchDateAge_string);

           case SNARL_SITE_DATE_AGE_ALERT_TRIGGER:
                long_ret = (aTask->matchDateAge_flag_trap);
                return (u_char *)&long_ret;


           case SNARL_SITE_DATE_SUSPEND_START:
               if( aTask->noUpdateBetweenStart != 0 ) {
			       snarl_ctime( message, &aTask->noUpdateBetweenStart, 128 );
			   } else {
				   message[0] = '\0';
			   }
               *var_len = strlen(message);
               return (u_char *)(message);

           case SNARL_SITE_DATE_SUSPEND_END:
               if( aTask->noUpdateBetweenEnd != 0 ) {
			       snarl_ctime( message, &aTask->noUpdateBetweenEnd, 128 );
			   } else {
				   message[0] = '\0';
			   }
               *var_len = strlen(message);
               return (u_char *)(message);


           case SNARL_SITE_STATUS_NORMAL:
                long_ret = (aTask->statusVars.statusNormal);
                return (u_char *)&long_ret;

           case SNARL_SITE_STATUS_WARNING:
                long_ret = (aTask->statusVars.statusWarning);
                return (u_char *)&long_ret;

           case SNARL_SITE_STATUS_CRITICAL:
                long_ret = (aTask->statusVars.statusCritical);
                return (u_char *)&long_ret;


           case SNARL_SITE_GROUP_STATUS_NORMAL:
                siteVars = aTask->siteVars;
                long_ret = (siteVars->statusVars.statusNormal);
                return (u_char *)&long_ret;

           case SNARL_SITE_GROUP_STATUS_WARNING:
                siteVars = aTask->siteVars;
                long_ret = (siteVars->statusVars.statusWarning);
                return (u_char *)&long_ret;

           case SNARL_SITE_GROUP_STATUS_CRITICAL:
                siteVars = aTask->siteVars;
                long_ret = (siteVars->statusVars.statusCritical);
                return (u_char *)&long_ret;


           case SNARL_SITE_RELIABILITY:
                if(  (aTask->statusVars.last_update) && ((aTask->statusVars.last_update - aTask->statusVars.first_run) > 0) ) {

                    long_ret = 10000 *  (double)(aTask->statusVars.last_update - aTask->statusVars.first_run - aTask->statusVars.accumulatedTimeWarning ) /
                                        (double)(aTask->statusVars.last_update - aTask->statusVars.first_run);
                } else {
                    long_ret = 10000;
                }
                return(u_char*)&long_ret;

           case SNARL_SITE_MTBF_W:
                if( aTask->statusVars.accumulatedWarnings > 1 ) {
                    long_ret = 100   * (double)(   aTask->statusVars.accumulatedTimeBetweenWarning) /
                                       (double)(   aTask->statusVars.accumulatedWarnings - 1 );
                } else {
                    long_ret = 0;
                }
                return(u_char*)&long_ret;

           case SNARL_SITE_MTTR_W:
                if( aTask->statusVars.accumulatedWarnings != 0 ) {
                    long_ret = 100   * (double)(   aTask->statusVars.accumulatedTimeWarning) /
                                       (double)(   aTask->statusVars.accumulatedWarnings );
                } else {
                    long_ret = 0;
                }
                return(u_char*)&long_ret;

           case SNARL_SITE_WARNINGS:
                long_ret = aTask->statusVars.accumulatedWarnings;
                return(u_char*)&long_ret;

           case SNARL_SITE_MTBF_C:
                if( aTask->statusVars.accumulatedCriticals > 1 ) {
                    long_ret = 100   * (double)(   aTask->statusVars.accumulatedTimeBetweenCritical) /
                                       (double)(   aTask->statusVars.accumulatedCriticals - 1 );
                } else {
                    long_ret = 0;
                }
                return(u_char*)&long_ret;

           case SNARL_SITE_MTTR_C:
                if( aTask->statusVars.accumulatedCriticals != 0 ) {
                    long_ret = 100   * (double)(   aTask->statusVars.accumulatedTimeCritical) /
                                       (double)(   aTask->statusVars.accumulatedCriticals );
                } else {
                    long_ret = 0;
                }
                return(u_char*)&long_ret;

           case SNARL_SITE_CRITICALS:
                long_ret = aTask->statusVars.accumulatedCriticals;
                return(u_char*)&long_ret;


           case SNARL_SITE_GROUP_RELIABILITY:
                siteVars = aTask->siteVars;

                if( (siteVars->statusVars.last_update) && ((siteVars->statusVars.last_update - siteVars->statusVars.first_run) > 0) ) {
                    long_ret = 10000 *  (double)(siteVars->statusVars.last_update - siteVars->statusVars.first_run - siteVars->statusVars.accumulatedTimeWarning ) /
                                        (double)(siteVars->statusVars.last_update - siteVars->statusVars.first_run);

                } else {
                    long_ret = 10000;
                }
                return(u_char*)&long_ret;

           case SNARL_SITE_GROUP_MTBF_W:
                siteVars = aTask->siteVars;
                if( siteVars->statusVars.accumulatedWarnings > 1 ) {
                    long_ret = 100   * (double)(   siteVars->statusVars.accumulatedTimeBetweenWarning) /
                                       (double)(   siteVars->statusVars.accumulatedWarnings - 1 );
                } else {
                    long_ret = 0;
                }
                return(u_char*)&long_ret;

           case SNARL_SITE_GROUP_MTTR_W:
                siteVars = aTask->siteVars;
                if( siteVars->statusVars.accumulatedWarnings != 0 ) {
                    long_ret = 100   * (double)(   siteVars->statusVars.accumulatedTimeWarning) /
                                       (double)(   siteVars->statusVars.accumulatedWarnings );
                } else {
                    long_ret = 0;
                }
                return(u_char*)&long_ret;

           case SNARL_SITE_GROUP_WARNINGS:
                siteVars = aTask->siteVars;
                long_ret = siteVars->statusVars.accumulatedWarnings;
                return(u_char*)&long_ret;

           case SNARL_SITE_GROUP_MTBF_C:
                siteVars = aTask->siteVars;
                if( siteVars->statusVars.accumulatedCriticals > 1 ) {
                    long_ret = 100   * (double)(   siteVars->statusVars.accumulatedTimeBetweenCritical) /
                                       (double)(   siteVars->statusVars.accumulatedCriticals - 1 );
                } else {
                    long_ret = 0;
                }
                return(u_char*)&long_ret;

           case SNARL_SITE_GROUP_MTTR_C:
                siteVars = aTask->siteVars;
                if( siteVars->statusVars.accumulatedCriticals != 0 ) {
                    long_ret = 100   * (double)(   siteVars->statusVars.accumulatedTimeCritical) /
                                       (double)(   siteVars->statusVars.accumulatedCriticals );
                } else {
                    long_ret = 0;
                }
                return(u_char*)&long_ret;

           case SNARL_SITE_GROUP_CRITICALS:
                siteVars = aTask->siteVars;
                long_ret = siteVars->statusVars.accumulatedCriticals;
                return(u_char*)&long_ret;


           case SNARL_SITE_LAST_RUN:
                if(aTask->hasData ) {
                    snarl_time(&now);
                    long_ret = ((now - aTask->statusVars.last_run)*100);
                } else {
                    long_ret = 0;
                }
                return (u_char *)&long_ret;

           case SNARL_SITE_LAST_RUN_ABS_TIME_STRING:
                if(aTask->hasData ) {
                    *var_len = strlen(aTask->statusVars.last_run_string);
                    return (u_char *)(aTask->statusVars.last_run_string);
                } else {
                    strcpy(message, "task has not started yet or was never at status NORMAL");
                    *var_len = strlen(message);
                    return (u_char *)(message);
                }


           case SNARL_SITE_NEXT_RUN:
                snarl_time(&now);
                if(   !( aTask->task_waitForExternalTrigger)
                    && ((long_ret = (aTask->statusVars.next_run - now) * 100) >= 0)
                  ) {
                    /* everything went well */
                } else {
                    long_ret = 0;
                }
                return (u_char *)&long_ret;

           case SNARL_SITE_NEXT_RUN_ABS_TIME_STRING:
                snarl_time(&now);
                if(   !( aTask->task_waitForExternalTrigger)
                    && ((aTask->statusVars.next_run - now) >= 0) ) {
                    *var_len = strlen(aTask->statusVars.next_run_string);
                    return (u_char *)(aTask->statusVars.next_run_string);
                } else {

                    if( !(aTask->task_waitForExternalTrigger) ) {
                        strcpy(message, "task is running...");
                    } else {
                        snprintf(message, 255, "task is waiting for %s", aTask->triggerTaskName );
                        if(strlen(message) >= 255) message[255]='\0';
                    }
                    *var_len = strlen(message);
                    return (u_char *)(message);
                }


           case SNARL_SITE_FIRST_RUN_ABS_TIME_STRING:
                if(aTask->hasData ) {
                    *var_len = strlen(aTask->statusVars.first_run_string);
                    return (u_char *)(aTask->statusVars.first_run_string);
                } else {
                    strcpy(message, "task has not started yet or was never at status NORMAL");
                    *var_len = strlen(message);
                    return (u_char *)(message);
                }


           case SNARL_SITE_TOTAL_TRANSFERS:
               long_ret = aTask->totalTransfers;
               return( u_char *)&long_ret;

           case SNARL_SITE_TOTAL_TRANSFER_VOLUME:
               long_ret = aTask->totalTransferVolume;
               return( u_char *)&long_ret;


           default:

               DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_aTask_table\n", vp->magic));
       }

       return NULL;
}


/***************************************************************
*                                                              *
* task / snmp variable identification by OID                   *
* and identification of the logical next OID / snmp variable;  *
* this function is a customized version of the similar snmp    *
* framework function 'header_table' designed to  cope with     *
* arbitrarily chosen but unique indices                        *
*                                                              *
***************************************************************/


/***************************************************************
 * header_snarl_table
 *
 * Parameters:
 *    *vp        Variable data.
 *    *name      Fully instantiated OID name.
 *    *length    Length of name.
 *     exact     TRUE if an exact match is desired.
 *    *var_len   Hook for size of returned data type.
 *  (**write_method) Hook for write method (UNUSED).
 *     max
 *
 * Returns:
 *  0   If name matches vp->name (accounting for 'exact') and is
 *          not greater in length than 'max'.
 *  1   Otherwise.
 *
 *
 * Compare 'name' to vp->name for the best match or an exact match (if
 *  requested).  Also check that 'name' is not longer than 'max' if
 *  max is greater-than/equal 0.
 * Store a successful match in 'name', and increment the OID instance if
 *  the match was not exact.
 *
 * 'name' and 'length' are undefined upon failure.
 *
 */

int header_snarl_table( struct variable   *vp,
                        oid               *name,
                        size_t            *length,
                        int                exact,
                        size_t            *var_len,
                        WriteMethod      **write_method,
                        int max)
{
       int     i,
       rtest;    /* Set to:     -1   If name < vp->name,
                  *              1   If name > vp->name,
                  *              0   Otherwise.
                  */

       int     currentMaxSubIndex;
       struct  snarltask   *someTask;
       oid     newname[MAX_OID_LEN];


       /* ------------------------------------ */
       /* 1. check for equality and find non   */
       /* matching trees                       */
       /* ------------------------------------ */

       for(i=0,rtest=0; i < (int) vp->namelen && i < (int)(*length) && !rtest; i++) {
         if (name[i] != vp->name[i]) {
           if (name[i] < vp->name[i])
             rtest = -1;
           else
             rtest = 1;
         }
       }


       /* ------------------------------------ */
       /* 2. check if exact match was required */
       /* ------------------------------------ */

       if  (rtest > 0 ||
           (rtest == 0 && !exact && (int)(vp->namelen+1) < (int) *length) ||
           (exact == 1 && (rtest || (int)*length != (int)(vp->namelen+1)))) {
         if (var_len)
         *var_len = 0;
             return MATCH_FAILED;
       }


       /* ------------------------------------ */
       /* 3. create new oid                    */
       /* ------------------------------------ */

       memset(newname, 0, sizeof(newname));


       /* ------------------------------------ */
       /* 4. copy the new oid with             */
       /* a) increased or                      */
       /* b) same tree structure               */
       /* ------------------------------------ */

       if (((int) *length) <= (int) vp->namelen || rtest == -1) {

       /* ------------------------------------ */
       /* start a new tree                     */
       /* ------------------------------------ */

         memmove(newname, vp->name, (int)vp->namelen * sizeof (oid));
         newname[vp->namelen] = find_minIndex(globalTaskArray);
         *length = vp->namelen+1;
       }
       else {


         *length = vp->namelen+1;
         memmove(newname, name, (*length) * sizeof(oid));

       /* ------------------------------------ */
       /* increase the last oid number         */
       /* ------------------------------------ */

         if (!exact) {
           newname[*length-1] = find_nextIndex( name[*length-1], globalTaskArray  );

           if( -1 == newname[*length-1] ) {

       /* ------------------------------------ */
       /* we're at the end                     */
       /* ------------------------------------ */

             newname[*length-1] = name[*length-1] + 1;
           }
         } else {

       /* ------------------------------------ */
       /* stick with exact oid number          */
       /* ------------------------------------ */

           newname[*length-1] = name[*length-1] + 1;

           if( NULL != find_IndexedTask(name[*length-1], EXACT, globalTaskArray ) ) {
               newname[*length-1] = name[*length-1];
           } else {
             if (var_len)
           *var_len = 0;
                return MATCH_FAILED;
           }
         }
       }

       /* ------------------------------------ */
       /* 5. if new tree exceeds max index fail*/
       /* ------------------------------------ */

       if (max >= 0 && ((int)newname[*length-1] > max)) {
         if(var_len)
           *var_len = 0;
         return MATCH_FAILED;
       }


       /* ------------------------------------ */
       /* everything o.k. return new oid       */
       /* ------------------------------------ */

       memmove(name, newname, (*length) * sizeof(oid));
       if (write_method)
         *write_method = 0;
       if (var_len)
         *var_len = sizeof(long);   /* default */
       return(MATCH_SUCCEEDED);
}
/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_manage_config.c                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* this code handles the interaction between net-snmp and       *
* SNARLsnmp MIB module                                         *
*                                                              *
* here the configuration takes place. snarl_parse_config() is  *
* called by net-snmp's snmpd once for each line in snmpd.conf  *
* beginning with 'sitewatch'. the token 'sitewatch' was        *
* registered with a macro in file snarl_module_mibapi.c, the   *
* main interface to the snmp daemon                            *
*                                                              *
***************************************************************/

void snarl_parse_config(const char *token, char* cptr)
{
    char                 sitename[SNARL_TASK_MAX_SITENAME+1];
    char                 urlname[SNARL_TASK_MAX_SITENAME+1];
    char                 agentname[SNARL_TASK_URL_MAX_LENGTH+1];
    char                 referer[SNARL_TASK_URL_MAX_LENGTH+1];

    char                 proxy[SNARL_TASK_PROXY_MAX_LENGTH+1];
    char                 url[SNARL_TASK_URL_MAX_LENGTH+1];
    char                 passphrase[STRINGMAX+1];

    char                 aToken[SNARL_TASK_MAX_LENGTH_TOKEN+1];

    char                 inbuf[STRINGMAX+1];
    char                 inbuf2[STRINGMAX+1];
    int                  tokenID, subTokenID;
    int                  newIndex;
    char*                configptr;

    int                  index         = 0;
    int                  index2        = 0;
    int                  len;
    int                  foundToken    = FALSE;
    int                  foundSiteVars = FALSE;
    int                  foundProxyVars= FALSE;
    int                  keepAlive     = FALSE;
    int                  done          = FALSE;

    snarltaskPtr         aTask         = NULL;
    snarltaskPtr         aTask2        = NULL;
    snarltaskPtr         newTask       = NULL;

    siteVarsPtr          siteVarsTemplate;
    proxyVarsPtr         proxyVarsTemplate;

    int                  matchSize;
    time_t               now;
    struct tm            *aTime;


    if (globalTaskArray->taskCount < MAXSNARL)
    {
       /* ------------------------------------ */
       /* is there any token at all ?          */
       /* ------------------------------------ */

        if( cptr  ) {

            configptr = copy_nword(cptr, sitename, SNARL_TASK_MAX_SITENAME );
            tokenID   = matchToken( sitename );

        } else {

            return;

        }

       /***************************************************************
       *                                                              *
       * "ResetDefaults" directive                                    *
       *                                                              *
       ***************************************************************/

       if( tokenID == SNARL_SITE_RESET_DEFAULTS ) {

       /* ------------------------------------ */
       /* this was global one                  */
       /* ------------------------------------ */

       /* ------------------------------------ */
       /* now we create the global task        */
       /* template as allocated memory struct  */
       /* from now on the original global      */
       /* task template is orphaned            */
       /* ------------------------------------ */
            if( newTask = create_snarltask( SNARL_SITE_DEFAULT_SITENAME, SNARL_SITE_DEFAULT_URLNAME, &globalTaskTemplateStruct, globalTaskArray ) ) {

       /* ------------------------------------ */
       /* now we need to detach the old        */
       /* defaults template.                   */
       /* The defaults template used to be the */
       /* very first task in the array now     */
       /* it will be somewhere in the middle   */
       /* ------------------------------------ */
               detach_snarltask( globalTaskTemplate, globalTaskArray );
               free( globalTaskTemplate );

               globalTaskTemplate = newTask;
               globalTaskTemplate->index = 0;

           } else {

               globalTaskTemplate = NULL;

           }

           return;
       }

       /* ------------------------------------ */
       /* now come the token that need one     */
       /* argument                             */
       /* ------------------------------------ */

       if( configptr == NULL ) {
           return;
       }

       /***************************************************************
       *                                                              *
       * "AddIndexOnProxy" directive                                  *
       *                                                              *
       ***************************************************************/

       if( tokenID == SNARL_SITE_PROXY_ADD_INDEX ) {

       /* ------------------------------------ */
       /* this was global one                  */
       /* ------------------------------------ */

           if( configptr ) {
               configptr = copy_nword( configptr, aToken, SNARL_TASK_MAX_LENGTH_TOKEN );
               globalAddIndex = abs(atoi(aToken));
	       }
           return;
       }


       /***************************************************************
       *                                                              *
       * "TrapAttack" directive                                       *
       *                                                              *
       ***************************************************************/

       if( tokenID == SNARL_SITE_TRAP_ATTACK ) {

       /* ------------------------------------ */
       /* this was global one                  */
       /* ------------------------------------ */

           if( configptr ) {
               configptr = copy_nword( configptr, aToken, SNARL_TASK_MAX_LENGTH_TOKEN );
               globalTrapAttack = abs(atoi(aToken));
		   }
           return;
       }



       /***************************************************************
       *                                                              *
       * "HeartBeat" directive                                        *
       *                                                              *
       ***************************************************************/

       if( tokenID == SNARL_SITE_HEARTBEAT_INTERVAL ) {

       /* ------------------------------------ */
       /* this was global one                  */
       /* ------------------------------------ */

           if( configptr ) {
               configptr = copy_nword( configptr, aToken, SNARL_TASK_MAX_LENGTH_TOKEN );
               globalHeartBeatInterval = abs(atoi(aToken));
		   }
           return;
       }

       /***************************************************************
       *                                                              *
       * "ConfigID" directive                                         *
       *                                                              *
       ***************************************************************/

      if( tokenID == SNARL_SITE_CONFIG_ID ) {

       /* ------------------------------------ */
       /* this was global one                  */
       /* ------------------------------------ */

           if( configptr ) {
			   configptr = copy_nword( configptr, aToken, SNARL_TASK_MAX_LENGTH_TOKEN );

			   snprintf( globalID, SNARL_TASK_MAX_SITENAME, "%s", aToken  );
			   len = min( SNARL_TASK_MAX_SITENAME - 1, strlen( aToken ));
			   globalID[len]                           = '\0';
		   }

           return;
       }

       /* ------------------------------------ */
       /* now come the token that need two     */
       /* arguments after sitewatch            */
       /* ------------------------------------ */

       if( configptr == NULL ) {
           return;
       }

       configptr = copy_nword( configptr, aToken, SNARL_TASK_MAX_LENGTH_TOKEN );

       /* ------------------------------------ */
       /* now we should have one of the        */
       /* following token in array "aToken"    */
       /*                                      */
       /*  - proxy affecting the whole site    */
       /*  - id specifying the url             */
       /* ------------------------------------ */


       /* ------------------------------------ */
       /* now it is time to decide what to do  */
       /* with this configuration              */
       /* ------------------------------------ */

       tokenID = matchToken( aToken );


       /***************************************************************
       *                                                              *
       * "proxy" directive                                            *
       *                                                              *
       ***************************************************************/

       if( tokenID == SNARL_SITE_PROXY ) {

          /* this is a proxy directive */

           foundToken = TRUE;
           if( configptr ) {
               configptr = copy_nword( configptr, proxy, SNARL_TASK_PROXY_MAX_LENGTH);

       /* ------------------------------------ */
       /* if there is a directive after        */
       /* proxy <proxy> it could be the        */
       /* KeepAlive directive                  */
       /* ------------------------------------ */

               keepAlive = FALSE;
	       }

		   if( configptr ) {

			   configptr = copy_nword( configptr, aToken, SNARL_TASK_MAX_LENGTH_TOKEN);
			   tokenID   = matchToken( aToken );

			   if( tokenID == SNARL_SITE_PROXY_KEEPALIVE ) {
				   keepAlive = TRUE;
			   }

		   }

           clone_proxy_task( sitename, proxy, keepAlive );

           done = TRUE;
       }


       /***************************************************************
       *                                                              *
       * "id" directive                                               *
       *                                                              *
       ***************************************************************/

        if( tokenID == SNARL_SITE_ID ) {

       /* ------------------------------------ */
       /* this is an ID directive              */
       /* ------------------------------------ */

           foundToken = TRUE;

           if( configptr ) {
               configptr  = copy_nword( configptr, urlname, SNARL_TASK_MAX_SITENAME );

       /* ------------------------------------ */
       /* now we have the ID <urlname> in      */
       /* the array "urlname                   */
       /* let's find the task - if it exists   */
       /* go ahead with the following directive*/
       /* if it doesn't exist create a task    */
       /* ------------------------------------ */

               index = 0;
               aTask = find_snarltask( sitename, urlname, &index, globalTaskArray );

               if( aTask == NULL) {

       /* ------------------------------------ */
       /* we have no matching task             */
       /* so let's create one                  */
       /* ------------------------------------ */

					index2 = 0;
					done   = FALSE;
					while(    (aTask2 = find_snarltask( sitename, urlname, &index2, globalRecycledTaskArray))
							&&(*aTask2->mainURL.proxy == '\0')  ) {

						newTask = create_snarltask( sitename, urlname, NULL, globalTaskArray );

					  /*  mask_snarltask_defaults( newTask );*/
						copy_masked_data( newTask, aTask2, globalTaskCopyMask, sizeof(snarltask) );

						detach_snarltask( aTask2, globalRecycledTaskArray );
						free( aTask2);

						done = TRUE;
						break;
					}


					if( !done ) {
						newTask = create_snarltask( sitename, urlname, globalTaskTemplate, globalTaskArray );
					}

					done = FALSE;

       /* ------------------------------------ */
       /* is this the first entry for this     */
       /* site? if yes create siteVars.        */
       /* that is is a structure holding common*/
       /* variables for all tasks in one site  */
       /* ------------------------------------ */

				   index         = 0;
				   foundSiteVars = FALSE;
				   while( aTask = find_snarltask(sitename, NULL, &index, globalTaskArray ) ) {

					  if( aTask->hasSiteVarsPtr ) {

						  foundSiteVars = TRUE;
						  break;
					  }
				   }


				   if( foundSiteVars && newTask) {
					   newTask->siteVars = aTask->siteVars;
				   } else {


					  siteVarsTemplate = NULL;
					  index = 0;
					  while( aTask = find_snarltask(sitename, NULL, &index, globalRecycledTaskArray ) ) {

						  if( aTask->holdsRecycableSiteVars ) {
							  siteVarsTemplate = &aTask->siteVarsGroup;
							  break;
						  }
					  }

					  if( newTask ) newTask->siteVars = create_siteVars( newTask, siteVarsTemplate );
					  if( newTask && newTask->siteVars ) {
						  newTask->hasSiteVarsPtr = TRUE;
					  }
				   }

       /* ------------------------------------ */
       /* is this the first entry for this     */
       /* site? if yes create proxyVars.       */
       /* that is is a structure holding common*/
       /* variables for all tasks in one proxy */
       /* ------------------------------------ */

				   index          = 0;
				   foundProxyVars = FALSE;
				   while( aTask = find_snarltask(sitename, NULL, &index, globalTaskArray ) ) {

					  if( (! strcmp(aTask->mainURL.proxy, "" )) && (aTask->hasProxyVarsPtr) ) {

						  foundProxyVars = TRUE;
						  break;
					  }
				   }


				   if( foundProxyVars && newTask) {
					   newTask->proxyVars = aTask->proxyVars;
				   } else {


					  proxyVarsTemplate = NULL;
					  index = 0;
					  while( aTask = find_snarltask(sitename, NULL, &index, globalRecycledTaskArray ) ) {

						  if( (! strcmp(aTask->mainURL.proxy, "" )) && (aTask->holdsRecycableProxyVars) ) {
							  proxyVarsTemplate = &aTask->proxyVarsGroup;
							  break;
						  }
					  }

					  if( newTask ) newTask->proxyVars = create_proxyVars( newTask, proxyVarsTemplate, proxyVarsTemplate ? proxyVarsTemplate->keepAlive : FALSE );
					  if( newTask && newTask->proxyVars ) {
						  newTask->hasProxyVarsPtr = TRUE;
					  }
				   }
			   }
           }

       /* ------------------------------------ */
       /* look if there is more - if not we're */
       /* done                                 */
       /* ------------------------------------ */

           if( configptr == NULL ) done = TRUE;
       }


       /**************************************************************/

       /* ------------------------------------ */
       /* if there was another token than      */
       /* "proxy" or "id" break silently       */
       /* ------------------------------------ */

       if( !foundToken ) return;

       if( done ) {

           return;

       } else {

       /* ------------------------------------ */
       /* we anticipate another token          */
       /* let's find interprete it and         */
       /* from here                            */
       /* ------------------------------------ */

           configptr = copy_nword( configptr, aToken, SNARL_TASK_MAX_LENGTH_TOKEN );

           tokenID   = matchToken( aToken );

           index     = 0;

           switch( tokenID ) {


               case SNARL_SITE_URL:
                     if( configptr ) {
                         configptr = copy_nword( configptr, url,  SNARL_TASK_URL_MAX_LENGTH );

                         while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray ) ) {
                             strcpy(aTask->mainURL.url, url);

                             aTask->initialized      = TRUE;
                             aTask->task_performance = TRUE;
                             aTask->task_httpcode    = TRUE;
                         }
                     }
                   break;

               case SNARL_SITE_REFERER:
                     if( configptr ) {
                         configptr = copy_nword( configptr, url,  SNARL_TASK_URL_MAX_LENGTH );

                         while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray ) ) {
                             strcpy(aTask->mainURL.referer, referer);
                             strcpy(aTask->contentMatchURL.referer, referer);
                             strcpy(aTask->sizeMatchURL.referer, referer);

                             aTask->initialized = TRUE;
                         }
                     }
                   break;

               case SNARL_SITE_ALL_AGENTS:
                     if( configptr ) {
                         configptr = copy_nword( configptr, agentname,  SNARL_TASK_URL_MAX_LENGTH );

                         while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray ) ) {
                             strcpy(aTask->mainURL.agent, agentname);
                             strcpy(aTask->sizeMatchURL.agent, agentname);
                             strcpy(aTask->contentMatchURL.agent, agentname);
                             aTask->initialized = TRUE;
                         }

                     }
                   break;

               case SNARL_SITE_MAIN_AGENT:
                     if( configptr ) {
                         configptr = copy_nword( configptr, agentname,  SNARL_TASK_URL_MAX_LENGTH );

                         while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray ) ) {
                             strcpy(aTask->mainURL.agent, agentname);
                             aTask->initialized = TRUE;
                         }

                     }
                   break;

               case SNARL_SITE_CONTENTMATCH_AGENT:
                     if( configptr ) {
                         configptr = copy_nword( configptr, agentname,  SNARL_TASK_URL_MAX_LENGTH );

                         while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray ) ) {
                             strcpy(aTask->contentMatchURL.agent, agentname);
                             aTask->initialized = TRUE;
                         }

                     }
                   break;

               case SNARL_SITE_SIZEMATCH_AGENT:
                     if( configptr ) {
                         configptr = copy_nword( configptr, agentname,  SNARL_TASK_URL_MAX_LENGTH );

                         while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray ) ) {
                             strcpy(aTask->sizeMatchURL.agent, agentname);
                             aTask->initialized = TRUE;
                         }

                     }
                   break;

               case SNARL_SITE_NO_HTTP_CODE_TEST:
                    while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->task_httpcode     = FALSE;
                            aTask->initialized       = TRUE;
                    }
                  break;

               case SNARL_SITE_URL_PASSPHRASE:
                     if( configptr ) {
                         configptr = copy_nword( configptr, passphrase, STRINGMAX );

                         while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray ) ) {
                             strcpy(aTask->mainURL.passphrase, passphrase);
                             aTask->initialized = TRUE;
                         }
                     }
                   break;

               case SNARL_SITE_PROXY:
                     if( configptr ) {
                         configptr = copy_nword( configptr, proxy, SNARL_TASK_PROXY_MAX_LENGTH );

                         while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray ) ) {
                             strcpy(aTask->mainURL.proxy, proxy);
                             aTask->initialized = TRUE;
                         }
                     }
                   break;

               case SNARL_SITE_TRIGGER_TASK:
                     if( configptr ) {
                         configptr = copy_nword( configptr, inbuf, STRINGMAX );

                         while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray ) ) {
                             strcpy(aTask->triggerTaskName, inbuf);
                             aTask->initialized                 = TRUE;
                             aTask->task_waitForExternalTrigger = TRUE;
                         }
                     }
               break;

               case SNARL_SITE_PROPAGATE_ERROR:
					 while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray ) ) {
						 aTask->initialized             = TRUE;
						 aTask->propagateError          = TRUE;
					 }
               break;

               case SNARL_SITE_IGNORE_PROXY_GROUP:
                    while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->ignoreProxyGroup     = TRUE;
                            aTask->initialized          = TRUE;
                    }
                  break;

               case SNARL_SITE_CONNECT_VIA_PROXY_ONLY:
                    while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {

						    if( ! aTask->mainURL.proxy[0] ) {

                                aTask->mainURL.viaProxyOnly = TRUE;
                                aTask->isOnlyTemplate       = TRUE;
                                aTask->initialized          = TRUE;
							}
                    }
                  break;

               case SNARL_SITE_FORM_DATA:
                      configptr = copy_nword( configptr, inbuf, STRINGMAX);

                     index=0;
                     while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray ) ) {
                         aTask->mainURL.isForm        = TRUE;
                         strcpy( aTask->mainURL.formData, inbuf );
                         aTask->initialized           = TRUE;
                     }
                   break;

               case SNARL_SITE_MP_FORM_DATA:

                     while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray ) ) {
                         aTask->mainURL.isMultiForm   = TRUE;
                         aTask->mainURL.firstFormItem = NULL;
                         aTask->mainURL.lastFormItem  = NULL;
                         aTask->initialized           = TRUE;
                      }

                      while( configptr ) {

                        configptr = copy_nword( configptr, inbuf, STRINGMAX );

                        index=0;
                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            curl_formparse( inbuf, &(aTask->mainURL.firstFormItem), &(aTask->mainURL.lastFormItem));
                            aTask->initialized           = TRUE;
                        }
                      }
                  break;

               case SNARL_SITE_PERFORMANCE_THRESHOLD_TOKEN:
                    if( configptr ) {
                        configptr = copy_nword( configptr, inbuf, STRINGMAX );

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->performance_warning_threshold_total  = abs(atoi( inbuf ));

                            if( configptr ) {
                                configptr = copy_nword( configptr, inbuf, STRINGMAX );

                                aTask->performance_critical_threshold_total = abs(atoi( inbuf ));
                                aTask->initialized                          = TRUE;
                            }
                        }
                    }
                  break;

               case SNARL_SITE_PERFORMANCE_TIMEOUT_MAIN:
                    if( configptr ) {
                        configptr = copy_nword( configptr, inbuf, STRINGMAX );

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->mainURL.timeout             = abs(atoi( inbuf ));
                            aTask->mainURL.connect_timeout     = abs(atoi( inbuf ));
                            aTask->initialized                 = TRUE;
                        }
                    }
                  break;

               case SNARL_SITE_PERFORMANCE_TIMEOUT_SMURL:
                    if( configptr ) {
                        configptr = copy_nword( configptr, inbuf, STRINGMAX );

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->sizeMatchURL.timeout             = abs(atoi( inbuf ));
                            aTask->sizeMatchURL.connect_timeout     = abs(atoi( inbuf ));
                            aTask->initialized                      = TRUE;
                        }
                    }
                  break;

               case SNARL_SITE_PERFORMANCE_TIMEOUT_CMURL:
                    if( configptr ) {
                        configptr = copy_nword( configptr, inbuf, STRINGMAX );

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->contentMatchURL.timeout             = abs(atoi( inbuf ));
                            aTask->contentMatchURL.connect_timeout     = abs(atoi( inbuf ));
                            aTask->initialized                         = TRUE;
                        }
                    }
                  break;

               case SNARL_SITE_LOG_TIMING:
                    while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->mainURL.logTiming    = TRUE;
                            aTask->initialized          = TRUE;

                    }
                  break;

               case SNARL_SITE_DUMP_CONTENT:
                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {

                                aTask->mainURL.dumpContent    = TRUE;
                                aTask->initialized            = TRUE;
                        }
                  break;

               case SNARL_SITE_KEEP_CONTENT:
                     if( configptr ) {
                         configptr = copy_nword( configptr, inbuf, min(STRINGMAX, SNARL_TASK_MAX_FILENAME_LENGTH) - 1 );

                         while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray ) ) {
                             strcpy(aTask->mainURL.keepContentFileName, inbuf);
                             aTask->mainURL.keepContent			= TRUE;
                             aTask->initialized                 = TRUE;
                         }
                     }
                  break;

               case SNARL_SITE_INHIBIT_GENERAL:
                    while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->general_trap_inhibit    = TRUE;
                            aTask->initialized             = TRUE;
                    }
                  break;

               case SNARL_SITE_SYNCH_TRAPS:
                    while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->task_synchronize        = TRUE;
                            aTask->initialized             = TRUE;
                    }
                  break;

               case SNARL_SITE_REGEX_MATCH:
                     if( configptr ) {

       /* ------------------------------------ */
       /* here we must be extremely careful    */
       /* everything left in configptr now is  */
       /* part of a regular expression         */
       /* ------------------------------------ */

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            strcpy(aTask->matchRegex_data.regex, configptr);
                            aTask->matchRegex_data.regexError            = regcomp( &(aTask->matchRegex_data.regexBuffer),
                                                                                      aTask->matchRegex_data.regex,
                                                                                      REG_EXTENDED | REG_NOSUB );
                            regerror(  aTask->matchRegex_data.regexError,
                                     &(aTask->matchRegex_data.regexBuffer),
                                       aTask->matchRegex_data.regexError_string,
                                sizeof(aTask->matchRegex_data.regexError_string));

                            aTask->task_matchRegex       = TRUE;
                            aTask->initialized           = TRUE;
                        }
                    }
                  break;

               case SNARL_SITE_REGEX_TRAP_ON_MATCH:
                    while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->matchRegex_flag_trap = TRUE;
                            aTask->initialized          = TRUE;
                    }
                  break;

               case SNARL_SITE_CONTENT_SIZE_MATCH:
                    if( configptr ) {
                        configptr = copy_nword( configptr, inbuf, STRINGMAX );

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->matchContentSize = abs(atoi( inbuf ));
                            aTask->task_matchContentSize = TRUE;
                            aTask->initialized           = TRUE;
                        }
                    }
                  break;

               case SNARL_SITE_CONTENT_SIZE_TRAP_ON_MATCH:
                    while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->matchContentSize_flag_trap = TRUE;
                            aTask->initialized                = TRUE;
                    }
                  break;

               case SNARL_SITE_CONTENT_URL_MATCH:
                    if( configptr ) {
                        configptr = copy_nword( configptr, url, SNARL_TASK_URL_MAX_LENGTH );

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            strcpy(aTask->contentMatchURL.url, url);
                            aTask->task_contentMatchURL  = TRUE;
                            aTask->initialized           = TRUE;
                        }
                    }
                  break;

               case SNARL_SITE_CONTENT_URL_TRAP_ON_MATCH:
                    while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->contentMatchURL_flag_trap = TRUE;
                            aTask->initialized               = TRUE;
                    }
                  break;

               case SNARL_SITE_CONTENT_URL_MATCH_PROXY:
                    if( configptr ) {
                        configptr = copy_nword( configptr, proxy, SNARL_TASK_PROXY_MAX_LENGTH );

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            strcpy(aTask->contentMatchURL.proxy, proxy);
                            aTask->initialized           = TRUE;
                        }
                    }
                  break;

               case SNARL_SITE_CONTENT_URL_MATCH_PASSPHRASE:
                    if( configptr ) {
                        configptr = copy_nword( configptr, passphrase, STRINGMAX );

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            strcpy(aTask->contentMatchURL.passphrase, passphrase);
                            aTask->initialized           = TRUE;
                        }
                    }
                  break;

               case SNARL_SITE_SIZE_URL_MATCH:
                    if( configptr ) {
                        configptr = copy_nword( configptr, url, SNARL_TASK_URL_MAX_LENGTH );

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            strcpy(aTask->sizeMatchURL.url, url);
                            aTask->task_sizeMatchURL     = TRUE;
                            aTask->initialized           = TRUE;
                        }
                    }
                  break;

               case SNARL_SITE_SIZE_URL_TRAP_ON_MATCH:
                    while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->sizeMatchURL_flag_trap = TRUE;
                            aTask->initialized            = TRUE;
                    }
                  break;

                case SNARL_SITE_SIZE_URL_MATCH_PROXY:
                    if( configptr ) {
                        configptr = copy_nword( configptr, proxy,  SNARL_TASK_PROXY_MAX_LENGTH );

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            strcpy(aTask->sizeMatchURL.proxy, proxy);
                            aTask->initialized           = TRUE;
                        }
                    }
                  break;

               case SNARL_SITE_SIZE_URL_MATCH_PASSPHRASE:
                    if( configptr ) {
                        configptr = copy_nword( configptr, passphrase, STRINGMAX );

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            strcpy(aTask->sizeMatchURL.passphrase, passphrase);
                            aTask->initialized           = TRUE;
                        }
                    }
                  break;

               case SNARL_SITE_INTERVAL:
                    if( configptr ) {
                        configptr = copy_nword( configptr, inbuf, STRINGMAX );

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->interval              = abs(atoi( inbuf ));
                            aTask->initialized           = TRUE;
                        }
                    }
                  break;

               case SNARL_SITE_ALERT_INTERVAL:
                    if( configptr ) {
                        configptr = copy_nword( configptr, inbuf, STRINGMAX );

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->alertInterval         = abs(atoi( inbuf ));
                            aTask->initialized           = TRUE;
                        }
                    }
                  break;

               case SNARL_SITE_WARNINGS_BEFORE_CRITICAL:
                    if( configptr ) {
                        configptr = copy_nword( configptr, inbuf, STRINGMAX );

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->warningsBeforeCritical= abs(atoi( inbuf ));
                            aTask->initialized           = TRUE;
                        }
                    }
                  break;

               case SNARL_SITE_INDEX:
                    if( configptr ) {
                        configptr = copy_nword( configptr, inbuf, STRINGMAX );
                        newIndex  = abs(atoi( inbuf ));

                        if( ! find_IndexedTask( newIndex, EXACT, globalTaskArray  )) {
                            if( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                                aTask->index             = newIndex;
                                setTrapIndex( aTask );
                                aTask->initialized       = TRUE;
                            }
                        }
                    }
                  break;

               case SNARL_SITE_DYNAMIC_URL:
                    if( configptr ) {

       /* ------------------------------------ */
       /* here we must be extremely careful    */
       /* everything left in configptr now is  */
       /* part of a regular expression         */
       /* attention: we use line mode here     */
       /* by supplying REG_NEWLINE             */
       /* thus (.*) never includes a newline   */
       /* ------------------------------------ */

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            strcpy(aTask->dynamicURLRegex.regex, configptr);
                            aTask->dynamicURLRegex.regexError            = regcomp( &(aTask->dynamicURLRegex.regexBuffer),
                                                                                      aTask->dynamicURLRegex.regex,
                                                                                       REG_EXTENDED | REG_NEWLINE);
                            regerror(  aTask->dynamicURLRegex.regexError,
                                     &(aTask->dynamicURLRegex.regexBuffer),
                                       aTask->dynamicURLRegex.regexError_string,
                                sizeof(aTask->dynamicURLRegex.regexError_string));

                            aTask->task_fetchDynamicURL                  = TRUE;
                            aTask->initialized                           = TRUE;
                        }
                    }
                  break;

               case SNARL_SITE_APPEND_DYNAMIC_URL_COMPONENT:

                    if( configptr ) {

                        while( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            len = min((SNARL_TASK_MAX_LENGTH_SUBCONFIG_LINE - 1), strlen(configptr));
                            strncpy( aTask->dynurlsubconfigline, configptr, len);
                            aTask->dynurlsubconfigline[len] = '\0';
                            aTask->task_appendDynamicURLComponent = TRUE;
                            aTask->initialized                    = TRUE;
                            aTask->task_performance               = TRUE;
                            aTask->task_httpcode                  = TRUE;
                        }
                    }

                  break;

               case SNARL_SITE_APPEND_DYNAMIC_FORM_COMPONENT:

                    if( configptr ) {

                        while( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            len = min((SNARL_TASK_MAX_LENGTH_SUBCONFIG_LINE - 1), strlen(configptr)) ;
                            strncpy( aTask->dynformsubconfigline, configptr, len);
                            aTask->dynformsubconfigline[len] = '\0';
                            aTask->task_appendDynamicFORMComponent = TRUE;
                            aTask->initialized                     = TRUE;
                        }
                    }

                  break;

               case SNARL_SITE_GET_DYNAMIC_URL_FROM:
                    if( configptr ) {
                        configptr = copy_nword( configptr, inbuf, STRINGMAX );

                        while( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            len = ((SNARL_TASK_MAX_SITENAME - 1) < strlen(inbuf)) ? (SNARL_TASK_MAX_SITENAME - 1): strlen(inbuf);
                            strncpy( aTask->getDynamicURLFrom, inbuf, len);
                            aTask->getDynamicURLFrom[len] = '\0';
                            aTask->task_getDynamicURLFrom = TRUE;
                            aTask->initialized            = TRUE;
                            aTask->task_performance       = TRUE;
                            aTask->task_httpcode          = TRUE;
                        }
                    }
                  break;

               case SNARL_SITE_WITH_COOKIES:
                    while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                        aTask->mainURL.noCookies     = FALSE;
                        aTask->initialized           = TRUE;
                    }
                  break;

               case SNARL_SITE_END_COOKIE_SESSION:
                    while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                        aTask->mainURL.task_killCookies   = TRUE;
                        aTask->initialized                = TRUE;
                    }
                  break;

               case SNARL_SITE_WITH_HEADER:
                    while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                        aTask->mainURL.withHeader    = TRUE;
                        aTask->initialized           = TRUE;
                    }
                  break;

               case SNARL_SITE_RANDOM_SOON_START:
                    while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                        aTask->randomSoonStart       = TRUE;
                        aTask->initialized           = TRUE;
                    }
                  break;

               case SNARL_SITE_FETCH_REGEX:
                    if( configptr ) {

       /* ------------------------------------ */
       /* here we must be extremely careful    */
       /* everything left in configptr now is  */
       /* part of a regular expression         */
       /* ------------------------------------ */

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            strcpy(aTask->varRegex.regex, configptr);
                            aTask->varRegex.regexError            = regcomp( &(aTask->varRegex.regexBuffer),
                                                                               aTask->varRegex.regex,
                                                                               REG_EXTENDED);
                            regerror(  aTask->varRegex.regexError,
                                     &(aTask->varRegex.regexBuffer),
                                       aTask->varRegex.regexError_string,
                                sizeof(aTask->varRegex.regexError_string));

                            aTask->task_fetchRegex                = TRUE;
                            aTask->initialized                    = TRUE;
                        }
                    }
                  break;


               case SNARL_SITE_NUMERICAL_COMPARE_STRING:

                    if( configptr ) {
                        configptr = copy_nword( configptr, inbuf, STRINGMAX );

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->matchNumic_withRegexIndex = (abs(atoi( inbuf )) & SNARL_TASK_MAX_REGEX_MATCHES_MASK);
                            aTask->initialized               = TRUE;
                        }

                        if( configptr ) {
                            configptr = copy_nword( configptr, inbuf, STRINGMAX );

                            index = 0;
                            while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                                aTask->matchNumic_flag_trap = ((inbuf[0] == '<') ? (-1) : (inbuf[0] == '>') ? (1) : (0));
                                aTask->initialized               = TRUE;
                            }

                            if( configptr ) {
                                configptr = copy_nword( configptr, inbuf, STRINGMAX );

                                index = 0;

                                while( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                                    len = ((SNARL_TASK_NUMERICAL_STRING_MAX_LENGTH - 1)< strlen(inbuf)) ? (SNARL_TASK_NUMERICAL_STRING_MAX_LENGTH - 1) : strlen(inbuf);
                                    strncpy( aTask->matchNumic_string, inbuf, len);
                                    aTask->matchNumic_string[len] = '\0';
                                    aTask->task_matchNumic        = TRUE;
                                }
                            } else {

                                while( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                                    aTask->initialized            = FALSE;
                                }
                            }
                        } else {
                                while( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                                    aTask->initialized            = FALSE;
                                }
                        }
                    }
                  break;

               case SNARL_SITE_DATE_AGE_COMPARE_STRING:

                    if( configptr ) {
                        configptr = copy_nword( configptr, inbuf, STRINGMAX );

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            len = ((SNARL_SITE_REGEX_DATE_AGE_STRING_MAX_LENGTH - 1) < strlen(inbuf)) ? (SNARL_SITE_REGEX_DATE_AGE_STRING_MAX_LENGTH - 1) : strlen(inbuf);
                            strncpy( aTask->matchDateAge_tokenline, inbuf, len);
                            aTask->matchDateAge_string[len] = '\0';
                            aTask->initialized               = TRUE;
                        }

                        if( configptr ) {
                            configptr = copy_nword( configptr, inbuf, STRINGMAX );

                            index = 0;
                            while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                                aTask->matchDateAge_flag_trap = ((inbuf[0] == '<') ? (-1) : (inbuf[0] == '>') ? (1) : (0));
                                aTask->initialized               = TRUE;
                            }

                            if( configptr ) {
                                configptr = copy_nword( configptr, inbuf, STRINGMAX );

                                index = 0;

                                while( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                                    len = ((SNARL_SITE_REGEX_DATE_AGE_STRING_MAX_LENGTH - 1) < strlen(inbuf)) ? (SNARL_SITE_REGEX_DATE_AGE_STRING_MAX_LENGTH - 1) : strlen(inbuf);
                                    strncpy( aTask->matchDateAge_string, inbuf, len);
                                    aTask->matchDateAge_string[len] = '\0';
                                    aTask->task_matchDateAge        = TRUE;
                                }
                            } else {

                                while( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                                    aTask->initialized            = FALSE;
                                }
                            }
                        } else {
                                while( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                                    aTask->initialized            = FALSE;
                                }
                        }
                    }
                  break;


               case SNARL_SITE_INDEX_NO_UPDATE_BETWEEN:

                        if( configptr ) {
                            configptr = copy_nword( configptr, inbuf, STRINGMAX );

                            index = 0;
                            while( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                                len = ((SNARL_SITE_REGEX_DATE_AGE_STRING_MAX_LENGTH - 1)< strlen(inbuf)) ? (SNARL_SITE_REGEX_DATE_AGE_STRING_MAX_LENGTH - 1) : strlen(inbuf);
                                strncpy( aTask->noUpdateBetweenStart_string, inbuf, len);
                                aTask->noUpdateBetweenStart_string[len] = '\0';
                            }

                            if( configptr ) {
                                configptr = copy_nword( configptr, inbuf, STRINGMAX );

                                index = 0;

                                while( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                                    len = ((SNARL_SITE_REGEX_DATE_AGE_STRING_MAX_LENGTH - 1 ) < strlen(inbuf)) ? (SNARL_SITE_REGEX_DATE_AGE_STRING_MAX_LENGTH - 1) : strlen(inbuf);
                                    strncpy( aTask->noUpdateBetweenEnd_string, inbuf, len);
                                    aTask->noUpdateBetweenEnd_string[len] = '\0';
                                    aTask->task_noUpdateBetween           = TRUE;

                                    snarl_time(&now);
                                    snarl_getdate( aTask->noUpdateBetweenStart_string, &now, &(aTask->noUpdateBetweenStart));
                                    snarl_getdate( aTask->noUpdateBetweenEnd_string,   &now, &(aTask->noUpdateBetweenEnd));
                                }
                            } else {

                                while( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                                    aTask->initialized                    = FALSE;
                                }
                            }
                        } else {
                                while( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                                    aTask->initialized                    = FALSE;
                                }
                        }
                  break;


               case SNARL_SITE_QUIT_ON_DATASIZE:
                    if( configptr ) {
                        configptr = copy_nword( configptr, inbuf, STRINGMAX );

                        while ( aTask = find_snarltask( sitename, urlname, &index, globalTaskArray) ) {
                            aTask->maxDownloadSize       = abs(atoi( inbuf ));
                            aTask->initialized           = TRUE;
                        }
                    }
                  break;

               default:
                  break;
          }
      }
   }
}

/***************************************************************
*                                                              *
* helper function for config line parsing                      *
*                                                              *
***************************************************************/


int matchToken( char* aToken ) {

           int   numToken = 66;
           int   i;

    static char* token[]  = {
                             "TrapAttack",
                             "Proxy",
                             "KeepAlive",
                             "ConnectViaProxyOnly",
                             "RandomSoonStart",
                             "ConfigurationID",
                             "ResetDefaults",
                             "SendSyncTraps",

                             "Id",

                             "Url",
                             "NoHTTPCodeTest",
                             "UrlPassphrase",

                             "MatchContent",
                             "AlertOnMatchContent",

                             "MatchSize",
                             "lertOnMatchSize",

                             "MatchURLContent",
                             "AlertOnMatchURLContent",
                             "MatchURLContentProxy",
                             "MatchURLContentPassphrase",

                             "MatchURLSize",
                             "AlertOnMatchURLSize",
                             "MatchURLSizeProxy",
                             "MatchURLSizePassphrase",

                             "Interval",
                             "AlertInterval",
                             "WarningsBeforeCritical",
                             "Index",
                             "AddIndexOnProxy",

                             "MultipartForm",
                             "FormData",

                             "FetchDynamicURL",
                             "GetDynamicURLFrom",
                             "IgnoreProxyGroup",

                             "WithCookies",
                             "EndCookieSession",
                             "WithHeader",

                             "PerformanceThreshold",
                             "MainPerformanceTimeout",
                             "SizeMatchUrlPerformanceTimeout",
                             "ContentMatchUrlPerformanceTimeout",

                             "LogTiming",
                             "DumpContent",
                             "KeepContent",
                             "NoGeneralError",

                             "AppendDynamicURLComponent",
                             "AppendDynamicFORMData",
                             "Monitoringtime",
                             "Timestamp",
                             "Clocktime",
                             "RegexOf",
                             "StaticText",
                             "&",

                             "FetchRegex",

                             "NumRegexCompare",
                             "AgeRegexDateCompare",

                             "NoUpdateBetween",
                             "HeartBeatInterval",

                             "TriggerTask",
                             "PropagateError",
                             "AllAgents",
                             "MainAgent",
                             "SizeMatchAgent",
                             "ContentMatchAgent",

                             "Referer",

                             "QuitOnDataSizeGreater"

                           };

    static int   tokenID[]= {
                             SNARL_SITE_TRAP_ATTACK,
                             SNARL_SITE_PROXY,
                             SNARL_SITE_PROXY_KEEPALIVE,
                             SNARL_SITE_CONNECT_VIA_PROXY_ONLY,
                             SNARL_SITE_RANDOM_SOON_START,
                             SNARL_SITE_CONFIG_ID,
                             SNARL_SITE_RESET_DEFAULTS,
                             SNARL_SITE_SYNCH_TRAPS,

                             SNARL_SITE_ID,

                             SNARL_SITE_URL,
                             SNARL_SITE_NO_HTTP_CODE_TEST,
                             SNARL_SITE_URL_PASSPHRASE,

                             SNARL_SITE_REGEX_MATCH,
                             SNARL_SITE_REGEX_TRAP_ON_MATCH,

                             SNARL_SITE_CONTENT_SIZE_MATCH,
                             SNARL_SITE_CONTENT_SIZE_TRAP_ON_MATCH,

                             SNARL_SITE_CONTENT_URL_MATCH,
                             SNARL_SITE_CONTENT_URL_TRAP_ON_MATCH,
                             SNARL_SITE_CONTENT_URL_MATCH_PROXY,
                             SNARL_SITE_CONTENT_URL_MATCH_PASSPHRASE,

                             SNARL_SITE_SIZE_URL_MATCH,
                             SNARL_SITE_SIZE_URL_TRAP_ON_MATCH,
                             SNARL_SITE_SIZE_URL_MATCH_PROXY,
                             SNARL_SITE_SIZE_URL_MATCH_PASSPHRASE,

                             SNARL_SITE_INTERVAL,
                             SNARL_SITE_ALERT_INTERVAL,
                             SNARL_SITE_WARNINGS_BEFORE_CRITICAL,
                             SNARL_SITE_INDEX,
                             SNARL_SITE_PROXY_ADD_INDEX,

                             SNARL_SITE_MP_FORM_DATA,
                             SNARL_SITE_FORM_DATA,

                             SNARL_SITE_DYNAMIC_URL,
                             SNARL_SITE_GET_DYNAMIC_URL_FROM,
                             SNARL_SITE_IGNORE_PROXY_GROUP,

                             SNARL_SITE_WITH_COOKIES,
                             SNARL_SITE_END_COOKIE_SESSION,
                             SNARL_SITE_WITH_HEADER,

                             SNARL_SITE_PERFORMANCE_THRESHOLD_TOKEN,
                             SNARL_SITE_PERFORMANCE_TIMEOUT_MAIN,
                             SNARL_SITE_PERFORMANCE_TIMEOUT_SMURL,
                             SNARL_SITE_PERFORMANCE_TIMEOUT_CMURL,

                             SNARL_SITE_LOG_TIMING,
                             SNARL_SITE_DUMP_CONTENT,
                             SNARL_SITE_KEEP_CONTENT,
                             SNARL_SITE_INHIBIT_GENERAL,

                             SNARL_SITE_APPEND_DYNAMIC_URL_COMPONENT,
                             SNARL_SITE_APPEND_DYNAMIC_FORM_COMPONENT,
                             SNARL_SITE_SUBTOKEN_DYN_URL_COMP_MTIME,
                             SNARL_SITE_SUBTOKEN_DYN_URL_COMP_TIMESTAMP,
                             SNARL_SITE_SUBTOKEN_DYN_URL_COMP_CTIME,
                             SNARL_SITE_SUBTOKEN_DYN_URL_COMP_REGEX,
                             SNARL_SITE_SUBTOKEN_DYN_URL_STATIC_TEXT,
                             SNARL_SITE_SUBTOKEN_DYN_URL_STATIC_AMPERSAND,

                             SNARL_SITE_FETCH_REGEX,

                             SNARL_SITE_NUMERICAL_COMPARE_STRING,
                             SNARL_SITE_DATE_AGE_COMPARE_STRING,

                             SNARL_SITE_INDEX_NO_UPDATE_BETWEEN,
                             SNARL_SITE_HEARTBEAT_INTERVAL,

                             SNARL_SITE_TRIGGER_TASK,
                             SNARL_SITE_PROPAGATE_ERROR,

                             SNARL_SITE_ALL_AGENTS,
                             SNARL_SITE_MAIN_AGENT,
                             SNARL_SITE_SIZEMATCH_AGENT,
                             SNARL_SITE_CONTENTMATCH_AGENT,

                             SNARL_SITE_REFERER,

                             SNARL_SITE_QUIT_ON_DATASIZE

                            };

    for( i=0; i < numToken; i++ ) {

        if( !strcasecmp(aToken, token[i])) {
           return( tokenID[i] );
        }
    }
    return( -1 );
}
/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_manage_switch.c                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/


/***************************************************************
*                                                              *
* this code handles the interaction between the main           *
* SNARLsnmp MIB thread as part of net-snmp's snmpd and         *
* the SNARLsnmp task manager. the task manager starts the      *
* monitoring task threads independently of the main snmpd      *
* thread                                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* start the task manager                                       *
*                                                              *
***************************************************************/

void   startScheduler( void ) {

       pthread_t                   task_registry;
       pthread_attr_t              scheduler_attributes;
       snarltaskPtr                aTask;
       int                         done, index;
       tasktrapdata                startUpTrap;
       time_t                      now;


       /* ------------------------------------ */
       /* remove the global task template      */
       /* ------------------------------------ */

       index = 0;
       while( aTask = find_snarltask(SNARL_SITE_DEFAULT_SITENAME, SNARL_SITE_DEFAULT_URLNAME, &index, globalTaskArray)) {

           invalidate_snarltask(aTask, globalTaskArray);
           free( aTask );
       }


       /* ------------------------------------ */
       /* remove all uninitialized tasks       */
       /* ------------------------------------ */

       index = 0;
       while( aTask = find_snarltask(NULL, NULL, &index, globalTaskArray)) {

           if(   ((aTask->initialized)    == FALSE )
              || ((aTask->isOnlyTemplate) == TRUE  )  ) {

               invalidate_snarltask(aTask, globalTaskArray);
               free( aTask );

           } else {
               aTask->aboutToCease = FALSE;
           }
       }


       /* ------------------------------------ */
       /* now empty the recycled list          */
       /* ------------------------------------ */

       index = 0;
       while( aTask = find_snarltask(NULL, NULL, &index, globalRecycledTaskArray)) {

           invalidate_snarltask(aTask, globalRecycledTaskArray);
           free( aTask );

       }


       /* ------------------------------------ */
       /* init curl                            */
       /* ------------------------------------ */

       if( !globalCURLinited ) {
           curl_global_init(CURL_GLOBAL_ALL);
           globalCURLinited = TRUE;
       }


       /* ------------------------------------ */
       /* prepare to create a scheduler        */
       /* ------------------------------------ */

       pthread_mutex_lock(    &globalSchedulerInitLock );


       /* ------------------------------------ */
       /* scheduler isn't running - so we can  */
       /* manipulate it's globals here         */
       /* ------------------------------------ */

       globalScheduleEvents = TRUE;

       pthread_attr_init(           &scheduler_attributes );
       pthread_attr_setdetachstate( &scheduler_attributes, PTHREAD_CREATE_JOINABLE );


	   if( pthread_create( &task_registry, &scheduler_attributes, (void* )startThreadManager, NULL ) ) {


       /* ------------------------------------ */
       /* problems:                            */
       /* the tasks just won't run - ignore    */
       /* ------------------------------------ */

	   } else {


       /* ------------------------------------ */
       /* wait for initialization to be done   */
       /* ------------------------------------ */

		   globalSchedulerRegistry = task_registry;

		   while( globalSchedulerInited != TRUE ) {
			   pthread_cond_wait( &globalSchedulerInitDone, &globalSchedulerInitLock );
		   }

		   startUpTrap.siteIndex         = 0;
		   snarl_time(&now);
           startUpTrap.eventTime         = (long)(now);
		   startUpTrap.subTaskID         = globalStartUpType;

		   startUpTrap.siteStatus        = SNARLSNMP_TRAP_GENERIC_INFO;
		   startUpTrap.proxyStatus       = SNARLSNMP_TRAP_GENERIC_INFO;
		   startUpTrap.siteGroupStatus   = SNARLSNMP_TRAP_GENERIC_INFO;

		   startUpTrap.siteEventNum      = globalStartUpCount;
		   startUpTrap.proxyEventNum     = globalStartUpCount;
		   startUpTrap.groupEventNum     = globalStartUpCount;

		   strcpy(     startUpTrap.siteMessage,   "Starting "SNARL_APP_NAME);
		   strcpy(     startUpTrap.siteGroupName, SNARL_APP_NAME );
		   sprintf(    startUpTrap.siteName,      "%s", globalID);
		   gethostname(startUpTrap.proxy,         SNARL_TASK_PROXY_MAX_LENGTH);
		   sprintf(    startUpTrap.effectiveURL,  "http://%s", startUpTrap.proxy );



		   snarl_sendtrap( SNARLSNMP_TRAP_GENERIC_INFO,
						   NULL,
						   &startUpTrap
						   );


       /* ------------------------------------ */
       /* queue start scheduling - don't wait  */
       /* for handshake                        */
       /* ------------------------------------ */

	   }

       pthread_mutex_unlock(  &globalSchedulerInitLock );
}

/***************************************************************
*                                                              *
* stop the task manager                                        *
*                                                              *
***************************************************************/

void   stopThreadManager( void ){

       snarltaskPtr       aTask;
       eventPtr           anEventPtr;

       int                pthread_return;
       void              *status_pthread = &pthread_return;

       int                index;
       int                taskCount;
       time_t             earlier, now;
       int                printedDot;
       int                schedulerInited;


       /* ------------------------------------ */
       /* first tell the scheduler             */
       /* to prevent new tasks being fired up  */
       /* ------------------------------------ */

       pthread_mutex_lock(    &globalSchedulerInitLock );
           schedulerInited =   globalSchedulerInited;
       pthread_mutex_unlock(  &globalSchedulerInitLock );


       if( schedulerInited ) {

           pthread_mutex_lock(   &globalScheduleEventsLock );

               if( globalScheduleEvents ) {

       /* ------------------------------------ */
       /* queueing is done with local time     */
       /* ------------------------------------ */
                   snarl_time(&now);
                   queueEvent( SNARL_STOP_SCHEDULE_EVENTS, now, NULL );

                   while( globalScheduleEvents == TRUE ) {
                       pthread_cond_wait( &globalScheduleEventsAck, &globalScheduleEventsLock );
                   }
               }

           pthread_mutex_unlock( &globalScheduleEventsLock );


       /* ------------------------------------ */
       /* then cancel the scheduler            */
       /* ------------------------------------ */

           pthread_cancel( globalSchedulerRegistry );
           pthread_join(   globalSchedulerRegistry, &status_pthread );

           globalSchedulerRegistry = 0;

           pthread_mutex_lock(    &globalSchedulerInitLock );
                                   globalSchedulerInited   = FALSE;

           pthread_mutex_unlock(  &globalSchedulerInitLock );
       }


       /* ------------------------------------ */
       /* wait for all running tasks to stop   */
       /* ------------------------------------ */

       waitForAllRunningTasks();


       /* ------------------------------------ */
       /* then flush all events                */
       /* ------------------------------------ */

       pthread_mutex_lock(   &globalEventLock );

           if( globalEventQueueInited ) {

               flushEvents();


       /* ------------------------------------ */
       /* then destroy priority queue          */
       /* ------------------------------------ */

               DestroyPriQueue(  &globalEventQueue );
                                  globalEventQueueInited = FALSE;
           }

       pthread_mutex_unlock( &globalEventLock );
}


/***************************************************************
*                                                              *
* task thread accounting                                       *
*                                                              *
***************************************************************/

void   checkInTask( void ) {

       pthread_mutex_lock(   &globalRunningTaskCountLock );
           globalRunningTaskCount++;
       pthread_mutex_unlock( &globalRunningTaskCountLock );
}

/***************************************************************
*                                                              *
* task thread accounting                                       *
*                                                              *
***************************************************************/

void   checkOutTask( void ) {

       pthread_mutex_lock(   &globalRunningTaskCountLock );
           globalRunningTaskCount--;
       pthread_mutex_unlock( &globalRunningTaskCountLock );
}

/***************************************************************
*                                                              *
* task thread accounting                                       *
*                                                              *
***************************************************************/

int    getRunningTasks( void ) {

       int    runningTasks;

       pthread_mutex_lock(     &globalRunningTaskCountLock );
           runningTasks = globalRunningTaskCount;
       pthread_mutex_unlock(   &globalRunningTaskCountLock );

       return( runningTasks );
}

/***************************************************************
*                                                              *
* task thread synchronization                                  *
*                                                              *
***************************************************************/

void   waitForAllRunningTasks( void ) {


       int             index;
       snarltaskPtr    aTask;

       /* ------------------------------------ */
       /* second tell all possible running     */
       /* tasks that their memory is about to  */
       /* to cease                            */
       /* ------------------------------------ */

       index     = 0;

       while( aTask = find_snarltask( NULL, NULL, &index, globalTaskArray )) {

           pthread_mutex_lock(   aTask->lock );
               aTask->aboutToCease = TRUE;
           pthread_mutex_unlock( aTask->lock );
       }

       while( getRunningTasks() ) {

       /* ------------------------------------ */
       /* just loop - this could be improved   */
       /* waiting for condition variables      */
       /* ------------------------------------ */

       }
}
/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_manage_equeue.c                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* this code handles the event queue keeping every scheduled    *
* monitoring and administrative tasks                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* fetch event from the queue                                   *
*                                                              *
***************************************************************/

eventPtr   getNextEvent( eventPtr yourEventPtr ) {


       eventPtr     anEventPtr;

       time_t       now, later;
struct timespec     untilNextEvent;

       pthread_mutex_lock(      &globalEventLock );
           anEventPtr = (eventPtr)globalEventQueue.first;
       pthread_mutex_unlock(    &globalEventLock );


           if( anEventPtr == NULL ) {

                snarl_time(&now);
                queueEvent( SNARL_NULL_EVENT, now, NULL);
                anEventPtr = (eventPtr)globalEventQueue.first;
           }


       /* ------------------------------------ */
       /* we have an event - when was it due?  */
       /* ------------------------------------ */

       pthread_mutex_lock(      &globalEventLock );

           snarl_time(&now);

           while( (later = (anEventPtr->when - now)) > 0 ) {

       /* ------------------------------------ */
       /* this event is due much later         */
       /* timed wait will sleep and wait       */
       /* subtract timezone - sleeping in      */
       /* pthread_cond_timedwait listens to    */
       /* local time                           */
       /* ------------------------------------ */

               untilNextEvent.tv_sec  = anEventPtr->when;
               untilNextEvent.tv_nsec = 0;

               pthread_cond_timedwait( &globalEventAvailable, &globalEventLock, &untilNextEvent );

       /* ------------------------------------ */
       /* slept out or woke up upon event      */
       /* reception update sleep time until    */
       /* new next event is due                */
       /* ------------------------------------ */

               anEventPtr     = (eventPtr)globalEventQueue.first;

               snarl_time(&now);
           }

       /* ------------------------------------ */
       /* all events will go through this      */
       /* ------------------------------------ */

           if( (anEventPtr = (eventPtr)globalEventQueue.first) != NULL ) {

               RemovePriQueue(  &globalEventQueue, anEventPtr->pos );

               memcpy( yourEventPtr, anEventPtr, sizeof(event));
               free(   anEventPtr );
           }
       pthread_mutex_unlock(  &globalEventLock );


       if( anEventPtr ) {
           return( yourEventPtr );
       } else {
           return( NULL );
       }
}

/***************************************************************
*                                                              *
* queue event for later retrieval                              *
*                                                              *
***************************************************************/

void   queueEvent( int what, time_t when, void* ptr ) {


       eventPtr   anEventPtr;

       anEventPtr = (eventPtr)malloc( sizeof(event) );

       if( anEventPtr ) {

           anEventPtr->what = what;
           anEventPtr->when = when;
           anEventPtr->ptr  = ptr;

           pthread_mutex_lock(      &globalEventLock );
               AddPriQueue(         &globalEventQueue, anEventPtr );
               pthread_cond_signal( &globalEventAvailable );
           pthread_mutex_unlock(    &globalEventLock );


       }
}

/***************************************************************
*                                                              *
* remove all events from event queue                           *
*                                                              *
***************************************************************/

void   flushEvents( void ) {

       eventPtr    anEventPtr;
       int         eventQueueInited;

       /* ------------------------------------ */
       /* remove all events from the event     */
       /* queue and destroy the queue          */
       /* ------------------------------------ */

       if( globalEventQueueInited ) {

           while( anEventPtr = (eventPtr)globalEventQueue.first ) {

                   RemovePriQueue(    &globalEventQueue, anEventPtr->pos );
                   free( anEventPtr );
           }
       }
}


/***************************************************************
*                                                              *
* callback function for determination the event order          *
*                                                              *
***************************************************************/

Bool    earlierEvent( const void* ptr1, const void* ptr2 ) {


        eventPtr        eventOne, eventTwo;
        Bool            result;

        eventOne = (eventPtr)ptr1;
        eventTwo = (eventPtr)ptr2;


        if(eventOne->when < eventTwo->when ) {
            result = true;
        } else {
            result = false;
        }


        return( result );
}


/***************************************************************
*                                                              *
* callback function for storing the event position             *
*                                                              *
***************************************************************/

void    setEventPosition( void* ptr, UInt   position ) {

        eventPtr        anEventPtr;

        anEventPtr      = (eventPtr)ptr;
        anEventPtr->pos = position;
}


/***************************************************************
*                                                              *
* initialize the event queue                                   *
*                                                              *
***************************************************************/

Void InitPriQueue (
    PriQueue_t *            priq,
    UInt                    init_count,
    BoolCVPtrCVPtr_fpt      is_less_item,
    VoidVPtrUInt_fpt        set_pos_item)
{
    UInt                    i;
    priq->items = malloc( sizeof(Void*)*init_count );
    for ( i = 0; i < init_count; i++ ) {
        priq->items[i] = 0;
    }
    priq->first = 0;
    priq->count = 0;
    priq->alloc = init_count;
    priq->is_less_item = is_less_item;
    priq->set_pos_item = set_pos_item;
}



/***************************************************************
*                                                              *
* destroy the event queue                                      *
*                                                              *
***************************************************************/

Void DestroyPriQueue (PriQueue_t* priq)
{
  if (priq->items != NULL)
  {
    free( priq->items );
  }
}


#ifndef IS_LESS_ITEM_PRIQ
#define IS_LESS_ITEM_PRIQ(heap,item1,item2) ((*((heap)->is_less_item))((item1),(item2)))
#endif

#ifndef SET_POS_ITEM_PRIQ
#define SET_POS_ITEM_PRIQ(heap,item,pos)    ((*((heap)->set_pos_item))((item),(pos)))
#endif

/***************************************************************
*                                                              *
* order the event queue                                        *
*                                                              *
***************************************************************/

Void FixPriQueue (
    PriQueue_t *            priq,
    UInt                    pos )
{
    Void *                  item;
    UInt                    i;

    /* this is the item that may be out of place */
    item = priq->items[pos-1];
    i = pos;

    /* check whether we need to move the item up, moving other items down */
    while ( 1 <= i/2 ) {
        if ( IS_LESS_ITEM_PRIQ(priq,item,priq->items[i/2-1]) ) {
            priq->items[i-1] = priq->items[i/2-1];
            SET_POS_ITEM_PRIQ( priq, priq->items[i-1], i );
            i = i/2;
        }
        else {
            break;
        }
    }

    /* if we did not move it up, check whether we need to move it down */
    if ( i == pos ) {
        while ( 2*i <= priq->count ) {
            if ( 2*i < priq->count
              && IS_LESS_ITEM_PRIQ( priq, priq->items[2*i], priq->items[2*i-1] )
              && IS_LESS_ITEM_PRIQ( priq, priq->items[2*i], item ) ) {
                priq->items[i-1] = priq->items[2*i];
                SET_POS_ITEM_PRIQ( priq, priq->items[i-1], i );
                i = 2*i+1;
            }
            else if ( IS_LESS_ITEM_PRIQ( priq, priq->items[2*i-1], item ) ) {
                priq->items[i-1] = priq->items[2*i-1];
                SET_POS_ITEM_PRIQ( priq, priq->items[i-1], i );
                i = 2*i;
            }
            else {
                break;
            }
        }
    }

    /* put the item at this position */
    priq->items[i-1] = item;
    SET_POS_ITEM_PRIQ( priq, priq->items[i-1], i );
    priq->first = priq->items[0];
}


/***************************************************************
*                                                              *
* add event to the event queue                                 *
*                                                              *
***************************************************************/

Void  AddPriQueue (
    PriQueue_t *            priq,
    Void *                  item )
{
    Void * *                new_items;
    UInt                    i;

    priq->count++;

    /* handle case that priq is too small                      */
    if ( priq->alloc < priq->count ) {
        new_items = malloc( sizeof(Void*) * 2 * priq->alloc );
        priq->alloc = 2 * priq->alloc;
        for ( i = 0; i < priq->count-1; i++ ) {
            new_items[i] = priq->items[i];
        }
        for ( i = priq->count-1; i < priq->alloc; i++ ) {
            new_items[i] = 0;
        }
        free( priq->items );
        priq->items = new_items;
    }

    /* add the item to the end                                 */
    priq->items[priq->count-1] = item;

    /* and fix the priority queue                              */
    FixPriQueue( priq, priq->count );
}


/***************************************************************
*                                                              *
* remove event from the event queue                            *
*                                                              *
***************************************************************/

Void RemovePriQueue (
    PriQueue_t *            priq,
    UInt                    pos )
{
    if ( pos == priq->count ) {
        priq->items[priq->count-1] = 0;
        priq->count--;
        priq->first = priq->items[0];
    }
    else if ( 1 <= pos && pos < priq->count ) {
        priq->items[pos-1] = priq->items[priq->count-1];
        priq->items[priq->count-1] = 0;
        priq->count--;
        FixPriQueue( priq, pos );
    }
}



/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_manage_memory.c                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/


/***************************************************************
*                                                              *
* this code manages the creation and destruction of task       *
* memory holding all task configuration and task result        *
* information                                                  *
*                                                              *
***************************************************************/


/***************************************************************
*                                                              *
* attach task memory to linked list                            *
*                                                              *
***************************************************************/

void attach_snarltask( snarltaskPtr aTask, snarltaskArrayPtr taskarray ) {

    snarltaskPtr    lastsnarltask;
    snarltaskPtr    firstsnarltask;
    int             taskCount;


    lastsnarltask                 = taskarray->lastsnarltask;
    firstsnarltask                = taskarray->firstsnarltask;
    taskCount                     = taskarray->taskCount;

    aTask->last_task              = lastsnarltask;
    aTask->next_task              = NULL;


    if( taskarray->lastsnarltask ) {
        lastsnarltask->next_task  = aTask;
    } else {
        taskarray->firstsnarltask = aTask;
    }

    taskarray->lastsnarltask      = aTask;

    (taskarray->taskCount)++;

    renumber_ordinalIndex( taskarray );
}


/***************************************************************
*                                                              *
* insert task memory into linked list                          *
*                                                              *
***************************************************************/

void insert_snarltask( snarltaskPtr aTask, snarltaskPtr afterTask, snarltaskArrayPtr taskarray ) {

    snarltaskPtr     old_next_task;
    snarltaskPtr     lastsnarltask;
    snarltaskPtr     firstsnarltask;
    int              taskCount;


    lastsnarltask    = taskarray->lastsnarltask;
    firstsnarltask   = taskarray->firstsnarltask;
    taskCount        = taskarray->taskCount;

    aTask->last_task = afterTask;

    if( afterTask ) {

        if( afterTask->next_task ) {

            old_next_task             = afterTask->next_task;
            old_next_task->last_task  = aTask;

        } else {
            taskarray->lastsnarltask  = aTask;
        }

        aTask->next_task              = afterTask->next_task;
        afterTask->next_task          = aTask;


    } else {
        aTask->next_task              = NULL;
        taskarray->firstsnarltask     = aTask;
        taskarray->lastsnarltask      = aTask;
    }

    (taskarray->taskCount)++;

    renumber_ordinalIndex( taskarray );

}


/***************************************************************
*                                                              *
* detach task memory from linked list                          *
*                                                              *
***************************************************************/

void detach_snarltask( snarltaskPtr aTask, snarltaskArrayPtr taskarray ) {

    snarltaskPtr    beforeThisTask;
    snarltaskPtr    afterThisTask;

    snarltaskPtr    lastsnarltask;
    snarltaskPtr    firstsnarltask;
    int             taskCount;


    lastsnarltask   = taskarray->lastsnarltask;
    firstsnarltask  = taskarray->firstsnarltask;
    taskCount       = taskarray->taskCount;

    beforeThisTask  = aTask->last_task;
    afterThisTask   = aTask->next_task;


    if( beforeThisTask ) {
        beforeThisTask->next_task = aTask->next_task;
    } else {
        taskarray->firstsnarltask = aTask->next_task;
    }

    if( afterThisTask ) {
        afterThisTask->last_task = aTask->last_task;
    } else {
        taskarray->lastsnarltask = aTask->last_task;
    }

    (taskarray->taskCount)--;

    renumber_ordinalIndex( taskarray );
}


/***************************************************************
*                                                              *
* creation, destruction and identification of task memory      *
*                                                              *
***************************************************************/

siteVarsPtr create_siteVars( snarltaskPtr aTask, siteVarsPtr template ) {

    siteVarsPtr  aSiteVars;
    snarltaskPtr anotherTask;
    int          index;


        aSiteVars = &aTask->siteVarsGroup;

		memset( aSiteVars, 0,        sizeof(siteVariables));

		if ( aTask->sitename ) strcpy( aSiteVars->sitename, aTask->sitename );

        if( template ) {
            copy_masked_data( aSiteVars, template, globalSiteCopyMask, sizeof(siteVariables));
        }

        if( aSiteVars->cookielock = (pthread_mutex_t*)malloc(sizeof(pthread_mutex_t))) {
            pthread_mutex_init( aSiteVars->cookielock, NULL );

            if( aSiteVars->statusVars.statusLock = (pthread_mutex_t*)malloc(sizeof(pthread_mutex_t))) {
                pthread_mutex_init( aSiteVars->statusVars.statusLock, NULL );

                 aSiteVars->parentTask = aTask;

                 return( aSiteVars );

            } else {
                free( aSiteVars->cookielock );
                return( NULL );
            }

        }

        return( NULL );
}


/***************************************************************
*                                                              *
* creation, destruction and identification of task memory      *
*                                                              *
***************************************************************/

proxyVarsPtr create_proxyVars( snarltaskPtr aTask, proxyVarsPtr template, int keepAlive ) {

    proxyVarsPtr  aProxyVars;

    aProxyVars = &aTask->proxyVarsGroup;

    aProxyVars->parentTask = aTask;

	memset( aProxyVars, 0,        sizeof(proxyVariables));

    if( template ) {
        copy_masked_data( aProxyVars, template, globalProxyCopyMask, sizeof(proxyVariables));
    }

    if( keepAlive ) {

		aProxyVars->keepAlive   = keepAlive;
		aProxyVars->curl_handle = curl_easy_init();
	}

    if( aProxyVars->statusVars.statusLock = (pthread_mutex_t*)malloc(sizeof(pthread_mutex_t))) {
        pthread_mutex_init( aProxyVars->statusVars.statusLock, NULL );

         return( aProxyVars );
    }

    return( NULL );
}


/***************************************************************
*                                                              *
* fix pointers of task memory                                  *
*                                                              *
***************************************************************/

void fixPointers( snarltaskPtr aTask ) {

       /* ------------------------------------ */
       /* urldata struct's                     */
       /* ------------------------------------ */
        aTask->mainURL.parentTask               = aTask;
        aTask->sizeMatchURL.parentTask          = aTask;
        aTask->contentMatchURL.parentTask       = aTask;


       /* ------------------------------------ */
       /* tasktrapdata struct's                */
       /* ------------------------------------ */
        aTask->general_trap.parentTask          = aTask;
        aTask->synchronize_trap.parentTask      = aTask;
        aTask->performance_trap.parentTask      = aTask;
        aTask->httperror_trap.parentTask        = aTask;
        aTask->matchRegex_trap.parentTask       = aTask;
        aTask->matchContentSize_trap.parentTask = aTask;
        aTask->sizeMatchURL_trap.parentTask     = aTask;
        aTask->contentMatchURL_trap.parentTask  = aTask;
        aTask->matchNumic_trap.parentTask       = aTask;
        aTask->matchDateAge_trap.parentTask     = aTask;
        aTask->noUpdateBetween_trap.parentTask  = aTask;


       /* ------------------------------------ */
       /* siteVariables struct's               */
       /* ------------------------------------ */
       if( aTask->hasSiteVarsPtr ) {
           aTask->siteVarsGroup.parentTask             = aTask;
           aTask->siteVarsGroup.statusVars.parentTask  = aTask;
       } else {
           aTask->siteVarsGroup.parentTask             = NULL;
           aTask->siteVarsGroup.statusVars.parentTask  = NULL;
       }


       if( aTask->hasProxyVarsPtr) {
       /* ------------------------------------ */
       /* proxyVariables struct's              */
       /* ------------------------------------ */
           aTask->proxyVarsGroup.parentTask            = aTask;
           aTask->proxyVarsGroup.statusVars.parentTask = aTask;
       } else {
           aTask->proxyVarsGroup.parentTask            = NULL;
           aTask->proxyVarsGroup.statusVars.parentTask = NULL;
       }


       /* ------------------------------------ */
       /* statusVariables struct's             */
       /* ------------------------------------ */
       aTask->statusVars.parentTask                = aTask;

}

void setTrapIndex( snarltaskPtr aTask) {

       /* ------------------------------------ */
       /* tasktrapdata struct's                */
       /* ------------------------------------ */

        aTask->general_trap.siteIndex          = aTask->index;
        aTask->synchronize_trap.siteIndex      = aTask->index;
        aTask->performance_trap.siteIndex      = aTask->index;
        aTask->httperror_trap.siteIndex        = aTask->index;
        aTask->matchRegex_trap.siteIndex       = aTask->index;
        aTask->matchContentSize_trap.siteIndex = aTask->index;
        aTask->sizeMatchURL_trap.siteIndex     = aTask->index;
        aTask->contentMatchURL_trap.siteIndex  = aTask->index;
        aTask->matchNumic_trap.siteIndex       = aTask->index;
        aTask->matchDateAge_trap.siteIndex     = aTask->index;
        aTask->noUpdateBetween_trap.siteIndex  = aTask->index;
}



/***************************************************************
*                                                              *
* mask task memory and setup default values                    *
*                                                              *
***************************************************************/

void mask_snarltask_defaults( snarltaskPtr aTask ) {

     char *byte;
     char *mask;
     char *def;
     long  index;

     byte = (char*)aTask;
     mask = (char*)globalDefaultsMask;
     def  = (char*)globalTaskTemplate;


     for( index = 0; index < sizeof(snarltask); index++) {

       /* ------------------------------------ */
       /* set defaults variables to zero       */
       /* ------------------------------------ */
       byte[index] = (byte[index] & mask[index]);

       /* ------------------------------------ */
       /* set defaults variables to default    */
       /* ------------------------------------ */
       byte[index] = (byte[index] | ( def[index] & ~(mask[index])));

     }
}

void copy_masked_data( void* dest, void* src, void* mask, long len ) {

     long  index;

     char* ldest = (char*)dest;
     char* lsrc  = (char*)src;
     char* lmask = (char*)mask;
       /* ------------------------------------ */
       /* copy mased data from src to dest     */
       /* dest will be untouched where mask    */
       /* is FALSE and will be src where maksk */
       /* equals TRUE (bitwise)                */
       /* ------------------------------------ */
     for( index = 0; index < len; index++) {

       ldest[index] = (ldest[index] & lmask[index]) | ( lsrc[index] & ~(lmask[index]));

     }
}


/***************************************************************
*                                                              *
* creation and destruction of task memory                      *
*                                                              *
***************************************************************/

snarltaskPtr create_snarltask( char* sitename, char* urlname, snarltaskPtr template, snarltaskArrayPtr taskarray) {

    snarltaskPtr           aTask;
    snarltaskPtr           someTask;
    snarltaskPtr           linkedTask;
    snarltaskPtr           lastTask;
           int             maxIndex;

    pthread_mutexattr_t    mutexattr;


    if( aTask = (snarltaskPtr)malloc( sizeof(snarltask)) ) {

        maxIndex = find_maxIndex( taskarray );
        if( maxIndex == -1 ) maxIndex = 0;

        if( template ) {

            memcpy( aTask, template, sizeof(snarltask) );

        } else {
            memcpy( aTask, globalTaskTemplate, sizeof(snarltask) );
        }

        if( sitename ) {

            strncpy( aTask->sitename,                            sitename, SNARL_TASK_MAX_SITENAME );

            strncpy( aTask->general_trap.siteGroupName,          sitename, SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->synchronize_trap.siteGroupName,      sitename, SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->performance_trap.siteGroupName,      sitename, SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->httperror_trap.siteGroupName,        sitename, SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->matchRegex_trap.siteGroupName,       sitename, SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->matchContentSize_trap.siteGroupName, sitename, SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->contentMatchURL_trap.siteGroupName,  sitename, SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->sizeMatchURL_trap.siteGroupName,     sitename, SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->matchNumic_trap.siteGroupName,       sitename, SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->matchDateAge_trap.siteGroupName,     sitename, SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->noUpdateBetween_trap.siteGroupName,  sitename, SNARL_TASK_MAX_SITENAME );
        }

        if( urlname  ) {

            strncpy( aTask->urlname,                             urlname,  SNARL_TASK_MAX_SITENAME );

            strncpy( aTask->general_trap.siteName,               urlname,  SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->synchronize_trap.siteName,           urlname,  SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->performance_trap.siteName,           urlname,  SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->httperror_trap.siteName,             urlname,  SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->matchRegex_trap.siteName,            urlname,  SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->matchContentSize_trap.siteName,      urlname,  SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->contentMatchURL_trap.siteName,       urlname,  SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->sizeMatchURL_trap.siteName,          urlname,  SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->matchNumic_trap.siteName,            urlname,  SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->matchDateAge_trap.siteName,          urlname,  SNARL_TASK_MAX_SITENAME );
            strncpy( aTask->noUpdateBetween_trap.siteName,       urlname,  SNARL_TASK_MAX_SITENAME );
        }

        aTask->index = maxIndex + 1;

        setTrapIndex( aTask );

        fixPointers( aTask );

        if( aTask->lock = (pthread_mutex_t*)malloc(sizeof(pthread_mutex_t))) {

            pthread_mutexattr_init(          &mutexattr );
            pthread_mutexattr_setkind_np(    &mutexattr,  PTHREAD_MUTEX_ERRORCHECK_NP );
            pthread_mutex_init( aTask->lock, &mutexattr );

            if( aTask->statusVars.statusLock = (pthread_mutex_t*)malloc(sizeof(pthread_mutex_t))) {
                pthread_mutex_init( aTask->statusVars.statusLock, NULL );

                if( aTask->init_done = (pthread_cond_t*)malloc(sizeof(pthread_cond_t))) {
                    pthread_cond_init( aTask->init_done, NULL );
                } else {
                    free( aTask->lock );
                    free( aTask->statusVars.statusLock );
                    return( NULL );
                }
            } else {
                free( aTask->lock );
                return( NULL );
            }
        } else {
            return( NULL );
        }

        attach_snarltask( aTask, taskarray );

        return( aTask );
    }

    return( NULL );
}


/***************************************************************
*                                                              *
* creation and destruction of task memory                      *
*                                                              *
***************************************************************/

void invalidate_curlhandles( snarltaskPtr aTask ) {


       proxyVarsPtr    aProxyVars;


       /* ------------------------------------ */
       /* mainURL                              */
       /* ------------------------------------ */

       /* ------------------------------------ */
       /* then cancel main URL connection      */
       /* ------------------------------------ */

       if( aTask->mainURL.curl_handle ) {

		   if(                    aTask->mainURL.curl_handle ) {
			   curl_easy_cleanup( aTask->mainURL.curl_handle );
								  aTask->mainURL.curl_handle = NULL;
		   }

	   }

       /* ------------------------------------ */
       /* contentMatchURL                      */
       /* ------------------------------------ */

       /* ------------------------------------ */
       /* then cancel contentMatchURL conn.    */
       /* ------------------------------------ */

       if( aTask->contentMatchURL.curl_handle ) {

		   if(                    aTask->contentMatchURL.curl_handle ) {
			   curl_easy_cleanup( aTask->contentMatchURL.curl_handle );
								  aTask->contentMatchURL.curl_handle = NULL;
		   }
	   }


       /* ------------------------------------ */
       /* contentMatchURL                      */
       /* ------------------------------------ */

       /* ------------------------------------ */
       /* then cancel sizeMatchURL conn.       */
       /* ------------------------------------ */

       if( aTask->sizeMatchURL.curl_handle ) {

		   if(                    aTask->sizeMatchURL.curl_handle ) {
			   curl_easy_cleanup( aTask->sizeMatchURL.curl_handle );
								  aTask->sizeMatchURL.curl_handle = NULL;
		   }
	   }

       /* ------------------------------------ */
       /* is there a KeepAlive connection?     */
       /* ------------------------------------ */

       if( aTask->hasProxyVarsPtr ) {

           if( aProxyVars    = aTask->proxyVars ) {

			   if( aProxyVars->curl_handle ) {

				   curl_easy_cleanup( aProxyVars->curl_handle );
				   aProxyVars->curl_handle = NULL;
			   }
		   }
	   }

/*
       aTask->time_DNS           = 0;
       aTask->time_connect       = 0;
       aTask->time_firstByte     = 0;
       aTask->time_total         = 0;

       aTask->isVirgin = TRUE;
*/
}


/***************************************************************
*                                                              *
* empty but do not free                                        *
*                                                              *
***************************************************************/

void invalidate_snarltask( snarltaskPtr aTask, snarltaskArrayPtr taskarray   ) {


       int             index, index2, noMemberFound;
       snarltaskPtr    anotherMemberTask;
       siteVarsPtr     aSiteVars;
       proxyVarsPtr    aProxyVars;

       detach_snarltask( aTask, taskarray );

       invalidate_curlhandles( aTask );

       /* ------------------------------------ */
       /* free common siteVars                 */
       /* ------------------------------------ */
       if(           aTask->hasSiteVarsPtr ) {

       /* ------------------------------------ */
       /* if there exists another member of    */
       /* sitename in the linked list, handle  */
       /* the pointer responsibility over to   */
       /* that member                          */
       /* also copy the siteVars to that member*/
       /* if there is no other member to hold  */
       /* the group data then delete the in    */
       /* siteVarsGroup contained pointers     */
       /* ------------------------------------ */
           index = 0;
           if( anotherMemberTask = find_snarltask(aTask->sitename, NULL, &index, taskarray) ) {

               anotherMemberTask->hasSiteVarsPtr = TRUE;
               aSiteVars                         = &anotherMemberTask->siteVarsGroup;
               memcpy(aSiteVars, aTask->siteVars, sizeof(siteVariables));

               index2 = 0;
               while( anotherMemberTask = find_snarltask(aTask->sitename, NULL, &index2, taskarray) ) {
                   anotherMemberTask->siteVars = aSiteVars;
               }

           } else {

               if( aSiteVars = aTask->siteVars ) {

                   free( aSiteVars->cookielock );
                   free( aSiteVars->statusVars.statusLock );

                   aTask->holdsRecycableSiteVars = TRUE;
               }
           }

           aTask->hasSiteVarsPtr                 = FALSE;
       }
       aTask->siteVars                           = NULL;



       /* ------------------------------------ */
       /* free common proxyVars                */
       /* ------------------------------------ */
       if(           aTask->hasProxyVarsPtr ) {

       /* ------------------------------------ */
       /* if there exists another member of    */
       /* sitename in the linked list, handle  */
       /* the pointer responsibility over to   */
       /* that member                          */
       /* also copy the proxyVars to that member*/
       /* if there is no other member to hold  */
       /* the group data then delete the in    */
       /* proxyVarsGroup contained pointers    */
       /* ------------------------------------ */
           index = 0;
           noMemberFound = TRUE;

           while( anotherMemberTask = find_snarltask(aTask->sitename, NULL, &index, taskarray) ) {

               if( !strcmp( anotherMemberTask->mainURL.proxy, aTask->mainURL.proxy) ) {

                   noMemberFound                      = FALSE;
                   anotherMemberTask->hasProxyVarsPtr = TRUE;
                   aProxyVars                         = &anotherMemberTask->proxyVarsGroup;

                   memcpy(aProxyVars, &aTask->proxyVarsGroup, sizeof(proxyVariables));

                   index2 = 0;
                   while( anotherMemberTask = find_snarltask(aTask->sitename, NULL, &index2, taskarray) ) {
                       if( !strcmp( anotherMemberTask->mainURL.proxy, aTask->mainURL.proxy) ) {

                           anotherMemberTask->proxyVars = aProxyVars;
                       }
                   }
                   break;
               }
           }


           if( (noMemberFound) && (aProxyVars = aTask->proxyVars) ) {

               free( aProxyVars->statusVars.statusLock );
               aProxyVars->statusVars.statusLock = NULL;
               aTask->holdsRecycableProxyVars    = TRUE;
/*
               curl_easy_cleanup( aTask->sizeMatchURL.curl_handle );
                                  aTask->sizeMatchURL.curl_handle = NULL;
*/
           }
           aTask->hasProxyVarsPtr                 = FALSE;
       }
       aTask->proxyVars                           = NULL;


       /* ------------------------------------ */
       /* then free main URL form structure    */
       /* ------------------------------------ */
       if( aTask->mainURL.isForm && aTask->mainURL.firstFormItem ) {
           curl_formfree(           aTask->mainURL.firstFormItem );
                                    aTask->mainURL.firstFormItem = NULL;
                                    aTask->mainURL.lastFormItem  = NULL;
       }

       /* ------------------------------------ */
       /* then free size MatchURL form struct  */
       /* ------------------------------------ */
       if( aTask->sizeMatchURL.isForm && aTask->sizeMatchURL.firstFormItem ) {
           curl_formfree(                aTask->sizeMatchURL.firstFormItem );
                                         aTask->sizeMatchURL.firstFormItem = NULL;
                                         aTask->sizeMatchURL.lastFormItem  = NULL;
       }

       /* ------------------------------------ */
       /* then free contentMatchURL form struct*/
       /* ------------------------------------ */
       if( aTask->contentMatchURL.isForm && aTask->contentMatchURL.firstFormItem ) {
           curl_formfree(                   aTask->contentMatchURL.firstFormItem );
                                            aTask->contentMatchURL.firstFormItem = NULL;
                                            aTask->contentMatchURL.lastFormItem  = NULL;
       }

       /* ------------------------------------ */
       /* match regular expression             */
       /* only the original task is owner of   */
       /* the regex buffer                     */
       /* ------------------------------------ */
       if(     aTask->task_matchRegex
           && (aTask->matchRegex_data.regexError == 0)) {

           regfree( &(aTask->matchRegex_data.regexBuffer));
       }

       /* ------------------------------------ */
       /* var regular expression               */
       /* only the original task is owner of   */
       /* the regex buffer                     */
       /* ------------------------------------ */
       if(     aTask->task_fetchRegex
           && (aTask->varRegex.regexError == 0 )) {

           regfree( &(aTask->varRegex.regexBuffer));
       }

       /* ------------------------------------ */
       /* regular expression for dynamic url's */
       /* only the original task is owner of   */
       /* the regex buffer                     */
       /* ------------------------------------ */
       if(     aTask->task_fetchDynamicURL
           && (aTask->dynamicURLRegex.regexError == 0)) {

           regfree( &(aTask->dynamicURLRegex.regexBuffer));
       }

       /* ------------------------------------ */
       /* then free pthread init_done structure*/
       /* ------------------------------------ */
       if(       aTask->init_done ) {
           free( aTask->init_done );
           aTask->init_done = NULL;
       }
       /* ------------------------------------ */
       /* then free pthread statusLock structure*/
       /* ------------------------------------ */
       if(       aTask->statusVars.statusLock ) {

           free( aTask->statusVars.statusLock );
                 aTask->statusVars.statusLock = NULL;
       }

       /* ------------------------------------ */
       /* then free pthread lock structure     */
       /* ------------------------------------ */

       if(       aTask->lock ) {
           free( aTask->lock );
                 aTask->lock = NULL;
       }
}





/***************************************************************
*                                                              *
* clone a group of tasks and change the proxy to query the     *
* url                                                          *
*                                                              *
***************************************************************/

void    clone_proxy_task( char* sitename, char* proxy, int keepAlive ) {

       int                  index, index2, index3, done;
       int                  indexbuffer;
       int                  maxOrdinalIndex;

       snarltask            nullTaskStruct;

       snarltaskPtr         aTask;
       snarltaskPtr         anOriginalTask;
       snarltaskPtr         anOriginalProxyTask;

       snarltaskPtr         aTask3;
       snarltaskPtr         newTask;
       snarltaskPtr         last_task_buf;
       snarltaskPtr         next_task_buf;

       proxyVarsPtr         proxyVarsTemplate;
       int                  foundProxyVars;

       pthread_mutex_t     *lock_buf;
       pthread_cond_t      *init_done_buf;

       char                *byte;
       char                *mask;
       char                *def;
       int                 bytepointer;

       proxyVarsPtr        aProxyVars;

       maxOrdinalIndex = find_maxOrdinalIndex(globalTaskArray);

       /* ------------------------------------ */
       /* a.) first find a task to be cloned   */
       /*     within the current               */
       /*     globalTaskArray                  */
       /* ------------------------------------ */
       index = 0;
       while( aTask = find_snarltask( sitename, NULL, &index, globalTaskArray  ) ) {

           if(    (aTask->ordinal_index <= maxOrdinalIndex)
               && (aTask->initialized )
               && (aTask->mainURL.proxy[0] == '\0')) {

       /* ------------------------------------ */
       /* b.) then try to find it's version in */
       /*     the current array                */
       /*     this is to prevent               */
       /*     double creation of proxy tasks   */
       /*     as in two consecutive equal      */
       /*     proxy directives                 */
       /* ------------------------------------ */
               index2         = 0;
               anOriginalTask = NULL;
               while( anOriginalTask = find_snarltask(sitename, aTask->urlname, &index2, globalTaskArray ) ) {

                  if( !strcmp(anOriginalTask->mainURL.proxy, proxy) ){
                      done = TRUE;
                      break;
                  }
               }

       /* ------------------------------------ */
       /* c.) and then find the proxie's       */
       /*     version in the recycled          */
       /*     variables stash                  */
       /* ------------------------------------ */
               index2 = 0;
               done   = FALSE;
               while( anOriginalProxyTask = find_snarltask(sitename, aTask->urlname, &index2, globalRecycledTaskArray ) ) {

                   if( ! anOriginalTask && (!strcmp(anOriginalProxyTask->mainURL.proxy, proxy))  ) {

       /* ------------------------------------ */
       /* we've found the proxy task -         */
       /* create the new proxy using the       */
       /* current valid task to be cloned as a */
       /* template                             */
       /* ------------------------------------ */
                       if( newTask = create_snarltask( sitename, aTask->urlname, aTask, globalTaskArray )) {

       /* ------------------------------------ */
       /* copy all OID relevant data           */
       /* from the old (original proxy) task   */
       /* to the new task                      */
       /* ------------------------------------ */
                           copy_masked_data( newTask, anOriginalProxyTask, globalTaskCopyMask, sizeof(snarltask) );

       /* ------------------------------------ */
       /* now get rid of the old data struct   */
       /* ------------------------------------ */
                           detach_snarltask( anOriginalProxyTask, globalRecycledTaskArray );
                           free( anOriginalProxyTask );
                       }

       /* ------------------------------------ */
       /* now setup all the data that need     */
       /* to be set only for proxies           */
       /* ------------------------------------ */
                       if( newTask ) {

       /* ------------------------------------ */
       /* give the task a valid index, check   */
       /* that the default index does not      */
       /* already exist                        */
       /* ------------------------------------ */

                           if( ! find_IndexedTask( aTask->index + globalAddIndex, EXACT, globalTaskArray  ) ) {
                               newTask->index = aTask->index + globalAddIndex;
                               setTrapIndex( newTask );
                           }

       /* ------------------------------------ */
       /* set the URL proxy and the proxy to   */
       /* be reported in traps                 */
       /* ------------------------------------ */
                           strcpy( newTask->mainURL.proxy, proxy );

                           strcpy( newTask->general_trap.proxy          , proxy);
                           strcpy( newTask->synchronize_trap.proxy      , proxy);
                           strcpy( newTask->performance_trap.proxy      , proxy);
                           strcpy( newTask->httperror_trap.proxy        , proxy);
                           strcpy( newTask->matchRegex_trap.proxy       , proxy);
                           strcpy( newTask->matchContentSize_trap.proxy , proxy);
                           strcpy( newTask->sizeMatchURL_trap.proxy     , proxy);
                           strcpy( newTask->contentMatchURL_trap.proxy  , proxy);
                           strcpy( newTask->matchNumic_trap.proxy       , proxy);
                           strcpy( newTask->matchDateAge_trap.proxy     , proxy);
                           strcpy( newTask->noUpdateBetween_trap.proxy  , proxy);

       /* ------------------------------------ */
       /* recompile regular expressions, etc.  */
       /* ------------------------------------ */
                           if( newTask->task_matchRegex ) {
                               newTask->matchRegex_data.regexError     = regcomp( &(newTask->matchRegex_data.regexBuffer),
                                                                                    newTask->matchRegex_data.regex,
                                                                                    REG_EXTENDED | REG_NOSUB );
                               regerror(  newTask->matchRegex_data.regexError,
                                        &(newTask->matchRegex_data.regexBuffer),
                                          newTask->matchRegex_data.regexError_string,
                                   sizeof(newTask->matchRegex_data.regexError_string));
                           }

                           if( newTask->task_fetchRegex ) {
                               newTask->varRegex.regexError            = regcomp( &(newTask->varRegex.regexBuffer),
                                                                                    newTask->varRegex.regex,
                                                                                    REG_EXTENDED);
                               regerror(  newTask->varRegex.regexError,
                                        &(newTask->varRegex.regexBuffer),
                                          newTask->varRegex.regexError_string,
                                   sizeof(newTask->varRegex.regexError_string));
                           }

                           if( newTask->task_fetchDynamicURL ) {
                               newTask->dynamicURLRegex.regexError     = regcomp( &(newTask->dynamicURLRegex.regexBuffer),
                                                                                    newTask->dynamicURLRegex.regex,
                                                                                    REG_EXTENDED);
                               regerror(  newTask->dynamicURLRegex.regexError,
                                        &(newTask->dynamicURLRegex.regexBuffer),
                                          newTask->dynamicURLRegex.regexError_string,
                                   sizeof(newTask->dynamicURLRegex.regexError_string));
                           }

       /* ------------------------------------ */
       /* make the proxy task member of the    */
       /* site group by letting it have access */
       /* to the site Variables                */
       /* ------------------------------------ */

                           newTask->siteVars       = aTask->siteVars;
                           newTask->hasSiteVarsPtr = FALSE;
                           newTask->initialized    = TRUE;
                           newTask->isOnlyTemplate = FALSE;

       /* ------------------------------------ */
       /* is this the first entry for this     */
       /* proxy? if yes create proxyVars.      */
       /* that is is a structure holding common*/
       /* variables for all tasks in one proxy */
       /* ------------------------------------ */
                           index3         = 0;
                           foundProxyVars = FALSE;

                           while( aTask3 = find_snarltask(sitename, NULL, &index3, globalTaskArray ) ) {

                              if( (aTask3 != newTask) && (!strcmp(aTask3->mainURL.proxy, proxy )) && (aTask3->hasProxyVarsPtr) ) {

                                  foundProxyVars = TRUE;
                                  break;
                              }
                           }

       /* ------------------------------------ */
       /* if proxy Variables already exist,    */
       /* just point to it                     */
       /* ------------------------------------ */
                           if( foundProxyVars && newTask) {
                               newTask->proxyVars = aTask3->proxyVars;
                           } else {

       /* ------------------------------------ */
       /* if proxy Variables do not exist,     */
       /* create new ones                      */
       /* ------------------------------------ */
                              proxyVarsTemplate = NULL;
                              index3 = 0;
                              while( aTask3 = find_snarltask(sitename, NULL, &index3, globalRecycledTaskArray ) ) {

       /* ------------------------------------ */
       /* try to copy over old proxy data      */
       /* ------------------------------------ */
                                  if( (! strcmp(aTask3->mainURL.proxy, proxy )) && (aTask3->holdsRecycableProxyVars) ) {
                                      proxyVarsTemplate = &aTask3->proxyVarsGroup;
                                      break;
                                  }
                              }

                              if( newTask ) {
                                  newTask->proxyVars = create_proxyVars( newTask, proxyVarsTemplate, keepAlive );
                              }
                              if( (newTask) && (newTask->proxyVars) ) {
                                  newTask->hasProxyVarsPtr = TRUE;
                              }
                           }

                           done = TRUE;
                       }
                   }
               }


       /* ------------------------------------ */
       /* proxy directive never creates two    */
       /* tasks with same sitename, urlname    */
       /* and proxy                            */
       /* if asked to do so we ignore creation */
       /* of a second instance                 */
       /* ------------------------------------ */
               index3 = 0;
               while ( aTask3 = find_snarltask(sitename, aTask->urlname, &index3, globalTaskArray ) ) {

                   if( !strcmp( aTask3->mainURL.proxy, proxy ) ) {
                       done  = TRUE;
                       break;
                   }
               }


       /* ------------------------------------ */
       /* we have not found the proxy task     */
       /* create the new proxy task using the  */
       /* aTask as a template                  */
       /* ------------------------------------ */

       /* ------------------------------------ */
       /* now setup all the data that need     */
       /* to be set only for proxies           */
       /* ------------------------------------ */
               if( !done ) {

                   if( newTask = create_snarltask( sitename, aTask->urlname, aTask, globalTaskArray ) ) {

       /* ------------------------------------ */
       /* make sure we do not copy status data */
       /* as this newly created task cannot    */
       /* possibly have any status             */
       /* ------------------------------------ */
                       memset( &nullTaskStruct, 0, sizeof(snarltask) );

                       copy_masked_data( newTask, &nullTaskStruct, globalTaskCopyMask, sizeof(snarltask));

		               newTask->statusVars.first_run = 0;
                       newTask->statusVars.first_run_string[0] = '\0';

                       newTask->hasData = FALSE;

       /* ------------------------------------ */
       /* give the task a valid index, check   */
       /* that the default index does not      */
       /* already exist                        */
       /* ------------------------------------ */
                       if( ! find_IndexedTask( aTask->index + globalAddIndex, EXACT, globalTaskArray  ) ) {
                           newTask->index = aTask->index + globalAddIndex;

                           setTrapIndex( newTask );
                       }
       /* ------------------------------------ */
       /* set the URL proxy and the proxy to   */
       /* be reported in traps                 */
       /* ------------------------------------ */
                       strcpy( newTask->mainURL.proxy, proxy );

                       strcpy( newTask->general_trap.proxy          , proxy);
                       strcpy( newTask->synchronize_trap.proxy      , proxy);
                       strcpy( newTask->performance_trap.proxy      , proxy);
                       strcpy( newTask->httperror_trap.proxy        , proxy);
                       strcpy( newTask->matchRegex_trap.proxy       , proxy);
                       strcpy( newTask->matchContentSize_trap.proxy , proxy);
                       strcpy( newTask->sizeMatchURL_trap.proxy     , proxy);
                       strcpy( newTask->contentMatchURL_trap.proxy  , proxy);
                       strcpy( newTask->matchNumic_trap.proxy       , proxy);
                       strcpy( newTask->matchDateAge_trap.proxy     , proxy);
                       strcpy( newTask->noUpdateBetween_trap.proxy  , proxy);

       /* ------------------------------------ */
       /* make the proxy task member of the    */
       /* site group by letting it have access */
       /* to the site Variables                */
       /* ------------------------------------ */

       /* attention newTask->siteVars       = aTask->siteVars; wasn't here to begin with */

                       newTask->siteVars       = aTask->siteVars;
                       newTask->hasSiteVarsPtr = FALSE;
                       newTask->initialized    = TRUE;
                       newTask->isOnlyTemplate = FALSE;

       /* ------------------------------------ */
       /* recompile regular expressions, etc.  */
       /* ------------------------------------ */
                       if( newTask->task_matchRegex ) {
                           newTask->matchRegex_data.regexError     = regcomp( &(newTask->matchRegex_data.regexBuffer),
                                                                                newTask->matchRegex_data.regex,
                                                                                REG_EXTENDED | REG_NOSUB );
                           regerror(  newTask->matchRegex_data.regexError,
                                    &(newTask->matchRegex_data.regexBuffer),
                                      newTask->matchRegex_data.regexError_string,
                               sizeof(newTask->matchRegex_data.regexError_string));
                       }

                       if( newTask->task_fetchRegex ) {
                           newTask->varRegex.regexError            = regcomp( &(newTask->varRegex.regexBuffer),
                                                                                newTask->varRegex.regex,
                                                                                REG_EXTENDED);
                           regerror(  newTask->varRegex.regexError,
                                    &(newTask->varRegex.regexBuffer),
                                      newTask->varRegex.regexError_string,
                               sizeof(newTask->varRegex.regexError_string));
                       }

                       if( newTask->task_fetchDynamicURL ) {
                           newTask->dynamicURLRegex.regexError     = regcomp( &(newTask->dynamicURLRegex.regexBuffer),
                                                                                newTask->dynamicURLRegex.regex,
                                                                                REG_EXTENDED);
                           regerror(  newTask->dynamicURLRegex.regexError,
                                    &(newTask->dynamicURLRegex.regexBuffer),
                                      newTask->dynamicURLRegex.regexError_string,
                               sizeof(newTask->dynamicURLRegex.regexError_string));
                       }

       /* ------------------------------------ */
       /* is this the first entry for this     */
       /* proxy? if yes create proxyVars.      */
       /* that is is a structure holding common*/
       /* variables for all tasks in one proxy */
       /* ------------------------------------ */
                       index3         = 0;
                       foundProxyVars = FALSE;
                       while( aTask3 = find_snarltask(sitename, NULL, &index3, globalTaskArray ) ) {

                          if( (aTask3 != newTask) && (!strcmp(aTask3->mainURL.proxy, proxy )) && (aTask3->hasProxyVarsPtr) ) {

                              foundProxyVars = TRUE;
                              break;
                          }
                       }


                       if( foundProxyVars && newTask) {
       /* ------------------------------------ */
       /* this was not the first entry for     */
       /* this proxy - just point to the first */
       /* proxy entry                          */
       /* ------------------------------------ */
                           newTask->proxyVars = aTask3->proxyVars;

                       } else {

       /* ------------------------------------ */
       /* this is the first proxy entry        */
       /* try to find the old proxy vars from  */
       /* stash if it exists                   */
       /* ------------------------------------ */
                           proxyVarsTemplate = NULL;
                           index3            = 0;
                           while( aTask3 = find_snarltask(sitename, NULL, &index3, globalRecycledTaskArray ) ) {

                               if( (aTask3 != newTask) && (! strcmp(aTask3->mainURL.proxy, proxy )) && (aTask3->holdsRecycableProxyVars) ) {
                                   proxyVarsTemplate = &aTask3->proxyVarsGroup;
                                   break;
                               }
                           }

                           if( newTask ) {
                               newTask->proxyVars = create_proxyVars( newTask, proxyVarsTemplate, keepAlive );
                           }

                           if( (newTask) && (newTask->proxyVars) ) {
                               newTask->hasProxyVarsPtr = TRUE;
                           }
                       }
                   }
               }
           }
       }
}


/***************************************************************
*                                                              *
* searching of task memory in linked list                      *
*                                                              *
***************************************************************/

void renumber_ordinalIndex( snarltaskArrayPtr taskarray  ) {

    snarltaskPtr        linkedTask;
    int                 ordinal_index = 1;

    snarltaskPtr        lastsnarltask;
    snarltaskPtr        firstsnarltask;
    int                 taskCount;

    lastsnarltask     = taskarray->lastsnarltask;
    firstsnarltask    = taskarray->firstsnarltask;
    taskCount         = taskarray->taskCount;

    if( linkedTask = firstsnarltask ) {

      do {

          linkedTask->ordinal_index = ordinal_index;
          ordinal_index++;

      }
      while( linkedTask = linkedTask->next_task  );

    }
}


/***************************************************************
*                                                              *
* searching of task memory in linked list                      *
*                                                              *
***************************************************************/

int find_maxOrdinalIndex( snarltaskArrayPtr taskarray  ) {


    snarltaskPtr        linkedTask;
    int                 maxOrdinalIndex;

    snarltaskPtr        lastsnarltask;
    snarltaskPtr        firstsnarltask;
    int                 taskCount;

    lastsnarltask     = taskarray->lastsnarltask;
    firstsnarltask    = taskarray->firstsnarltask;
    taskCount         = taskarray->taskCount;



    if( linkedTask = firstsnarltask ) {
      maxOrdinalIndex = linkedTask->ordinal_index;
      do {
          if( linkedTask->ordinal_index >= maxOrdinalIndex ) {
            maxOrdinalIndex = linkedTask->ordinal_index;
          }
      }
      while( linkedTask = linkedTask->next_task  );
      return( maxOrdinalIndex );
    }
    return( -1 );
}


/***************************************************************
*                                                              *
* searching of task memory in linked list                      *
*                                                              *
***************************************************************/

int find_maxIndex( snarltaskArrayPtr taskarray  ) {


    snarltaskPtr        linkedTask;
    int                 maxIndex;

    snarltaskPtr        lastsnarltask;
    snarltaskPtr        firstsnarltask;
    int                 taskCount;

    lastsnarltask     = taskarray->lastsnarltask;
    firstsnarltask    = taskarray->firstsnarltask;
    taskCount         = taskarray->taskCount;



    if( linkedTask = firstsnarltask ) {
      maxIndex = linkedTask->index;
      do {
          if( linkedTask->index >= maxIndex ) {
            maxIndex = linkedTask->index;
          }
      }
      while( linkedTask = linkedTask->next_task  );
      return( maxIndex );
    }
    return( -1 );
}


/***************************************************************
*                                                              *
* searching of task memory in linked list                      *
*                                                              *
***************************************************************/

int find_minIndex( snarltaskArrayPtr taskarray  ) {

    snarltaskPtr        linkedTask;
    int                 minIndex;

    snarltaskPtr        lastsnarltask;
    snarltaskPtr        firstsnarltask;
    int                 taskCount;

    lastsnarltask     = taskarray->lastsnarltask;
    firstsnarltask    = taskarray->firstsnarltask;
    taskCount         = taskarray->taskCount;


   if( linkedTask = firstsnarltask ) {
      minIndex = linkedTask->index;
      do {
          if( linkedTask->index <= minIndex ) {
            minIndex = linkedTask->index;
          }
      }
      while( linkedTask = linkedTask->next_task  );
      return( minIndex );
    }
    return( -1 );
}


/***************************************************************
*                                                              *
* searching of task memory in linked list                      *
*                                                              *
***************************************************************/

int find_lastIndex( int index, snarltaskArrayPtr taskarray ) {

    snarltaskPtr        linkedTask;
    int                 lastIndex;

    snarltaskPtr        lastsnarltask;
    snarltaskPtr        firstsnarltask;
    int                 taskCount;

    lastsnarltask     = taskarray->lastsnarltask;
    firstsnarltask    = taskarray->firstsnarltask;
    taskCount         = taskarray->taskCount;


    if( (linkedTask = firstsnarltask) && (index >= 0) ) {

      lastIndex = find_minIndex(taskarray);
      do {
          if( (linkedTask->index < index) && (linkedTask->index >= lastIndex) ) {
            lastIndex = linkedTask->index;
          }
      }
      while( linkedTask = linkedTask->next_task  );

      if( index != lastIndex ) {
        return( lastIndex );
      } else {
        return( - 1);
      }
    }
    return( -1 );
}


/***************************************************************
*                                                              *
* searching of task memory in linked list                      *
*                                                              *
***************************************************************/

int find_nextIndex( int index, snarltaskArrayPtr taskarray  ) {

    snarltaskPtr        linkedTask;
    int                 nextIndex;

    snarltaskPtr        lastsnarltask;
    snarltaskPtr        firstsnarltask;
    int                 taskCount;

    lastsnarltask     = taskarray->lastsnarltask;
    firstsnarltask    = taskarray->firstsnarltask;
    taskCount         = taskarray->taskCount;


    if( (linkedTask = firstsnarltask) && (index >= 0) ) {

      nextIndex = find_maxIndex(taskarray);

      do {
          if( (linkedTask->index > index) && (linkedTask->index <= nextIndex) ) {
            nextIndex = linkedTask->index;
          }
      }
      while( linkedTask = linkedTask->next_task  );

      if( index != nextIndex ) {

        return( nextIndex );
      } else {

        return( - 1);
      }
    }
    return( -1 );
}


/***************************************************************
*                                                              *
* searching of task memory in linked list                      *
*                                                              *
***************************************************************/

snarltaskPtr find_IndexedTask( int index, int exact, snarltaskArrayPtr taskarray ) {

    snarltaskPtr        linkedTask;
    snarltaskPtr        foundTask = NULL;
    int                 nextIndex;

    snarltaskPtr        lastsnarltask;
    snarltaskPtr        firstsnarltask;
    int                 taskCount;

    lastsnarltask     = taskarray->lastsnarltask;
    firstsnarltask    = taskarray->firstsnarltask;
    taskCount         = taskarray->taskCount;


    if( (linkedTask = firstsnarltask) && (index >= 0) ) {

    /* try to find index                       */
    /*-----------------------------------------*/
      do {
          if( linkedTask->index == index ) {
            foundTask = linkedTask;
            break;
          }
      } while( linkedTask = linkedTask->next_task );


    /* successfull ? then return               */
    /*-----------------------------------------*/

      if( foundTask ) {
        return( foundTask );

      } else {

        if( exact ) return ( NULL );

    /* find closest upper neighbor             */
    /*-----------------------------------------*/

        nextIndex = find_nextIndex( index, taskarray );

        if( nextIndex == -1 ) return( NULL );
   /*-----------------------------------------*/

        linkedTask = firstsnarltask;

        do {
          if( linkedTask->index == nextIndex ) {
            foundTask = linkedTask;
            break;
          }
        } while( linkedTask = linkedTask->next_task );

   /*-----------------------------------------*/

      if( foundTask ) {
        return( foundTask );
      } else {
        /* should'nt get here */
        return( NULL );
      }

   /*-----------------------------------------*/

      }
    }

    return( NULL );
}


/***************************************************************
*                                                              *
* searching of task memory in linked list                      *
*                                                              *
* find task by three parameters                                *
*                                                              *
***************************************************************/

snarltaskPtr find_snarltask(char* sitename, char* urlname, int* startindex, snarltaskArrayPtr taskarray ) {


    int                 index;
    snarltaskPtr        linkedTask;
    snarltaskPtr        startTask;

     snarltaskPtr        lastsnarltask;
    snarltaskPtr        firstsnarltask;
    int                 taskCount;

    lastsnarltask     = taskarray->lastsnarltask;
    firstsnarltask    = taskarray->firstsnarltask;
    taskCount         = taskarray->taskCount;


    if( (firstsnarltask == NULL) || (*startindex < 0) ) return( NULL );

    /* search for the first snarltask with     */
    /* index == *startindex                    */

    if( startTask = find_IndexedTask( *startindex, NEAR, taskarray ) ) {

       for( index = startTask->index; index != -1; index = find_nextIndex(index, taskarray)) {

          linkedTask = find_IndexedTask( index, EXACT, taskarray );

          if( sitename ) {
              if( !strcmp(sitename, linkedTask->sitename)) {

                  if( urlname ) {
                     if(!strcmp(urlname, linkedTask->urlname ) ) {
                        *startindex = find_nextIndex( index, taskarray );

                        /* sitname and urlname */
                        /* was given in args   */

                        return( linkedTask );
                     }
                  } else {
                      *startindex = find_nextIndex( index, taskarray );

                      /* only sitename was given */
                      /* in args                 */

                      return( linkedTask );
                   }
              }
          } else {

              /* only index was given in args  */
              *startindex = find_nextIndex( index, taskarray );

              return( linkedTask );
          }
      }
   }
    return( NULL );
}

/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_manage_thread.c                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* this code directs the task invocation by translating         *
* events in the event queue into concrete actions              *
*                                                              *
***************************************************************/


/***************************************************************
*                                                              *
* entry point of the task manager                              *
*                                                              *
***************************************************************/

void   startThreadManager( void* null ) {

       snarltaskPtr               aTask;
       tasktrapdataPtr            aTrap;

       sigset_t                   sigs_to_block;

       pthread_attr_t             task_attributes;
       pthread_t                  task_registry;

       event                      anEvent;
       eventPtr                   anEventPtr;

       struct timeval             idletime;

       int                        scheduleTasks;

       int                        message_level;

       long                       anIndex;
       long                       aSubTaskID;

       char                       protstring[SNARL_TASK_PROT_MAX_LENGTH];
       char                       hoststring[SNARL_TASK_HOST_MAX_LENGTH];
       char                       pathstring[SNARL_TASK_PATH_MAX_LENGTH];

       char                       message[SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH];
       char                       agent[SNARL_TASK_PROXY_MAX_LENGTH];

       char                       *onBehalfAgent = agent;
       tasktrapdata               heartBeatTrapTrap;
       time_t                     now;


       /* ------------------------------------ */
       /* prepare                              */
       /* first of all don't listen to signals */
       /* ------------------------------------ */

       sigfillset(                 &sigs_to_block );
       pthread_sigmask( SIG_BLOCK, &sigs_to_block, NULL );

       initializeThreadManager();

       /* ------------------------------------ */
       /* now continuously reschedule          */
       /* ------------------------------------ */

       while( TRUE ) {

       /* ------------------------------------ */
       /* get next event in priority queue     */
       /* ------------------------------------ */

           pthread_mutex_lock(   &globalScheduleEventsLock );
               scheduleTasks = globalScheduleEvents;
           pthread_mutex_unlock( &globalScheduleEventsLock );

           if( scheduleTasks && (anEventPtr = getNextEvent( &anEvent )) ) {

               switch( anEvent.what ) {

                   case SNARL_UPDATE_TASK_EVENT:

       /* ------------------------------------ */
       /* create thread for one run            */
       /* ------------------------------------ */

                       aTask = (snarltaskPtr)(anEvent.ptr);

                       pthread_mutex_lock(   aTask->lock );

                           checkInTask();

                           pthread_attr_init(             &task_attributes );
                           pthread_attr_setdetachstate(   &task_attributes, PTHREAD_CREATE_DETACHED );

                           if( getuid() == 0 ) {
       /* ------------------------------------ */
       /* try to use maxiumum priority sched.  */
       /* ------------------------------------ */
                               pthread_attr_setschedpolicy(   &task_attributes, SNARL_TASK_SCHEDULING_POLICY );
					       }
                           pthread_attr_setschedparam(    &task_attributes, &(aTask->schedulingParameters) );

                           if( pthread_create(        &task_registry,
                                                      &task_attributes,
                                               (void* )updateSnarlTask,
                                               (void* )aTask            )) {

       /* ------------------------------------ */
       /* at this point we are in serious      */
       /* trouble just ignore because there    */
       /* is nothing we can do                 */
       /* however checkOutTask from global     */
       /* running tasks counter                */
       /* ------------------------------------ */

                               checkOutTask();


                           } else {
                               aTask->task_registry = task_registry;
                               aTask->isRunning     = TRUE;
                               pthread_detach(        task_registry );
                           }
                       pthread_mutex_unlock( aTask->lock );
                   break;


                   case SNARL_STOP_SCHEDULE_EVENTS:
                       pthread_mutex_lock(   &globalScheduleEventsLock );
                           globalScheduleEvents = FALSE;
                           pthread_cond_signal( &globalScheduleEventsAck );
                       pthread_mutex_unlock( &globalScheduleEventsLock );
                   break;


                   case SNARL_TRAP_EVENT:

                       onBehalfAgent = agent;

                       aTrap = (tasktrapdataPtr)(anEvent.ptr);
                       aTask = aTrap->parentTask;

                       pthread_mutex_lock(    aTask->lock );

                           strncpy( message,  aTrap->siteMessage, SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 );
                           message_level =    aTrap->specificTrap;

                               /* ------------------------------------ */
                               /* if the task has a well defined proxy */
                               /* use it as the agent/virtual server   */
                               /* address                              */
                               /* ------------------------------------ */

                           if( strlen(aTask->mainURL.proxy) != 0 ) {
                               strcpy( agent, aTask->mainURL.proxy );

                           } else {

                               /* ------------------------------------ */
                               /* if the task has no well defined proxy*/
                               /* use the host part of the url as agent*/
                               /* address                              */
                               /* ------------------------------------ */

                               if (2 > sscanf(aTask->mainURL.currenturl, "%64[^\n:]://%256[^\n/]%256[^\n]",

                                               protstring, hoststring, pathstring)) {

                               /* ------------------------------------ */
                               /* the url was badly formed             */
                               /* probably because it is in browser    */
                               /* style omitting protocoll and host    */
                               /* we have no idea about the host       */
                               /* let's default to ourself             */
                               /* ------------------------------------ */

                                   onBehalfAgent = NULL;

                               } else {

                                   strcpy( agent, hoststring );
                               }
                           }

                        pthread_mutex_unlock(  aTask->lock );

                        snarl_sendtrap( message_level, onBehalfAgent, aTrap );

                   break;


                   case SNARL_HEART_BEAT_EVENT:

                       globalHeartBeatCount++;

                       heartBeatTrapTrap.siteIndex         = 0;
                       snarl_time(&now);
                       heartBeatTrapTrap.eventTime         = now;
                       heartBeatTrapTrap.subTaskID         = SNARLSNMP_TRAP_SUBTASK_ID_HEARTBEAT;

                       heartBeatTrapTrap.siteStatus        = SNARLSNMP_TRAP_GENERIC_INFO;
                       heartBeatTrapTrap.proxyStatus       = SNARLSNMP_TRAP_GENERIC_INFO;
                       heartBeatTrapTrap.siteGroupStatus   = SNARLSNMP_TRAP_GENERIC_INFO;

                       heartBeatTrapTrap.siteEventNum      = globalHeartBeatCount;
                       heartBeatTrapTrap.proxyEventNum     = globalHeartBeatCount;
                       heartBeatTrapTrap.groupEventNum     = globalHeartBeatCount;

                       strcpy(      heartBeatTrapTrap.siteMessage,   "Heart Beat "SNARL_APP_NAME);

                       strcpy(      heartBeatTrapTrap.siteGroupName, SNARL_APP_NAME );

                       sprintf(     heartBeatTrapTrap.siteName,      "%s", globalID);

                       gethostname( heartBeatTrapTrap.proxy,         SNARL_TASK_PROXY_MAX_LENGTH);

                       sprintf(     heartBeatTrapTrap.effectiveURL,  "http://%s", heartBeatTrapTrap.proxy );


                       snarl_sendtrap( SNARLSNMP_TRAP_GENERIC_INFO,
                                       NULL,
                                       &heartBeatTrapTrap
                                       );

                      snarl_time(&now);
                      queueEvent( SNARL_HEART_BEAT_EVENT, now + globalHeartBeatInterval, NULL );
                   break;


                   case SNARL_NULL_EVENT:

                        idletime.tv_sec  = SNARL_SITE_DEFAULT_WAKEUP_SEC;
                        idletime.tv_usec = SNARL_SITE_DEFAULT_WAKEUP_USEC;
                        select(1, NULL, NULL, NULL, &idletime );
                   break;

                   default:
                   break;

               }
           }
       /* ------------------------------------ */
       /* this is a good point to be cancelled */
       /* check for canncellation              */
       /* ------------------------------------ */

           pthread_testcancel();
       }
}

/***************************************************************
*                                                              *
* initialize the task manager                                  *
*                                                              *
***************************************************************/

int    initializeThreadManager( void ) {

       int             index;
       snarltaskPtr    aTask;
       siteVarsPtr     siteVars;
       time_t          now;

       pthread_mutex_lock(   &globalSchedulerInitLock );


       /* ------------------------------------ */
       /* first setup an event queue           */
       /* ------------------------------------ */

           pthread_mutex_lock(   &globalEventLock );

               InitPriQueue (      &globalEventQueue,
                              (UInt)(globalTaskArray->taskCount),
                                    earlierEvent,
                                    setEventPosition );

               globalEventQueueInited = TRUE;

           pthread_mutex_unlock( &globalEventLock );

       /* ------------------------------------ */
       /* initialize all tasks                 */
       /* ------------------------------------ */

           index = 0;

           while( aTask = find_snarltask(NULL, NULL, &index, globalTaskArray) ) {

               initSnarlTask( aTask );
               invalidate_traps( aTask );


               if( !(aTask->task_waitForExternalTrigger )) {


                   if( aTask->randomSoonStart ) {

       /* ------------------------------------ */
       /* still randomly started               */
       /* to prevent traffic jam on startup    */
       /* ------------------------------------ */

                       snarl_time(&now);
                       aTask->statusVars.next_run     = now + ((globalTaskArray->taskCount) * ( 1.0 - ( rand() / (RAND_MAX + 1.0))));

                   } else {

       /* ------------------------------------ */
       /* randomly started depending on        */
       /* restart interval                     */
       /* to prevent traffic jam on startup    */
       /* ------------------------------------ */

                       snarl_time(&now);
                       aTask->statusVars.next_run    = now + ((aTask->interval) * ( 1.0 - ( rand() / (RAND_MAX  + 1.0))));
                   }

			   } else {

       /* ------------------------------------ */
       /* task is waiting for external trigger */
       /* ------------------------------------ */
				   aTask->statusVars.next_run        = 0;
			   }

       /* ------------------------------------ */
       /* setup next run of task               */
       /* ------------------------------------ */
               if( aTask->statusVars.next_run ) {

			       snarl_ctime( aTask->statusVars.next_run_string,  &aTask->statusVars.next_run, SNARL_TASK_MAX_DATE_STRING_LENGTH );

       /* ------------------------------------ */
       /* setup first run of task              */
       /* ------------------------------------ */
			       if(!(aTask->statusVars.first_run)) aTask->statusVars.first_run             = aTask->statusVars.next_run;
			       snarl_ctime( aTask->statusVars.first_run_string, &aTask->statusVars.first_run, SNARL_TASK_MAX_DATE_STRING_LENGTH );

			   } else {

       /* ------------------------------------ */
       /* task is waiting for external trigger */
       /* ------------------------------------ */

					snprintf(aTask->statusVars.first_run_string, SNARL_TASK_MAX_DATE_STRING_LENGTH, "task is waiting for %s", aTask->triggerTaskName );
				    if(strlen(aTask->statusVars.next_run_string)) aTask->statusVars.next_run_string[strlen(aTask->statusVars.next_run_string)-1]='\0';

					snprintf(aTask->statusVars.next_run_string, SNARL_TASK_MAX_DATE_STRING_LENGTH, "task is waiting for %s", aTask->triggerTaskName );
				    if(strlen(aTask->statusVars.next_run_string)) aTask->statusVars.next_run_string[strlen(aTask->statusVars.next_run_string)-1]='\0';
			   }

       /* ------------------------------------ */
       /* setup last run of task               */
       /* ------------------------------------ */
			   if(!(aTask->statusVars.last_run) )    aTask->statusVars.last_run           = globalFirstRun;
			   snarl_ctime( aTask->statusVars.last_run_string, &aTask->statusVars.last_run, SNARL_TASK_MAX_DATE_STRING_LENGTH );

       /* ------------------------------------ */
       /* setup last status update of task     */
       /* ------------------------------------ */
			   if(!(aTask->statusVars.last_update) ) aTask->statusVars.last_update        = globalFirstRun;


			   siteVars = aTask->siteVars;

			   pthread_mutex_lock(   siteVars->statusVars.statusLock );

       /* ------------------------------------ */
       /* setup next run of site except if     */
       /* task is waiting for external trigger */
       /* ------------------------------------ */
				   if(     aTask->statusVars.next_run
				       && (siteVars->statusVars.next_run > aTask->statusVars.next_run) )  {
					   siteVars->statusVars.next_run = aTask->statusVars.next_run;
					   snarl_ctime( siteVars->statusVars.next_run_string,  &siteVars->statusVars.next_run, SNARL_TASK_MAX_DATE_STRING_LENGTH );
				   }

       /* ------------------------------------ */
       /* setup first run of site              */
       /* ------------------------------------ */
				   if( siteVars->statusVars.first_run > aTask->statusVars.first_run )  {
					   siteVars->statusVars.first_run = aTask->statusVars.first_run;
					   snarl_ctime( siteVars->statusVars.first_run_string,  &siteVars->statusVars.first_run, SNARL_TASK_MAX_DATE_STRING_LENGTH );
				   }

       /* ------------------------------------ */
       /* setup last run of site               */
       /* ------------------------------------ */
				   if( siteVars->statusVars.last_run < aTask->statusVars.last_run )  {
					   siteVars->statusVars.last_run = aTask->statusVars.last_run;
					   snarl_ctime( siteVars->statusVars.last_run_string,  &siteVars->statusVars.last_run, SNARL_TASK_MAX_DATE_STRING_LENGTH );
				   }

       /* ------------------------------------ */
       /* setup last status update of site     */
       /* ------------------------------------ */
				   if( siteVars->statusVars.last_update < aTask->statusVars.last_update )  {
					   siteVars->statusVars.last_update = aTask->statusVars.last_update;
				   }

			   pthread_mutex_unlock( siteVars->statusVars.statusLock );

       /* ------------------------------------ */
       /* eventually queue up the task         */
       /* ------------------------------------ */
               if( !(aTask->task_waitForExternalTrigger )) {
			       queueEvent( SNARL_UPDATE_TASK_EVENT, aTask->statusVars.next_run, aTask );
			   }

           }


           globalSchedulerInited = TRUE;

       /* ------------------------------------ */
       /* are we asked to send heartbeat traps */
       /* ------------------------------------ */

           if( globalHeartBeatInterval ) {

               snarl_time(&now);
               queueEvent( SNARL_HEART_BEAT_EVENT, now, NULL );
           }

       pthread_mutex_unlock( &globalSchedulerInitLock );

       pthread_cond_signal(  &globalSchedulerInitDone );

       return( TRUE );
}


/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_manage_v1trap.c                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* this code manages sending traps                              *
*                                                              *
***************************************************************/


/***************************************************************
*                                                              *
* send a trap                                                  *
*                                                              *
***************************************************************/

void snarl_sendtrap( int specificTrap, char* agent, tasktrapdataPtr theTaskTrapdata )
{

       static trapdata    snarlTrapData;

      /* ------------------------------------ */
      /* varbind 1: siteIndex                 */
      /* ------------------------------------ */
       snarlTrapData.var_siteIndex.name            = globalOID_snarlsnmp_siteIndex;
       snarlTrapData.var_siteIndex.name_length     = globalOID_snarlsnmp_siteIndex_length;

       snarlTrapData.var_siteIndex.type            = ASN_INTEGER;

       snarlTrapData.var_siteIndex.val.integer     = &(theTaskTrapdata->siteIndex);
       snarlTrapData.var_siteIndex.val_len         = sizeof(theTaskTrapdata->siteIndex);

       snarlTrapData.var_siteIndex.next_variable   = &(snarlTrapData.var_subTaskID);

      /* ------------------------------------ */
      /* varbind 2: subTaskID                 */
      /* ------------------------------------ */
       snarlTrapData.var_subTaskID.name            = globalOID_snarlsnmp_subTaskID;
       snarlTrapData.var_subTaskID.name_length     = globalOID_snarlsnmp_subTaskID_length;

       snarlTrapData.var_subTaskID.type            = ASN_INTEGER;

       snarlTrapData.var_subTaskID.val.integer     = &(theTaskTrapdata->subTaskID);
       snarlTrapData.var_subTaskID.val_len         = sizeof(theTaskTrapdata->subTaskID);

       snarlTrapData.var_subTaskID.next_variable   = &(snarlTrapData.var_siteMessage);

      /* ------------------------------------ */
      /* varbind 3: message                   */
      /* ------------------------------------ */
       snarlTrapData.var_siteMessage.name          = globalOID_snarlsnmp_siteTrapMessage;
       snarlTrapData.var_siteMessage.name_length   = globalOID_snarlsnmp_siteTrapMessage_length;

       snarlTrapData.var_siteMessage.type          = ASN_OCTET_STR;

       snarlTrapData.var_siteMessage.val.string    = theTaskTrapdata->siteMessage;
       snarlTrapData.var_siteMessage.val_len       = strlen(theTaskTrapdata->siteMessage);

       snarlTrapData.var_siteMessage.next_variable = &(snarlTrapData.var_siteGroupStatus);

      /* ------------------------------------ */
      /* varbind 4: groupStatus               */
      /* ------------------------------------ */
       snarlTrapData.var_siteGroupStatus.name          = globalOID_snarlsnmp_siteGroupStatus;
       snarlTrapData.var_siteGroupStatus.name_length   = globalOID_snarlsnmp_siteGroupStatus_length;

       snarlTrapData.var_siteGroupStatus.type          = ASN_INTEGER;

       snarlTrapData.var_siteGroupStatus.val.integer   = &(theTaskTrapdata->siteGroupStatus);
       snarlTrapData.var_siteGroupStatus.val_len       = sizeof(theTaskTrapdata->siteGroupStatus);

       snarlTrapData.var_siteGroupStatus.next_variable = &(snarlTrapData.var_siteGroupName);

      /* ------------------------------------ */
      /* varbind 5: group name                */
      /* ------------------------------------ */
       snarlTrapData.var_siteGroupName.name        = globalOID_snarlsnmp_siteGroupName;
       snarlTrapData.var_siteGroupName.name_length = globalOID_snarlsnmp_siteGroupName_length;

       snarlTrapData.var_siteGroupName.type        = ASN_OCTET_STR;

       snarlTrapData.var_siteGroupName.val.string  = theTaskTrapdata->siteGroupName;
       snarlTrapData.var_siteGroupName.val_len     = strlen(theTaskTrapdata->siteGroupName);

       snarlTrapData.var_siteGroupName.next_variable = &(snarlTrapData.var_siteName);

      /* ------------------------------------ */
      /* varbind 6: site name                 */
      /* ------------------------------------ */
       snarlTrapData.var_siteName.name             = globalOID_snarlsnmp_siteName;
       snarlTrapData.var_siteName.name_length      = globalOID_snarlsnmp_siteName_length;

       snarlTrapData.var_siteName.type             = ASN_OCTET_STR;

       snarlTrapData.var_siteName.val.string       = theTaskTrapdata->siteName;
       snarlTrapData.var_siteName.val_len          = strlen(theTaskTrapdata->siteName);

       snarlTrapData.var_siteName.next_variable    = &(snarlTrapData.var_proxy);

      /* ------------------------------------ */
      /* varbind 7: proxy                     */
      /* ------------------------------------ */
       snarlTrapData.var_proxy.name                = globalOID_snarlsnmp_proxy;
       snarlTrapData.var_proxy.name_length         = globalOID_snarlsnmp_proxy_length;

       snarlTrapData.var_proxy.type                = ASN_OCTET_STR;

       snarlTrapData.var_proxy.val.string          = theTaskTrapdata->proxy;
       snarlTrapData.var_proxy.val_len             = strlen(theTaskTrapdata->proxy);

       snarlTrapData.var_proxy.next_variable       = &(snarlTrapData.var_eventTime);

      /* ------------------------------------ */
      /* varbind 8: eventTime                 */
      /* ------------------------------------ */
       snarlTrapData.var_eventTime.name            = globalOID_snarlsnmp_siteEventTime;
       snarlTrapData.var_eventTime.name_length     = globalOID_snarlsnmp_siteEventTime_length;

       snarlTrapData.var_eventTime.type            = ASN_INTEGER;

       snarlTrapData.var_eventTime.val.integer     = &(theTaskTrapdata->eventTime);
       snarlTrapData.var_eventTime.val_len         = sizeof(theTaskTrapdata->eventTime);

       snarlTrapData.var_eventTime.next_variable   = &(snarlTrapData.var_siteStatus);

      /* ------------------------------------ */
      /* varbind 9: site status               */
      /* ------------------------------------ */
       snarlTrapData.var_siteStatus.name           = globalOID_snarlsnmp_siteSiteStatus;
       snarlTrapData.var_siteStatus.name_length    = globalOID_snarlsnmp_siteSiteStatus_length;

       snarlTrapData.var_siteStatus.type           = ASN_INTEGER;

       snarlTrapData.var_siteStatus.val.integer    = &(theTaskTrapdata->siteStatus);
       snarlTrapData.var_siteStatus.val_len        = sizeof(theTaskTrapdata->siteStatus);

       snarlTrapData.var_siteStatus.next_variable  = &(snarlTrapData.var_proxyStatus);

      /* ------------------------------------ */
      /* varbind 10: proxyStatus              */
      /* ------------------------------------ */
       snarlTrapData.var_proxyStatus.name          = globalOID_snarlsnmp_siteProxyStatus;
       snarlTrapData.var_proxyStatus.name_length   = globalOID_snarlsnmp_siteProxyStatus_length;

       snarlTrapData.var_proxyStatus.type          = ASN_INTEGER;

       snarlTrapData.var_proxyStatus.val.integer   = &(theTaskTrapdata->proxyStatus);
       snarlTrapData.var_proxyStatus.val_len       = sizeof(theTaskTrapdata->proxyStatus);


       snarlTrapData.var_proxyStatus.next_variable = &(snarlTrapData.var_siteEventNum);

      /* ------------------------------------ */
      /* varbind 11: siteEventNum             */
      /* ------------------------------------ */
       snarlTrapData.var_siteEventNum.name         = globalOID_snarlsnmp_siteEventNum;
       snarlTrapData.var_siteEventNum.name_length  = globalOID_snarlsnmp_siteEventNum_length;

       snarlTrapData.var_siteEventNum.type         = ASN_INTEGER;

       snarlTrapData.var_siteEventNum.val.integer  = &(theTaskTrapdata->siteEventNum);
       snarlTrapData.var_siteEventNum.val_len      = sizeof(theTaskTrapdata->siteEventNum);

       snarlTrapData.var_siteEventNum.next_variable = &(snarlTrapData.var_proxyEventNum);

      /* ------------------------------------ */
      /* varbind 12: proxyEventNum            */
      /* ------------------------------------ */
       snarlTrapData.var_proxyEventNum.name          = globalOID_snarlsnmp_proxyEventNum;
       snarlTrapData.var_proxyEventNum.name_length   = globalOID_snarlsnmp_proxyEventNum_length;

       snarlTrapData.var_proxyEventNum.type          = ASN_INTEGER;

       snarlTrapData.var_proxyEventNum.val.integer   = &(theTaskTrapdata->proxyEventNum);
       snarlTrapData.var_proxyEventNum.val_len       = sizeof(theTaskTrapdata->proxyEventNum);

       snarlTrapData.var_proxyEventNum.next_variable = &(snarlTrapData.var_groupEventNum);

      /* ------------------------------------ */
      /* varbind 13: groupEventNum            */
      /* ------------------------------------ */
       snarlTrapData.var_groupEventNum.name          = globalOID_snarlsnmp_groupEventNum;
       snarlTrapData.var_groupEventNum.name_length   = globalOID_snarlsnmp_groupEventNum_length;

       snarlTrapData.var_groupEventNum.type          = ASN_INTEGER;

       snarlTrapData.var_groupEventNum.val.integer   = &(theTaskTrapdata->groupEventNum);
       snarlTrapData.var_groupEventNum.val_len       = sizeof(theTaskTrapdata->groupEventNum);

       snarlTrapData.var_groupEventNum.next_variable = &(snarlTrapData.var_effectiveURL);;

      /* ------------------------------------ */
      /* varbind 14: effectiveURL             */
      /* ------------------------------------ */
       snarlTrapData.var_effectiveURL.name           = globalOID_snarlsnmp_effectiveURL;
       snarlTrapData.var_effectiveURL.name_length    = globalOID_snarlsnmp_effectiveURL_length;

       snarlTrapData.var_effectiveURL.type           = ASN_OCTET_STR;

       snarlTrapData.var_effectiveURL.val.string     = theTaskTrapdata->effectiveURL;
       snarlTrapData.var_effectiveURL.val_len        = strlen(theTaskTrapdata->effectiveURL);

       snarlTrapData.var_effectiveURL.next_variable  = NULL;



       snarl_send_enterprise_trap_vars(  6, specificTrap, agent,

                                         globalOID_snarlsnmp_enterprise,
                                         OID_LENGTH( globalOID_snarlsnmp_enterprise),

                                         &(snarlTrapData.var_siteIndex)

                                      );

}



/***************************************************************
*                                                              *
* send a trap                                                  *
*                                                              *
* modified version of the snmpd agent_trap.c function          *
* 'send_enterprise_trap_vars                                   *
*                                                              *
***************************************************************/

void snarl_send_enterprise_trap_vars ( int                    trap,
                                       int                    specific,
                                       char*                  agent,
                                       oid                   *enterprise, int enterprise_length,
                                       netsnmp_variable_list *vars)
{
               netsnmp_variable_list   uptime_var, snmptrap_var, enterprise_var;
               netsnmp_variable_list  *v2_vars, *last_var=NULL;
               netsnmp_pdu            *template_pdu;


   struct      timeval                 now;
   long        uptime;
   struct      sockaddr_in            *pduIp;

               /* netsnmp v 5.x has made a backstep in compatibility */
               /* -------------------------------------------------- */
               u_int32_t              *agentaddr;
               /* -------------------------------------------------- */

   struct      trap_sink              *sink;
   oid                                 temp_oid[MAX_OID_LEN];

   struct      hostent                *agenthostent;

   struct      snmp_session           *mysesp;

      /* ------------------------------------ */
      /* initialise SNMPv2 required variables */
      /* ------------------------------------ */

      /* ------------------------------------ */
      /* create uptime variable               */
      /* ------------------------------------ */

   gettimeofday(&now, NULL);
   uptime                    = calculate_time_diff(&now, &starttime);

   memset(             &uptime_var, 0, sizeof (struct variable_list));
   snmp_set_var_objid( &uptime_var, &sysuptime_oid, OID_LENGTH(sysuptime_oid));
   snmp_set_var_value( &uptime_var, (u_char *)&uptime, sizeof(uptime) );

   uptime_var.type           = ASN_TIMETICKS;
   uptime_var.next_variable  = &snmptrap_var;


      /* ------------------------------------ */
      /* create snmpTrapOID variable          */
      /* ------------------------------------ */

   memset (&snmptrap_var, 0, sizeof (struct variable_list));
   snmp_set_var_objid( &snmptrap_var, &snmptrap_oid, OID_LENGTH(snmptrap_oid));
      /* value set later ....                 */
   snmptrap_var.type         = ASN_OBJECT_ID;


      /* ------------------------------------ */
      /* bind custom variables or the requred */
      /* last variable or the enterprise var  */
      /* if no variable is available          */
      /* ------------------------------------ */

   if ( vars )
   snmptrap_var.next_variable  = vars;
   else
   snmptrap_var.next_variable  = &enterprise_var;


      /* ------------------------------------ */
      /* find end of provided varbind list,   */
      /* ready to append the enterprise info  */
      /* if necessary                         */
      /* ------------------------------------ */


   last_var = vars;
   while ( last_var && last_var->next_variable ) {

       last_var = last_var->next_variable;
   }


      /* ------------------------------------ */
      /* create the enterprise variable       */
      /* ------------------------------------ */

   memset(             &enterprise_var, 0, sizeof (struct variable_list));
   snmp_set_var_objid( &enterprise_var, &snmptrapenterprise_oid, OID_LENGTH(snmptrapenterprise_oid) );
   snmp_set_var_value( &enterprise_var, (u_char *)enterprise, enterprise_length*sizeof(oid));

   enterprise_var.type           = ASN_OBJECT_ID;
   enterprise_var.next_variable  = NULL;

   v2_vars = &uptime_var;


      /* ------------------------------------ */
      /* Create a template PDU, ready for     */
      /* sending                              */
      /* ------------------------------------ */

   template_pdu = snmp_pdu_create( SNMP_MSG_TRAP );

   if ( template_pdu == NULL ) {

      /* ------------------------------------ */
      /* free memory if value stored dynamically*/
      /* ------------------------------------ */
       snmp_set_var_value(&enterprise_var, NULL, 0 );
       return;
   }

   template_pdu->trap_type     = trap;
   template_pdu->specific_type = specific;

   if ( snmp_clone_mem((void **)&template_pdu->enterprise,
                                 enterprise,
                                 enterprise_length*sizeof(oid))  ) {
       snmp_free_pdu( template_pdu );
       snmp_set_var_value(&enterprise_var, NULL, 0);
       return;
   }


   template_pdu->enterprise_length = enterprise_length;
   template_pdu->flags |= UCD_MSG_FLAG_FORCE_PDU_COPY;


      /* ------------------------------------ */
      /* append the IP address of the entity  */
      /* this trap was meant for              */
      /* ------------------------------------ */

   /*
   pduIp = (struct sockaddr_in *)&template_pdu->agent_addr;
   pduIp->sin_family               = AF_INET;
   */

   agentaddr       = (u_int32_t*)&template_pdu->agent_addr;

   if( agent && (agenthostent = gethostbyname( agent )) ) {

      /* ------------------------------------ */
      /* an agent was given on whose behalf   */
      /* we are sending this trap for         */
      /* ------------------------------------ */

       /*
       memcpy( &(pduIp->sin_addr.s_addr),
                 agenthostent->h_addr_list[0],
                 agenthostent->h_length
             );
       */
       memcpy( agentaddr,
               agenthostent->h_addr_list[0],
               4
             );

   } else {

      /* ------------------------------------ */
      /* we are sending the trap for ourself  */
      /* ------------------------------------ */

       /*
       pduIp->sin_addr.s_addr          = get_myaddr();
       */

       *agentaddr = (u_int32_t)get_myaddr();
   }

   template_pdu->time                  = uptime;



      /* ------------------------------------ */
      /*  now use the parameters to determine */
      /*  which v2 variables are needed,      */
      /*  and what values they should take.   */
      /* ------------------------------------ */

   switch ( trap ) {

       case -1:

          /* ------------------------------------ */
          /* SNMPv2 only                          */
          /* check to see whether the variables   */
          /* provided are sufficient for SNMPv2   */
          /* notifications                        */
          /* ------------------------------------ */


          /* ------------------------------------ */
          /* if the first variable is uptime      */
          /* we assume the rest is filled out     */
          /* properly                             */
          /* ------------------------------------ */

           if (   vars
               && snmp_oid_compare(vars->name,
                                   vars->name_length,
                                   &sysuptime_oid,
                                   OID_LENGTH(sysuptime_oid) ) == 0 ) {
               v2_vars = vars;

           } else {


          /* ------------------------------------ */
          /* if the first variable is snmpTrapOID */
          /* prepend optinal uptime to the vars   */
          /* append the enterprise var to it      */
          /* ------------------------------------ */

               if (vars && snmp_oid_compare(vars->name, vars->name_length,
                       &snmptrap_oid, OID_LENGTH(snmptrap_oid)) == 0 )
                   uptime_var.next_variable = vars;
               else {

          /* ------------------------------------ */
          /* we don't even have a snmpTrapOID     */
          /* thogh required in snmpV2             */
          /* we'll send out an incomplete         */
          /* trap with just the info              */
          /* ------------------------------------ */

                   snmptrap_var.next_variable = vars;
               }
           }

          /* ------------------------------------ */
          /* we always append the enterprise info */
          /* ------------------------------------ */

               last_var->next_variable = &enterprise_var;
               last_var                = NULL;

               convert_v2_to_v1( vars, template_pdu );
           break;




          /* ------------------------------------ */
          /* "Standard" SNMPv1 traps              */
          /* ------------------------------------ */

       case SNMP_TRAP_COLDSTART:
           snmp_set_var_value( &snmptrap_var,
                       (u_char *)cold_start_oid,
                       sizeof(cold_start_oid));
           break;

       case SNMP_TRAP_WARMSTART:
           snmp_set_var_value( &snmptrap_var,
                       (u_char *)warm_start_oid,
                       sizeof(warm_start_oid));
           break;

       case SNMP_TRAP_LINKDOWN:
           snmp_set_var_value( &snmptrap_var,
                       (u_char *)link_down_oid,
                       sizeof(link_down_oid));
           break;

       case SNMP_TRAP_LINKUP:
           snmp_set_var_value( &snmptrap_var,
                       (u_char *)link_up_oid,
                       sizeof(link_up_oid));
           break;

       case SNMP_TRAP_AUTHFAIL:
           if (snmp_enableauthentraps == SNMP_AUTHENTICATED_TRAPS_DISABLED) {
               snmp_set_var_value(&enterprise_var, NULL, 0);
               snmp_free_pdu(template_pdu);
               return;
           }
           snmp_set_var_value( &snmptrap_var,
                       (u_char *)auth_fail_oid,
                       sizeof(auth_fail_oid));
           break;

       case SNMP_TRAP_EGPNEIGHBORLOSS:
           snmp_set_var_value( &snmptrap_var,
                       (u_char *)egp_xxx_oid,
                       sizeof(egp_xxx_oid));
           break;

       case SNMP_TRAP_ENTERPRISESPECIFIC:
           memcpy(     temp_oid,
                       (char *)enterprise,
                       (enterprise_length)*sizeof(oid));
           temp_oid[ enterprise_length   ] = 0;
           temp_oid[ enterprise_length+1 ] = specific;
           snmp_set_var_value( &snmptrap_var,
                       (u_char *)temp_oid,
                       (enterprise_length+2)*sizeof(oid));
           snmptrap_var.next_variable  = vars;

          /* ------------------------------------ */
          /* Don't need version/enterprise info   */
          /* ------------------------------------ */

           last_var = NULL;
           break;
   }




          /* ------------------------------------ */
          /* now loop through the list of trap    */
          /* sinks, sending an appropriately      */
          /* formatted PDU to each                */
          /* ------------------------------------ */

   for ( sink = sinks ; sink ; sink=sink->next ) {

       if ( sink->version == SNMP_VERSION_1 && trap == -1 ) {

          /* ------------------------------------ */
          /* Skip v1 sinks for v2 only traps      */
          /* ------------------------------------ */
           continue;
       }

       template_pdu->command = sink->pdutype;

       if ( sink->version != SNMP_VERSION_1 ) {

           template_pdu->variables = v2_vars;

           if ( last_var )
           last_var->next_variable = &enterprise_var;
       }
       else {
           template_pdu->variables = vars;
       }

       send_trap_to_sess(sink->sesp, template_pdu);

       if ( sink->version != SNMP_VERSION_1 && last_var )
            last_var->next_variable = NULL;
   }

          /* ------------------------------------ */
          /* send stuff to registered callbacks   */
          /* v2 traps/informs                     */
          /* ------------------------------------ */

   template_pdu->variables = v2_vars;

   if ( last_var )
        last_var->next_variable = &enterprise_var;

   snmp_call_callbacks( SNMP_CALLBACK_APPLICATION,
                        SNMPD_CALLBACK_SEND_TRAP2,
                        template_pdu );

   if ( last_var )
        last_var->next_variable = NULL;


          /* ------------------------------------ */
          /* v1 traps                             */
          /* ------------------------------------ */

   template_pdu->command   = SNMP_MSG_TRAP;
   template_pdu->variables = vars;


   snmp_call_callbacks( SNMP_CALLBACK_APPLICATION,
                        SNMPD_CALLBACK_SEND_TRAP1,
                        template_pdu );

   snmp_set_var_value(&enterprise_var, NULL, 0);
   snmp_set_var_value(&snmptrap_var, NULL, 0);

          /* ------------------------------------ */
          /* ensure we don't free anything we     */
          /* shouldn't                            */
          /* ------------------------------------ */

   if ( last_var )
        last_var->next_variable = NULL;

   template_pdu->variables = NULL;
   snmp_free_pdu( template_pdu );
}

/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_manage_getdat.c                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* The GNU Public License may be overriden by the statement of  *
* Steven M. Bellovin, the author of this module                *
*                                                              *
***************************************************************/

/**************************************************************
*														      *
* Originally written by Steven M. Bellovin             	      *
* <smb@research.att.com> while at the University of North     *
* Carolina at Chapel Hill.  Later tweaked by a couple of      *
* people on Usenet.  Completely overhauled by Rich $alz	      *
* <rsalz@bbn.com> and Jim Berets <jberets@bbn.com> in	      *
* August, 1990.											      *
*														      *
* This code is in the public domain and has no copyright.     *
*														      *
**************************************************************/


void snarl_ctime( char* buffer, time_t* aTime, int maxlen ) {

    struct tm *tmp;

    pthread_mutex_lock(   &globalSnarlCtimeLock );

        tmp = gmtime(aTime);

        strncpy( buffer, asctime(tmp), maxlen );

        if(strlen(buffer) && (strlen(buffer) < maxlen + 4) ) {

            strcpy( &buffer[strlen(buffer)-1], " UTC" );
        }

    pthread_mutex_unlock( &globalSnarlCtimeLock );

}

void snarl_time( time_t* now ) {

	struct tm *tmp;
	time_t dummy;

    pthread_mutex_lock( &globalSnarlTimeLock );

        snarl_getdate( "", 0, now );

    pthread_mutex_unlock( &globalSnarlTimeLock );
}



/* A Bison parser, made from getdate.y
   by GNU bison 1.35.  */

#define YYBISON 1  /* Identify Bison output.  */

# define	tAGO	257
# define	tDAY	258
# define	tDAY_UNIT	259
# define	tDAYZONE	260
# define	tDST	261
# define	tHOUR_UNIT	262
# define	tID	263
# define	tMERIDIAN	264
# define	tMINUTE_UNIT	265
# define	tMONTH	266
# define	tMONTH_UNIT	267
# define	tSEC_UNIT	268
# define	tSNUMBER	269
# define	tUNUMBER	270
# define	tYEAR_UNIT	271
# define	tZONE	272
# define	tUPCOMING	273
# define	tNEW	274
# define	tWDAY_UNIT	275
# define	tWEDAY_UNIT	276
# define	tSEPARATOR	277

#line 1 "getdate.y"

/*
**  Originally written by Steven M. Bellovin <smb@research.att.com> while
**  at the University of North Carolina at Chapel Hill.  Later tweaked by
**  a couple of people on Usenet.  Completely overhauled by Rich $alz
**  <rsalz@bbn.com> and Jim Berets <jberets@bbn.com> in August, 1990;
**
**  This grammar has 13 shift/reduce conflicts.
**
**  This code is in the public domain and has no copyright.
*/

/*
**  Many updates by Stephan Wenzel while developing SNARLsnmp
**
**  the changes reflect that all times are interpreted and
**  calculated to UTC
**
**  added operator new, upcoming, workingday, weekendday and ctime
*/

#if HAVE_CONFIG_H
# include <config.h>
#endif

#ifndef PARAMS
# if defined PROTOTYPES || (defined __STDC__ && __STDC__)
#  define PARAMS(Args) Args
# else
#  define PARAMS(Args) ()
# endif
#endif

# include <sys/types.h>

# if TIME_WITH_SYS_TIME
#  include <sys/time.h>
#  include <time.h>
# else
#  if HAVE_SYS_TIME_H
#   include <sys/time.h>
#  else
#   include <time.h>
#  endif
# endif


#ifdef HAVE_CONFIG_H
# ifdef FORCE_ALLOCA_H
#  include <alloca.h>
# endif
#endif

/* Since the code of getdate.y is not included in the Emacs executable
   itself, there is no need to #define static in this file.  Even if
   the code were included in the Emacs executable, it probably
   wouldn't do any harm to #undef it here; this will only cause
   problems if we try to write to a static variable, which I don't
   think this code needs to do.  */
#ifdef emacs
# undef static
#endif

#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif

#ifdef HAVE_CTYPE_H
#include <ctype.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#if defined (STDC_HEADERS) || (!defined (isascii) && !defined (HAVE_ISASCII))
# define IN_CTYPE_DOMAIN(c) 1
#else
# define IN_CTYPE_DOMAIN(c) isascii(c)
#endif

#define ISSPACE(c) (IN_CTYPE_DOMAIN (c) && isspace (c))
#define ISALPHA(c) (IN_CTYPE_DOMAIN (c) && isalpha (c))
#define ISUPPER(c) (IN_CTYPE_DOMAIN (c) && isupper (c))
#define ISDIGIT_LOCALE(c) (IN_CTYPE_DOMAIN (c) && isdigit (c))

/* ISDIGIT differs from ISDIGIT_LOCALE, as follows:
   - Its arg may be any int or unsigned int; it need not be an unsigned char.
   - It's guaranteed to evaluate its argument exactly once.
   - It's typically faster.
   Posix 1003.2-1992 section 2.5.2.1 page 50 lines 1556-1558 says that
   only '0' through '9' are digits.  Prefer ISDIGIT to ISDIGIT_LOCALE unless
   it's important to use the locale's definition of `digit' even when the
   host does not conform to Posix.  */
#define ISDIGIT(c) ((unsigned) (c) - '0' <= 9)

/*
#include "getdate.h"
*/

#if defined (STDC_HEADERS) || defined (USG)
# include <string.h>
#endif

/* Some old versions of bison generate parsers that use bcopy.
   That loses on systems that don't provide the function, so we have
   to redefine it here.  */
#if !defined (HAVE_BCOPY) && defined (HAVE_MEMCPY) && !defined (bcopy)
# define bcopy(from, to, len) memcpy ((to), (from), (len))
#endif

/* Remap normal yacc parser interface names (yyparse, yylex, yyerror, etc),
   as well as gratuitiously global symbol names, so we can have multiple
   yacc generated parsers in the same program.  Note that these are only
   the variables produced by yacc.  If other parser generators (bison,
   byacc, etc) produce additional global names that conflict at link time,
   then those parser generators need to be fixed instead of adding those
   names to this list. */

#define yymaxdepth gd_maxdepth
#define yyparse gd_parse
#define yylex   gd_lex
#define yyerror gd_error
#define yylval  gd_lval
#define yychar  gd_char
#define yydebug gd_debug
#define yypact  gd_pact
#define yyr1    gd_r1
#define yyr2    gd_r2
#define yydef   gd_def
#define yychk   gd_chk
#define yypgo   gd_pgo
#define yyact   gd_act
#define yyexca  gd_exca
#define yyerrflag gd_errflag
#define yynerrs gd_nerrs
#define yyps    gd_ps
#define yypv    gd_pv
#define yys     gd_s
#define yy_yys  gd_yys
#define yystate gd_state
#define yytmp   gd_tmp
#define yyv     gd_v
#define yy_yyv  gd_yyv
#define yyval   gd_val
#define yylloc  gd_lloc
#define yyreds  gd_reds          /* With YYDEBUG defined */
#define yytoks  gd_toks          /* With YYDEBUG defined */
#define yylhs   gd_yylhs
#define yylen   gd_yylen
#define yydefred gd_yydefred
#define yydgoto gd_yydgoto
#define yysindex gd_yysindex
#define yyrindex gd_yyrindex
#define yygindex gd_yygindex
#define yytable  gd_yytable
#define yycheck  gd_yycheck

static int yylex (void);
static int yyerror (char *s);
extern int yyparse (void);

#define EPOCH		    1970
#define TM_YEAR_ORIGIN  1900
#define HOUR(x)		   ((x) * 60)

#define MAX_BUFF_LEN    128   /* size of buffer to read the date into */

/*
**  An entry in the lexical lookup table.
*/
typedef struct _TABLE {
    const char	*name;
    int		type;
    int		value;
} TABLE;


/*
**  Meridian:  am, pm, or 24-hour style.
*/
typedef enum _MERIDIAN {
    MERam, MERpm, MER24
} MERIDIAN;

/* exp */
static int ToYear (int Year);
static int ToHour (int Hours, MERIDIAN Meridian);
void       corr_zone( struct tm* tm );
void       clear_flags(void);
/* /exp */


/*
**  Global variables.  We could get rid of most of these by using a good
**  union as the yacc stack.  (This routine was originally written before
**  yacc had the %union construct.)  Maybe someday; right now we only use
**  the %union very rarely.
*/
static const char	*yyInput;

static int       yyAbsoluteDate;

static int       yyDayOrdinal;
static int       yyDayNumber;

static int       yyHaveDate;
static int       yyHaveDay;
static int       yyHaveRel;
static int       yyHaveFixRel;
static int       yyHaveWday;
static int       yyHaveWEday;
static int       yyHaveTime;
static int       yyHaveZone;
static int       yyHaveUpcoming;

static int       yyTimezone;

static int       yyDay;
static int       yyWDay;
static int       yyHour;
static int       yyMinutes;
static int       yyMonth;
static int       yySeconds;
static int       yyYear;
static MERIDIAN  yyMeridian;

static int       yyRelDay;
static int       yyRelHour;
static int       yyRelMinutes;
static int       yyRelSeconds;
static int       yyRelMonth;
static int       yyRelYear;

static int       yyFixRelDay;
static int       yyFixRelHour;
static int       yyFixRelMinutes;
static int       yyFixRelMonth;
static int       yyFixRelYear;

static int       yyRelWday;
static int       yyRelWEday;

static struct tm curr_gmtm;
static struct tm curr_localtm;

static int       w,r,x;

time_t          curr_time;
time_t          mktime2gmdiff;



#line 255 "getdate.y"
#ifndef YYSTYPE
typedef union {
    int			    Number;
    int             Flag;
    enum _MERIDIAN	Meridian;
} yystype;
# define YYSTYPE yystype
# define YYSTYPE_IS_TRIVIAL 1
#endif
#ifndef YYDEBUG
# define YYDEBUG 0
#endif



#define	YYFINAL		131
#define	YYFLAG		-32768
#define	YYNTBASE	28

/* YYTRANSLATE(YYLEX) -- Bison token number corresponding to YYLEX. */
#define YYTRANSLATE(x) ((unsigned)(x) <= 277 ? yytranslate[x] : 48)

/* YYTRANSLATE[YYLEX] -- Bison token number corresponding to YYLEX. */
static const char yytranslate[] =
{
       0,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,    25,     2,    26,    27,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,    24,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     1,     3,     4,     5,
       6,     7,     8,     9,    10,    11,    12,    13,    14,    15,
      16,    17,    18,    19,    20,    21,    22,    23
};

#if YYDEBUG
static const short yyprhs[] =
{
       0,     0,     1,     4,     6,     8,    10,    12,    14,    16,
      18,    20,    22,    24,    26,    28,    38,    41,    46,    51,
      58,    65,    67,    69,    72,    74,    77,    80,    84,    88,
      93,    99,   105,   109,   113,   116,   121,   124,   128,   132,
     137,   140,   142,   145,   148,   150,   153,   156,   158,   161,
     164,   166,   169,   172,   174,   177,   180,   182,   185,   188,
     190,   193,   196,   200,   205,   208,   211,   215,   218,   221,
     224,   226,   229,   232,   234,   237,   240,   242,   245,   248,
     250,   253,   256,   258,   261,   264,   266,   269,   272,   274,
     277,   279,   282,   285,   287,   290,   292,   295,   298,   300,
     302,   303,   305
};
static const short yyrhs[] =
{
      -1,    28,    29,     0,    30,     0,    31,     0,    32,     0,
      34,     0,    33,     0,    35,     0,    37,     0,    39,     0,
      41,     0,    43,     0,    45,     0,    47,     0,     4,    12,
      16,    16,    24,    16,    24,    16,    16,     0,    16,    10,
       0,    16,    24,    16,    46,     0,    16,    24,    16,    15,
       0,    16,    24,    16,    24,    16,    46,     0,    16,    24,
      16,    24,    16,    15,     0,    18,     0,     6,     0,    18,
       7,     0,     4,     0,     4,    25,     0,    16,     4,     0,
      16,    26,     4,     0,    16,    27,    16,     0,    16,    26,
      16,    26,     0,    16,    27,    16,    27,    16,     0,    16,
      26,    16,    26,    16,     0,    16,    15,    15,     0,    16,
      12,    15,     0,    12,    16,     0,    12,    16,    25,    16,
       0,    16,    12,     0,    16,    26,    12,     0,    16,    12,
      16,     0,    16,    26,    12,    16,     0,    36,     3,     0,
      36,     0,    16,    17,     0,    15,    17,     0,    17,     0,
      16,    13,     0,    15,    13,     0,    13,     0,    16,     5,
       0,    15,     5,     0,     5,     0,    16,     8,     0,    15,
       8,     0,     8,     0,    16,    11,     0,    15,    11,     0,
      11,     0,    16,    14,     0,    15,    14,     0,    14,     0,
      19,    38,     0,    19,    31,     0,    16,    27,    16,     0,
      16,    26,    16,    26,     0,    12,    16,     0,    16,    12,
       0,    16,    26,    12,     0,    20,    40,     0,    16,    17,
       0,    15,    17,     0,    17,     0,    16,    13,     0,    15,
      13,     0,    13,     0,    16,     5,     0,    15,     5,     0,
       5,     0,    16,     8,     0,    15,     8,     0,     8,     0,
      16,    11,     0,    15,    11,     0,    11,     0,    16,    21,
       0,    15,    21,     0,    21,     0,    16,    22,     0,    15,
      22,     0,    22,     0,    42,     3,     0,    42,     0,    16,
      21,     0,    15,    21,     0,    21,     0,    44,     3,     0,
      44,     0,    16,    22,     0,    15,    22,     0,    22,     0,
      16,     0,     0,    10,     0,    23,     0
};

#endif

#if YYDEBUG
/* YYRLINE[YYN] -- source line where rule number YYN was defined. */
static const short yyrline[] =
{
       0,   274,   275,   278,   294,   303,   308,   317,   325,   337,
     340,   426,   459,   495,   500,   508,   529,   535,   541,   548,
     554,   564,   567,   570,   576,   580,   584,   588,   594,   598,
     602,   622,   640,   646,   653,   657,   663,   667,   671,   677,
     685,   694,   698,   701,   704,   707,   710,   713,   716,   719,
     722,   725,   728,   731,   734,   737,   740,   743,   746,   749,
     754,   757,   772,   788,   804,   820,   836,   853,   858,   861,
     864,   867,   870,   873,   876,   879,   882,   885,   888,   891,
     894,   897,   900,   903,   906,   909,   912,   915,   918,   923,
     928,   931,   934,   937,   945,   950,   953,   956,   959,   967,
    1015,  1019,  1024
};
#endif


#if (YYDEBUG) || defined YYERROR_VERBOSE

/* YYTNAME[TOKEN_NUM] -- String name of the token TOKEN_NUM. */
static const char *const yytname[] =
{
  "$", "error", "$undefined.", "tAGO", "tDAY", "tDAY_UNIT", "tDAYZONE",
  "tDST", "tHOUR_UNIT", "tID", "tMERIDIAN", "tMINUTE_UNIT", "tMONTH",
  "tMONTH_UNIT", "tSEC_UNIT", "tSNUMBER", "tUNUMBER", "tYEAR_UNIT",
  "tZONE", "tUPCOMING", "tNEW", "tWDAY_UNIT", "tWEDAY_UNIT", "tSEPARATOR",
  "':'", "','", "'.'", "'/'", "spec", "item", "ctime", "time", "zone",
  "day", "date", "rel", "relunit", "upcoming", "upcomtime", "fixed",
  "fixedunit", "relwday", "wday", "relweday", "weday", "number",
  "o_merid", "sepchar", 0
};
#endif

/* YYR1[YYN] -- Symbol number of symbol that rule YYN derives. */
static const short yyr1[] =
{
       0,    28,    28,    29,    29,    29,    29,    29,    29,    29,
      29,    29,    29,    29,    29,    30,    31,    31,    31,    31,
      31,    32,    32,    32,    33,    33,    33,    33,    34,    34,
      34,    34,    34,    34,    34,    34,    34,    34,    34,    34,
      35,    35,    36,    36,    36,    36,    36,    36,    36,    36,
      36,    36,    36,    36,    36,    36,    36,    36,    36,    36,
      37,    37,    38,    38,    38,    38,    38,    39,    40,    40,
      40,    40,    40,    40,    40,    40,    40,    40,    40,    40,
      40,    40,    40,    40,    40,    40,    40,    40,    40,    41,
      41,    42,    42,    42,    43,    43,    44,    44,    44,    45,
      46,    46,    47
};

/* YYR2[YYN] -- Number of symbols composing right hand side of rule YYN. */
static const short yyr2[] =
{
       0,     0,     2,     1,     1,     1,     1,     1,     1,     1,
       1,     1,     1,     1,     1,     9,     2,     4,     4,     6,
       6,     1,     1,     2,     1,     2,     2,     3,     3,     4,
       5,     5,     3,     3,     2,     4,     2,     3,     3,     4,
       2,     1,     2,     2,     1,     2,     2,     1,     2,     2,
       1,     2,     2,     1,     2,     2,     1,     2,     2,     1,
       2,     2,     3,     4,     2,     2,     3,     2,     2,     2,
       1,     2,     2,     1,     2,     2,     1,     2,     2,     1,
       2,     2,     1,     2,     2,     1,     2,     2,     1,     2,
       1,     2,     2,     1,     2,     1,     2,     2,     1,     1,
       0,     1,     1
};

/* YYDEFACT[S] -- default rule to reduce with in state S when YYTABLE
   doesn't specify something else to do.  Zero means the default is an
   error. */
static const short yydefact[] =
{
       1,     0,    24,    50,    22,    53,    56,     0,    47,    59,
       0,    99,    44,    21,     0,     0,    93,    98,   102,     2,
       3,     4,     5,     7,     6,     8,    41,     9,    10,    11,
      90,    12,    95,    13,    14,     0,    25,    34,    49,    52,
      55,    46,    58,    43,    92,    97,    26,    48,    51,    16,
      54,    36,    45,    57,     0,    42,    91,    96,     0,     0,
       0,    23,     0,     0,    61,    60,    76,    79,    82,    73,
       0,     0,    70,    85,    88,    67,    40,    89,    94,     0,
       0,    33,    38,    32,   100,    27,    37,     0,    28,    64,
      65,     0,     0,    75,    78,    81,    72,    69,    84,    87,
      74,    77,    80,    71,    68,    83,    86,     0,    35,   101,
      18,     0,    17,    39,    29,     0,    66,     0,    62,     0,
     100,    31,    30,    63,     0,    20,    19,     0,     0,    15,
       0,     0
};

static const short yydefgoto[] =
{
       1,    19,    20,    21,    22,    23,    24,    25,    26,    27,
      65,    28,    75,    29,    30,    31,    32,    33,   112,    34
};

static const short yypact[] =
{
  -32768,     0,    14,-32768,-32768,-32768,-32768,   -15,-32768,-32768,
      55,    20,-32768,    -4,    24,    40,-32768,-32768,-32768,-32768,
  -32768,-32768,-32768,-32768,-32768,-32768,    26,-32768,-32768,-32768,
      46,-32768,    56,-32768,-32768,    11,-32768,    33,-32768,-32768,
  -32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
  -32768,    -6,-32768,-32768,    49,-32768,-32768,-32768,    58,    88,
      64,-32768,    65,    79,-32768,-32768,-32768,-32768,-32768,-32768,
      62,    77,-32768,-32768,-32768,-32768,-32768,-32768,-32768,    70,
      71,-32768,-32768,-32768,    28,-32768,    80,    39,    44,-32768,
  -32768,    38,    81,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
  -32768,-32768,-32768,-32768,-32768,-32768,-32768,    54,-32768,-32768,
  -32768,    85,-32768,-32768,    86,    91,-32768,    67,-32768,    92,
      -8,-32768,-32768,-32768,    87,-32768,-32768,    93,    94,-32768,
      95,-32768
};

static const short yypgoto[] =
{
  -32768,-32768,-32768,    98,-32768,-32768,-32768,-32768,-32768,-32768,
  -32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,    -7,-32768
};


#define	YYLAST		113


static const short yytable[] =
{
     130,    37,   109,    61,     2,     3,     4,   125,     5,    81,
      82,     6,     7,     8,     9,    10,    11,    12,    13,    14,
      15,    16,    17,    18,    46,    47,    35,    79,    48,    76,
      49,    50,    51,    52,    53,    54,    62,    55,   109,    36,
      63,    56,    57,   110,    58,    66,    59,    60,    67,    77,
     116,    68,   111,    69,   117,    70,    71,    72,    80,    78,
      38,    73,    74,    39,    83,   114,    40,    93,    41,    42,
      94,   115,    43,    95,    84,    96,    44,    45,   119,    97,
      88,    89,   100,    98,    99,   101,   107,   108,   102,    49,
     103,    90,    85,   123,   104,   131,   113,   118,   105,   106,
      86,   120,   121,    58,    87,    91,    92,   122,   124,   128,
     129,   127,    64,   126
};

static const short yycheck[] =
{
       0,    16,    10,     7,     4,     5,     6,    15,     8,    15,
      16,    11,    12,    13,    14,    15,    16,    17,    18,    19,
      20,    21,    22,    23,     4,     5,    12,    16,     8,     3,
      10,    11,    12,    13,    14,    15,    12,    17,    10,    25,
      16,    21,    22,    15,    24,     5,    26,    27,     8,     3,
      12,    11,    24,    13,    16,    15,    16,    17,    25,     3,
       5,    21,    22,     8,    15,    26,    11,     5,    13,    14,
       8,    27,    17,    11,    16,    13,    21,    22,    24,    17,
      16,    16,     5,    21,    22,     8,    16,    16,    11,    10,
      13,    12,     4,    26,    17,     0,    16,    16,    21,    22,
      12,    16,    16,    24,    16,    26,    27,    16,    16,    16,
      16,    24,    14,   120
};
/* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
#line 3 "/usr/share/bison/bison.simple"

/* Skeleton output parser for bison,

   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002 Free Software
   Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

/* As a special exception, when this file is copied by Bison into a
   Bison output file, you may use that output file without restriction.
   This special exception was added by the Free Software Foundation
   in version 1.24 of Bison.  */

/* This is the parser code that is written into each bison parser when
   the %semantic_parser declaration is not specified in the grammar.
   It was written by Richard Stallman by simplifying the hairy parser
   used when %semantic_parser is specified.  */

/* All symbols defined below should begin with yy or YY, to avoid
   infringing on user name space.  This should be done even for local
   variables, as they might otherwise be expanded by user macros.
   There are some unavoidable exceptions within include files to
   define necessary library symbols; they are noted "INFRINGES ON
   USER NAME SPACE" below.  */

#ifndef YYPARSE_RETURN_TYPE
#define YYPARSE_RETURN_TYPE int
#endif

#if ! defined (yyoverflow) || defined (YYERROR_VERBOSE)

/* The parser invokes alloca or malloc; define the necessary symbols.  */

# if YYSTACK_USE_ALLOCA
#  define YYSTACK_ALLOC alloca
# else
#  ifndef YYSTACK_USE_ALLOCA
#   if defined (alloca) || defined (_ALLOCA_H)
#    define YYSTACK_ALLOC alloca
#   else
#    ifdef __GNUC__
#     define YYSTACK_ALLOC __builtin_alloca
#    endif
#   endif
#  endif
# endif

# ifdef YYSTACK_ALLOC
   /* Pacify GCC's `empty if-body' warning. */
#  define YYSTACK_FREE(Ptr) do { /* empty */; } while (0)
# else
#  if defined (__STDC__) || defined (__cplusplus)
#   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
#   define YYSIZE_T size_t
#  endif
#  define YYSTACK_ALLOC malloc
#  define YYSTACK_FREE free
# endif
#endif /* ! defined (yyoverflow) || defined (YYERROR_VERBOSE) */


#if (! defined (yyoverflow) \
     && (! defined (__cplusplus) \
	 || ((YYLTYPE_IS_TRIVIAL || ! YYLSP_NEEDED) && YYSTYPE_IS_TRIVIAL)))

/* A type that is properly aligned for any stack member.  */
union yyalloc
{
  short yyss;
  YYSTYPE yyvs;
# if YYLSP_NEEDED
  YYLTYPE yyls;
# endif
};

/* The size of the maximum gap between one aligned stack and the next.  */
# define YYSTACK_GAP_MAX (sizeof (union yyalloc) - 1)

/* The size of an array large to enough to hold all stacks, each with
   N elements.  */
# if YYLSP_NEEDED
#  define YYSTACK_BYTES(N) \
     ((N) * (sizeof (short) + sizeof (YYSTYPE) + sizeof (YYLTYPE))	\
      + 2 * YYSTACK_GAP_MAX)
# else
#  define YYSTACK_BYTES(N) \
     ((N) * (sizeof (short) + sizeof (YYSTYPE))				\
      + YYSTACK_GAP_MAX)
# endif

/* Copy COUNT objects from FROM to TO.  The source and destination do
   not overlap.  */
# ifndef YYCOPY
#  if 1 < __GNUC__
#   define YYCOPY(To, From, Count) \
      __builtin_memcpy (To, From, (Count) * sizeof (*(From)))
#  else
#   define YYCOPY(To, From, Count)		\
      do					\
	{					\
	  register YYSIZE_T yyi;		\
	  for (yyi = 0; yyi < (Count); yyi++)	\
	    (To)[yyi] = (From)[yyi];		\
	}					\
      while (0)
#  endif
# endif

/* Relocate STACK from its old location to the new one.  The
   local variables YYSIZE and YYSTACKSIZE give the old and new number of
   elements in the stack, and YYPTR gives the new location of the
   stack.  Advance YYPTR to a properly aligned location for the next
   stack.  */
# define YYSTACK_RELOCATE(Stack)					\
    do									\
      {									\
	YYSIZE_T yynewbytes;						\
	YYCOPY (&yyptr->Stack, Stack, yysize);				\
	Stack = &yyptr->Stack;						\
	yynewbytes = yystacksize * sizeof (*Stack) + YYSTACK_GAP_MAX;	\
	yyptr += yynewbytes / sizeof (*yyptr);				\
      }									\
    while (0)

#endif


#if ! defined (YYSIZE_T) && defined (__SIZE_TYPE__)
# define YYSIZE_T __SIZE_TYPE__
#endif
#if ! defined (YYSIZE_T) && defined (size_t)
# define YYSIZE_T size_t
#endif
#if ! defined (YYSIZE_T)
# if defined (__STDC__) || defined (__cplusplus)
#  include <stddef.h> /* INFRINGES ON USER NAME SPACE */
#  define YYSIZE_T size_t
# endif
#endif
#if ! defined (YYSIZE_T)
# define YYSIZE_T unsigned int
#endif

#define yyerrok		(yyerrstatus = 0)
#define yyclearin	(yychar = YYEMPTY)
#define YYEMPTY		-2
#define YYEOF		0
#define YYACCEPT	goto yyacceptlab
#define YYABORT 	goto yyabortlab
#define YYERROR		goto yyerrlab1
/* Like YYERROR except do call yyerror.  This remains here temporarily
   to ease the transition to the new meaning of YYERROR, for GCC.
   Once GCC version 2 has supplanted version 1, this can go.  */
#define YYFAIL		goto yyerrlab
#define YYRECOVERING()  (!!yyerrstatus)
#define YYBACKUP(Token, Value)					\
do								\
  if (yychar == YYEMPTY && yylen == 1)				\
    {								\
      yychar = (Token);						\
      yylval = (Value);						\
      yychar1 = YYTRANSLATE (yychar);				\
      YYPOPSTACK;						\
      goto yybackup;						\
    }								\
  else								\
    { 								\
      yyerror ("syntax error: cannot back up");			\
      YYERROR;							\
    }								\
while (0)

#define YYTERROR	1
#define YYERRCODE	256


/* YYLLOC_DEFAULT -- Compute the default location (before the actions
   are run).

   When YYLLOC_DEFAULT is run, CURRENT is set the location of the
   first token.  By default, to implement support for ranges, extend
   its range to the last symbol.  */

#ifndef YYLLOC_DEFAULT
# define YYLLOC_DEFAULT(Current, Rhs, N)       	\
   Current.last_line   = Rhs[N].last_line;	\
   Current.last_column = Rhs[N].last_column;
#endif


/* YYLEX -- calling `yylex' with the right arguments.  */

#if YYPURE
# if YYLSP_NEEDED
#  ifdef YYLEX_PARAM
#   define YYLEX		yylex (&yylval, &yylloc, YYLEX_PARAM)
#  else
#   define YYLEX		yylex (&yylval, &yylloc)
#  endif
# else /* !YYLSP_NEEDED */
#  ifdef YYLEX_PARAM
#   define YYLEX		yylex (&yylval, YYLEX_PARAM)
#  else
#   define YYLEX		yylex (&yylval)
#  endif
# endif /* !YYLSP_NEEDED */
#else /* !YYPURE */
# define YYLEX			yylex ()
#endif /* !YYPURE */


/* Enable debugging if requested.  */
#if YYDEBUG

# ifndef YYFPRINTF
#  include <stdio.h> /* INFRINGES ON USER NAME SPACE */
#  define YYFPRINTF fprintf
# endif

# define YYDPRINTF(Args)			\
do {						\
  if (yydebug)					\
    YYFPRINTF Args;				\
} while (0)
/* Nonzero means print parse trace.  It is left uninitialized so that
   multiple parsers can coexist.  */
int yydebug;
#else /* !YYDEBUG */
# define YYDPRINTF(Args)
#endif /* !YYDEBUG */

/* YYINITDEPTH -- initial size of the parser's stacks.  */
#ifndef	YYINITDEPTH
# define YYINITDEPTH 200
#endif

/* YYMAXDEPTH -- maximum size the stacks can grow to (effective only
   if the built-in stack extension method is used).

   Do not make this value too large; the results are undefined if
   SIZE_MAX < YYSTACK_BYTES (YYMAXDEPTH)
   evaluated with infinite-precision integer arithmetic.  */

#if YYMAXDEPTH == 0
# undef YYMAXDEPTH
#endif

#ifndef YYMAXDEPTH
# define YYMAXDEPTH 10000
#endif

#ifdef YYERROR_VERBOSE

# ifndef yystrlen
#  if defined (__GLIBC__) && defined (_STRING_H)
#   define yystrlen strlen
#  else
/* Return the length of YYSTR.  */
static YYSIZE_T
#   if defined (__STDC__) || defined (__cplusplus)
yystrlen (const char *yystr)
#   else
yystrlen (yystr)
     const char *yystr;
#   endif
{
  register const char *yys = yystr;

  while (*yys++ != '\0')
    continue;

  return yys - yystr - 1;
}
#  endif
# endif

# ifndef yystpcpy
#  if defined (__GLIBC__) && defined (_STRING_H) && defined (_GNU_SOURCE)
#   define yystpcpy stpcpy
#  else
/* Copy YYSRC to YYDEST, returning the address of the terminating '\0' in
   YYDEST.  */
static char *
#   if defined (__STDC__) || defined (__cplusplus)
yystpcpy (char *yydest, const char *yysrc)
#   else
yystpcpy (yydest, yysrc)
     char *yydest;
     const char *yysrc;
#   endif
{
  register char *yyd = yydest;
  register const char *yys = yysrc;

  while ((*yyd++ = *yys++) != '\0')
    continue;

  return yyd - 1;
}
#  endif
# endif
#endif

#line 319 "/usr/share/bison/bison.simple"


/* The user can define YYPARSE_PARAM as the name of an argument to be passed
   into yyparse.  The argument should have type void *.
   It should actually point to an object.
   Grammar actions can access the variable by casting it
   to the proper pointer type.  */

#ifdef YYPARSE_PARAM
# if defined (__STDC__) || defined (__cplusplus)
#  define YYPARSE_PARAM_ARG void *YYPARSE_PARAM
#  define YYPARSE_PARAM_DECL
# else
#  define YYPARSE_PARAM_ARG YYPARSE_PARAM
#  define YYPARSE_PARAM_DECL void *YYPARSE_PARAM;
# endif
#else /* !YYPARSE_PARAM */
# define YYPARSE_PARAM_ARG
# define YYPARSE_PARAM_DECL
#endif /* !YYPARSE_PARAM */

/* Prevent warning if -Wstrict-prototypes.  */
#ifdef __GNUC__
# ifdef YYPARSE_PARAM
YYPARSE_RETURN_TYPE yyparse (void *);
# else
YYPARSE_RETURN_TYPE yyparse (void);
# endif
#endif

/* YY_DECL_VARIABLES -- depending whether we use a pure parser,
   variables are global, or local to YYPARSE.  */

#define YY_DECL_NON_LSP_VARIABLES			\
/* The lookahead symbol.  */				\
int yychar;						\
							\
/* The semantic value of the lookahead symbol. */	\
YYSTYPE yylval;						\
							\
/* Number of parse errors so far.  */			\
int yynerrs;

#if YYLSP_NEEDED
# define YY_DECL_VARIABLES			\
YY_DECL_NON_LSP_VARIABLES			\
						\
/* Location data for the lookahead symbol.  */	\
YYLTYPE yylloc;
#else
# define YY_DECL_VARIABLES			\
YY_DECL_NON_LSP_VARIABLES
#endif


/* If nonreentrant, generate the variables here. */

#if !YYPURE
YY_DECL_VARIABLES
#endif  /* !YYPURE */

YYPARSE_RETURN_TYPE
yyparse (YYPARSE_PARAM_ARG)
     YYPARSE_PARAM_DECL
{
  /* If reentrant, generate the variables here. */
#if YYPURE
  YY_DECL_VARIABLES
#endif  /* !YYPURE */

  register int yystate;
  register int yyn;
  int yyresult;
  /* Number of tokens to shift before error messages enabled.  */
  int yyerrstatus;
  /* Lookahead token as an internal (translated) token number.  */
  int yychar1 = 0;

  /* Three stacks and their tools:
     `yyss': related to states,
     `yyvs': related to semantic values,
     `yyls': related to locations.

     Refer to the stacks thru separate pointers, to allow yyoverflow
     to reallocate them elsewhere.  */

  /* The state stack. */
  short	yyssa[YYINITDEPTH];
  short *yyss = yyssa;
  register short *yyssp;

  /* The semantic value stack.  */
  YYSTYPE yyvsa[YYINITDEPTH];
  YYSTYPE *yyvs = yyvsa;
  register YYSTYPE *yyvsp;

#if YYLSP_NEEDED
  /* The location stack.  */
  YYLTYPE yylsa[YYINITDEPTH];
  YYLTYPE *yyls = yylsa;
  YYLTYPE *yylsp;
#endif

#if YYLSP_NEEDED
# define YYPOPSTACK   (yyvsp--, yyssp--, yylsp--)
#else
# define YYPOPSTACK   (yyvsp--, yyssp--)
#endif

  YYSIZE_T yystacksize = YYINITDEPTH;


  /* The variables used to return semantic value and location from the
     action routines.  */
  YYSTYPE yyval;
#if YYLSP_NEEDED
  YYLTYPE yyloc;
#endif

  /* When reducing, the number of symbols on the RHS of the reduced
     rule. */
  int yylen;

  YYDPRINTF ((stderr, "Starting parse\n"));

  yystate = 0;
  yyerrstatus = 0;
  yynerrs = 0;
  yychar = YYEMPTY;		/* Cause a token to be read.  */

  /* Initialize stack pointers.
     Waste one element of value and location stack
     so that they stay on the same level as the state stack.
     The wasted elements are never initialized.  */

  yyssp = yyss;
  yyvsp = yyvs;
#if YYLSP_NEEDED
  yylsp = yyls;
#endif
  goto yysetstate;

/*------------------------------------------------------------.
| yynewstate -- Push a new state, which is found in yystate.  |
`------------------------------------------------------------*/
 yynewstate:
  /* In all cases, when you get here, the value and location stacks
     have just been pushed. so pushing a state here evens the stacks.
     */
  yyssp++;

 yysetstate:
  *yyssp = yystate;

  if (yyssp >= yyss + yystacksize - 1)
    {
      /* Get the current used size of the three stacks, in elements.  */
      YYSIZE_T yysize = yyssp - yyss + 1;

#ifdef yyoverflow
      {
	/* Give user a chance to reallocate the stack. Use copies of
	   these so that the &'s don't force the real ones into
	   memory.  */
	YYSTYPE *yyvs1 = yyvs;
	short *yyss1 = yyss;

	/* Each stack pointer address is followed by the size of the
	   data in use in that stack, in bytes.  */
# if YYLSP_NEEDED
	YYLTYPE *yyls1 = yyls;
	/* This used to be a conditional around just the two extra args,
	   but that might be undefined if yyoverflow is a macro.  */
	yyoverflow ("parser stack overflow",
		    &yyss1, yysize * sizeof (*yyssp),
		    &yyvs1, yysize * sizeof (*yyvsp),
		    &yyls1, yysize * sizeof (*yylsp),
		    &yystacksize);
	yyls = yyls1;
# else
	yyoverflow ("parser stack overflow",
		    &yyss1, yysize * sizeof (*yyssp),
		    &yyvs1, yysize * sizeof (*yyvsp),
		    &yystacksize);
# endif
	yyss = yyss1;
	yyvs = yyvs1;
      }
#else /* no yyoverflow */
# ifndef YYSTACK_RELOCATE
      goto yyoverflowlab;
# else
      /* Extend the stack our own way.  */
      if (yystacksize >= YYMAXDEPTH)
	goto yyoverflowlab;
      yystacksize *= 2;
      if (yystacksize > YYMAXDEPTH)
	yystacksize = YYMAXDEPTH;

      {
	short *yyss1 = yyss;
	union yyalloc *yyptr =
	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
	if (! yyptr)
	  goto yyoverflowlab;
	YYSTACK_RELOCATE (yyss);
	YYSTACK_RELOCATE (yyvs);
# if YYLSP_NEEDED
	YYSTACK_RELOCATE (yyls);
# endif
# undef YYSTACK_RELOCATE
	if (yyss1 != yyssa)
	  YYSTACK_FREE (yyss1);
      }
# endif
#endif /* no yyoverflow */

      yyssp = yyss + yysize - 1;
      yyvsp = yyvs + yysize - 1;
#if YYLSP_NEEDED
      yylsp = yyls + yysize - 1;
#endif

      YYDPRINTF ((stderr, "Stack size increased to %lu\n",
		  (unsigned long int) yystacksize));

      if (yyssp >= yyss + yystacksize - 1)
	YYABORT;
    }

  YYDPRINTF ((stderr, "Entering state %d\n", yystate));

  goto yybackup;


/*-----------.
| yybackup.  |
`-----------*/
yybackup:

/* Do appropriate processing given the current state.  */
/* Read a lookahead token if we need one and don't already have one.  */
/* yyresume: */

  /* First try to decide what to do without reference to lookahead token.  */

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yydefault;

  /* Not known => get a lookahead token if don't already have one.  */

  /* yychar is either YYEMPTY or YYEOF
     or a valid token in external form.  */

  if (yychar == YYEMPTY)
    {
      YYDPRINTF ((stderr, "Reading a token: "));
      yychar = YYLEX;
    }

  /* Convert token to internal form (in yychar1) for indexing tables with */

  if (yychar <= 0)		/* This means end of input. */
    {
      yychar1 = 0;
      yychar = YYEOF;		/* Don't call YYLEX any more */

      YYDPRINTF ((stderr, "Now at end of input.\n"));
    }
  else
    {
      yychar1 = YYTRANSLATE (yychar);

#if YYDEBUG
     /* We have to keep this `#if YYDEBUG', since we use variables
	which are defined only if `YYDEBUG' is set.  */
      if (yydebug)
	{
	  YYFPRINTF (stderr, "Next token is %d (%s",
		     yychar, yytname[yychar1]);
	  /* Give the individual parser a way to print the precise
	     meaning of a token, for further debugging info.  */
# ifdef YYPRINT
	  YYPRINT (stderr, yychar, yylval);
# endif
	  YYFPRINTF (stderr, ")\n");
	}
#endif
    }

  yyn += yychar1;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != yychar1)
    goto yydefault;

  yyn = yytable[yyn];

  /* yyn is what to do for this token type in this state.
     Negative => reduce, -yyn is rule number.
     Positive => shift, yyn is new state.
       New state is final state => don't bother to shift,
       just return success.
     0, or most negative number => error.  */

  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrlab;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrlab;

  if (yyn == YYFINAL)
    YYACCEPT;

  /* Shift the lookahead token.  */
  YYDPRINTF ((stderr, "Shifting token %d (%s), ",
	      yychar, yytname[yychar1]));

  /* Discard the token being shifted unless it is eof.  */
  if (yychar != YYEOF)
    yychar = YYEMPTY;

  *++yyvsp = yylval;
#if YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  /* Count tokens shifted since error; after three, turn off error
     status.  */
  if (yyerrstatus)
    yyerrstatus--;

  yystate = yyn;
  goto yynewstate;


/*-----------------------------------------------------------.
| yydefault -- do the default action for the current state.  |
`-----------------------------------------------------------*/
yydefault:
  yyn = yydefact[yystate];
  if (yyn == 0)
    goto yyerrlab;
  goto yyreduce;


/*-----------------------------.
| yyreduce -- Do a reduction.  |
`-----------------------------*/
yyreduce:
  /* yyn is the number of a rule to reduce with.  */
  yylen = yyr2[yyn];

  /* If YYLEN is nonzero, implement the default value of the action:
     `$$ = $1'.

     Otherwise, the following line sets YYVAL to the semantic value of
     the lookahead token.  This behavior is undocumented and Bison
     users should not rely upon it.  Assigning to YYVAL
     unconditionally makes the parser a bit smaller, and it avoids a
     GCC warning that YYVAL may be used uninitialized.  */
  yyval = yyvsp[1-yylen];

#if YYLSP_NEEDED
  /* Similarly for the default location.  Let the user run additional
     commands if for instance locations are ranges.  */
  yyloc = yylsp[1-yylen];
  YYLLOC_DEFAULT (yyloc, (yylsp - yylen), yylen);
#endif

#if YYDEBUG
  /* We have to keep this `#if YYDEBUG', since we use variables which
     are defined only if `YYDEBUG' is set.  */
  if (yydebug)
    {
      int yyi;

      YYFPRINTF (stderr, "Reducing via rule %d (line %d), ",
		 yyn, yyrline[yyn]);

      /* Print the symbols being reduced, and their result.  */
      for (yyi = yyprhs[yyn]; yyrhs[yyi] > 0; yyi++)
	YYFPRINTF (stderr, "%s ", yytname[yyrhs[yyi]]);
      YYFPRINTF (stderr, " -> %s\n", yytname[yyr1[yyn]]);
    }
#endif

  switch (yyn) {

case 3:
#line 280 "getdate.y"
{

		  curr_gmtm.tm_year  = ToYear (yyYear) - TM_YEAR_ORIGIN;
		  curr_gmtm.tm_mon   = yyMonth - 1;

          curr_gmtm.tm_mday += ((yyDayNumber - curr_gmtm.tm_wday + 7) % 7
		                + 7 * (yyDayOrdinal - (0 < yyDayOrdinal)));

		  curr_gmtm.tm_hour  = ToHour (yyHour, yyMeridian) + curr_gmtm.tm_isdst;
		  curr_gmtm.tm_min   = yyMinutes;
		  curr_gmtm.tm_sec   = yySeconds;

		  mktime(&curr_gmtm);
	;
    break;}
case 4:
#line 294 "getdate.y"
{
	    yyHaveTime++;

		  curr_gmtm.tm_hour  = ToHour (yyHour, yyMeridian) + curr_gmtm.tm_isdst;
		  curr_gmtm.tm_min   = yyMinutes;
		  curr_gmtm.tm_sec   = yySeconds;

		  mktime(&curr_gmtm);
	;
    break;}
case 5:
#line 303 "getdate.y"
{
	    yyHaveZone++;

		  mktime(&curr_gmtm);
	;
    break;}
case 6:
#line 308 "getdate.y"
{
	    yyHaveDate++;

		  curr_gmtm.tm_year  = ToYear (yyYear) - TM_YEAR_ORIGIN;
		  curr_gmtm.tm_mon   = yyMonth - 1;
		  curr_gmtm.tm_mday  = yyDay;

		  mktime(&curr_gmtm);
	;
    break;}
case 7:
#line 317 "getdate.y"
{
	    yyHaveDay++;

          curr_gmtm.tm_mday += ((yyDayNumber - curr_gmtm.tm_wday + 7) % 7
		                + 7 * (yyDayOrdinal - (0 < yyDayOrdinal)));

		  mktime(&curr_gmtm);
	;
    break;}
case 8:
#line 325 "getdate.y"
{
	    yyHaveRel++;

          curr_gmtm.tm_year += yyRelYear;
          curr_gmtm.tm_mon  += yyRelMonth;
          curr_gmtm.tm_mday += yyRelDay;
          curr_gmtm.tm_hour += yyRelHour;
          curr_gmtm.tm_min  += yyRelMinutes;
          curr_gmtm.tm_sec  += yyRelSeconds;

          mktime(&curr_gmtm);
	;
    break;}
case 9:
#line 337 "getdate.y"
{
	      yyHaveUpcoming++;
	;
    break;}
case 10:
#line 340 "getdate.y"
{
	    yyHaveFixRel++;

   	      if( yyFixRelYear    ) { curr_gmtm.tm_year = (int)(curr_gmtm.tm_year / yyFixRelYear)    * yyFixRelYear    + yyFixRelYear;    }
		  if( yyFixRelMonth   ) { curr_gmtm.tm_mon  = (int)(curr_gmtm.tm_mon  / yyFixRelMonth)   * yyFixRelMonth   + yyFixRelMonth;   }
		  if( yyFixRelDay     ) { curr_gmtm.tm_mday = (int)(curr_gmtm.tm_mday / yyFixRelDay)     * yyFixRelDay     + yyFixRelDay;     }
		  if( yyFixRelHour    ) { curr_gmtm.tm_hour = (int)(curr_gmtm.tm_hour / yyFixRelHour)    * yyFixRelHour    + yyFixRelHour;    }
		  if( yyFixRelMinutes ) { curr_gmtm.tm_min  = (int)(curr_gmtm.tm_min  / yyFixRelMinutes) * yyFixRelMinutes + yyFixRelMinutes; }

		  if( yyRelWday )
			{
			  mktime( &curr_gmtm );

			  w = curr_gmtm.tm_wday;
			  r = yyRelWday;

			  if( r > 0 ) {

				x  = ((int)((r-1)/5))*7;    /* 7 days per 5 workingdays */
				x += ((r-1)%5 + 1);         /* add fractional working day weeks */
				x += ((w+(r-1)%5)>=5?2:0);  /* correct for days close to a weekend */
				x -= (w==6?1:w==7?2:0);     /* correct for sat. and sun. */
				curr_gmtm.tm_mday += x;

			  } else {

				r  = -yyRelWday;
				w  = (12-w)%7+1;            /* trick: negative relative */
											/* workingdays are off-symmetrical */
											/* distributed */

				x  =  ((int)((r-1)/5))*7;    /* 7 days per 5 workingdays */
				x +=  ((r-1)%5 + 1);         /* add fractional working day weeks */
				x +=  ((w+(r-1)%5)>=5?2:0);  /* correct for days close to a weekend */
				x -=  (w==6?1:w==7?2:0);     /* correct for sat. and sun. */

				curr_gmtm.tm_mday -= x;
			  }
			}

		  if( yyRelWEday )
			{
			  mktime( &curr_gmtm );

			  w = curr_gmtm.tm_wday;
			  r = yyRelWEday;

			  if( r > 0 ) {

				x  = ((int)((r-1)/2))*7;    /* 7 days per 2 weekenddays */
				x += ((r-1)%2 + 1);         /* add fractional weekend day weeks */
				x += ((w+(r-1)%2)>=7?5:0);  /* correct for days close to a workingday */
				x += (w<5?5-w:0);           /* correct for mon to thu */

				curr_gmtm.tm_mday += x;


			  } else {

				r  = -yyRelWEday;
				w  = (12-w)%7+1;            /* trick: negative relative */
											/* workingdays are off-symmetrical */
											/* distributed */

				x  = ((int)((r-1)/2))*7;    /* 7 days per 2 weekenddays */
				x += ((r-1)%2 + 1);         /* add fractional weekend day weeks */
				x += ((w+(r-1)%2)>=7?5:0);  /* correct for days close to a workingday */
				x += (w<5?5-w:0);           /* correct for mon to thu */

				curr_gmtm.tm_mday -= x;
			  }
			}

		  while( 1 ) {
			  if( yyFixRelMinutes ) { curr_gmtm.tm_sec = 0; break;}
			  if( yyFixRelHour    ) { curr_gmtm.tm_sec = curr_gmtm.tm_min = 0; break;}
			  if( yyFixRelDay     ) { curr_gmtm.tm_sec = curr_gmtm.tm_min = 0; curr_gmtm.tm_hour = curr_gmtm.tm_isdst; break;}
			  if( yyRelWday       ) { curr_gmtm.tm_sec = curr_gmtm.tm_min = 0; curr_gmtm.tm_hour = curr_gmtm.tm_isdst; break;}
			  if( yyRelWEday      ) { curr_gmtm.tm_sec = curr_gmtm.tm_min = 0; curr_gmtm.tm_hour = curr_gmtm.tm_isdst; break;}
			  if( yyFixRelMonth   ) { curr_gmtm.tm_sec = curr_gmtm.tm_min = 0; curr_gmtm.tm_hour = curr_gmtm.tm_isdst; curr_gmtm.tm_mday = 1; break;}
			  if( yyFixRelYear    ) { curr_gmtm.tm_sec = curr_gmtm.tm_min = 0; curr_gmtm.tm_hour = curr_gmtm.tm_isdst; curr_gmtm.tm_mday = curr_gmtm.tm_mon = 1; break;}
			  break;
		  }

		  mktime(&curr_gmtm);
    ;
    break;}
case 11:
#line 426 "getdate.y"
{
	    yyHaveWday++;

			  mktime( &curr_gmtm );

			  w = curr_gmtm.tm_wday;
			  r = yyRelWday;

			  if( r > 0 ) {

				x  = ((int)((r-1)/5))*7;    /* 7 days per 5 workingdays */
				x += ((r-1)%5 + 1);         /* add fractional working day weeks */
				x += ((w+(r-1)%5)>=5?2:0);  /* correct for days close to a weekend */
				x -= (w==6?1:w==7?2:0);     /* correct for sat. and sun. */
				curr_gmtm.tm_mday += x;

			  } else {

				r  = -yyRelWday;
				w  = (12-w)%7+1;            /* trick: negative relative */
											/* workingdays are off-symmetrical */
											/* distributed */

				x  =  ((int)((r-1)/5))*7;    /* 7 days per 5 workingdays */
				x +=  ((r-1)%5 + 1);         /* add fractional working day weeks */
				x +=  ((w+(r-1)%5)>=5?2:0);  /* correct for days close to a weekend */
				x -=  (w==6?1:w==7?2:0);     /* correct for sat. and sun. */

				curr_gmtm.tm_mday -= x;
			  }

		      mktime(&curr_gmtm);
	;
    break;}
case 12:
#line 459 "getdate.y"
{
	    yyHaveWEday++;

			  mktime( &curr_gmtm );

			  w = curr_gmtm.tm_wday;
			  r = yyRelWEday;

			  if( r > 0 ) {

				x  = ((int)((r-1)/2))*7;    /* 7 days per 2 weekenddays */
				x += ((r-1)%2 + 1);         /* add fractional weekend day weeks */
				x += ((w+(r-1)%2)>=7?5:0);  /* correct for days close to a workingday */
				x += (w<5?5-w:0);           /* correct for mon to thu */

				curr_gmtm.tm_mday += x;


			  } else {

				r  = -yyRelWEday;
				w  = (12-w)%7+1;            /* trick: negative relative */
											/* workingdays are off-symmetrical */
											/* distributed */

				x  = ((int)((r-1)/2))*7;    /* 7 days per 2 weekenddays */
				x += ((r-1)%2 + 1);         /* add fractional weekend day weeks */
				x += ((w+(r-1)%2)>=7?5:0);  /* correct for days close to a workingday */
				x += (w<5?5-w:0);           /* correct for mon to thu */

				curr_gmtm.tm_mday -= x;
			  }

		  mktime(&curr_gmtm);

	;
    break;}
case 13:
#line 495 "getdate.y"
{
	    /* see in 'number' */

		  mktime(&curr_gmtm);
	;
    break;}
case 14:
#line 500 "getdate.y"
{

		  mktime(&curr_gmtm);
          corr_zone(&curr_gmtm);
          clear_flags();
	;
    break;}
case 15:
#line 508 "getdate.y"
{

		yyAbsoluteDate = 1;
	    yyDayOrdinal   = 1;

	    yyDayNumber    = yyvsp[-8].Number;
	    yyMonth        = yyvsp[-7].Number;
	    yyDay          = yyvsp[-6].Number;
	    yyHour         = yyvsp[-5].Number;

	    yyMinutes      = yyvsp[-3].Number;

	    yySeconds      = yyvsp[-1].Number;
	    yyMeridian     = MER24;

		yyYear         = yyvsp[0].Number;

		curr_gmtm.tm_year = ToYear (yyYear) - TM_YEAR_ORIGIN;
    ;
    break;}
case 16:
#line 529 "getdate.y"
{
	    yyHour = yyvsp[-1].Number;
	    yyMinutes = 0;
	    yySeconds = 0;
	    yyMeridian = yyvsp[0].Meridian;
	;
    break;}
case 17:
#line 535 "getdate.y"
{
	    yyHour = yyvsp[-3].Number;
	    yyMinutes = yyvsp[-1].Number;
	    yySeconds = 0;
	    yyMeridian = yyvsp[0].Meridian;
	;
    break;}
case 18:
#line 541 "getdate.y"
{
	    yyHour = yyvsp[-3].Number;
	    yyMinutes = yyvsp[-1].Number;
	    yyMeridian = MER24;
	    yyHaveZone++;
	    yyTimezone += (yyvsp[0].Number % 100 + (yyvsp[0].Number / 100) * 60);
	;
    break;}
case 19:
#line 548 "getdate.y"
{
	    yyHour = yyvsp[-5].Number;
	    yyMinutes = yyvsp[-3].Number;
	    yySeconds = yyvsp[-1].Number;
	    yyMeridian = yyvsp[0].Meridian;
	;
    break;}
case 20:
#line 554 "getdate.y"
{
	    yyHour = yyvsp[-5].Number;
	    yyMinutes = yyvsp[-3].Number;
	    yySeconds = yyvsp[-1].Number;
	    yyMeridian = MER24;
	    yyHaveZone++;
	    yyTimezone += (yyvsp[0].Number % 100 + (yyvsp[0].Number / 100) * 60);
	;
    break;}
case 21:
#line 564 "getdate.y"
{
	    yyTimezone = yyvsp[0].Number;
	;
    break;}
case 22:
#line 567 "getdate.y"
{
	    yyTimezone = (yyvsp[0].Number - 60);
	;
    break;}
case 23:
#line 571 "getdate.y"
{
	    yyTimezone = (yyvsp[-1].Number - 60);
	;
    break;}
case 24:
#line 576 "getdate.y"
{
	    yyDayOrdinal = 1;
	    yyDayNumber  = yyvsp[0].Number;
	;
    break;}
case 25:
#line 580 "getdate.y"
{
	    yyDayOrdinal = 1;
	    yyDayNumber  = yyvsp[-1].Number;
	;
    break;}
case 26:
#line 584 "getdate.y"
{
	    yyDayOrdinal = yyvsp[-1].Number;
	    yyDayNumber  = yyvsp[0].Number;
	;
    break;}
case 27:
#line 588 "getdate.y"
{
	    yyDayOrdinal = yyvsp[-2].Number;
	    yyDayNumber  = yyvsp[0].Number;
	;
    break;}
case 28:
#line 594 "getdate.y"
{
	    yyMonth = yyvsp[-2].Number;
	    yyDay   = yyvsp[0].Number;
	;
    break;}
case 29:
#line 598 "getdate.y"
{
	    yyMonth = yyvsp[-1].Number;
	    yyDay   = yyvsp[-3].Number;
	;
    break;}
case 30:
#line 602 "getdate.y"
{
	  /* Interpret as YYYY/MM/DD if $1 >= 1000, otherwise as MM/DD/YY.
	     The goal in recognizing YYYY/MM/DD is solely to support legacy
	     machine-generated dates like those in an RCS log listing.  If
	     you want portability, use the ISO 8601 format.  */
	  if (yyvsp[-4].Number >= 1000)
	    {
	      yyAbsoluteDate = 1;
	      yyYear  = yyvsp[-4].Number;
	      yyMonth = yyvsp[-2].Number;
	      yyDay   = yyvsp[0].Number;
	    }
	  else
	    {
	      yyAbsoluteDate = 1;
	      yyMonth        = yyvsp[-4].Number;
	      yyDay          = yyvsp[-2].Number;
	      yyYear         = yyvsp[0].Number;
	    }
	;
    break;}
case 31:
#line 622 "getdate.y"
{

	  if (yyvsp[0].Number >= 1000)
	    {
	      yyAbsoluteDate = 1;
	      yyYear         = yyvsp[0].Number;
	      yyMonth        = yyvsp[-2].Number;
	      yyDay          = yyvsp[-4].Number;
	    }
	  else
	    {
	      yyAbsoluteDate = 1;
	      yyYear         = ToYear(yyvsp[0].Number) - TM_YEAR_ORIGIN;
	      yyMonth        = yyvsp[-2].Number;
	      yyDay          = yyvsp[-4].Number;
	    }

	;
    break;}
case 32:
#line 640 "getdate.y"
{
	    /* ISO 8601 format.  yyyy-mm-dd.  */
	    yyYear  = yyvsp[-2].Number;
	    yyMonth = -yyvsp[-1].Number;
	    yyDay   = -yyvsp[0].Number;
	;
    break;}
case 33:
#line 646 "getdate.y"
{
	    /* e.g. 17-JUN-1992.  */
	    yyAbsoluteDate = 1;
	    yyDay   = yyvsp[-2].Number;
	    yyMonth = yyvsp[-1].Number;
	    yyYear  = -yyvsp[0].Number;
	;
    break;}
case 34:
#line 653 "getdate.y"
{
	    yyMonth = yyvsp[-1].Number;
	    yyDay   = yyvsp[0].Number;
	;
    break;}
case 35:
#line 657 "getdate.y"
{
	    yyAbsoluteDate = 1;
	    yyMonth = yyvsp[-3].Number;
	    yyDay   = yyvsp[-2].Number;
	    yyYear  = yyvsp[0].Number;
	;
    break;}
case 36:
#line 663 "getdate.y"
{
	    yyMonth = yyvsp[0].Number;
	    yyDay   = yyvsp[-1].Number;
	;
    break;}
case 37:
#line 667 "getdate.y"
{
	    yyMonth = yyvsp[0].Number;
	    yyDay   = yyvsp[-2].Number;
	;
    break;}
case 38:
#line 671 "getdate.y"
{
	    yyAbsoluteDate = 1;
	    yyMonth = yyvsp[-1].Number;
	    yyDay   = yyvsp[-2].Number;
	    yyYear  = yyvsp[0].Number;
	;
    break;}
case 39:
#line 677 "getdate.y"
{
	    yyAbsoluteDate = 1;
	    yyMonth = yyvsp[-1].Number;
	    yyDay   = yyvsp[-3].Number;
	    yyYear  = yyvsp[0].Number;
	;
    break;}
case 40:
#line 685 "getdate.y"
{
	    yyRelSeconds = -yyRelSeconds;
	    yyRelMinutes = -yyRelMinutes;
	    yyRelHour    = -yyRelHour;
	    yyRelDay     = -yyRelDay;
	    yyRelMonth   = -yyRelMonth;
	    yyRelYear    = -yyRelYear;
	;
    break;}
case 42:
#line 698 "getdate.y"
{
	    yyRelYear += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 43:
#line 701 "getdate.y"
{
	    yyRelYear += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 44:
#line 704 "getdate.y"
{
	    yyRelYear += yyvsp[0].Number;
	;
    break;}
case 45:
#line 707 "getdate.y"
{
	    yyRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 46:
#line 710 "getdate.y"
{
	    yyRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 47:
#line 713 "getdate.y"
{
	    yyRelMonth += yyvsp[0].Number;
	;
    break;}
case 48:
#line 716 "getdate.y"
{
	    yyRelDay += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 49:
#line 719 "getdate.y"
{
	    yyRelDay += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 50:
#line 722 "getdate.y"
{
	    yyRelDay += yyvsp[0].Number;
	;
    break;}
case 51:
#line 725 "getdate.y"
{
	    yyRelHour += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 52:
#line 728 "getdate.y"
{
	    yyRelHour += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 53:
#line 731 "getdate.y"
{
	    yyRelHour += yyvsp[0].Number;
	;
    break;}
case 54:
#line 734 "getdate.y"
{
	    yyRelMinutes += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 55:
#line 737 "getdate.y"
{
	    yyRelMinutes += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 56:
#line 740 "getdate.y"
{
	    yyRelMinutes += yyvsp[0].Number;
	;
    break;}
case 57:
#line 743 "getdate.y"
{
	    yyRelSeconds += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 58:
#line 746 "getdate.y"
{
	    yyRelSeconds += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 59:
#line 749 "getdate.y"
{
	    yyRelSeconds += yyvsp[0].Number;
	;
    break;}
case 60:
#line 754 "getdate.y"
{

    ;
    break;}
case 61:
#line 757 "getdate.y"
{

	    curr_time = mktime(&curr_gmtm);

		curr_gmtm.tm_hour = ToHour (yyHour, yyMeridian) + curr_gmtm.tm_isdst;
		curr_gmtm.tm_min  = yyMinutes;
		curr_gmtm.tm_sec  = yySeconds;

	    if( curr_time > mktime(&curr_gmtm) ) {
	        curr_gmtm.tm_mday += 1;
	        mktime(&curr_gmtm);
	    }
    ;
    break;}
case 62:
#line 772 "getdate.y"
{
	    yyMonth = yyvsp[-2].Number;
	    yyDay = yyvsp[0].Number;

	    curr_time = mktime(&curr_gmtm);

	    curr_gmtm.tm_mon = yyMonth - 1;
	    curr_gmtm.tm_mday= yyDay;

	    mktime(&curr_gmtm);

	    if( curr_time > mktime(&curr_gmtm) ) {
	        curr_gmtm.tm_year += 1;
	        mktime(&curr_gmtm);
	    }
	;
    break;}
case 63:
#line 788 "getdate.y"
{
	    yyMonth = yyvsp[-1].Number;
	    yyDay = yyvsp[-3].Number;

	    curr_time = mktime(&curr_gmtm);

	    curr_gmtm.tm_mon = yyMonth - 1;
	    curr_gmtm.tm_mday= yyDay;

	    mktime(&curr_gmtm);

	    if( curr_time > mktime(&curr_gmtm) ) {
	        curr_gmtm.tm_year += 1;
	        mktime(&curr_gmtm);
	    }
	;
    break;}
case 64:
#line 804 "getdate.y"
{
	    yyMonth = yyvsp[-1].Number;
	    yyDay = yyvsp[0].Number;

	    curr_time = mktime(&curr_gmtm);

	    curr_gmtm.tm_mon = yyMonth - 1;
	    curr_gmtm.tm_mday= yyDay;

	    mktime(&curr_gmtm);

	    if( curr_time > mktime(&curr_gmtm) ) {
	        curr_gmtm.tm_year += 1;
	        mktime(&curr_gmtm);
	    }
	;
    break;}
case 65:
#line 820 "getdate.y"
{
	    yyMonth = yyvsp[0].Number;
	    yyDay = yyvsp[-1].Number;

	    curr_time = mktime(&curr_gmtm);

	    curr_gmtm.tm_mon = yyMonth - 1;
	    curr_gmtm.tm_mday= yyDay;

	    mktime(&curr_gmtm);

	    if( curr_time > mktime(&curr_gmtm) ) {
	        curr_gmtm.tm_year += 1;
	        mktime(&curr_gmtm);
	    }
	;
    break;}
case 66:
#line 836 "getdate.y"
{
	    yyMonth = yyvsp[0].Number;
	    yyDay = yyvsp[-2].Number;

	    curr_time = mktime(&curr_gmtm);

	    curr_gmtm.tm_mon = yyMonth - 1;
	    curr_gmtm.tm_mday= yyDay;

	    mktime(&curr_gmtm);

	    if( curr_time > mktime(&curr_gmtm) ) {
	        curr_gmtm.tm_year += 1;
	        mktime(&curr_gmtm);
	    }
	;
    break;}
case 67:
#line 853 "getdate.y"
{

    ;
    break;}
case 68:
#line 858 "getdate.y"
{
	    yyFixRelYear += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 69:
#line 861 "getdate.y"
{
	    yyFixRelYear += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 70:
#line 864 "getdate.y"
{
	    yyFixRelYear += yyvsp[0].Number;
	;
    break;}
case 71:
#line 867 "getdate.y"
{
	    yyFixRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 72:
#line 870 "getdate.y"
{
	    yyFixRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 73:
#line 873 "getdate.y"
{
	    yyFixRelMonth += yyvsp[0].Number;
	;
    break;}
case 74:
#line 876 "getdate.y"
{
	    yyFixRelDay += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 75:
#line 879 "getdate.y"
{
	    yyFixRelDay += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 76:
#line 882 "getdate.y"
{
	    yyFixRelDay += yyvsp[0].Number;
	;
    break;}
case 77:
#line 885 "getdate.y"
{
	    yyFixRelHour += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 78:
#line 888 "getdate.y"
{
	    yyFixRelHour += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 79:
#line 891 "getdate.y"
{
	    yyFixRelHour += yyvsp[0].Number;
	;
    break;}
case 80:
#line 894 "getdate.y"
{
	    yyFixRelMinutes += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 81:
#line 897 "getdate.y"
{
	    yyFixRelMinutes += yyvsp[-1].Number * yyvsp[0].Number;
	;
    break;}
case 82:
#line 900 "getdate.y"
{
	    yyFixRelMinutes += yyvsp[0].Number;
	;
    break;}
case 83:
#line 903 "getdate.y"
{
	    yyRelWday += yyvsp[-1].Number * yyvsp[0].Flag;
	;
    break;}
case 84:
#line 906 "getdate.y"
{
	    yyRelWday += yyvsp[-1].Number * yyvsp[0].Flag;
	;
    break;}
case 85:
#line 909 "getdate.y"
{
	    yyRelWday += yyvsp[0].Flag;
	;
    break;}
case 86:
#line 912 "getdate.y"
{
	    yyRelWEday += yyvsp[-1].Number * yyvsp[0].Flag;
	;
    break;}
case 87:
#line 915 "getdate.y"
{
	    yyRelWEday += yyvsp[-1].Number * yyvsp[0].Flag;
	;
    break;}
case 88:
#line 918 "getdate.y"
{
	    yyRelWEday += yyvsp[0].Flag;
	;
    break;}
case 89:
#line 923 "getdate.y"
{

        yyRelWday *= -1;
    ;
    break;}
case 91:
#line 931 "getdate.y"
{
	    yyRelWday += yyvsp[-1].Number * yyvsp[0].Flag;
	;
    break;}
case 92:
#line 934 "getdate.y"
{
	    yyRelWday += yyvsp[-1].Number * yyvsp[0].Flag;
	;
    break;}
case 93:
#line 937 "getdate.y"
{

	    if( (curr_gmtm.tm_wday == 0) || (curr_gmtm.tm_wday == 6) ) {
	       yyRelWday += yyvsp[0].Flag;
	    }
    ;
    break;}
case 94:
#line 945 "getdate.y"
{

        yyRelWEday *= -1;
    ;
    break;}
case 96:
#line 953 "getdate.y"
{
	    yyRelWEday += yyvsp[-1].Number * yyvsp[0].Flag;
	;
    break;}
case 97:
#line 956 "getdate.y"
{
	    yyRelWEday += yyvsp[-1].Number * yyvsp[0].Flag;
	;
    break;}
case 98:
#line 959 "getdate.y"
{

	    if( (curr_gmtm.tm_wday != 0) && (curr_gmtm.tm_wday != 6) ) {
	       yyRelWEday += yyvsp[0].Flag;
	    }
    ;
    break;}
case 99:
#line 968 "getdate.y"
{
			if (yyHaveTime && yyHaveDate && !yyHaveRel)
			  {
				 yyAbsoluteDate = 1;
				 yyYear = yyvsp[0].Number;

				 curr_gmtm.tm_year = ToYear (yyYear) - TM_YEAR_ORIGIN;
			  }
			else
			  {
			if (yyvsp[0].Number>10000)
			  {
				yyAbsoluteDate = 1;
				yyHaveDate++;

				yyDay= (yyvsp[0].Number)%100;
				yyMonth= (yyvsp[0].Number/100)%100;
				yyYear = yyvsp[0].Number/10000;

				curr_gmtm.tm_year = ToYear (yyYear) - TM_YEAR_ORIGIN;
				curr_gmtm.tm_mon  = yyMonth - 1;
				curr_gmtm.tm_mday = yyDay;
			  }
			else
			  {
				yyHaveTime++;

				if (yyvsp[0].Number < 100)
				  {
				yyHour = yyvsp[0].Number;
				yyMinutes = 0;
				  }
				else
				  {
					yyHour = yyvsp[0].Number / 100;
					yyMinutes = yyvsp[0].Number % 100;
				  }
				yySeconds = 0;
				yyMeridian = MER24;

				curr_gmtm.tm_hour = yyHour + curr_gmtm.tm_isdst;
				curr_gmtm.tm_min  = yyMinutes;
			  }
	      }
    ;
    break;}
case 100:
#line 1016 "getdate.y"
{
	    yyval.Meridian = MER24;
	  ;
    break;}
case 101:
#line 1020 "getdate.y"
{
	    yyval.Meridian = yyvsp[0].Meridian;
	  ;
    break;}
case 102:
#line 1024 "getdate.y"
{

       /* stuff is done above */

    ;
    break;}
}

#line 709 "/usr/share/bison/bison.simple"


  yyvsp -= yylen;
  yyssp -= yylen;
#if YYLSP_NEEDED
  yylsp -= yylen;
#endif

#if YYDEBUG
  if (yydebug)
    {
      short *yyssp1 = yyss - 1;
      YYFPRINTF (stderr, "state stack now");
      while (yyssp1 != yyssp)
	YYFPRINTF (stderr, " %d", *++yyssp1);
      YYFPRINTF (stderr, "\n");
    }
#endif

  *++yyvsp = yyval;
#if YYLSP_NEEDED
  *++yylsp = yyloc;
#endif

  /* Now `shift' the result of the reduction.  Determine what state
     that goes to, based on the state we popped back to and the rule
     number reduced by.  */

  yyn = yyr1[yyn];

  yystate = yypgoto[yyn - YYNTBASE] + *yyssp;
  if (yystate >= 0 && yystate <= YYLAST && yycheck[yystate] == *yyssp)
    yystate = yytable[yystate];
  else
    yystate = yydefgoto[yyn - YYNTBASE];

  goto yynewstate;


/*------------------------------------.
| yyerrlab -- here on detecting error |
`------------------------------------*/
yyerrlab:
  /* If not already recovering from an error, report this error.  */
  if (!yyerrstatus)
    {
      ++yynerrs;

#ifdef YYERROR_VERBOSE
      yyn = yypact[yystate];

      if (yyn > YYFLAG && yyn < YYLAST)
	{
	  YYSIZE_T yysize = 0;
	  char *yymsg;
	  int yyx, yycount;

	  yycount = 0;
	  /* Start YYX at -YYN if negative to avoid negative indexes in
	     YYCHECK.  */
	  for (yyx = yyn < 0 ? -yyn : 0;
	       yyx < (int) (sizeof (yytname) / sizeof (char *)); yyx++)
	    if (yycheck[yyx + yyn] == yyx)
	      yysize += yystrlen (yytname[yyx]) + 15, yycount++;
	  yysize += yystrlen ("parse error, unexpected ") + 1;
	  yysize += yystrlen (yytname[YYTRANSLATE (yychar)]);
	  yymsg = (char *) YYSTACK_ALLOC (yysize);
	  if (yymsg != 0)
	    {
	      char *yyp = yystpcpy (yymsg, "parse error, unexpected ");
	      yyp = yystpcpy (yyp, yytname[YYTRANSLATE (yychar)]);

	      if (yycount < 5)
		{
		  yycount = 0;
		  for (yyx = yyn < 0 ? -yyn : 0;
		       yyx < (int) (sizeof (yytname) / sizeof (char *));
		       yyx++)
		    if (yycheck[yyx + yyn] == yyx)
		      {
			const char *yyq = ! yycount ? ", expecting " : " or ";
			yyp = yystpcpy (yyp, yyq);
			yyp = yystpcpy (yyp, yytname[yyx]);
			yycount++;
		      }
		}
	      yyerror (yymsg);
	      YYSTACK_FREE (yymsg);
	    }
	  else
	    yyerror ("parse error; also virtual memory exhausted");
	}
      else
#endif /* defined (YYERROR_VERBOSE) */
	yyerror ("parse error");
    }
  goto yyerrlab1;


/*--------------------------------------------------.
| yyerrlab1 -- error raised explicitly by an action |
`--------------------------------------------------*/
yyerrlab1:
  if (yyerrstatus == 3)
    {
      /* If just tried and failed to reuse lookahead token after an
	 error, discard it.  */

      /* return failure if at end of input */
      if (yychar == YYEOF)
	YYABORT;
      YYDPRINTF ((stderr, "Discarding token %d (%s).\n",
		  yychar, yytname[yychar1]));
      yychar = YYEMPTY;
    }

  /* Else will try to reuse lookahead token after shifting the error
     token.  */

  yyerrstatus = 3;		/* Each real token shifted decrements this */

  goto yyerrhandle;


/*-------------------------------------------------------------------.
| yyerrdefault -- current state does not do anything special for the |
| error token.                                                       |
`-------------------------------------------------------------------*/
yyerrdefault:
#if 0
  /* This is wrong; only states that explicitly want error tokens
     should shift them.  */

  /* If its default is to accept any token, ok.  Otherwise pop it.  */
  yyn = yydefact[yystate];
  if (yyn)
    goto yydefault;
#endif


/*---------------------------------------------------------------.
| yyerrpop -- pop the current state because it cannot handle the |
| error token                                                    |
`---------------------------------------------------------------*/
yyerrpop:
  if (yyssp == yyss)
    YYABORT;
  yyvsp--;
  yystate = *--yyssp;
#if YYLSP_NEEDED
  yylsp--;
#endif

#if YYDEBUG
  if (yydebug)
    {
      short *yyssp1 = yyss - 1;
      YYFPRINTF (stderr, "Error: state stack now");
      while (yyssp1 != yyssp)
	YYFPRINTF (stderr, " %d", *++yyssp1);
      YYFPRINTF (stderr, "\n");
    }
#endif

/*--------------.
| yyerrhandle.  |
`--------------*/
yyerrhandle:
  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yyerrdefault;

  yyn += YYTERROR;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != YYTERROR)
    goto yyerrdefault;

  yyn = yytable[yyn];
  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrpop;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrpop;

  if (yyn == YYFINAL)
    YYACCEPT;

  YYDPRINTF ((stderr, "Shifting error token, "));

  *++yyvsp = yylval;
#if YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  yystate = yyn;
  goto yynewstate;


/*-------------------------------------.
| yyacceptlab -- YYACCEPT comes here.  |
`-------------------------------------*/
yyacceptlab:
  yyresult = 0;
  goto yyreturn;

/*-----------------------------------.
| yyabortlab -- YYABORT comes here.  |
`-----------------------------------*/
yyabortlab:
  yyresult = 1;
  goto yyreturn;

/*---------------------------------------------.
| yyoverflowab -- parser overflow comes here.  |
`---------------------------------------------*/
yyoverflowlab:
  yyerror ("parser stack overflow");
  yyresult = 2;
  /* Fall through.  */

yyreturn:
#ifndef yyoverflow
  if (yyss != yyssa)
    YYSTACK_FREE (yyss);
#endif
  return yyresult;
}
#line 1031 "getdate.y"


/* Month and day table. */
static TABLE const MonthDayTable[] = {
    { "january",	tMONTH,  1 },
    { "february",	tMONTH,  2 },
    { "march",		tMONTH,  3 },
    { "april",		tMONTH,  4 },
    { "may",		tMONTH,  5 },
    { "june",		tMONTH,  6 },
    { "july",		tMONTH,  7 },
    { "august",		tMONTH,  8 },
    { "september",	tMONTH,  9 },
    { "sept",		tMONTH,  9 },
    { "october",	tMONTH, 10 },
    { "november",	tMONTH, 11 },
    { "december",	tMONTH, 12 },

    { "januar",	    tMONTH,  1 },
    { "februar",	tMONTH,  2 },
    { "maerz",		tMONTH,  3 },
    { "marz",		tMONTH,  3 },
    { "april",		tMONTH,  4 },
    { "mai",		tMONTH,  5 },
    { "juni",		tMONTH,  6 },
    { "juli",		tMONTH,  7 },
    { "august",		tMONTH,  8 },
 /* { "september",	tMONTH,  9 }, */
 /* { "sept",		tMONTH,  9 }, */
    { "oktober",	tMONTH, 10 },
 /* { "november",	tMONTH, 11 }, */
    { "dezember",	tMONTH, 12 },

    { "sunday",		tDAY, 0 },
    { "monday",		tDAY, 1 },
    { "tuesday",	tDAY, 2 },
    { "tues",		tDAY, 2 },
    { "wednesday",	tDAY, 3 },
    { "wednes",		tDAY, 3 },
    { "thursday",	tDAY, 4 },
    { "thur",		tDAY, 4 },
    { "thurs",		tDAY, 4 },
    { "friday",		tDAY, 5 },
    { "saturday",	tDAY, 6 },

    { "sonntag",	tDAY, 0 },
    { "so",         tDAY, 0 },
    { "montag",		tDAY, 1 },
    { "mo",	     	tDAY, 1 },
    { "dienstag",	tDAY, 2 },
    { "di",     	tDAY, 2 },
    { "mittwoch",   tDAY, 3 },
    { "mi",         tDAY, 3 },
    { "donnerstag",	tDAY, 4 },
    { "do",	        tDAY, 4 },
    { "freitag",	tDAY, 5 },
    { "fr",     	tDAY, 5 },
    { "samstag",	tDAY, 6 },
    { "sa",         tDAY, 6 },

    { NULL }
};

/* Time units table. */
static TABLE const UnitsTable[] = {
    { "year",		tYEAR_UNIT,	    1 },
    { "month",		tMONTH_UNIT,	1 },
    { "quarter",    tMONTH_UNIT,    3 },
    { "quartal",    tMONTH_UNIT,    3 },
    { "fortnight",	tDAY_UNIT,	   14 },
    { "week",		tDAY_UNIT,	    7 },
    { "day",		tDAY_UNIT,	    1 },
    { "hour",		tHOUR_UNIT,	    1 },
    { "minute",		tMINUTE_UNIT,	1 },
    { "min",		tMINUTE_UNIT,   1 },
    { "second",		tSEC_UNIT,	    1 },
    { "sec",		tSEC_UNIT,	    1 },
    { "workingday", tWDAY_UNIT,     1 },
    { "weekendday", tWEDAY_UNIT,    1 },
    { NULL }
};

/* Assorted relative-time words. */
static TABLE const OtherTable[] = {
    { "tomorrow",	tMINUTE_UNIT,	 1 * 24 * 60 },
    { "yesterday",	tMINUTE_UNIT,	-1 * 24 * 60 },
    { "today",		tMINUTE_UNIT,	0 },
    { "now",		tMINUTE_UNIT,	0 },
    { "last",		tUNUMBER,	   -1 },
    { "this",		tMINUTE_UNIT,	0 },
    { "next",		tUNUMBER,	    2 },
    { "first",		tUNUMBER,	    1 },
/*  { "second",		tUNUMBER,   	2 }, */
    { "third",		tUNUMBER,	    3 },
    { "fourth",		tUNUMBER,	    4 },
    { "fifth",		tUNUMBER,	    5 },
    { "sixth",		tUNUMBER,	    6 },
    { "seventh",	tUNUMBER,	    7 },
    { "eighth",		tUNUMBER,	    8 },
    { "ninth",		tUNUMBER,	    9 },
    { "tenth",		tUNUMBER,      10 },
    { "eleventh",	tUNUMBER,	   11 },
    { "twelfth",	tUNUMBER,	   12 },
    { "ago",		tAGO,	        1 },
    { "upcoming",   tUPCOMING,      1 },
    { "new",        tNEW,           1 },
    { NULL }
};

/* The timezone table. */
static TABLE const TimezoneTable[] = {
    { "gmt",	tZONE,     HOUR ( 0) },	     /* Greenwich Mean */
    { "ut",	    tZONE,     HOUR ( 0) },	     /* Universal (Coordinated) */
    { "utc",	tZONE,     HOUR ( 0) },
    { "wet",	tZONE,     HOUR ( 0) },	     /* Western European */
    { "bst",	tDAYZONE,  HOUR ( 0) },	     /* British Summer */
    { "wat",	tZONE,     HOUR ( 1) },	     /* West Africa */
    { "at",	    tZONE,     HOUR ( 2) },    	 /* Azores */
#if	0
    /* For completeness.  BST is also British Summer, and GST is
     * also Guam Standard. */
    { "bst",	tZONE,     HOUR ( 3) },	     /* Brazil Standard */
    { "gst",	tZONE,     HOUR ( 3) },	     /* Greenland Standard */
#endif
#if 0
    { "nft",	tZONE,     HOUR (3.5) },	 /* Newfoundland */
    { "nst",	tZONE,     HOUR (3.5) },	 /* Newfoundland Standard */
    { "ndt",	tDAYZONE,  HOUR (3.5) },	 /* Newfoundland Daylight */
#endif
    { "ast",	tZONE,     HOUR ( 4) },	     /* Atlantic Standard */
    { "adt",	tDAYZONE,  HOUR ( 4) },	     /* Atlantic Daylight */
    { "est",	tZONE,     HOUR ( 5) },	     /* Eastern Standard */
    { "edt",	tDAYZONE,  HOUR ( 5) },	     /* Eastern Daylight */
    { "cst",	tZONE,     HOUR ( 6) },	     /* Central Standard */
    { "cdt",	tDAYZONE,  HOUR ( 6) },	     /* Central Daylight */
    { "mst",	tZONE,     HOUR ( 7) },	     /* Mountain Standard */
    { "mdt",	tDAYZONE,  HOUR ( 7) },	     /* Mountain Daylight */
    { "pst",	tZONE,     HOUR ( 8) },	     /* Pacific Standard */
    { "pdt",	tDAYZONE,  HOUR ( 8) },	     /* Pacific Daylight */
    { "yst",	tZONE,     HOUR ( 9) },	     /* Yukon Standard */
    { "ydt",	tDAYZONE,  HOUR ( 9) },	     /* Yukon Daylight */
    { "hst",	tZONE,     HOUR (10) },	     /* Hawaii Standard */
    { "hdt",	tDAYZONE,  HOUR (10) },	     /* Hawaii Daylight */
    { "cat",	tZONE,     HOUR (10) },	     /* Central Alaska */
    { "ahst",	tZONE,     HOUR (10) },	     /* Alaska-Hawaii Standard */
    { "nt",	    tZONE,     HOUR (11) },	     /* Nome */
    { "idlw",	tZONE,     HOUR (12) },	     /* International Date Line West */
    { "cet",	tZONE,    -HOUR (1)	 },	     /* Central European */
    { "cest",   tDAYZONE, -HOUR (1)  },      /* Central European Summer Time */
    { "met",	tZONE,    -HOUR (1)	 },	     /* Middle European */
    { "mewt",	tZONE,    -HOUR (1)	 },	     /* Middle European Winter */
    { "mest",	tDAYZONE, -HOUR (1)	 },	     /* Middle European Summer */
    { "mesz",	tDAYZONE, -HOUR (1)	 },	     /* Middle European Summer */
    { "swt",	tZONE,    -HOUR (1)	 },	     /* Swedish Winter */
    { "sst",	tDAYZONE, -HOUR (1)	 },	     /* Swedish Summer */
    { "fwt",	tZONE,    -HOUR (1)	 },	     /* French Winter */
    { "fst",	tDAYZONE, -HOUR (1)	 },	     /* French Summer */
    { "eet",	tZONE,    -HOUR (2)	 },	     /* Eastern Europe, USSR Zone 1 */
    { "bt",	    tZONE,    -HOUR (3)	 },	     /* Baghdad, USSR Zone 2 */
#if 0
    { "it",	    tZONE,    -HOUR (3.5) },   /* Iran */
#endif
    { "zp4",	tZONE,    -HOUR (4	) },	 /* USSR Zone 3 */
    { "zp5",	tZONE,    -HOUR (5	) },	 /* USSR Zone 4 */
#if 0
    { "ist",	tZONE,    -HOUR (5.5) },   /* Indian Standard */
#endif
    { "zp6",	tZONE,    -HOUR (6	) },	 /* USSR Zone 5 */
#if	0
    /* For completeness.  NST is al	so Newfoundland Standard, and SST is
     * also Swedish Summer. */
    { "nst",	tZONE,    -HOUR (6.5) },   /* North Sumatra */
    { "sst",	tZONE,    -HOUR (7	) },	 /* South Sumatra, USSR Zone 6 */
#endif	/* 0 */
    { "wast",	tZONE,    -HOUR (7	) },	 /* West Australian Standard */
    { "wadt",	tDAYZONE, -HOUR (7	) },	 /* West Australian Daylight */
#if 0
    { "jt",	    tZONE,    -HOUR (7.5) },     /* Java (3pm in Cronusland!) */
#endif
    { "cct",	tZONE,    -HOUR (8) },   	 /* China Coast, USSR Zone 7 */
    { "jst",	tZONE,    -HOUR (9) },	     /* Japan Standard, USSR Zone 8 */
#if 0
    { "cast",	tZONE,    -HOUR (9.5) },     /* Central Australian Standard */
    { "cadt",	tDAYZONE, -HOUR (9.5) },     /* Central Australian Daylight */
#endif
    { "east",	tZONE,    -HOUR (10) },	     /* Eastern Australian Standard */
    { "eadt",	tDAYZONE, -HOUR (10) },	     /* Eastern Australian Daylight */
    { "gst",	tZONE,    -HOUR (10) },	     /* Guam Standard, USSR Zone 9 */
    { "nzt",	tZONE,    -HOUR (12) },	     /* New Zealand */
    { "nzst",	tZONE,    -HOUR (12) },	     /* New Zealand Standard */
    { "nzdt",	tDAYZONE, -HOUR (12) },	     /* New Zealand Daylight */
    { "idle",	tZONE,    -HOUR (12) },	     /* International Date Line East */
    {  NULL  }
};

/* Military timezone table. */
static TABLE const MilitaryTable[] = {
    { "a",	tZONE,	HOUR ( -1) },
    { "b",	tZONE,	HOUR ( -2) },
    { "c",	tZONE,	HOUR ( -3) },
    { "d",	tZONE,	HOUR ( -4) },
    { "e",	tZONE,	HOUR ( -5) },
    { "f",	tZONE,	HOUR ( -6) },
    { "g",	tZONE,	HOUR ( -7) },
    { "h",	tZONE,	HOUR ( -8) },
    { "i",	tZONE,	HOUR ( -9) },
    { "k",	tZONE,	HOUR (-10) },
    { "l",	tZONE,	HOUR (-11) },
    { "m",	tZONE,	HOUR (-12) },
    { "n",	tZONE,	HOUR (  1) },
    { "o",	tZONE,	HOUR (  2) },
    { "p",	tZONE,	HOUR (  3) },
    { "q",	tZONE,	HOUR (  4) },
    { "r",	tZONE,	HOUR (  5) },
    { "s",	tZONE,	HOUR (  6) },
    { "t",	tZONE,	HOUR (  7) },
    { "u",	tZONE,	HOUR (  8) },
    { "v",	tZONE,	HOUR (  9) },
    { "w",	tZONE,	HOUR ( 10) },
    { "x",	tZONE,	HOUR ( 11) },
    { "y",	tZONE,	HOUR ( 12) },
    { "z",	tZONE,	HOUR (  0) },
    { NULL }
};



/* ARGSUSED */
static int yyerror (char *s)
{
  return 0;
}

static int ToHour (int Hours, MERIDIAN Meridian)
{
  switch (Meridian)
    {
    case MER24:
      if (Hours < 0 || Hours > 23)
	return -1;
      return Hours;
    case MERam:
      if (Hours < 1 || Hours > 12)
	return -1;
      if (Hours == 12)
	Hours = 0;
      return Hours;
    case MERpm:
      if (Hours < 1 || Hours > 12)
	return -1;
      if (Hours == 12)
	Hours = 0;
      return Hours + 12;
    default:
      abort ();
    }
  /* NOTREACHED */
}

static int ToYear (int Year)
{
  if (Year < 0)
    Year = -Year;

  /* XPG4 suggests that years 00-68 map to 2000-2068, and
     years 69-99 map to 1969-1999.  */
  if (Year < 69)
    Year += 2000;
  else if (Year < 100)
    Year += 1900;

  return Year;
}

static int LookupWord (char *buff)
{
  register char *p;
  register char *q;
  register const TABLE *tp;
  int i;
  int abbrev;

  /* Make it lowercase. */
  for (p = buff; *p; p++)
    if (ISUPPER (*p))
      *p = tolower (*p);

  if (strcmp (buff, "am") == 0 || strcmp (buff, "a.m.") == 0)
    {
      yylval.Meridian = MERam;
      return tMERIDIAN;
    }
  if (strcmp (buff, "pm") == 0 || strcmp (buff, "p.m.") == 0)
    {
      yylval.Meridian = MERpm;
      return tMERIDIAN;
    }

  /* See if we have an abbreviation for a month. */
  if (strlen (buff) == 3)
    abbrev = 1;
  else if (strlen (buff) == 4 && buff[3] == '.')
    {
      abbrev = 1;
      buff[3] = '\0';
    }
  else
    abbrev = 0;

  for (tp = MonthDayTable; tp->name; tp++)
    {
      if (abbrev)
	{
	  if (strncmp (buff, tp->name, 3) == 0)
	    {
	      yylval.Number = tp->value;
	      return tp->type;
	    }
	}
      else if (strcmp (buff, tp->name) == 0)
	{
	  yylval.Number = tp->value;
	  return tp->type;
	}
    }

  for (tp = TimezoneTable; tp->name; tp++)
    if (strcmp (buff, tp->name) == 0)
      {
	yylval.Number = tp->value;
	return tp->type;
      }

  if (strcmp (buff, "dst") == 0)
    return tDST;

  for (tp = UnitsTable; tp->name; tp++)
    if (strcmp (buff, tp->name) == 0)
      {
	yylval.Number = tp->value;
	return tp->type;
      }

  /* Strip off any plural and try the units table again. */
  i = strlen (buff) - 1;
  if (buff[i] == 's')
    {
      buff[i] = '\0';
      for (tp = UnitsTable; tp->name; tp++)
	if (strcmp (buff, tp->name) == 0)
	  {
	    yylval.Number = tp->value;
	    return tp->type;
	  }
      buff[i] = 's';		/* Put back for "this" in OtherTable. */
    }

  for (tp = OtherTable; tp->name; tp++)
    if (strcmp (buff, tp->name) == 0)
      {
	yylval.Number = tp->value;
	return tp->type;
      }

  /* Military timezones. */
  if (buff[1] == '\0' && ISALPHA (*buff))
    {
      for (tp = MilitaryTable; tp->name; tp++)
	if (strcmp (buff, tp->name) == 0)
	  {
	    yylval.Number = tp->value;
	    return tp->type;
	  }
    }

  /* Drop out any periods and try the timezone table again. */
  for (i = 0, p = q = buff; *q; q++)
    if (*q != '.')
      *p++ = *q;
    else
      i++;
  *p = '\0';
  if (i)
    for (tp = TimezoneTable; tp->name; tp++)
      if (strcmp (buff, tp->name) == 0)
	{
	  yylval.Number = tp->value;
	  return tp->type;
	}

  return tID;
}

static int yylex (void)
{
  register char c;
  register char *p;
  char buff[20];
  int Count;
  int sign;

  for (;;)
    {
      while (ISSPACE (*yyInput))
	yyInput++;

      if (ISDIGIT (c = *yyInput) || c == '-' || c == '+')
	{
	  if (c == '-' || c == '+')
	    {
	      sign = c == '-' ? -1 : 1;
	      if (!ISDIGIT (*++yyInput))
		/* skip the '-' sign */
		continue;
	    }
	  else
	    sign = 0;
	  for (yylval.Number = 0; ISDIGIT (c = *yyInput++);)
	    yylval.Number = 10 * yylval.Number + c - '0';
	  yyInput--;
	  if (sign < 0)
	    yylval.Number = -yylval.Number;
	  return sign ? tSNUMBER : tUNUMBER;
	}
      if (ISALPHA (c))
	{
	  for (p = buff; (c = *yyInput++, ISALPHA (c)) || c == '.';)
	    if (p < &buff[sizeof buff - 1])
	      *p++ = c;
	  *p = '\0';
	  yyInput--;
	  return LookupWord (buff);
	}
	  if ( c == ';' )
	{ yyInput++;
      return tSEPARATOR;
    }

      if (c != '(')
	return *yyInput++;
      Count = 0;
      do
	{
	  c = *yyInput++;
	  if (c == '\0')
	    return c;
	  if (c == '(')
	    Count++;
	  else if (c == ')')
	    Count--;
	}
      while (Count > 0);
    }
}


/* Yield A - B, measured in seconds.  */
static long difftm (struct tm *a, struct tm *b)
{
  int ay = a->tm_year + (TM_YEAR_ORIGIN - 1);
  int by = b->tm_year + (TM_YEAR_ORIGIN - 1);
  long days = (
  /* difference in day of year */
		a->tm_yday - b->tm_yday
  /* + intervening leap days */
		+ ((ay >> 2) - (by >> 2))
		- (ay / 100 - by / 100)
		+ ((ay / 100 >> 2) - (by / 100 >> 2))
  /* + difference in years * 365 */
		+ (long) (ay - by) * 365
  );
  return (60 * (60 * (24 * days + (a->tm_hour - b->tm_hour))
		+ (a->tm_min - b->tm_min))
	  + (a->tm_sec - b->tm_sec));
}

void corr_zone (struct tm* atm) {

  time_t Start;
  struct tm* tm;

  Start = mktime(atm);

  if (Start == (time_t) -1)
    {

      /* Guard against falsely reporting errors near the time_t boundaries
         when parsing times in other time zones.  For example, if the min
         time_t value is 1970-01-01 00:00:00 UTC and we are 8 hours ahead
         of UTC, then the min localtime value is 1970-01-01 08:00:00; if
         we apply mktime to 1970-01-01 00:00:00 we will get an error, so
         we apply mktime to 1970-01-02 08:00:00 instead and adjust the time
         zone by 24 hours to compensate.  This algorithm assumes that
         there is no DST transition within a day of the time_t boundaries.  */
      if (yyHaveZone)
	    {

	      if (atm->tm_year <= EPOCH - TM_YEAR_ORIGIN)
	        {
	          atm->tm_mday++;
	          yyTimezone -= 24 * 60;
	        }
	      else
	        {
	          atm->tm_mday--;
	          yyTimezone += 24 * 60;
	        }
	      mktime (atm);
	      printf("-1, zone\n");
	    }
    }


  if (yyHaveZone)
    {
      long delta = yyTimezone * 60L;// + difftm (atm, gmtime (&Start));
      if ((Start + delta < Start) != (delta < 0))
        {
	      /*return -1;*/		/* time_t overflow */
	    }

      Start += delta;

      tm = localtime(&Start); //gmtime(&Start);

      memcpy( atm, tm, sizeof(struct tm) );
    }
}

void clear_flags(void) {

  yyMeridian       = MER24;

  yyRelSeconds     = 0;
  yyRelMinutes     = 0;
  yyRelHour        = 0;
  yyRelDay         = 0;
  yyRelMonth       = 0;
  yyRelYear        = 0;

  yyFixRelMinutes  = 0;
  yyFixRelHour     = 0;
  yyFixRelDay      = 0;
  yyFixRelMonth    = 0;
  yyFixRelYear     = 0;

  yyRelWday        = 0;
  yyRelWEday       = 0;

  yyHaveDate       = 0;
  yyHaveDay        = 0;
  yyHaveRel        = 0;
  yyHaveTime       = 0;
  yyHaveZone       = 0;
  yyHaveFixRel     = 0;
  yyHaveWday       = 0;
  yyHaveWEday      = 0;
  yyAbsoluteDate   = 0;

}

void snarl_getdate (const char *p, const time_t *now, time_t *result )
{
  struct tm tm, tm0, *tmp;
  time_t Start;
  int err;

  pthread_mutex_lock( &globalGetDateLock );

  yyInput = p;

  /* ------------------------------ */
  /* initialize tzname and timezone */
  /* initialize variables           */
  /* ------------------------------ */

  tzset();
  clear_flags();


  /* ------------------------------ */
  /* default to current UTC time    */
  /* ------------------------------ */

  Start = now ? *now : time ((time_t *) NULL);


  /* ------------------------------ */
  /* create a struct holding time   */
  /*                                */
  /* we work in gmtime, however     */
  /* we must frequently adjust      */
  /* the result of mktime to        */
  /* reflect that the mktime inputs */
  /* are gmtime not localtime       */
  /* ------------------------------ */

  /* ------------------------------ */
  /* make the current time          */
  /* ------------------------------ */

  tmp                     = gmtime (&Start);

  yyYear                  = tmp->tm_year + TM_YEAR_ORIGIN;
  yyMonth                 = tmp->tm_mon + 1;
  yyDay                   = tmp->tm_mday;
  yyWDay                  = tmp->tm_wday;
  yyHour                  = tmp->tm_hour;
  yyMinutes               = tmp->tm_min;
  yySeconds               = tmp->tm_sec;

  curr_gmtm.tm_year       = tmp->tm_year;
  curr_gmtm.tm_mon        = tmp->tm_mon;
  curr_gmtm.tm_mday       = tmp->tm_mday;
  curr_gmtm.tm_wday       = tmp->tm_wday;
  curr_gmtm.tm_hour       = tmp->tm_hour;
  curr_gmtm.tm_min        = tmp->tm_min;
  curr_gmtm.tm_sec        = tmp->tm_sec;
  curr_gmtm.tm_isdst      = tmp->tm_isdst;

  mktime( &curr_gmtm );

  /* ------------------------------ */
  /* make the current time          */
  /* ------------------------------ */

  tmp                     = localtime (&Start);

  curr_localtm.tm_year    = tmp->tm_year;
  curr_localtm.tm_mon     = tmp->tm_mon;
  curr_localtm.tm_mday    = tmp->tm_mday;
  curr_localtm.tm_wday    = tmp->tm_wday;
  curr_localtm.tm_hour    = tmp->tm_hour;
  curr_localtm.tm_min     = tmp->tm_min;
  curr_localtm.tm_sec     = tmp->tm_sec;
  curr_localtm.tm_isdst   = tmp->tm_isdst;

  mktime( &curr_localtm );

  /* ------------------------------ */
  /* calculate the time difference  */
  /* to add later to the result of  */
  /* mktime                         */
  /* ------------------------------ */

  mktime2gmdiff = (time_t) difftm( &curr_localtm, &curr_gmtm);

  /* ------------------------------ */
  /* now parse the time             */
  /* ------------------------------ */

  err = yyparse();

  corr_zone(&curr_gmtm);


  /* ------------------------------ */
  /* pickup data and retranslate    */
  /* time result into num of secs   */
  /* as in UTC                      */
  /* ------------------------------ */
/*
  *abs = yyAbsoluteDate;
*/
  Start = mktime(&curr_gmtm) + mktime2gmdiff;

  if( err ) {
	  *result = -1;
  } else {
	  *result = Start;
  }

  pthread_mutex_unlock( &globalGetDateLock );
}



/***************************************************************
*                                                              *
*  SNARLsnmp MIB module for net-snmp                           *
*                                                              *
*     with recognition of the makers of libcurl and net-snmp   *
*     is a web monitoring tool that can check URLs for errors  *
*     and detect errors in dynamic content and was written by  *
*     the                                                      *
*                                                              *
*  SNARLsnmp Dynamic Web Application Monitor Developers Group  *
*                                                              *
*  Stephan Wenzel and Siegfried Steinmueller                   *
*                                                              *
*  stephanwenzel@snarl-snmp.org                                *
*   steinmueller@snarl-snmp.org                                *
*                                                              *
*  http://snarl-snmp.org                                       *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* This file is part of SNARLsnmp MIB module for net-snmp.      *
*                                                              *
* SNARLsnmp MIB module for net-snmp is free software; you      *
* can redistribute it and/or modify it under the terms         *
* of the GNU General Public License as published by the        *
* Free Software Foundation; either version 2 of the            *
* License, or (at your option) any later version.              *
*                                                              *
* SNARLsnmp MIB module for net-snmp is distributed in the      *
* hope that it will be useful, but WITHOUT ANY WARRANTY;       *
* without even the implied warranty of MERCHANTABILITY or      *
* FITNESS FOR A PARTICULAR PURPOSE.                            *
*                                                              *
* See the GNU General Public License for more details.         *
*                                                              *
* You should have received a copy of the GNU General           *
* Public License along with SNARLsnmp; if not, write to the    *
*                                                              *
* Free Software Foundation, Inc.,                              *
* 59 Temple Place,                                             *
* Suite 330, Boston,                                           *
* MA  02111-1307  USA                                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* snarl_manage_dotask.c                                        *
*                                                              *
* this is a module for net-snmpd-5.0.3 or higher               *
* snarl is a web monitoring tool that can check                *
* urls for errors and detect errors in dynamic content         *
* see doc/snarldoc.html for reference                          *
*                                                              *
***************************************************************/

/***************************************************************
*                                                              *
* this code is the main part of the transient task threads     *
* running for the lifetime of a single monitoring transaction  *
*                                                              *
***************************************************************/



/***************************************************************
*                                                              *
* this is the main workhorse of one monitoring task            *
*                                                              *
***************************************************************/

void updateSnarlTask(snarltaskPtr theTask)
{

   snarlWebData        snarlWebData1;
   snarlWebData        snarlWebData2;

   snarltask           taskData;
   snarltaskPtr        aTask = (snarltaskPtr)&taskData;

   snarltaskPtr        currentTask;

   sigset_t            sigs_to_block;

   char               *URLContent1;
   char               *URLContent2;

   char                timestring[SNARL_TASK_MAX_URL_APPENDIX_LENGTH];
   double              difference;

   time_t              timeDifference, timeAge;
   time_t              now, trap_time;

   char               *configptr;
   char               *numicCompareString;

   char               *tokenline;
   char               *token;

   FILE               *dumpFile;
   FILE               *keepfile;

   char                inbuf[SNARL_TASK_MAX_LENGTH_SUBCONFIG_LINE];
   char                inbuf2[SNARL_TASK_MAX_LENGTH_SUBCONFIG_LINE];
   int                 subTokenID, len, index2;

   regmatch_t          myMatch;
   regmatch_t          regexResult[SNARL_TASK_MAX_REGEX_MATCHES];


   int                 result = 0;
   int                 breakUp;
   int                 index;
   int                 length;
   int                 flag;
   int                 currentStatus;

   siteVarsPtr         aSiteVars;
   siteVariables       aSiteVarsGroupCopy;

   proxyVarsPtr        aProxyVars;
   proxyVariables      aProxyVarsGroupCopy;

   statusVariables     aStatusVarsCopy;

       /* ------------------------------------ */
       /* prepare                              */
       /* first of all don't listen to cancel  */
       /* ------------------------------------ */

       /*--------------------------------------*/
       /* prepare                              */
       /* first of all don't listen to signals */
       /*--------------------------------------*/

   sigfillset(                 &sigs_to_block );
   pthread_sigmask( SIG_BLOCK, &sigs_to_block, NULL );

   if( theTask) {


       /* ------------------------------------ */
       /* !!! FIRST DO PREPARATIONS !!!        */
       /* ------------------------------------ */

       /* ------------------------------------ */
       /* prepare                              */
       /* lock your memory and make a local    */
       /* copy                                 */
       /* ------------------------------------ */

       pthread_mutex_lock(   theTask->lock );

       /* ------------------------------------ */
       /* record current running time          */
       /* ------------------------------------ */

           if( theTask->task_waitForExternalTrigger ) {

       /* ------------------------------------ */
       /* this task is not scheduled by time   */
       /* but by an external event             */
       /* ------------------------------------ */
               snarl_time( &theTask->statusVars.current_run );

           } else {

               theTask->statusVars.current_run = theTask->statusVars.next_run;
		   }

       /* ------------------------------------ */
       /* make a copy of aTask                 */
       /* ------------------------------------ */

           memcpy( (void*)aTask, (void*)theTask, sizeof(snarltask) );
           fixPointers( aTask );

       pthread_mutex_unlock(   theTask->lock );


       /* ------------------------------------ */
       /* now do your checks with the copy     */
       /* of theTask                           */
       /* ------------------------------------ */


       /* ------------------------------------ */
       /* all the actual webbing goes here     */
       /* start an infinite loop that can      */
       /* only be exited with a cancel         */
       /* request                              */
       /* ------------------------------------ */

       while( TRUE ) {

       /* ------------------------------------ */
       /* if we should hapen to get here       */
       /* and our task has been set to be      */
       /* freed get out immediately            */
       /* ------------------------------------ */

           if( aTask->aboutToCease ) {
               break;
           }


           if( ! initSnarlTask( aTask )) {
               break;
           }


       /* ------------------------------------ */
       /* ------------------------------------ */
       /* are we asking for dynamic url's?     */
       /* ------------------------------------ */

           index   = 0;
           breakUp = FALSE;

           if( aTask->task_getDynamicURLFrom ) {

           index = 0;
                 while( currentTask = find_snarltask( aTask->sitename, NULL, &index, globalTaskArray )) {

                     pthread_mutex_lock(   currentTask->lock );

       /* ------------------------------------ */
       /* the url providers  should be in the  */
       /* same proxy group                     */
       /* ------------------------------------ */
                         if(   !strcmp( aTask->getDynamicURLFrom, currentTask->urlname)
                            && (   (aTask->ignoreProxyGroup)
                                || (!strcmp( aTask->mainURL.proxy,currentTask->mainURL.proxy))
                               )
                           ) {

       /* ------------------------------------ */
       /* does the task we are getting the     */
       /* dynamic url from have data at all?   */
       /* ------------------------------------ */
                             if( currentTask->hasData ) {

                                 strcpy( aTask->mainURL.currenturl, currentTask->dynamicURL );

                             } else {

       /* ------------------------------------ */
       /* we don't do anything here because    */
       /* we must unlock the current data first*/
       /* ------------------------------------ */
                                 breakUp = TRUE;
                             }
                         }
                     pthread_mutex_unlock( currentTask->lock );
                 }
           }

       /* ------------------------------------ */
       /* does the task we are getting the     */
       /* dynamic url from have data at all?   */
       /* (see above) if not break!!!!         */
       /* we don't have a valid url yet        */
       /* ------------------------------------ */

           if( breakUp ) {
               break;
           }



       /* ------------------------------------ */
       /* ------------------------------------ */
       /* should we append dynamic URL compo-  */
       /* nents to the URL?                    */
       /* ------------------------------------ */

           if( aTask->task_appendDynamicURLComponent ) {

               strncpy( aTask->mainURL.currenturl, aTask->mainURL.url, SNARL_TASK_URL_MAX_LENGTH );
               if(strlen(aTask->mainURL.currenturl) == SNARL_TASK_URL_MAX_LENGTH) aTask->mainURL.currenturl[strlen(aTask->mainURL.currenturl)-1]='\0';

               if( !(createDynamicURLData(aTask, aTask->mainURL.currenturl, SNARL_TASK_URL_MAX_LENGTH, aTask->dynurlsubconfigline )) ) {

       /* ------------------------------------ */
       /* silently perish - probably we        */
       /* tried to fetch data from a not yet   */
       /* run task                             */
       /* ------------------------------------ */
                   break;
               }
           }




       /* ------------------------------------ */
       /* ------------------------------------ */
       /* should we append dynamic FORM compo- */
       /* nents to the URL FormData?           */
       /* ------------------------------------ */

           if( aTask->task_appendDynamicFORMComponent ) {

               strncpy( aTask->mainURL.currentformData, aTask->mainURL.formData, SNARL_TASK_URL_MAX_LENGTH );
               if(strlen(aTask->mainURL.currentformData) == SNARL_TASK_URL_MAX_LENGTH) aTask->mainURL.currentformData[strlen(aTask->mainURL.currentformData)-1]='\0';

               if( !(createDynamicURLData(aTask, aTask->mainURL.currentformData, SNARL_TASK_URL_MAX_LENGTH, aTask->dynformsubconfigline )) ) {

       /* ------------------------------------ */
       /* silently perish - probably we        */
       /* tried to fetch data from a not yet   */
       /* run task                             */
       /* ------------------------------------ */
                   break;
               }
           }



       /* ------------------------------------ */
       /* ------------------------------------ */
       /* are we scheduled to run or to        */
       /* suspend this task                    */
       /* ------------------------------------ */

           if( aTask->task_noUpdateBetween ) {

               snarl_time(&now);

               if( (now >= aTask->noUpdateBetweenStart) && (now < aTask->noUpdateBetweenEnd )) {


                   if( aTask->noUpdateBetween_trap.specificTrap != SNARLSNMP_TRAP_INHIBITED ) {
                       aTask->noUpdateBetween_trap.specificTrap =  SNARLSNMP_TRAP_INHIBITED;

                       snarl_ctime( inbuf, &aTask->noUpdateBetweenEnd, SNARL_TASK_MAX_LENGTH_SUBCONFIG_LINE );

                       snprintf(   aTask->noUpdateBetween_trap.siteMessage,
                                   SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                   "%s - %s: until %s task is inhibited",
                                   aTask->sitename,
                                   aTask->urlname,
                                   inbuf
                               );
       /* ------------------------------------ */
       /* this suspension trap will be         */
       /* submitted below                      */
       /* ------------------------------------ */

                       aTask->noUpdateBetween_trap_available  = globalTrapAttack;
                       aTask->noUpdateBetween_trap.eventTime  = aTask->statusVars.current_run;


       /* ------------------------------------ */
       /* now walk through all subtasks and    */
       /* send an appropiate trap              */
       /* ------------------------------------ */

                       snprintf( aTask->general_trap.siteMessage,
                                 SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                 "%s - %s: until %s general error catching is inhibited",
                                 aTask->sitename,
                                 aTask->urlname,
                                 inbuf
                               );

                       aTask->general_trap.specificTrap  = SNARLSNMP_TRAP_INHIBITED;
                       aTask->general_trap.eventTime     = aTask->statusVars.current_run;
                       aTask->general_trap_available     = globalTrapAttack;


                       if( aTask->task_performance ) {

                           snprintf( aTask->performance_trap.siteMessage,
                                     SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                     "%s - %s: until %s timing is inhibited",
                                     aTask->sitename,
                                     aTask->urlname,
                                     inbuf
                                   );
                           aTask->performance_trap.specificTrap = SNARLSNMP_TRAP_INHIBITED;
                           aTask->performance_trap.eventTime  = aTask->statusVars.current_run;
                           aTask->performance_trap_available  = globalTrapAttack;
                       }

                       if( aTask->task_httpcode ) {

                           snprintf( aTask->httperror_trap.siteMessage,
                                     SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                     "%s - %s: until %s http code fetching is inhibited",
                                     aTask->sitename,
                                     aTask->urlname,
                                     inbuf
                                   );
                           aTask->httperror_trap.specificTrap = SNARLSNMP_TRAP_INHIBITED;
                           aTask->httperror_trap.eventTime    = aTask->statusVars.current_run;
                           aTask->httperror_trap_available    = globalTrapAttack;

                       }

                       if( aTask->task_matchRegex) {

                           snprintf( aTask->matchRegex_trap.siteMessage,
                                     SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                     "%s - %s: until %s regex matching is inhibited",
                                     aTask->sitename,
                                     aTask->urlname,
                                     inbuf
                                   );
                           aTask->matchRegex_trap.specificTrap = SNARLSNMP_TRAP_INHIBITED;
                           aTask->matchRegex_trap.eventTime    = aTask->statusVars.current_run;
                           aTask->matchRegex_trap_available    = globalTrapAttack;
                       }

                       if( aTask->task_matchContentSize) {

                           snprintf( aTask->matchContentSize_trap.siteMessage,
                                     SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                     "%s - %s: until %s static content size matching is inhibited",
                                     aTask->sitename,
                                     aTask->urlname,
                                     inbuf
                                   );
                           aTask->matchContentSize_trap.specificTrap = SNARLSNMP_TRAP_INHIBITED;
                           aTask->matchContentSize_trap.eventTime    = aTask->statusVars.current_run;
                           aTask->matchContentSize_trap_available    = globalTrapAttack;
                       }

                       if( aTask->task_sizeMatchURL) {

                           snprintf( aTask->sizeMatchURL_trap.siteMessage,
                                     SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                     "%s - %s: until %s url content size comparison is inhibited",
                                     aTask->sitename,
                                     aTask->urlname,
                                     inbuf
                                   );
                           aTask->sizeMatchURL_trap.specificTrap = SNARLSNMP_TRAP_INHIBITED;
                           aTask->sizeMatchURL_trap.eventTime    = aTask->statusVars.current_run;
                           aTask->sizeMatchURL_trap_available    = globalTrapAttack;
                       }

                       if( aTask->task_contentMatchURL) {

                           snprintf( aTask->contentMatchURL_trap.siteMessage,
                                     SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                     "%s - %s: until %s url content comparison is inhibited",
                                     aTask->sitename,
                                     aTask->urlname,
                                     inbuf
                                   );
                           aTask->contentMatchURL_trap.specificTrap = SNARLSNMP_TRAP_INHIBITED;
                           aTask->contentMatchURL_trap.eventTime    = aTask->statusVars.current_run;
                           aTask->contentMatchURL_trap_available    = globalTrapAttack;
                       }

                       if( aTask->task_matchNumic) {

                           snprintf( aTask->matchNumic_trap.siteMessage,
                                     SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                     "%s - %s: until %s numerical regex result comparison is inhibited",
                                     aTask->sitename,
                                     aTask->urlname,
                                     inbuf
                                   );
                           aTask->matchNumic_trap.specificTrap = SNARLSNMP_TRAP_INHIBITED;
                           aTask->matchNumic_trap.eventTime    = aTask->statusVars.current_run;
                           aTask->matchNumic_trap_available    = globalTrapAttack;
                       }

                       if( aTask->task_matchDateAge) {

                           snprintf( aTask->matchDateAge_trap.siteMessage,
                                     SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                     "%s - %s: until %s age of regex result date comparison is inhibited",
                                     aTask->sitename,
                                     aTask->urlname,
                                     inbuf
                                   );
                           aTask->matchDateAge_trap.specificTrap = SNARLSNMP_TRAP_INHIBITED;
                           aTask->matchDateAge_trap.eventTime    = aTask->statusVars.current_run;
                           aTask->matchDateAge_trap_available    = globalTrapAttack;
                       }

       /* ------------------------------------ */
       /* we're inactive - break and quit      */
       /* ------------------------------------ */

                       break;


                   } else {

       /* ------------------------------------ */
       /* just ignore                          */
       /* don't report status inhibited        */
       /* repeatedly                           */
       /* ------------------------------------ */

                   }


       /* ------------------------------------ */
       /* quit this task - we're inactive      */
       /* ------------------------------------ */
                   break;


               } else {

       /* ------------------------------------ */
       /* we are active and should report this */
       /* only once                            */
       /* ------------------------------------ */

                   if( aTask->noUpdateBetween_trap.specificTrap != SNARLSNMP_TRAP_RESUMED ) {

       /* ------------------------------------ */
       /* recalculate the new inhibition time. */
       /*                                      */
       /*                                      */
       /* ATTENTION: strategy change:          */
       /* now new data may be interpreted to   */
       /* ly in the past - therefore time      */
       /* interpretation has been updated      */
       /*                                      */
       /* OBSOLETE:                            */
       /* Since this suspension is over, the   */
       /* next inhibition time will start in   */
       /* earliest 24 hours from now           */
       /* we do not know here what time        */
       /* directive we get (i.e. tomorrw, day  */
       /* week etc.),                          */
       /* so in case the calculated time is in */
       /* the past we add 24*60*60 seconds     */
       /* (one day) to avoid a new suspension  */
       /* time in the past                     */
       /* ------------------------------------ */

                       snarl_time(&now);

                       invalidate_traps( aTask );

                       if(  (*(aTask->noUpdateBetweenStart_string) == '\0')
                          ||(*(aTask->noUpdateBetweenStart_string) == '\0') ) {

						   snarl_getdate( aTask->noUpdateBetweenStart_string, &now, &(aTask->noUpdateBetweenStart));
						   snarl_getdate( aTask->noUpdateBetweenEnd_string,   &now, &(aTask->noUpdateBetweenEnd));

						   snarl_ctime( inbuf,  &aTask->noUpdateBetweenStart, SNARL_TASK_MAX_LENGTH_SUBCONFIG_LINE );
						   snarl_ctime( inbuf2, &aTask->noUpdateBetweenEnd,   SNARL_TASK_MAX_LENGTH_SUBCONFIG_LINE );

					       flag = TRUE;

					   } else {

						   flag = FALSE;
					   }
					   snprintf(   aTask->noUpdateBetween_trap.siteMessage,
								   SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
								   flag ? "%s - %s: between %s and %s task will be inhibited next" : "%s - %s: task is resumed",
								   aTask->sitename,
								   aTask->urlname,
								   inbuf,
								   inbuf2
							   );

					   aTask->noUpdateBetween_trap.specificTrap = SNARLSNMP_TRAP_RESUMED;
					   aTask->noUpdateBetween_trap.eventTime    = aTask->statusVars.current_run;
					   aTask->noUpdateBetween_trap_available    = globalTrapAttack;


					   snprintf( aTask->general_trap.siteMessage,
								 SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
									 flag ? "%s - %s: between %s and %s general error catching will be inhibited next" : "%s - %s: general error catching is resumed",
									 aTask->sitename,
									 aTask->urlname,
									 inbuf,
									 inbuf2
							   );

					   aTask->general_trap.specificTrap  = SNARLSNMP_TRAP_RESUMED;
					   aTask->general_trap.eventTime     = aTask->statusVars.current_run;
					   aTask->general_trap_available     = globalTrapAttack;


					   if( aTask->task_performance ) {

						   snprintf( aTask->performance_trap.siteMessage,
									 SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
									 flag ? "%s - %s: between %s and %s performance measurement will be inhibited next" : "%s - %s: performance measurement is resumed",
									 aTask->sitename,
									 aTask->urlname,
									 inbuf,
									 inbuf2
								   );
						   aTask->performance_trap.specificTrap = SNARLSNMP_TRAP_RESUMED;
						   aTask->performance_trap.eventTime  = aTask->statusVars.current_run;
						   aTask->performance_trap_available  = globalTrapAttack;
					   }

					   if( aTask->task_httpcode ) {
						   snprintf( aTask->httperror_trap.siteMessage,
									 SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
									 flag ? "%s - %s: between %s and %s http code fetching will be inhibited next" : "%s - %s: http code fetching is resumed",
									 aTask->sitename,
									 aTask->urlname,
									 inbuf,
									 inbuf2
								   );
						   aTask->httperror_trap.specificTrap = SNARLSNMP_TRAP_RESUMED;
						   aTask->httperror_trap.eventTime    = aTask->statusVars.current_run;
						   aTask->httperror_trap_available    = globalTrapAttack;

					   }

					   if( aTask->task_matchRegex) {

						   snprintf( aTask->matchRegex_trap.siteMessage,
									 SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
									 flag ? "%s - %s: between %s and %s content matching will be inhibited next" : "%s - %s: content matching is resumed",
									 aTask->sitename,
									 aTask->urlname,
									 inbuf,
									 inbuf2
								   );
						   aTask->matchRegex_trap.specificTrap = SNARLSNMP_TRAP_RESUMED;
						   aTask->matchRegex_trap.eventTime    = aTask->statusVars.current_run;
						   aTask->matchRegex_trap_available    = globalTrapAttack;
					   }

					   if( aTask->task_matchContentSize) {

						   snprintf( aTask->matchContentSize_trap.siteMessage,
									 SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
									 flag ? "%s - %s: between %s and %s static content size matching will be inhibited next" : "%s - %s: content size matching is resumed",
									 aTask->sitename,
									 aTask->urlname,
									 inbuf,
									 inbuf2
								   );
						   aTask->matchContentSize_trap.specificTrap = SNARLSNMP_TRAP_RESUMED;
						   aTask->matchContentSize_trap.eventTime    = aTask->statusVars.current_run;
						   aTask->matchContentSize_trap_available    = globalTrapAttack;
					   }

					   if( aTask->task_sizeMatchURL) {

						   snprintf( aTask->sizeMatchURL_trap.siteMessage,
									 SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
									 flag ? "%s - %s: between %s and %s content size comparison will be inhibited next" : "%s - %s: content size comparison is resumed",
									 aTask->sitename,
									 aTask->urlname,
									 inbuf,
									 inbuf2
								   );
						   aTask->sizeMatchURL_trap.specificTrap = SNARLSNMP_TRAP_RESUMED;
						   aTask->sizeMatchURL_trap.eventTime    = aTask->statusVars.current_run;
						   aTask->sizeMatchURL_trap_available    = globalTrapAttack;
					   }

					   if( aTask->task_contentMatchURL) {

						   snprintf( aTask->contentMatchURL_trap.siteMessage,
									 SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
									 flag ? "%s - %s: between %s and %s content comparison will be inhibited next" : "%s - %s: content comparison is resumed",
									 aTask->sitename,
									 aTask->urlname,
									 inbuf,
									 inbuf2
								   );
						   aTask->contentMatchURL_trap.specificTrap = SNARLSNMP_TRAP_RESUMED;
						   aTask->contentMatchURL_trap.eventTime    = aTask->statusVars.current_run;
						   aTask->contentMatchURL_trap_available    = globalTrapAttack;
					   }

					   if( aTask->task_matchNumic) {

						   snprintf( aTask->matchNumic_trap.siteMessage,
									 SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
									 flag ? "%s - %s: between %s and %s numerical result comparison will be inhibited next" : "%s - %s: numerical result comparison is resumed",
									 aTask->sitename,
									 aTask->urlname,
									 inbuf,
									 inbuf2
								   );
						   aTask->matchNumic_trap.specificTrap = SNARLSNMP_TRAP_RESUMED;
						   aTask->matchNumic_trap.eventTime    = aTask->statusVars.current_run;
						   aTask->matchNumic_trap_available    = globalTrapAttack;
					   }

					   if( aTask->task_matchDateAge) {

						   snprintf( aTask->matchDateAge_trap.siteMessage,
									 SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
									 flag ? "%s - %s: between %s and %s regex result date comparison will be inhibited next" : "%s - %s: date result comparison is resumed",
									 aTask->sitename,
									 aTask->urlname,
									 inbuf,
									 inbuf2
								   );
						   aTask->matchDateAge_trap.specificTrap = SNARLSNMP_TRAP_RESUMED;
						   aTask->matchDateAge_trap.eventTime    = aTask->statusVars.current_run;
						   aTask->matchDateAge_trap_available    = globalTrapAttack;
					   }
       /* ------------------------------------ */
       /* we break at this point -             */
       /* most likely this run was scheduled   */
       /* above at exact the resume time       */
       /* ------------------------------------ */

                       break;
                   }
               }
           }

       /* ------------------------------------ */
       /* if the URL is empty we're done       */
       /* ------------------------------------ */

           if( aTask->mainURL.url[0] == '\0') {
               break;
           }




       /* ------------------------------------ */
       /* !!! NOW HERE COMES THE WEBBING       */
       /* since we are running tests log the   */
       /* the time ticks                       */
       /* ------------------------------------ */


       /* ------------------------------------ */
       /* do the transaction with mainURL once */
       /* only and do all the other tests      */
       /* with the result of this main         */
       /* transaction                          */
       /* ------------------------------------ */

           snarl_setup_connection_data( &aTask->mainURL,
                                         aTask->sitename,
                                         aTask->urlname,
                                         SNARL_TASK_MAIN_URL );

           if( (!(aTask->force_status_unknown)) && (aTask->mainURL.curl_handle) ) {

               result = snarl_getContentOfURL( &aTask->mainURL, &snarlWebData1 );
           }

           if(   (!(aTask->force_status_unknown)) &&
                 (( aTask->general_trap_inhibit) || !(result) ) ) {

               if(  ! (aTask->general_trap_inhibit)
                   && (aTask->general_trap.specificTrap > SNARLSNMP_TRAP_NORMAL) ) {
                   snprintf(   aTask->general_trap.siteMessage,
                               SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                               "%s - %s: no general error",
                               aTask->sitename, aTask->urlname );

                   aTask->general_trap.specificTrap          = SNARLSNMP_TRAP_NORMAL;
                   aTask->general_trap.currentWarnings       = 0;
                   aTask->general_trap_available             = globalTrapAttack;
                   aTask->general_trap.eventTime             = aTask->statusVars.current_run;
               }


               if(   (URLContent1 = snarlWebData1.data)
                   &&(snarlWebData1.size)

               ) {
                   /* make sure the content ends zero terminated */
		           realloc(snarlWebData1.data, snarlWebData1.size + 1);
                   URLContent1[(snarlWebData1.size)] = '\0';
               }

       /* ------------------------------------ */
       /* log performance                      */
       /* be careful: curl has done some weird */
       /* hacks and returns pointers to locals */
       /* don't call this function much later  */
       /* ------------------------------------ */


               if( aTask->task_performance ) {

                   snarl_log_statistics( &snarlWebData1, aTask );


                   if(   (aTask->performance_total > aTask->performance_warning_threshold_total)
                      || (aTask->performance_total > aTask->performance_critical_threshold_total)
                     ) {

                        snprintf(   aTask->performance_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: total transaction time exceeded by %d ms (%d ms of max. %d (%s) ms)",
                                    aTask->sitename, aTask->urlname,
                                    aTask->performance_total - (aTask->performance_total > aTask->performance_critical_threshold_total ? aTask->performance_critical_threshold_total : aTask->performance_warning_threshold_total),
                                    aTask->performance_total,
                                    (aTask->performance_total > aTask->performance_critical_threshold_total ? aTask->performance_critical_threshold_total : aTask->performance_warning_threshold_total),
                                    (aTask->performance_total > aTask->performance_critical_threshold_total ? "critical threshold" : "warning threshold")
                                );

                        if( aTask->performance_total > aTask->performance_critical_threshold_total ) {
                            aTask->performance_trap.currentWarnings = aTask->warningsBeforeCritical;
                        }

                        (aTask->performance_trap.currentWarnings)++;
                        if( aTask->performance_trap.currentWarnings > aTask->warningsBeforeCritical) {

                            aTask->performance_trap.specificTrap      = SNARLSNMP_TRAP_CRITICAL;
                        } else {
                            aTask->performance_trap.specificTrap      = SNARLSNMP_TRAP_WARNING;
                        }

                        aTask->performance_trap_available             = globalTrapAttack;
                        aTask->performance_trap.eventTime             = aTask->statusVars.current_run;
                        aTask->useAlertInterval                       = TRUE;

                   } else {

                        if( aTask->performance_trap.specificTrap > SNARLSNMP_TRAP_NORMAL ) {
                            snprintf(   aTask->performance_trap.siteMessage,
                                        SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                        "%s - %s: total transaction time is within limits (%d ms of max. %d ms warning threshold)",
                                        aTask->sitename, aTask->urlname,
                                        aTask->performance_total,
                                        aTask->performance_warning_threshold_total
                                    );

                            aTask->performance_trap.specificTrap      = SNARLSNMP_TRAP_NORMAL;
                            aTask->performance_trap.currentWarnings   = 0;
                            aTask->performance_trap.eventTime         = aTask->statusVars.current_run;
                            aTask->performance_trap_available         = globalTrapAttack;
                        }
                   }
               }


               if( aTask->task_httpcode ) {

                   if(    sscanf(aTask->mainURL.currenturl, "%64[^:]://", inbuf)
                       && (  (curl_strequal(inbuf, "https"))
                           ||(curl_strequal(inbuf, "http"))
                          )
                     ) {
                       if( (aTask->mainURL.HTTPCode == 0) || (aTask->mainURL.HTTPCode > 302) ) {

                            snprintf(   aTask->httperror_trap.siteMessage,
                                        SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                        "%s - %s: HTTP code is %ld",
                                        aTask->sitename, aTask->urlname,
                                        aTask->mainURL.HTTPCode
                                    );


                            (aTask->httperror_trap.currentWarnings)++;
                            if( aTask->httperror_trap.currentWarnings > aTask->warningsBeforeCritical) {

                                aTask->httperror_trap.specificTrap        = SNARLSNMP_TRAP_CRITICAL;
                            } else {
                                aTask->httperror_trap.specificTrap        = SNARLSNMP_TRAP_WARNING;
                            }

                            aTask->httperror_trap_available             = globalTrapAttack;
                            aTask->httperror_trap.eventTime             = aTask->statusVars.current_run;
                            aTask->useAlertInterval                     = TRUE;

                       } else {

                            if( aTask->httperror_trap.specificTrap > SNARLSNMP_TRAP_NORMAL ) {

                                snprintf(   aTask->httperror_trap.siteMessage,
                                            SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                            "%s - %s: HTTP code is %ld",
                                            aTask->sitename, aTask->urlname,
                                            aTask->mainURL.HTTPCode
                                        );

                                aTask->httperror_trap.specificTrap        = SNARLSNMP_TRAP_NORMAL;
                                aTask->httperror_trap.currentWarnings     = 0;
                                aTask->httperror_trap_available           = globalTrapAttack;
                                aTask->httperror_trap.eventTime           = aTask->statusVars.current_run;
                            }
                       }
                   }
               }

       /* ------------------------------------ */
       /* fetch dynamic URL                    */
       /* ------------------------------------ */


               if(     (aTask->task_fetchDynamicURL)
                   && !(aTask->dynamicURLRegex.regexError)
                   &&  (URLContent1)
                   &&  (snarlWebData1.size)
                 ) {

                    aTask->dynamicURLRegex.regexMatchingError = regexec( &(aTask->dynamicURLRegex.regexBuffer),
                                                                           URLContent1,
                                                                           1,
                                                                           &myMatch,
                                                                           0);

                   regerror(  aTask->dynamicURLRegex.regexMatchingError,
                            &(aTask->dynamicURLRegex.regexBuffer),
                              aTask->dynamicURLRegex.regexMatchingError_string,
                       sizeof(aTask->dynamicURLRegex.regexMatchingError_string));

                   if( !(aTask->dynamicURLRegex.regexMatchingError) ) {
                       length = (myMatch.rm_eo - myMatch.rm_so) < SNARL_TASK_URL_MAX_LENGTH-1 ? (myMatch.rm_eo - myMatch.rm_so) : (SNARL_TASK_URL_MAX_LENGTH-1);

                       strncpy(aTask->dynamicURL, &(URLContent1[myMatch.rm_so]), length);
                       aTask->dynamicURL[(myMatch.rm_eo - myMatch.rm_so)] = '\0';
                       snarl_make_absolute_url( aTask );
                   } else {

       /* ------------------------------------ */
       /* no match -> no URL                   */
       /* ------------------------------------ */

                       aTask->dynamicURL[0] = '\0';
                   }
               }

        /* ------------------------------------ */
        /* matchRegEx                           */
        /* ------------------------------------ */

               if(     (aTask->task_matchRegex)
                   &&  (URLContent1)
                   &&  (snarlWebData1.size) ) {

                   if( !(aTask->matchRegex_data.regexError) ) {

                       result = regexec( &(aTask->matchRegex_data.regexBuffer),
                            URLContent1,
                            0,
                            &myMatch,
                            0);

                       if( result ) {
                         aTask->matchRegex_flag = FALSE;
                       } else {
                         aTask->matchRegex_flag = TRUE;
                       }
                   } else {
                       aTask->matchRegex_flag = FALSE;
                   }

                   if( (aTask->matchRegex_flag_trap) == (aTask->matchRegex_flag) ) {

                        snprintf(   aTask->matchRegex_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: web content %s match %s",
                                    aTask->sitename, aTask->urlname,
                                    aTask->matchRegex_flag ? "does" : "doesn't",
                                    aTask->matchRegex_data.regex
                                );

                        (aTask->matchRegex_trap.currentWarnings)++;
                        if( aTask->matchRegex_trap.currentWarnings > aTask->warningsBeforeCritical) {

                            aTask->matchRegex_trap.specificTrap      = SNARLSNMP_TRAP_CRITICAL;
                        } else {
                            aTask->matchRegex_trap.specificTrap      = SNARLSNMP_TRAP_WARNING;
                        }

                        aTask->matchRegex_trap_available           = globalTrapAttack;
                        aTask->matchRegex_trap.eventTime           = aTask->statusVars.current_run;
                        aTask->useAlertInterval                    = TRUE;

                   } else {

                        if( aTask->matchRegex_trap.specificTrap > SNARLSNMP_TRAP_NORMAL ) {
                        snprintf(   aTask->matchRegex_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: web content %s match %s",
                                    aTask->sitename, aTask->urlname,
                                    aTask->matchRegex_flag ? "does" : "doesn't",
                                    aTask->matchRegex_data.regex
                                 );

                        aTask->matchRegex_trap.specificTrap          = SNARLSNMP_TRAP_NORMAL;
                        aTask->matchRegex_trap.currentWarnings     = 0;
                        aTask->matchRegex_trap_available           = globalTrapAttack;
                        aTask->matchRegex_trap.eventTime           = aTask->statusVars.current_run;
                        }
                   }
               }


       /* ------------------------------------ */
       /* match content size                   */
       /* ------------------------------------ */

               if( aTask->task_matchContentSize ) {

                   aTask->matchContentSize_actual = snarlWebData1.size;

                   if( aTask->matchContentSize_actual == aTask->matchContentSize ) {

                       aTask->matchContentSize_flag    = 1; /* equal */

                   } else {

                       if( aTask->matchContentSize_actual < aTask->matchContentSize ) {

                           aTask->matchContentSize_flag = 0; /* smaller */

                       } else {

                           aTask->matchContentSize_flag   = 2; /* greater */

                       }
                   }

                   if( (aTask->matchContentSize_flag_trap) == (aTask->matchContentSize_flag) ) {

                        snprintf(   aTask->matchContentSize_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s: - %s: web content size %s %d bytes",
                                    aTask->sitename, aTask->urlname,
                                    aTask->matchContentSize_flag ? "is" : "is not",
                                    aTask->matchContentSize
                                );

                        (aTask->matchContentSize_trap.currentWarnings)++;
                        if( aTask->matchContentSize_trap.currentWarnings > aTask->warningsBeforeCritical) {

                            aTask->matchContentSize_trap.specificTrap      = SNARLSNMP_TRAP_CRITICAL;
                        } else {
                            aTask->matchContentSize_trap.specificTrap      = SNARLSNMP_TRAP_WARNING;
                        }

                        aTask->matchContentSize_trap_available           = globalTrapAttack;
                        aTask->matchContentSize_trap.eventTime           = aTask->statusVars.current_run;
                        aTask->useAlertInterval                          = TRUE;

                   } else {

                        if( aTask->matchContentSize_trap.specificTrap > SNARLSNMP_TRAP_NORMAL ) {
                        snprintf(   aTask->matchContentSize_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s web content size %s %d bytes",
                                    aTask->sitename, aTask->urlname,
                                    aTask->matchContentSize_flag ? "is" : "is not",
                                    aTask->matchContentSize
                                );

                        aTask->matchContentSize_trap.specificTrap          = SNARLSNMP_TRAP_NORMAL;
                        aTask->matchContentSize_trap.currentWarnings     = 0;
                        aTask->matchContentSize_trap_available           = globalTrapAttack;
                        aTask->matchContentSize_trap.eventTime           = aTask->statusVars.current_run;
                        }
                   }
               }


       /* ------------------------------------ */
       /* match content size of another URL    */
       /* ------------------------------------ */

               if( aTask->task_sizeMatchURL ) {

                   if( !(result = snarl_getContentOfURL( &aTask->sizeMatchURL, &snarlWebData2 )) ) {

                       if( snarlWebData1.size == snarlWebData2.size ) {
                           aTask->sizeMatchURL_flag = TRUE;
                       } else {
                           aTask->sizeMatchURL_flag = FALSE;
                       }

                       free( snarlWebData2.data );
                   }

                   if( (aTask->sizeMatchURL_flag_trap) == (aTask->sizeMatchURL_flag) ) {

                        snprintf(   aTask->sizeMatchURL_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: web content size %s match the size of %s",
                                    aTask->sitename, aTask->urlname,
                                    aTask->sizeMatchURL_flag ? "does" : "doesn't",
                                    aTask->sizeMatchURL.url
                                );

                        (aTask->sizeMatchURL_trap.currentWarnings)++;
                        if( aTask->sizeMatchURL_trap.currentWarnings > aTask->warningsBeforeCritical) {

                            aTask->sizeMatchURL_trap.specificTrap      = SNARLSNMP_TRAP_CRITICAL;
                        } else {
                            aTask->sizeMatchURL_trap.specificTrap      = SNARLSNMP_TRAP_WARNING;
                        }

                        aTask->sizeMatchURL_trap_available           = globalTrapAttack;
                        aTask->sizeMatchURL_trap.eventTime           = aTask->statusVars.current_run;
                        aTask->useAlertInterval                      = TRUE;

                   } else {

                        if( aTask->sizeMatchURL_trap.specificTrap > SNARLSNMP_TRAP_NORMAL ) {
                        snprintf(   aTask->sizeMatchURL_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: web content size %s match the size of %s",
                                    aTask->sitename, aTask->urlname,
                                    aTask->sizeMatchURL_flag ? "does" : "doesn't",
                                    aTask->sizeMatchURL.url
                                );

                        aTask->sizeMatchURL_trap.specificTrap          = SNARLSNMP_TRAP_NORMAL;
                        aTask->sizeMatchURL_trap.currentWarnings     = 0;
                        aTask->sizeMatchURL_trap_available           = globalTrapAttack;
                        aTask->sizeMatchURL_trap.eventTime           = aTask->statusVars.current_run;
                        }
                   }
               }

       /* ------------------------------------ */
       /* match content of other URL           */
       /* ------------------------------------ */

               if( aTask->task_contentMatchURL ) {

                   if( !(result = snarl_getContentOfURL( &aTask->contentMatchURL, &snarlWebData2 )) ) {

                       URLContent2 = snarlWebData2.data;


                       if( (URLContent1) && (snarlWebData1.size) && (snarlWebData1.size == snarlWebData2.size) ) {

                           if( !memcmp(URLContent1, URLContent2, snarlWebData1.size) ) {
                               aTask->contentMatchURL_flag = TRUE;
                           } else {
                               aTask->contentMatchURL_flag = FALSE;
                           }
                       } else {
                           /* cannot be equal */
                          aTask->contentMatchURL_flag = FALSE;
                       }

                       free( URLContent2 );
                   }

                   if( (aTask->contentMatchURL_flag_trap) == (aTask->contentMatchURL_flag) ) {

                        snprintf(   aTask->contentMatchURL_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: web content %s match the content of %s",
                                    aTask->sitename, aTask->urlname,
                                    aTask->contentMatchURL_flag ? "does" : "doesn't",
                                    aTask->contentMatchURL.url
                                );

                        (aTask->contentMatchURL_trap.currentWarnings)++;
                        if( aTask->contentMatchURL_trap.currentWarnings > aTask->warningsBeforeCritical) {

                            aTask->contentMatchURL_trap.specificTrap      = SNARLSNMP_TRAP_CRITICAL;
                        } else {
                            aTask->contentMatchURL_trap.specificTrap      = SNARLSNMP_TRAP_WARNING;
                        }

                        aTask->contentMatchURL_trap_available           = globalTrapAttack;
                        aTask->contentMatchURL_trap.eventTime           = aTask->statusVars.current_run;
                        aTask->useAlertInterval                         = TRUE;

                   } else {

                        if( aTask->contentMatchURL_trap.specificTrap > SNARLSNMP_TRAP_NORMAL ) {
                        snprintf(   aTask->contentMatchURL_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: web content %s match the content of %s",
                                    aTask->sitename, aTask->urlname,
                                    aTask->contentMatchURL_flag ? "does" : "doesn't",
                                    aTask->contentMatchURL.url
                                );

                        aTask->contentMatchURL_trap.specificTrap          = SNARLSNMP_TRAP_NORMAL;
                        aTask->contentMatchURL_trap.currentWarnings     = 0;
                        aTask->contentMatchURL_trap_available           = globalTrapAttack;
                        aTask->contentMatchURL_trap.eventTime           = aTask->statusVars.current_run;
                        }
                   }
               }


       /* ------------------------------------ */
       /* fetch regex                          */
       /* ------------------------------------ */

               if( aTask->task_fetchRegex ) {
                   if(   !(aTask->varRegex.regexError)
                       && (URLContent1)
                       && (snarlWebData1.size) ) {

                       aTask->varRegex.regexMatchingError = regexec( &(aTask->varRegex.regexBuffer),
                                                                     URLContent1,
                                                                     SNARL_TASK_MAX_REGEX_MATCHES,
                                                                     regexResult,
                                                                     0);


                       regerror(  aTask->varRegex.regexMatchingError,
                                &(aTask->varRegex.regexBuffer),
                                  aTask->varRegex.regexMatchingError_string,
                           sizeof(aTask->varRegex.regexMatchingError_string));

                       if( !(aTask->varRegex.regexMatchingError) ) {

                           for(  index=0;
                                (regexResult[index].rm_so != -1) && (index < SNARL_TASK_MAX_REGEX_MATCHES);
                                index++ ) {

                               length = (regexResult[index].rm_eo - regexResult[index].rm_so) < SNARL_TASK_MAX_REGEX_RESULT_LENGTH-1 ? (regexResult[index].rm_eo - regexResult[index].rm_so) : (SNARL_TASK_MAX_REGEX_RESULT_LENGTH-1);

                               strncpy(aTask->regexMatchContent[index], &(URLContent1[regexResult[index].rm_so]), length);
                               aTask->regexMatchContent[index][(regexResult[index].rm_eo - regexResult[index].rm_so)] = '\0';
                           }
                       } else {
                           for( index = 0; index < SNARL_TASK_MAX_REGEX_MATCHES; index++ ) {
                               aTask->regexMatchContent[index][0] = '\0';
                           }
                       }
                   }
               }



        /* ------------------------------------ */
        /* matchNumic                           */
        /* ------------------------------------ */

               if( aTask->task_matchNumic ) {

                   numicCompareString = aTask->regexMatchContent[aTask->matchNumic_withRegexIndex];

                   difference =  strtod(numicCompareString, NULL) - strtod(aTask->matchNumic_string, NULL );

                   if( difference > 0 ) {
                       aTask->matchNumic_flag = 1;
                   } else {

                       if( difference < 0 ) {
                           aTask->matchNumic_flag = -1;
                       } else {
                           aTask->matchNumic_flag = 0;
                       }
                   }
                   if( (aTask->matchNumic_flag_trap) == (aTask->matchNumic_flag) ) {

                        snprintf(   aTask->matchNumic_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: regular expression result \"%s\" is %s \"%s\"",
                                    aTask->sitename, aTask->urlname,
                                    numicCompareString,
                                    aTask->matchNumic_flag > 0 ? "greater than" : aTask->matchNumic_flag == 0 ? "equal to" : "less than",
                                    aTask->matchNumic_string );

                        (aTask->matchNumic_trap.currentWarnings)++;
                        if( aTask->matchNumic_trap.currentWarnings > aTask->warningsBeforeCritical) {

                            aTask->matchNumic_trap.specificTrap      = SNARLSNMP_TRAP_CRITICAL;
                        } else {
                            aTask->matchNumic_trap.specificTrap      = SNARLSNMP_TRAP_WARNING;
                        }

                        aTask->matchNumic_trap_available           = globalTrapAttack;

                        aTask->matchNumic_trap.eventTime           = aTask->statusVars.current_run;
                        aTask->useAlertInterval                    = TRUE;

                   } else {

                        if( aTask->matchNumic_trap.specificTrap > SNARLSNMP_TRAP_NORMAL ) {

                        snprintf(   aTask->matchNumic_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: regular expression result \"%s\" is %s \"%s\"",
                                    aTask->sitename, aTask->urlname,
                                    numicCompareString,
                                    aTask->matchNumic_flag > 0 ? "greater than" : aTask->matchNumic_flag == 0 ? "equal to" : "less than",
                                    aTask->matchNumic_string );

                        aTask->matchNumic_trap.specificTrap          = SNARLSNMP_TRAP_NORMAL;
                        aTask->matchNumic_trap.currentWarnings     = 0;
                        aTask->matchNumic_trap_available           = globalTrapAttack;
                        aTask->matchNumic_trap.eventTime           = aTask->statusVars.current_run;
                        }
                   }
               }


        /* ------------------------------------ */
        /* matchDateAge                         */
        /* ------------------------------------ */
               if( aTask->task_matchDateAge ) {


        /* ------------------------------------ */
        /* first build the string from all      */
        /* FetchRegex submatches                */
        /* ------------------------------------ */

                    if( tokenline = strdup(aTask->matchDateAge_tokenline)) {

                       aTask->matchDateAge_construct[0]='\0';
                       token = strtok_r(tokenline, "$", (char**)&inbuf);

                       while( token ) {

                           index = strtol(token, (char**)&token, 10) & SNARL_TASK_MAX_REGEX_MATCHES_MASK;

                           if( (strlen(aTask->matchDateAge_construct) +
                                strlen(aTask->regexMatchContent[index] ) )
                                <= (SNARL_SITE_REGEX_DATE_AGE_STRING_MAX_LENGTH - 1)
                             ) {
                                   sprintf( (char*)&aTask->matchDateAge_construct[strlen(aTask->matchDateAge_construct)] ,"%s", aTask->regexMatchContent[index]);
                           }
                           if( (strlen(aTask->matchDateAge_construct) +
                                strlen(token ) )
                                <= (SNARL_SITE_REGEX_DATE_AGE_STRING_MAX_LENGTH - 1)
                             ) {
                                   sprintf( (char*)&aTask->matchDateAge_construct[strlen(aTask->matchDateAge_construct)] ,"%s", token);
                           }

                           token = strtok_r(NULL,  "$", (char**)&inbuf);
                       }

                       free( tokenline );

                       snarl_time(&now);

                       snarl_getdate(aTask->matchDateAge_construct, &now, &timeAge);
                       timeAge        =  now - timeAge;



                       snarl_getdate(aTask->matchDateAge_string,    &now, &timeDifference);

                       timeDifference =  timeDifference - now;










                       if( timeAge > timeDifference  ) /* constructed date age is older */ {
                           aTask->matchDateAge_flag = 1;
                       } else {

                           if( timeAge < timeDifference ) /* constructed date age is younger  */{
                               aTask->matchDateAge_flag = -1;
                           } else {
                               aTask->matchDateAge_flag = 0;
                           }
                       }
                       if( (aTask->matchDateAge_flag_trap) == (aTask->matchDateAge_flag) ) {

                            snprintf(   aTask->matchDateAge_trap.siteMessage,
                                        SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                        "%s - %s: regular expression result \"%s\" is %s \"%s\"",
                                        aTask->sitename, aTask->urlname,
                                        aTask->matchDateAge_construct,
                                        aTask->matchDateAge_flag > 0 ? "older than" : aTask->matchDateAge_flag == 0 ? "equal to" : "younger than",
                                        aTask->matchDateAge_string );

                        (aTask->matchDateAge_trap.currentWarnings)++;
                        if( aTask->matchDateAge_trap.currentWarnings > aTask->warningsBeforeCritical) {

                            aTask->matchDateAge_trap.specificTrap    = SNARLSNMP_TRAP_CRITICAL;
                        } else {
                            aTask->matchDateAge_trap.specificTrap    = SNARLSNMP_TRAP_WARNING;
                        }

                        aTask->matchDateAge_trap_available           = globalTrapAttack;
                        aTask->matchDateAge_trap.eventTime           = aTask->statusVars.current_run;
                        aTask->useAlertInterval                      = TRUE;

                       } else {

                            if( aTask->matchDateAge_trap.specificTrap > SNARLSNMP_TRAP_NORMAL ) {

                            snprintf(   aTask->matchDateAge_trap.siteMessage,
                                        SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                        "%s - %s: regular expression result \"%s\" is %s \"%s\"",
                                        aTask->sitename, aTask->urlname,
                                        aTask->matchDateAge_construct,
                                        aTask->matchDateAge_flag > 0 ? "older than" : aTask->matchDateAge_flag == 0 ? "equal to" : "younger than",
                                        aTask->matchDateAge_string );

                            aTask->matchDateAge_trap.specificTrap    = SNARLSNMP_TRAP_NORMAL;
                            aTask->matchDateAge_trap.currentWarnings = 0;
                            aTask->matchDateAge_trap_available       = globalTrapAttack;
                            aTask->matchDateAge_trap.eventTime       = aTask->statusVars.current_run;
                            }
                       }
                   }
               }


       /* ------------------------------------ */
       /* end of testing - free mainURL        */
       /* ------------------------------------ */

               if( URLContent1  ) {

       /* ------------------------------------ */
       /* the keep content task is to monitor  */
       /* simple change                        */
       /* very last chance to save the content */
       /* ------------------------------------ */
				   if( aTask->mainURL.keepContent ) {

						  if( keepfile = fopen( aTask->mainURL.keepContentFileName, "w+")) {

							  if( (snarlWebData1.data) && (snarlWebData1.size)) {
								  fwrite(  snarlWebData1.data, snarlWebData1.size, 1, keepfile);
							  }

							  fflush(  keepfile );
							  fclose(  keepfile );
						  }
				   }

				   free( URLContent1 );
				   URLContent1 = NULL;
				   snarlWebData1.data = NULL;
				   snarlWebData1.size = 0;
               }


       /* ------------------------------------ */
       /* this is signalling snmpd that        */
       /* data are valid from now on           */
       /* ------------------------------------ */

               aTask->hasData = TRUE;


       /* ------------------------------------ */
       /* at the very last cause this task to  */
       /* send synchronize traps if desired    */
       /* ------------------------------------ */

               if( aTask->task_synchronize ) {

                    snarl_ctime( inbuf,  &aTask->statusVars.current_run, SNARL_TASK_MAX_LENGTH_SUBCONFIG_LINE );

                    snprintf(   aTask->synchronize_trap.siteMessage,
                                SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                "%s - %s: synchronization request, created at %s",
                                aTask->sitename, aTask->urlname,
                                inbuf
                             );

                   aTask->synchronize_trap.eventTime       = aTask->statusVars.current_run;
                   aTask->synchronize_trap.specificTrap    = SNARLSNMP_TRAP_GENERIC_INFO;
                   aTask->synchronize_trap_available       = globalTrapAttack;
               }

           } else {

       /* ------------------------------------ */
       /* this is the end of the very very     */
       /* long if structure.                   */
       /* when here then something went wrong  */
       /* downloading the main url             */
       /* ------------------------------------ */

                if( ! (aTask->general_trap_inhibit) ) {

                   if( aTask->force_status_unknown ) {
                       aTask->force_status_unknown               = FALSE;
                       result = -4;
				   }


                   aTask->general_trap.specificTrap          = SNARLSNMP_TRAP_STATE_UNKNOWN;
                   aTask->performance_trap.specificTrap      = SNARLSNMP_TRAP_STATE_UNKNOWN;
                   aTask->httperror_trap.specificTrap        = SNARLSNMP_TRAP_STATE_UNKNOWN;
                   aTask->matchRegex_trap.specificTrap       = SNARLSNMP_TRAP_STATE_UNKNOWN;
                   aTask->matchContentSize_trap.specificTrap = SNARLSNMP_TRAP_STATE_UNKNOWN;
                   aTask->sizeMatchURL_trap.specificTrap     = SNARLSNMP_TRAP_STATE_UNKNOWN;
                   aTask->contentMatchURL_trap.specificTrap  = SNARLSNMP_TRAP_STATE_UNKNOWN;
                   aTask->matchNumic_trap.specificTrap       = SNARLSNMP_TRAP_STATE_UNKNOWN;
                   aTask->matchDateAge_trap.specificTrap     = SNARLSNMP_TRAP_STATE_UNKNOWN;
       /* ------------------------------------ */
       /* we don't want the NoUpdateBetweeen   */
       /* trap to always notify it's status    */
       /* after a general error, because       */
       /* it is annoying not informative       */
       /* ------------------------------------ */
                 //aTask->noUpdateBetween_trap.specificTrap  = SNARLSNMP_TRAP_STATE_UNKNOWN;

                   aTask->general_trap_available          = 0;
                   aTask->performance_trap_available      = 0;
                   aTask->httperror_trap_available        = 0;
                   aTask->matchRegex_trap_available       = 0;
                   aTask->matchContentSize_trap_available = 0;
                   aTask->sizeMatchURL_trap_available     = 0;
                   aTask->contentMatchURL_trap_available  = 0;
                   aTask->matchNumic_trap_available       = 0;
                   aTask->matchDateAge_trap_available     = 0;
                 //aTask->noUpdateBetween_trap_available  = 0;

                    switch( result ) {


                        case CURLE_READ_ERROR:
                        snprintf(   aTask->general_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: data exceeded %ld bytes, transaction prematurely aborted",
                                    aTask->sitename, aTask->urlname,
                                    aTask->maxDownloadSize
                                    );
                        break;

                        case -4:
                        snprintf(   aTask->general_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: triggering task %s is at error",
                                    aTask->sitename, aTask->urlname, aTask->triggerTaskName
                                    );
                        break;

                        case -3:
                        snprintf(   aTask->general_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: general error: could not allocate download buffer - program is in bad shape or overloaded",
                                    aTask->sitename, aTask->urlname
                                    );
                        break;

                        case -2:
                        snprintf(   aTask->general_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: general error: could not initialize curl - probably internal program error",
                                    aTask->sitename, aTask->urlname
                                    );
                        break;

                        case -1:
                        snprintf(   aTask->general_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: general error: URL is missing",
                                    aTask->sitename, aTask->urlname
                                    );
                        break;

                        default:
                        snprintf(   aTask->general_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: general error: %s (CURLCode: %d)",
                                    aTask->sitename, aTask->urlname,
                                    aTask->mainURL.CurlError,
                                    result);
                        break;
                    }

       /* ------------------------------------ */
       /* curl goofes up ending the string     */
       /* with '\0'. so do it here for the     */
       /* next time                            */
       /* ------------------------------------ */
                    memset( aTask->mainURL.CurlError, 0, CURL_ERROR_SIZE);

                    (aTask->general_trap.currentWarnings)++;
                    if( aTask->general_trap.currentWarnings > aTask->warningsBeforeCritical) {

                        aTask->general_trap.specificTrap      = SNARLSNMP_TRAP_CRITICAL;
                    } else {
                        aTask->general_trap.specificTrap      = SNARLSNMP_TRAP_WARNING;
                    }

                    aTask->general_trap_available             = globalTrapAttack;
                    aTask->general_trap.eventTime             = aTask->statusVars.current_run;
                    aTask->useAlertInterval                   = TRUE;
                }

                if( result ) {

                    if( aTask->task_performance ) {
                        aTask->performance_trap_available          = globalTrapAttack;
                        aTask->performance_trap.eventTime          = aTask->statusVars.current_run;

                        snprintf(   aTask->performance_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: performance unknown",
                                    aTask->sitename, aTask->urlname);
                    }

                    if( aTask->task_httpcode ) {
                        if(    sscanf(aTask->mainURL.currenturl, "%64[^:]://", inbuf)
                           && (  (curl_strequal(inbuf, "https"))
                               ||(curl_strequal(inbuf, "http"))
                              )
                          ) {

                              aTask->httperror_trap_available          = globalTrapAttack;
                              aTask->httperror_trap.eventTime          = aTask->statusVars.current_run;
                              snprintf(   aTask->httperror_trap.siteMessage,
                                          SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                          "%s - %s: http code unknown",
                                          aTask->sitename, aTask->urlname);
                        }
                    }

                    if( aTask->task_matchRegex ) {
                        aTask->matchRegex_trap_available           = globalTrapAttack;
                        aTask->matchRegex_trap.eventTime           = aTask->statusVars.current_run;
                        snprintf(   aTask->matchRegex_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: match regex result unknown",
                                    aTask->sitename, aTask->urlname);
                    }

                    if( aTask->task_matchContentSize ) {
                        aTask->matchContentSize_trap_available           = globalTrapAttack;
                        aTask->matchContentSize_trap.eventTime           = aTask->statusVars.current_run;
                        snprintf(   aTask->matchContentSize_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: static content size match result unknown",
                                    aTask->sitename, aTask->urlname);
                    }

                    if( aTask->task_sizeMatchURL ) {
                        aTask->sizeMatchURL_trap_available           = globalTrapAttack;
                        aTask->sizeMatchURL_trap.eventTime           = aTask->statusVars.current_run;
                        snprintf(   aTask->sizeMatchURL_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: url content size comparison result unknown",
                                    aTask->sitename, aTask->urlname);
                    }

                    if( aTask->task_contentMatchURL ) {
                        aTask->contentMatchURL_trap_available           = globalTrapAttack;
                        aTask->contentMatchURL_trap.eventTime           = aTask->statusVars.current_run;
                        snprintf(   aTask->contentMatchURL_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: url content comparison result unknown",
                                    aTask->sitename, aTask->urlname);
                    }

                    if( aTask->task_matchNumic ) {
                        aTask->matchNumic_trap_available           = globalTrapAttack;
                        aTask->matchNumic_trap.eventTime           = aTask->statusVars.current_run;
                        snprintf(   aTask->matchNumic_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: numerical comparison result unknown",
                                    aTask->sitename, aTask->urlname);
                    }

                    if( aTask->task_matchDateAge ) {
                        aTask->matchDateAge_trap_available           = globalTrapAttack;
                        aTask->matchDateAge_trap.eventTime           = aTask->statusVars.current_run;
                        snprintf(   aTask->matchDateAge_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: date age comparison result unknown",
                                    aTask->sitename, aTask->urlname);
                    }

                    /*
                    if( aTask->task_noUpdateBetween ) {
                        aTask->noUpdateBetween_trap_available           = globalTrapAttack;
                        aTask->noUpdateBetween_trap.eventTime           = aTask->statusVars.current_run;
                        snprintf(   aTask->noUpdateBetween_trap.siteMessage,
                                    SNARL_TASK_MASK_TRAP_MESSAGE_LENGTH - 1 ,
                                    "%s - %s: suspension state unknown",
                                    aTask->sitename, aTask->urlname);
                    }
                    */
                }

       /* ------------------------------------ */
       /* don't forget to let go off the data  */
       /* ------------------------------------ */
               if(   (URLContent1 = snarlWebData1.data) ) {

                  free( URLContent1 );
                  URLContent1        = NULL;
                  snarlWebData1.data = NULL;
               }
           }

       /* ------------------------------------ */
       /* end of if mainURL.curl_handle        */
       /* ------------------------------------ */

       /* ------------------------------------ */
       /* always finish while loop             */
       /* ------------------------------------ */

           break;

       }

       /* ------------------------------------ */
       /* end of infinite loop -               */
       /* will never be called twice           */
       /* ------------------------------------ */


       pthread_mutex_lock(         theTask->lock );
           pthread_mutex_lock(     theTask->statusVars.statusLock );

               if( theTask->hasSiteVarsPtr ) {
                   if( aSiteVars = theTask->siteVars ) {
                       pthread_mutex_lock( aSiteVars->statusVars.statusLock );
                   }
               }

       /* ------------------------------------ */
       /* store back new data in task theTask  */
       /* ------------------------------------ */


       /* ------------------------------------ */
       /* safe the group data if this task has */
       /* the responsibility for it            */
       /* this data may have been changed      */
       /* while this task was running          */
       /* ------------------------------------ */
				   if( theTask->hasSiteVarsPtr ) {
					   if( aSiteVars = theTask->siteVars ) {
						   memcpy( &aSiteVarsGroupCopy, &theTask->siteVarsGroup, sizeof(siteVariables));
					   }
				   }

				   if( theTask->hasProxyVarsPtr ) {
					   if( aProxyVars = theTask->proxyVars ) {
						   memcpy( &aProxyVarsGroupCopy, &theTask->proxyVarsGroup, sizeof(proxyVariables));
					   }
				   }
       /* ------------------------------------ */
       /* also do not copy back the status     */
       /* these may have changed meanwhile     */
       /* by an update request of status       */
       /* calling recordStatus( -1, ...)       */
       /* ------------------------------------ */
				   memcpy( &aStatusVarsCopy, (void*)&theTask->statusVars, sizeof(statusVariables));

				   memcpy( (void*)theTask,  (void*)aTask, sizeof(snarltask));

				   memcpy( (void*)&theTask->statusVars, &aStatusVarsCopy, sizeof(statusVariables));


				   if( theTask->hasSiteVarsPtr ) {

					   if( aSiteVars = theTask->siteVars ) {
						   memcpy( &theTask->siteVarsGroup, &aSiteVarsGroupCopy, sizeof(siteVariables));
					   }
				   }

				   if( theTask->hasProxyVarsPtr ) {

					   if( aProxyVars = theTask->proxyVars ) {
						   memcpy( &theTask->proxyVarsGroup, &aProxyVarsGroupCopy, sizeof(proxyVariables));
					   }
				   }

				   fixPointers( theTask );

			   if( theTask->hasSiteVarsPtr ) {
				   if( aSiteVars = theTask->siteVars ) {
					   pthread_mutex_unlock( aSiteVars->statusVars.statusLock );
				   }
			   }

		   pthread_mutex_unlock(      theTask->statusVars.statusLock );

		/* ------------------------------------ */
       /*                                      */
       /*  --- !!! ATTENTION !!! ---           */
       /*                                      */
       /*  starting here do not tamper with    */
       /*  aTask - all info will be lost       */
       /*                                      */
       /*  use the original theTask instead    */
       /*                                      */
       /* ------------------------------------ */

       /* ------------------------------------ */
       /* consolidate status for this group    */
       /* ------------------------------------ */


       /* ------------------------------------ */
       /* 1. consolidated status for this task */
       /* ------------------------------------ */

		   currentStatus =   prioritizeTrap(                                  theTask->general_trap.specificTrap,

							 prioritizeTrap( theTask->task_performance      ? theTask->performance_trap.specificTrap      : 0,

							 prioritizeTrap( theTask->task_httpcode         ? theTask->httperror_trap.specificTrap        : 0,

							 prioritizeTrap( theTask->task_matchRegex       ? theTask->matchRegex_trap.specificTrap       : 0,

							 prioritizeTrap( theTask->task_matchContentSize ? theTask->matchContentSize_trap.specificTrap : 0,

							 prioritizeTrap( theTask->task_sizeMatchURL     ? theTask->sizeMatchURL_trap.specificTrap     : 0,

							 prioritizeTrap( theTask->task_contentMatchURL  ? theTask->contentMatchURL_trap.specificTrap  : 0,

							 prioritizeTrap( theTask->task_matchNumic       ? theTask->matchNumic_trap.specificTrap       : 0,

											 theTask->task_matchDateAge     ? theTask->matchDateAge_trap.specificTrap     : 0

								  ))))))));

		   recordStatus( currentStatus, &theTask->statusVars );


       /* ------------------------------------ */
       /* 2. consolidated status for this proxy*/
       /* ------------------------------------ */
		   currentStatus = 0;
		   index = 0;
		   while( currentTask = find_snarltask( theTask->sitename, NULL, &index, globalTaskArray) ) {

			   if( !strcmp(currentTask->mainURL.proxy, theTask->mainURL.proxy)) {
				   pthread_mutex_lock( currentTask->statusVars.statusLock );

					   currentStatus = prioritizeTrap( currentStatus, currentTask->statusVars.currentStatus );

				   pthread_mutex_unlock( currentTask->statusVars.statusLock );
			   }
		   }

		   if(!currentStatus ) currentStatus = SNARLSNMP_TRAP_NORMAL;

		   if( aProxyVars = theTask->proxyVars) {
			   recordStatus( currentStatus, &(aProxyVars->statusVars));
		   }


       /* ------------------------------------ */
       /* 3. consolidated status for this site */
       /* ------------------------------------ */
		   currentStatus = 0;
		   index = 0;
		   while( currentTask = find_snarltask( theTask->sitename, NULL, &index, globalTaskArray) ) {

			   pthread_mutex_lock( currentTask->statusVars.statusLock );

				   currentStatus = prioritizeTrap( currentStatus, currentTask->statusVars.currentStatus );

			   pthread_mutex_unlock( currentTask->statusVars.statusLock );
		   }

		   if(!currentStatus ) currentStatus = SNARLSNMP_TRAP_NORMAL;

		   if( aSiteVars = theTask->siteVars ) {
			   recordStatus( currentStatus, &(aSiteVars->statusVars));
		   }


		   if( !(theTask->task_waitForExternalTrigger)) {
       /* ------------------------------------ */
       /* now reschedule next run              */
       /* ------------------------------------ */
			   if( (theTask->task_noUpdateBetween) &&
				   (theTask->noUpdateBetween_trap.specificTrap == SNARLSNMP_TRAP_INHIBITED) ) {

					   theTask->statusVars.last_run      = theTask->statusVars.current_run;
					   theTask->statusVars.next_run      = theTask->noUpdateBetweenEnd;

			   } else {


				   if( (theTask->task_noUpdateBetween) &&
					   (theTask->noUpdateBetween_trap.specificTrap == SNARLSNMP_TRAP_RESUMED) &&
					   (theTask->noUpdateBetween_trap_available )) {

       /* ------------------------------------ */
       /* this was the resumed task trap       */
       /* reschedule very soon but remember    */
       /* that globalTrapAttack many traps     */
       /* will be sent within globalTrapAttack */
       /* seconds of time (one trap persecond) */
       /* ------------------------------------ */

					   theTask->statusVars.last_run      = theTask->statusVars.current_run;

					   if( theTask->randomSoonStart ) {
       /* ------------------------------------ */
       /* restart randomly soon                */
       /* ------------------------------------ */
						   snarl_time(&now);
						   theTask->statusVars.next_run  = now + (globalTrapAttack + 1.0) + ((globalTaskArray->taskCount) * ( 1.0 - ( rand() / (RAND_MAX + 1.0))));


					   } else {
       /* ------------------------------------ */
       /* restart randomly within one interval */
       /* ------------------------------------ */
						   snarl_time(&now);
						   theTask->statusVars.next_run  = now + ((theTask->interval) * ( 1.0 - ( rand() / (RAND_MAX + 1.0))));
					   }

				   } else {

					   if( theTask->useAlertInterval ) {

						   theTask->statusVars.last_run  = theTask->statusVars.current_run;
						   snarl_time(&now);
						   theTask->statusVars.next_run  = now + ((theTask->alertInterval) * (1.0 + 0.1 * ( 0.5 - ( rand() / (RAND_MAX + 1.0)))));

						   theTask->useAlertInterval     = FALSE;

					   } else {

						   theTask->statusVars.last_run  = theTask->statusVars.current_run;
						   snarl_time(&now);
						   theTask->statusVars.next_run  = now + ((theTask->interval)      * (1.0 + 0.1 * ( 0.5 - ( rand() / (RAND_MAX + 1.0)))));
					   }
				   }
			   }

			   snarl_ctime( theTask->statusVars.last_run_string,  &theTask->statusVars.last_run, SNARL_TASK_MAX_DATE_STRING_LENGTH );
			   snarl_ctime( theTask->statusVars.next_run_string,  &theTask->statusVars.next_run, SNARL_TASK_MAX_DATE_STRING_LENGTH );


			   queueEvent( SNARL_UPDATE_TASK_EVENT, theTask->statusVars.next_run, theTask );

		   } else {

       /* ------------------------------------ */
       /* this task is waiting to be fired up  */
       /* ------------------------------------ */

			   theTask->statusVars.last_run      = theTask->statusVars.current_run;
			   snarl_ctime( theTask->statusVars.last_run_string,  &theTask->statusVars.last_run, SNARL_TASK_MAX_DATE_STRING_LENGTH );

			   theTask->statusVars.next_run      = 0;
			   snprintf(theTask->statusVars.next_run_string, SNARL_TASK_MAX_DATE_STRING_LENGTH, "task is waiting for %s", theTask->triggerTaskName );
			   if(strlen(theTask->statusVars.next_run_string) >= SNARL_TASK_MAX_DATE_STRING_LENGTH) theTask->statusVars.next_run_string[SNARL_TASK_MAX_DATE_STRING_LENGTH]='\0';

			   if( !(theTask->statusVars.first_run)) {
				   theTask->statusVars.first_run      = theTask->statusVars.current_run;
				   snarl_ctime( theTask->statusVars.first_run_string,  &theTask->statusVars.first_run, SNARL_TASK_MAX_DATE_STRING_LENGTH );
			   }
		   }

	   pthread_mutex_unlock(   theTask->lock );



       /* ------------------------------------ */
       /* look if anyone is waiting for us     */
       /* ------------------------------------ */

		   index = 0;
		   while( currentTask = find_snarltask( theTask->sitename, NULL, &index, globalTaskArray) ) {
			   if(    ((currentTask->ignoreProxyGroup) || (!strcmp(currentTask->mainURL.proxy, theTask->mainURL.proxy)))
				   &&  (currentTask->task_waitForExternalTrigger)
				   && !(currentTask->isRunning)
				   && !strcmp(currentTask->triggerTaskName, theTask->urlname ) ){

       /* ------------------------------------ */
       /* set the referer URL in the task to   */
       /* to be fired up                       */
       /* set all URL's                        */
       /* ------------------------------------ */

    		      pthread_mutex_lock( currentTask->lock );

    			      strcpy( currentTask->mainURL.referer,         theTask->mainURL.effectiveURL ) ;
    			      strcpy( currentTask->sizeMatchURL.referer,    theTask->mainURL.effectiveURL );
    			      strcpy( currentTask->contentMatchURL.referer, theTask->mainURL.effectiveURL );

    			      if(   theTask->propagateError
    			         && (
						        (theTask->statusVars.currentStatus == SNARLSNMP_TRAP_WARNING)
    			             || (theTask->statusVars.currentStatus == SNARLSNMP_TRAP_CRITICAL) ) ) {

    				    	currentTask->force_status_unknown = TRUE;


    			      }

				      if( (theTask->task_noUpdateBetween) &&
					      (theTask->noUpdateBetween_trap.specificTrap == SNARLSNMP_TRAP_INHIBITED) &&
					      (theTask->noUpdateBetween_trap_available )) {


						  if( (theTask->noUpdateBetweenEnd          > currentTask->noUpdateBetweenEnd) ) {

							  currentTask->noUpdateBetweenEnd       = theTask->noUpdateBetweenEnd;
							  currentTask->task_noUpdateBetween     = TRUE;

						  } else {

                              if( currentTask->noUpdateBetweenStart > theTask->noUpdateBetweenEnd ) {
							      currentTask->noUpdateBetweenEnd   = theTask->noUpdateBetweenEnd;
							      currentTask->task_noUpdateBetween = TRUE;
                              }
						  }

						  currentTask->noUpdateBetweenStart         = theTask->statusVars.last_run;

					  }

    		      pthread_mutex_unlock( currentTask->lock );


       /* ------------------------------------ */
       /* eventually fire up the task          */
       /* ------------------------------------ */
    		      snarl_time(&now);
    		      queueEvent( SNARL_UPDATE_TASK_EVENT, now, currentTask );


               }
           }



       /* ------------------------------------ */
       /* if notify conditions were met        */
       /* queue traps up for sending now       */
       /* ------------------------------------ */

	   if( theTask->general_trap_available ) {

		   if( theTask->general_trap_available == globalTrapAttack ) {

			   theTask->general_trap.siteStatus                          = theTask->statusVars.currentStatus;
			   theTask->general_trap.siteEventNum                        = theTask->statusVars.statusChangeCount;

               if( -1 == snprintf( theTask->general_trap.effectiveURL, SNARL_TASK_URL_MAX_LENGTH, "%s", theTask->mainURL.effectiveURL[0] ?  theTask->mainURL.effectiveURL : (theTask->mainURL.currenturl[0] ? theTask->mainURL.currenturl : "empty") )) {
                   theTask->general_trap.effectiveURL[SNARL_TASK_URL_MAX_LENGTH-1] = '\0';
		       }

			   aProxyVars = theTask->proxyVars;
			   if( aProxyVars ) {
				   theTask->general_trap.proxyStatus                     = aProxyVars->statusVars.currentStatus;
				   theTask->general_trap.proxyEventNum                   = aProxyVars->statusVars.statusChangeCount;
			   }

			   aSiteVars  = theTask->siteVars;
			   if( aSiteVars ) {
				   theTask->general_trap.siteGroupStatus                 = aSiteVars->statusVars.currentStatus;
				   theTask->general_trap.groupEventNum                   = aSiteVars->statusVars.statusChangeCount;
			   }
		   }

		   snarl_time(&trap_time);
		   if( theTask->general_trap.eventTime > trap_time ) {
			   trap_time = theTask->general_trap.eventTime;
		   }

		   index = 0;
		   while( theTask->general_trap_available ) {

			   queueEvent( SNARL_TRAP_EVENT, trap_time+index, &(theTask->general_trap) );

			   theTask->general_trap_available                 -= 1;
			   index                                           += 1;
		   }
	   }

	   if( theTask->performance_trap_available ) {

		   if( theTask->performance_trap_available == globalTrapAttack ) {
			   theTask->performance_trap.siteStatus                      = theTask->statusVars.currentStatus;
			   theTask->performance_trap.siteEventNum                    = theTask->statusVars.statusChangeCount;

               if( -1 == snprintf( theTask->performance_trap.effectiveURL, SNARL_TASK_URL_MAX_LENGTH, "%s", theTask->mainURL.effectiveURL[0] ?  theTask->mainURL.effectiveURL : (theTask->mainURL.currenturl[0] ? theTask->mainURL.currenturl : "empty") )) {
                   theTask->performance_trap.effectiveURL[SNARL_TASK_URL_MAX_LENGTH-1] = '\0';
		       }

			   aProxyVars = theTask->proxyVars;
			   if( aProxyVars ) {
				   theTask->performance_trap.proxyStatus                 = aProxyVars->statusVars.currentStatus;
				   theTask->performance_trap.proxyEventNum               = aProxyVars->statusVars.statusChangeCount;
			   }

			   aSiteVars  = theTask->siteVars;
			   if( aSiteVars ) {
				   theTask->performance_trap.siteGroupStatus             = aSiteVars->statusVars.currentStatus;
				   theTask->performance_trap.groupEventNum               = aSiteVars->statusVars.statusChangeCount;
			   }
		   }

		   snarl_time(&trap_time);
		   if( theTask->performance_trap.eventTime > trap_time ) {
			   trap_time = theTask->performance_trap.eventTime;
		   }

		   index = 0;
		   while( theTask->performance_trap_available ) {

			   queueEvent( SNARL_TRAP_EVENT, trap_time+index, &(theTask->performance_trap) );

			   theTask->performance_trap_available             -= 1;
			   index                                           += 1;
		   }
	   }

	   if( theTask->httperror_trap_available ) {

		   if( theTask->httperror_trap_available == globalTrapAttack ) {
			   theTask->httperror_trap.siteStatus                        = theTask->statusVars.currentStatus;
			   theTask->httperror_trap.siteEventNum                      = theTask->statusVars.statusChangeCount;

               if( -1 == snprintf( theTask->httperror_trap.effectiveURL, SNARL_TASK_URL_MAX_LENGTH, "%s", theTask->mainURL.effectiveURL[0] ?  theTask->mainURL.effectiveURL : (theTask->mainURL.currenturl[0] ? theTask->mainURL.currenturl : "empty") )) {
                   theTask->httperror_trap.effectiveURL[SNARL_TASK_URL_MAX_LENGTH-1] = '\0';
		       }

			   aProxyVars = theTask->proxyVars;
			   if( aProxyVars ) {
				   theTask->httperror_trap.proxyStatus                   = aProxyVars->statusVars.currentStatus;
				   theTask->httperror_trap.proxyEventNum                 = aProxyVars->statusVars.statusChangeCount;
			   }

			   aSiteVars  = theTask->siteVars;
			   if( aSiteVars ) {
				   theTask->httperror_trap.siteGroupStatus               = aSiteVars->statusVars.currentStatus;
				   theTask->httperror_trap.groupEventNum                 = aSiteVars->statusVars.statusChangeCount;
			   }
		   }

		   snarl_time(&trap_time);
		   if( theTask->httperror_trap.eventTime > trap_time ) {
			   trap_time = theTask->httperror_trap.eventTime;
		   }

		   index = 0;
		   while( theTask->httperror_trap_available ) {

			   queueEvent( SNARL_TRAP_EVENT, trap_time+index, &(theTask->httperror_trap) );

			   theTask->httperror_trap_available               -= 1;
			   index                                           += 1;
		   }
	   }

	   if( theTask->matchRegex_trap_available ) {

		   if( theTask->matchRegex_trap_available == globalTrapAttack ) {
			   theTask->matchRegex_trap.siteStatus                       = theTask->statusVars.currentStatus;
			   theTask->matchRegex_trap.siteEventNum                     = theTask->statusVars.statusChangeCount;

               if( -1 == snprintf( theTask->matchRegex_trap.effectiveURL, SNARL_TASK_URL_MAX_LENGTH, "%s", theTask->mainURL.effectiveURL[0] ?  theTask->mainURL.effectiveURL : (theTask->mainURL.currenturl[0] ? theTask->mainURL.currenturl : "empty") )) {
                   theTask->matchRegex_trap.effectiveURL[SNARL_TASK_URL_MAX_LENGTH-1] = '\0';
		       }

			   aProxyVars = theTask->proxyVars;
			   if( aProxyVars ) {
				   theTask->matchRegex_trap.proxyStatus                  = aProxyVars->statusVars.currentStatus;
				   theTask->matchRegex_trap.proxyEventNum                = aProxyVars->statusVars.statusChangeCount;
			   }

			   aSiteVars  = theTask->siteVars;
			   if( aSiteVars ) {
				   theTask->matchRegex_trap.siteGroupStatus              = aSiteVars->statusVars.currentStatus;
				   theTask->matchRegex_trap.groupEventNum                = aSiteVars->statusVars.statusChangeCount;
			   }
		   }

		   snarl_time(&trap_time);
		   if( theTask->matchRegex_trap.eventTime > trap_time ) {
			   trap_time = theTask->matchRegex_trap.eventTime;
		   }

		   index = 0;
		   while( theTask->matchRegex_trap_available ) {

			   queueEvent( SNARL_TRAP_EVENT, trap_time+index, &(theTask->matchRegex_trap) );

			   theTask->matchRegex_trap_available              -= 1;
			   index                                           += 1;
		   }
	   }

	   if( theTask->matchContentSize_trap_available ) {

		   if( theTask->matchContentSize_trap_available == globalTrapAttack ) {
			   theTask->matchContentSize_trap.siteStatus                 = theTask->statusVars.currentStatus;
			   theTask->matchContentSize_trap.siteEventNum               = theTask->statusVars.statusChangeCount;

               if( -1 == snprintf( theTask->matchContentSize_trap.effectiveURL, SNARL_TASK_URL_MAX_LENGTH, "%s", theTask->mainURL.effectiveURL[0] ?  theTask->mainURL.effectiveURL : (theTask->mainURL.currenturl[0] ? theTask->mainURL.currenturl : "empty") )) {
                   theTask->matchContentSize_trap.effectiveURL[SNARL_TASK_URL_MAX_LENGTH-1] = '\0';
		       }

			   aProxyVars = theTask->proxyVars;
			   if( aProxyVars ) {
				   theTask->matchContentSize_trap.proxyStatus            = aProxyVars->statusVars.currentStatus;
				   theTask->matchContentSize_trap.proxyEventNum          = aProxyVars->statusVars.statusChangeCount;
			   }

			   aSiteVars  = theTask->siteVars;
			   if( aSiteVars ) {
				   theTask->matchContentSize_trap.siteGroupStatus        = aSiteVars->statusVars.currentStatus;
				   theTask->matchContentSize_trap.groupEventNum          = aSiteVars->statusVars.statusChangeCount;
			   }
		   }

		   snarl_time(&trap_time);
		   if( theTask->matchContentSize_trap.eventTime > trap_time ) {
			   trap_time = theTask->matchContentSize_trap.eventTime;
		   }

		   index = 0;
		   while( theTask->matchContentSize_trap_available ) {

			   queueEvent( SNARL_TRAP_EVENT, trap_time+index, &(theTask->matchContentSize_trap) );

			   theTask->matchContentSize_trap_available        -= 1;
			   index                                           += 1;
		   }
	   }

	   if( theTask->sizeMatchURL_trap_available ) {

		   if( theTask->sizeMatchURL_trap_available == globalTrapAttack ) {
			   theTask->sizeMatchURL_trap.siteStatus                     = theTask->statusVars.currentStatus;
			   theTask->sizeMatchURL_trap.siteEventNum                   = theTask->statusVars.statusChangeCount;

               if( -1 == snprintf( theTask->sizeMatchURL_trap.effectiveURL, SNARL_TASK_URL_MAX_LENGTH, "%s", theTask->mainURL.effectiveURL[0] ?  theTask->mainURL.effectiveURL : (theTask->mainURL.currenturl[0] ? theTask->mainURL.currenturl : "empty") )) {
                   theTask->sizeMatchURL_trap.effectiveURL[SNARL_TASK_URL_MAX_LENGTH-1] = '\0';
		       }

			   aProxyVars = theTask->proxyVars;
			   if( aProxyVars ) {
				   theTask->sizeMatchURL_trap.proxyStatus                = aProxyVars->statusVars.currentStatus;
				   theTask->sizeMatchURL_trap.proxyEventNum              = aProxyVars->statusVars.statusChangeCount;
			   }

			   aSiteVars  = theTask->siteVars;
			   if( aSiteVars ) {
				   theTask->sizeMatchURL_trap.siteGroupStatus            = aSiteVars->statusVars.currentStatus;
				   theTask->sizeMatchURL_trap.groupEventNum              = aSiteVars->statusVars.statusChangeCount;
			   }
		   }

		   snarl_time(&trap_time);
		   if( theTask->sizeMatchURL_trap.eventTime > trap_time ) {
			   trap_time = theTask->sizeMatchURL_trap.eventTime;
		   }

		   index = 0;
		   while( theTask->sizeMatchURL_trap_available ) {

			   queueEvent( SNARL_TRAP_EVENT, trap_time+index, &(theTask->sizeMatchURL_trap) );

			   theTask->sizeMatchURL_trap_available            -= 1;
			   index                                           += 1;
		   }
	   }

	   if( theTask->contentMatchURL_trap_available ) {

		   if( theTask->contentMatchURL_trap_available == globalTrapAttack ) {
			   theTask->contentMatchURL_trap.siteStatus                       = theTask->statusVars.currentStatus;
			   theTask->contentMatchURL_trap.siteEventNum                     = theTask->statusVars.statusChangeCount;

               if( -1 == snprintf( theTask->contentMatchURL_trap.effectiveURL, SNARL_TASK_URL_MAX_LENGTH, "%s", theTask->mainURL.effectiveURL[0] ?  theTask->mainURL.effectiveURL : (theTask->mainURL.currenturl[0] ? theTask->mainURL.currenturl : "empty") )) {
                   theTask->contentMatchURL_trap.effectiveURL[SNARL_TASK_URL_MAX_LENGTH-1] = '\0';
		       }

			   aProxyVars = theTask->proxyVars;
			   if( aProxyVars ) {
				   theTask->contentMatchURL_trap.proxyStatus                  = aProxyVars->statusVars.currentStatus;
				   theTask->contentMatchURL_trap.proxyEventNum                = aProxyVars->statusVars.statusChangeCount;
			   }

			   aSiteVars  = theTask->siteVars;
			   if( aSiteVars ) {
				   theTask->contentMatchURL_trap.siteGroupStatus              = aSiteVars->statusVars.currentStatus;
				   theTask->contentMatchURL_trap.groupEventNum                = aSiteVars->statusVars.statusChangeCount;
			   }
		   }

		   snarl_time(&trap_time);
		   if( theTask->contentMatchURL_trap.eventTime > trap_time ) {
			   trap_time = theTask->contentMatchURL_trap.eventTime;
		   }

		   index = 0;
		   while( theTask->contentMatchURL_trap_available ) {

			   queueEvent( SNARL_TRAP_EVENT, trap_time+index, &(theTask->contentMatchURL_trap) );

			   theTask->contentMatchURL_trap_available         -= 1;
			   index                                           += 1;
		   }
	   }

	   if( theTask->matchNumic_trap_available ) {

		   if( theTask->matchNumic_trap_available == globalTrapAttack ) {
			   theTask->matchNumic_trap.siteStatus                       = theTask->statusVars.currentStatus;
			   theTask->matchNumic_trap.siteEventNum                     = theTask->statusVars.statusChangeCount;

               if( -1 == snprintf( theTask->matchNumic_trap.effectiveURL, SNARL_TASK_URL_MAX_LENGTH, "%s", theTask->mainURL.effectiveURL[0] ?  theTask->mainURL.effectiveURL : (theTask->mainURL.currenturl[0] ? theTask->mainURL.currenturl : "empty") )) {
                   theTask->matchNumic_trap.effectiveURL[SNARL_TASK_URL_MAX_LENGTH-1] = '\0';
		       }

			   aProxyVars = theTask->proxyVars;
			   if( aProxyVars ) {
				   theTask->matchNumic_trap.proxyStatus                  = aProxyVars->statusVars.currentStatus;
				   theTask->matchNumic_trap.proxyEventNum                = aProxyVars->statusVars.statusChangeCount;
			   }

			   aSiteVars  = theTask->siteVars;
			   if( aSiteVars ) {
				   theTask->matchNumic_trap.siteGroupStatus              = aSiteVars->statusVars.currentStatus;
				   theTask->matchNumic_trap.groupEventNum                = aSiteVars->statusVars.statusChangeCount;
			   }
		   }

		   snarl_time(&trap_time);
		   if( theTask->matchNumic_trap.eventTime > trap_time ) {
			   trap_time = theTask->matchNumic_trap.eventTime;
		   }

		   index = 0;
		   while( theTask->matchNumic_trap_available ) {

			   queueEvent( SNARL_TRAP_EVENT, trap_time+index, &(theTask->matchNumic_trap) );

			   theTask->matchNumic_trap_available              -= 1;
			   index                                           += 1;
		   }
	   }

	   if( theTask->matchDateAge_trap_available ) {

		   if( theTask->matchDateAge_trap_available == globalTrapAttack ) {
			   theTask->matchDateAge_trap.siteStatus                      = theTask->statusVars.currentStatus;
			   theTask->matchDateAge_trap.siteEventNum                    = theTask->statusVars.statusChangeCount;

               if( -1 == snprintf( theTask->matchDateAge_trap.effectiveURL, SNARL_TASK_URL_MAX_LENGTH, "%s", theTask->mainURL.effectiveURL[0] ?  theTask->mainURL.effectiveURL : (theTask->mainURL.currenturl[0] ? theTask->mainURL.currenturl : "empty") )) {
                   theTask->matchDateAge_trap.effectiveURL[SNARL_TASK_URL_MAX_LENGTH-1] = '\0';
		       }

			   aProxyVars = theTask->proxyVars;
			   if( aProxyVars ) {
				   theTask->matchDateAge_trap.proxyStatus                 = aProxyVars->statusVars.currentStatus;
				   theTask->matchDateAge_trap.proxyEventNum               = aProxyVars->statusVars.statusChangeCount;
			   }
			   aSiteVars  = theTask->siteVars;
			   if( aSiteVars ) {
				   theTask->matchDateAge_trap.siteGroupStatus             = aSiteVars->statusVars.currentStatus;
				   theTask->matchDateAge_trap.groupEventNum               = aSiteVars->statusVars.statusChangeCount;
			   }
		   }

		   snarl_time(&trap_time);
		   if( theTask->matchDateAge_trap.eventTime > trap_time ) {
			   trap_time = theTask->matchDateAge_trap.eventTime;
		   }

		   index = 0;
		   while( theTask->matchDateAge_trap_available ) {


			   queueEvent( SNARL_TRAP_EVENT, trap_time+index, &(theTask->matchDateAge_trap) );

			   theTask->matchDateAge_trap_available            -= 1;
			   index                                           += 1;
		   }
	   }

	   if( theTask->noUpdateBetween_trap_available ) {

		   if( theTask->noUpdateBetween_trap_available == globalTrapAttack ) {
			   theTask->noUpdateBetween_trap.siteStatus                        = theTask->statusVars.currentStatus;
			   theTask->noUpdateBetween_trap.siteEventNum                      = theTask->statusVars.statusChangeCount;

               if( -1 == snprintf( theTask->noUpdateBetween_trap.effectiveURL, SNARL_TASK_URL_MAX_LENGTH, "%s", theTask->mainURL.effectiveURL[0] ?  theTask->mainURL.effectiveURL : (theTask->mainURL.currenturl[0] ? theTask->mainURL.currenturl : "empty") )) {
                   theTask->noUpdateBetween_trap.effectiveURL[SNARL_TASK_URL_MAX_LENGTH-1] = '\0';
		       }

			   aProxyVars = theTask->proxyVars;
			   if( aProxyVars ) {
				   theTask->noUpdateBetween_trap.proxyStatus                   = aProxyVars->statusVars.currentStatus;
				   theTask->noUpdateBetween_trap.proxyEventNum                 = aProxyVars->statusVars.statusChangeCount;
			   }

			   aSiteVars  = theTask->siteVars;
			   if( aSiteVars ) {
				   theTask->noUpdateBetween_trap.siteGroupStatus               = aSiteVars->statusVars.currentStatus;
				   theTask->noUpdateBetween_trap.groupEventNum                 = aSiteVars->statusVars.statusChangeCount;
			   }
		   }

		   snarl_time(&trap_time);
		   if( theTask->noUpdateBetween_trap.eventTime > trap_time ) {
			   trap_time = theTask->noUpdateBetween_trap.eventTime;
		   }

		   index = 0;
		   while( theTask->noUpdateBetween_trap_available ) {

			   queueEvent( SNARL_TRAP_EVENT, trap_time+index, &(theTask->noUpdateBetween_trap) );

			   theTask->noUpdateBetween_trap_available         -= 1;
			   index                                           += 1;
		   }
	   }

	   if( theTask->synchronize_trap_available ) {

		   if( theTask->synchronize_trap_available == globalTrapAttack ) {
			   theTask->synchronize_trap.siteStatus                      = theTask->statusVars.currentStatus;
			   theTask->synchronize_trap.siteEventNum                    = theTask->statusVars.statusChangeCount;

               if( -1 == snprintf( theTask->synchronize_trap.effectiveURL, SNARL_TASK_URL_MAX_LENGTH, "%s", theTask->mainURL.effectiveURL[0] ?  theTask->mainURL.effectiveURL : (theTask->mainURL.currenturl[0] ? theTask->mainURL.currenturl : "empty") )) {
                   theTask->synchronize_trap.effectiveURL[SNARL_TASK_URL_MAX_LENGTH-1] = '\0';
		       }

			   aProxyVars = theTask->proxyVars;
			   if( aProxyVars ) {
				   theTask->synchronize_trap.proxyStatus                 = aProxyVars->statusVars.currentStatus;
				   theTask->synchronize_trap.proxyEventNum               = aProxyVars->statusVars.statusChangeCount;
			   }

			   aSiteVars  = theTask->siteVars;
			   if( aSiteVars ) {
				   theTask->synchronize_trap.siteGroupStatus             = aSiteVars->statusVars.currentStatus;
				   theTask->synchronize_trap.groupEventNum               = aSiteVars->statusVars.statusChangeCount;
			   }
		   }

		   snarl_time(&trap_time);
		   if( theTask->synchronize_trap.eventTime > trap_time ) {
			   trap_time = theTask->synchronize_trap.eventTime;
		   }

		   index = 0;
		   while( theTask->synchronize_trap_available ) {

			   queueEvent( SNARL_TRAP_EVENT, trap_time+index, &(theTask->synchronize_trap) );

			   theTask->synchronize_trap_available             -= 1;
			   index                                           += 1;
		   }
	   }

       /* ------------------------------------ */
       /* are we asked to kill old cookies ?   */
       /* ------------------------------------ */

       if( !(theTask->mainURL.noCookies) && (theTask->mainURL.task_killCookies)) {

           aSiteVars = theTask->siteVars;

           pthread_mutex_lock(     aSiteVars->cookielock );

               if( *(aSiteVars->cookiefilename) ) {

                   cleanup_cookiefile(aSiteVars->cookiefilename);
               }

           pthread_mutex_unlock(   aSiteVars->cookielock );
       }

       /* ------------------------------------ */
       /* we are done - if we were asked to    */
       /* quit then acknowledge quitting       */
       /* ------------------------------------ */



       invalidate_curlhandles( theTask );


       /* ------------------------------------ */
       /* check out from running tasks counter */
       /* ------------------------------------ */

       checkOutTask();


       /* ------------------------------------ */
       /* unlock memory and reenable paging    */
       /* ------------------------------------ */

       if( getuid() == 0 ) {
           munlockall();
       }

       /* ------------------------------------ */
       /* tell others that this task isn't     */
       /* running                              */
       /* ------------------------------------ */

       theTask->isRunning = FALSE;
   }

       /* ------------------------------------ */
       /* silently perish                      */
       /* DONT PUT CODE BEYOND THIS LINE       */
       /* ------------------------------------ */

   pthread_exit( NULL );
}


/***************************************************************
* ------------------------------------------------------------ *
* private functions - for thread use only                      *
* ------------------------------------------------------------ *
***************************************************************/

/***************************************************************
*                                                              *
* !!! ---   PRIVATE  --- !!!                                   *
* !!! --- ASYNCHRONOUS---!!!                                   *
*                                                              *
* store status data                                            *
***************************************************************/

void recordStatus( int status, statusVarsPtr statusVars ) {


       time_t     now;
       time_t     deltaWarning;
       time_t     deltaCritical;


       pthread_mutex_lock( statusVars->statusLock );

           snarl_time(&now);

           if( !(statusVars->last_update)) {
               statusVars->last_update = now;
           }

       /* ------------------------------------ */
       /* at very first record the status      */
       /* ------------------------------------ */

           if( status >= 0 ) statusVars->currentStatus = status;


       /* ------------------------------------ */
       /* first find out about validity of     */
       /* the status                           */
       /* ------------------------------------ */
           switch( statusVars->currentStatus ) {


       /* ------------------------------------ */
       /* ------------------------------------ */
               case SNARLSNMP_TRAP_WARNING:

                   statusVars->statusNormal   = FALSE;
                   statusVars->statusWarning  = TRUE;
                   statusVars->statusCritical = FALSE;

                   switch( statusVars->lastStatus ) {

                       case  SNARLSNMP_TRAP_WARNING:

       /* ------------------------------------ */
       /* this is a continuation of status     */
       /* WARNING: time counts full            */
       /* ------------------------------------ */
                           deltaWarning                             = (now - statusVars->last_update);
                           (statusVars->accumulatedTimeWarning)    += deltaWarning;
                       break;

                       case SNARLSNMP_TRAP_CRITICAL:
                       case SNARLSNMP_TRAP_NORMAL:
                       default:

       /* ------------------------------------ */
       /* this is a new status WARNING:        */
       /* if there was another completed       */
       /* status WARNING before then record    */
       /* data to calculate MTBF(WARNING)      */
       /* ------------------------------------ */
                           if(statusVars->accumulatedWarnings) {
                               (statusVars->accumulatedTimeBetweenWarning) += (now - statusVars->lastTimeWarningStart);
                           }

                           (statusVars->accumulatedWarnings)       += 1;
                           (statusVars->lastTimeWarningStart)       = now;

                       break;

                   }
               break;


       /* ------------------------------------ */
       /* ------------------------------------ */
               case SNARLSNMP_TRAP_CRITICAL:

                   statusVars->statusNormal   = FALSE;
                   statusVars->statusWarning  = TRUE;
                   statusVars->statusCritical = TRUE;

                   switch( statusVars->lastStatus ) {

                       case SNARLSNMP_TRAP_CRITICAL:

       /* ------------------------------------ */
       /* this is a continuation of status     */
       /* CRITICAL: time counts full           */
       /* ------------------------------------ */
                           deltaCritical                            = (now - statusVars->last_update);
                           (statusVars->accumulatedTimeCritical)   += deltaCritical;
                           deltaWarning                             = (now - statusVars->last_update);
                           (statusVars->accumulatedTimeWarning)    += deltaWarning;
                       break;

                       case SNARLSNMP_TRAP_NORMAL:
                       case SNARLSNMP_TRAP_WARNING:
                       default:

       /* ------------------------------------ */
       /* this is a new status CRITICAL:       */
       /* if there was another completed       */
       /* status WARNING before then record    */
       /* data to calculate MTBF(CRITICAL)     */
       /* ------------------------------------ */
                           if(statusVars->accumulatedCriticals) {
                               (statusVars->accumulatedTimeBetweenCritical)    += (now - statusVars->lastTimeCriticalStart);
                           }

                           (statusVars->accumulatedCriticals)      += 1;
                           (statusVars->lastTimeCriticalStart)      = now;

                           if(statusVars->accumulatedWarnings) {
                               (statusVars->accumulatedTimeBetweenWarning)     += (now - statusVars->lastTimeWarningStart);
                           }

                           (statusVars->accumulatedWarnings)       += 1;
                           (statusVars->lastTimeWarningStart)       = now;

                       break;

                   }
               break;


       /* ------------------------------------ */
       /* ------------------------------------ */

               case SNARLSNMP_TRAP_NORMAL:

               case SNARLSNMP_TRAP_INFO:
               case SNARLSNMP_TRAP_INHIBITED:
               case SNARLSNMP_TRAP_RESUMED:
               case SNARLSNMP_TRAP_GENERIC_INFO:
               case SNARLSNMP_TRAP_STATE_UNKNOWN:

                   statusVars->statusNormal   = TRUE;
                   statusVars->statusWarning  = FALSE;
                   statusVars->statusCritical = FALSE;

                   switch( statusVars->lastStatus) {

                       case SNARLSNMP_TRAP_CRITICAL:

       /* ------------------------------------ */
       /* this is an end of CRITICAL           */
       /* (and counts as end of WARNING, too)  */
       /* record half of time                  */
       /* ------------------------------------ */
                            deltaCritical                           = (now - statusVars->last_update);
                            (statusVars->accumulatedTimeCritical)  += deltaCritical;
                            /* dont break */

                       case SNARLSNMP_TRAP_WARNING:

       /* ------------------------------------ */
       /* this is an end of WARNING only       */
       /* record half of time                  */
       /* ------------------------------------ */
                            deltaWarning                            = (now - statusVars->last_update);
                            (statusVars->accumulatedTimeWarning)   += deltaWarning;
                       break;

                       default:
       /* ------------------------------------ */
       /* any other status change is           */
       /* irrelevant for WARNING and CRITICAL  */
       /* ------------------------------------ */
                       break;
                   }
               break;


               default:

                   statusVars->statusNormal   = FALSE;
                   statusVars->statusWarning  = FALSE;
                   statusVars->statusCritical = FALSE;

               break;

           }


       if( status != -1 ) {
           statusVars->lastStatus = status;
       }
       statusVars->last_update   = now;

       (statusVars->statusChangeCount) += 1;

       pthread_mutex_unlock( statusVars->statusLock );
}


/***************************************************************
*                                                              *
* !!! ---   PRIVATE  --- !!!                                   *
* !!! --- SYNCHRONOUS ---!!!                                   *
*                                                              *
* setup connection data                                        *
* setup libraries and curl options                             *
*                                                              *
***************************************************************/


int snarl_setup_connection_data( urldataPtr urldata, char* sitename, char* urlname, int urltype) {


    int            index = 0;
    int            len;
    snarltaskPtr   aTask;
    siteVarsPtr    siteVars;
    proxyVarsPtr   proxyVars;


       /* ------------------------------------ */
       /* check if we need to instanciate      */
       /* a curl handle at all or if we have   */
       /* one from a keep alive proxy          */
       /* ------------------------------------ */

   aTask    = (snarltaskPtr)(urldata->parentTask);

   proxyVars = aTask->proxyVars;

   if( proxyVars && proxyVars->keepAlive ) {

	   if( proxyVars->curl_handle ) {

           urldata->curl_handle = proxyVars->curl_handle;

	   } else {

		   if( ! (urldata->curl_handle = curl_easy_init()) ) {
			   return( SNARL_TASK_NO_CURL_HANDLE );
		   }
	   }

   } else {

	   if( ! (urldata->curl_handle = curl_easy_init()) ) {
		   return( SNARL_TASK_NO_CURL_HANDLE );
	   }
   }


       /* ------------------------------------ */
       /* set general options for the transer  */
       /* ------------------------------------ */

   if( urldata->curl_handle ) {

       curl_easy_setopt( urldata->curl_handle, CURLOPT_WRITEFUNCTION, snarlDataAcquisition );

       curl_easy_setopt( urldata->curl_handle, CURLOPT_FAILONERROR, TRUE );
       curl_easy_setopt( urldata->curl_handle, CURLOPT_ERRORBUFFER, urldata->CurlError );

       curl_easy_setopt( urldata->curl_handle, CURLOPT_FOLLOWLOCATION, TRUE );

       curl_easy_setopt( urldata->curl_handle, CURLOPT_MAXREDIRS, SNARL_TASK_MAX_HTTP_REDIRECTIONS);

       curl_easy_setopt( urldata->curl_handle, CURLOPT_USERAGENT, urldata->agent );

       if( proxyVars && proxyVars->keepAlive ) {

		   curl_easy_setopt( urldata->curl_handle, CURLOPT_FORBID_REUSE, FALSE );
           curl_easy_setopt( urldata->curl_handle, CURLOPT_FRESH_CONNECT, FALSE );

       } else {

		   curl_easy_setopt( urldata->curl_handle, CURLOPT_FORBID_REUSE, TRUE );
           curl_easy_setopt( urldata->curl_handle, CURLOPT_FRESH_CONNECT, TRUE );
           curl_easy_setopt( urldata->curl_handle, CURLOPT_CLOSEPOLICY, CURLCLOSEPOLICY_OLDEST );
	   }

       curl_easy_setopt( urldata->curl_handle, CURLOPT_TIMEOUT, urldata->timeout );
       curl_easy_setopt( urldata->curl_handle, CURLOPT_CONNECTTIMEOUT, urldata->connect_timeout );


        /* ------------------------------------ */
        /* the first task found with            */
        /* sitename == sitename will            */
        /* hold all cookie data                 */
        /* ------------------------------------ */

       if( !(urldata->noCookies) ) {

           index    = 0;
           aTask    = (snarltaskPtr)(urldata->parentTask);
           siteVars = aTask->siteVars;

               switch( urltype ) {
                  case SNARL_TASK_MAIN_URL:
                  case SNARL_TASK_SIZEMATCH_URL:
                  case SNARL_TASK_CONTENTMATCH_URL:
                     sprintf( siteVars->cookiefilename, "%s/%s_cookies.txt", PERSISTENT_DIRECTORY, siteVars->sitename );

                     len = strlen(siteVars->cookiefilename);

                     for( index = sizeof(PERSISTENT_DIRECTORY); index < len; index++ ) {

						 if( (! isalnum(siteVars->cookiefilename[index])) && (siteVars->cookiefilename[index] != '.' )  ) {

							 siteVars->cookiefilename[index] = '_';
						 }
					 }


                  break;

                  default:
                      return ( SNARL_TASK_BAD_COOKIEFILE_SUFFIX_TYPE );
                  break;
               }

               curl_easy_setopt( urldata->curl_handle, CURLOPT_HEADERFUNCTION, snarl_filter_cookie_from_header );
               curl_easy_setopt( urldata->curl_handle, CURLOPT_COOKIEFILE,  siteVars->cookiefilename );

       } else {

        /* ------------------------------------ */
        /* just ignore                          */
        /* we had problems with cURL when not   */
        /* supplying cookiefiles                */
        /* we circumvent the problem by totally */
        /* reinitializing a new cURL handle     */
        /* for each new configuration           */
        /* instead of reusing the old handle    */
        /* ------------------------------------ */

       }
   }

        /* ------------------------------------ */
        /* set configured options               */
        /* ------------------------------------ */

  if( *urldata->currenturl && urldata->curl_handle ) {
      curl_easy_setopt( urldata->curl_handle, CURLOPT_URL, urldata->currenturl);
   }

   if( *urldata->proxy && urldata->curl_handle ) {
      curl_easy_setopt( urldata->curl_handle, CURLOPT_PROXY, urldata->proxy );
   }

   if( *urldata->passphrase && urldata->curl_handle ) {
      curl_easy_setopt( urldata->curl_handle, CURLOPT_USERPWD, urldata->passphrase );
   }

   if( urldata->isMultiForm && urldata->curl_handle ) {
      curl_easy_setopt( urldata->curl_handle, CURLOPT_HTTPPOST, urldata->firstFormItem );
   }

   if( urldata->isForm && urldata->curl_handle ) {
      curl_easy_setopt( urldata->curl_handle, CURLOPT_POST, TRUE );
      curl_easy_setopt( urldata->curl_handle, CURLOPT_POSTFIELDS, urldata->currentformData );
   }

   if( urldata->withHeader && urldata->curl_handle ) {
      curl_easy_setopt( urldata->curl_handle, CURLOPT_HEADER, TRUE );
   }

   if( *urldata->referer && urldata->curl_handle ) {
      curl_easy_setopt( urldata->curl_handle, CURLOPT_REFERER, urldata->referer );
   }

   return( 0 );
}



/***************************************************************
*                                                              *
* !!! ---   PRIVATE  --- !!!                                   *
* !!! --- SYNCHRONOUS ---!!!                                   *
*                                                              *
* snarl_log_statistics                                         *
* copy performance data from webData struct to snarltask struct*
*                                                              *
* this is a good place to dump task results to file, ODBC etc. *
*                                                              *
***************************************************************/


void  snarl_log_statistics(  snarlWebData *webData,  snarltaskPtr aTask ) {


    static char           *effectiveURL;
           int             len;

       /* ------------------------------------ */
       /* get effective url                    */
       /* this has been coded badly in curl    */
       /* curl returns a pointer to a local    */
       /* ------------------------------------ */

   curl_easy_getinfo( aTask->mainURL.curl_handle, CURLINFO_EFFECTIVE_URL,    &effectiveURL );
   if( *effectiveURL ) {
        len = min( strlen(effectiveURL), SNARL_TASK_URL_MAX_LENGTH-1 );

        strncpy( aTask->mainURL.effectiveURL, effectiveURL, len );
        aTask->mainURL.effectiveURL[len] = '\0';
   }

   curl_easy_getinfo( aTask->mainURL.curl_handle, CURLINFO_HTTP_CODE,        &aTask->mainURL.HTTPCode );
   curl_easy_getinfo( aTask->mainURL.curl_handle, CURLINFO_FILETIME,         &aTask->mainURL.fileTime );



       /* ------------------------------------ */
       /* get timing data                      */
       /* ------------------------------------ */

   curl_easy_getinfo( aTask->mainURL.curl_handle, CURLINFO_NAMELOOKUP_TIME,  &webData->DNSTime );
   curl_easy_getinfo( aTask->mainURL.curl_handle, CURLINFO_CONNECT_TIME,     &webData->connectTime );
   curl_easy_getinfo( aTask->mainURL.curl_handle, CURLINFO_PRETRANSFER_TIME, &webData->transferStartTime );

   /* this call is documented to be buggy
      timing data is already stored in webData->totalTime
      by snarl_getContentOfURL
   curl_easy_getinfo( aTask->mainURL.curl_handle, CURLINFO_TOTAL_TIME,       &webData->totalTime );
   */


       /* ------------------------------------ */
       /* first calculate the performance data */
       /* curl sums up DNS, Connect and        */
       /* First Byte. Total is given absolute  */
       /* ------------------------------------ */

   aTask->time_DNS           = webData->DNSTime;
   aTask->time_connect       = webData->connectTime;
   aTask->time_firstByte     = webData->transferStartTime;
   aTask->time_total         = webData->totalTime;


       /* ------------------------------------ */
       /* we are taking the gross times        */
       /* directly                             */
       /* ------------------------------------ */

   aTask->performance_DNS       = 1000 * aTask->time_DNS;
   aTask->performance_connect   = 1000 * aTask->time_connect;
   aTask->performance_firstByte = 1000 * aTask->time_firstByte;
   aTask->performance_total     = 1000 * aTask->time_total;

}


/***************************************************************
*                                                              *
* !!! ---   PRIVATE  --- !!!                                   *
* !!! --- SYNCHRONOUS ---!!!                                   *
*                                                              *
* invalidate_traps                                             *
* just causes new traps to be sent on next task invocation     *
*                                                              *
***************************************************************/


void invalidate_traps( snarltaskPtr aTask ) {


       aTask->general_trap.specificTrap          = SNARLSNMP_TRAP_STATE_UNKNOWN;
    // aTask->synchronize_trap.specificTrap      = SNARLSNMP_TRAP_STATE_UNKNOWN;
       aTask->performance_trap.specificTrap      = SNARLSNMP_TRAP_STATE_UNKNOWN;
       aTask->httperror_trap.specificTrap        = SNARLSNMP_TRAP_STATE_UNKNOWN;
       aTask->matchRegex_trap.specificTrap       = SNARLSNMP_TRAP_STATE_UNKNOWN;
       aTask->matchContentSize_trap.specificTrap = SNARLSNMP_TRAP_STATE_UNKNOWN;
       aTask->sizeMatchURL_trap.specificTrap     = SNARLSNMP_TRAP_STATE_UNKNOWN;
       aTask->contentMatchURL_trap.specificTrap  = SNARLSNMP_TRAP_STATE_UNKNOWN;
       aTask->matchNumic_trap.specificTrap       = SNARLSNMP_TRAP_STATE_UNKNOWN;
       aTask->matchDateAge_trap.specificTrap     = SNARLSNMP_TRAP_STATE_UNKNOWN;
       aTask->noUpdateBetween_trap.specificTrap  = SNARLSNMP_TRAP_STATE_UNKNOWN;

       aTask->general_trap_available             = 0;
       aTask->synchronize_trap_available         = 0;
       aTask->performance_trap_available         = 0;
       aTask->httperror_trap_available           = 0;
       aTask->matchRegex_trap_available          = 0;
       aTask->matchContentSize_trap_available    = 0;
       aTask->sizeMatchURL_trap_available        = 0;
       aTask->contentMatchURL_trap_available     = 0;
       aTask->matchNumic_trap_available          = 0;
       aTask->matchDateAge_trap_available        = 0;
       aTask->noUpdateBetween_trap_available     = 0;
}


/***************************************************************
*                                                              *
* !!! ---   PRIVATE  --- !!!                                   *
* !!! --- ASYNCHRONOUS HELPERFUNCTIONS ---!!!                  *
*                                                              *
* snarl_getContentOfURL                                        *
* do one single transaction                                    *
*                                                              *
***************************************************************/

int snarl_getContentOfURL (urldataPtr    urldata,
                           snarlWebData *snarlWebData ) {

           fakeStream      data;
           CURLcode        result;
           int             len,i;
           int             index;
           int             lockedCookieFile = FALSE;
           snarltaskPtr    aTask;
           siteVarsPtr     siteVars;
           FILE*           cookiefile;
           time_t          now;

           double          thisDNSTime, thisConnectTime, thisFirstByte, thisTotalTime, thisDownloadSize;
           long            thisHeaderSize;

           FILE*           statfile;
           FILE*           dumpfile;
           FILE*           keepfile;
           char            filename[SNARL_TASK_MAX_FILENAME_LENGTH];
           struct timeval  sometime;
           struct timezone tz;
           char            myinfo[SNARL_TASK_MAX_STATISTICS_FILE_LINE_LENGTH];


       /* ------------------------------------ */
       /* first create a data space for the    */
       /* transfer                             */
       /* let's guess it for 48 kB large       */
       /* enough for most HTML-content         */
       /* ------------------------------------ */

   if( !(*urldata->currenturl) ) {
       return( -1 );
   }

   if( !(urldata->curl_handle) ) {

       return( -2 );

   } else {


       /* ------------------------------------ */
       /* we need this prior to updating       */
       /* ------------------------------------ */

       aTask = (snarltaskPtr)(urldata->parentTask);

       if( data.start = malloc( 48 * 1024 ) ) {

           data.currentSize  = 0;
           data.leftSize     = 48 * 1024;
           data.currentPtr   = data.start;
           data.maxSize      = aTask->maxDownloadSize;


       /* ------------------------------------ */
       /* let's try to trick curl and pass it  */
       /* a data pointer to be used with our   */
       /* data function                        */
       /* ------------------------------------ */


           curl_easy_setopt( urldata->curl_handle, CURLOPT_FILE, &data );


       /* ------------------------------------ */
       /* let's setup a cookiefile if requested*/
       /* ------------------------------------ */

           if( !(urldata->noCookies) ) {
               siteVars = aTask->siteVars;

               pthread_mutex_lock(     siteVars->cookielock );
               lockedCookieFile = TRUE;

               if( *(  siteVars->cookiefilename) ) {
                   if( cookiefile = fopen( siteVars->cookiefilename, "a+") ) {

                       curl_easy_setopt( urldata->curl_handle, CURLOPT_WRITEHEADER, cookiefile );
                   }
               }
           }



       /* ------------------------------------ */
       /* init timing statistics timer         */
       /* ------------------------------------ */

           gettimeofday( &sometime, &tz);

           if( data.statisticsTxtPtr = (char*)malloc( 1 ) ) {
               *data.statisticsTxtPtr = '\0';

                data.firstInvocation.tv_sec  = sometime.tv_sec;
                data.firstInvocation.tv_usec = sometime.tv_usec;
           }


       /* ------------------------------------ */
       /* ------------------------------------ */
       /*  and now we are ready to go          */
       /* ------------------------------------ */

           result = curl_easy_perform( urldata->curl_handle );

       /* ------------------------------------ */
       /* this is a replacement for            */
       /* CURLINFO_TOTAL_TIME                  */
       /* ------------------------------------ */

           gettimeofday( &sometime, &tz);

            thisTotalTime = ((double)(sometime.tv_sec  - data.firstInvocation.tv_sec ) * 1000000)  +
                            ((double)(sometime.tv_usec - data.firstInvocation.tv_usec));

       /* ------------------------------------ */
       /* this is a replacement for            */
       /* CURLINFO_TOTAL_TIME                  */
       /* ------------------------------------ */

           (aTask->totalTransfers)++;
           (aTask->totalTransferVolume)  += (data.currentSize);

           pthread_mutex_lock(&globalTotalTransferLock);
               globalTotalTransfers++;
               globalTotalTransferVolume += (data.currentSize);
           pthread_mutex_unlock(&globalTotalTransferLock);

       /* ------------------------------------ */
       /*  and now we are ready to go          */
       /* ------------------------------------ */
       /* ------------------------------------ */


           if( data.statisticsTxtPtr ) {

               sprintf( filename, "%s/%s_%s_"SNARL_TASK_PROFILING_FILE_SUFFIX, PERSISTENT_DIRECTORY, aTask->sitename, aTask->urlname );

               len = strlen( filename );

               for( i=sizeof(PERSISTENT_DIRECTORY); i < len; i++ ){

                   if( (! isalnum(filename[i])) && (filename[i] != '.') ) {

                       filename[i] = '_';

                   }

               }


               if( aTask->mainURL.logTiming && (statfile = fopen( filename, "a")) ) {

                   curl_easy_getinfo( urldata->curl_handle, CURLINFO_NAMELOOKUP_TIME,  &thisDNSTime );
                   curl_easy_getinfo( urldata->curl_handle, CURLINFO_CONNECT_TIME,     &thisConnectTime );
                   curl_easy_getinfo( urldata->curl_handle, CURLINFO_PRETRANSFER_TIME, &thisFirstByte );
                   /*
                   this call is documented to be somewhat buggy in the curl mailinglist
                   curl_easy_getinfo( urldata->curl_handle, CURLINFO_TOTAL_TIME,       &thisTotalTime );
                   */

                   curl_easy_getinfo( urldata->curl_handle, CURLINFO_SIZE_DOWNLOAD,    &thisDownloadSize );
                   curl_easy_getinfo( urldata->curl_handle, CURLINFO_HEADER_SIZE,      &thisHeaderSize );

       /* ------------------------------------ */
       /* at runtime of these lines            */
       /* aTask->next_run still contains       */
       /* the current invocation time of this  */
       /* task                                 */
       /* ------------------------------------ */

                   snarl_ctime( myinfo, &aTask->statusVars.next_run, SNARL_TASK_MAX_STATISTICS_FILE_LINE_LENGTH );

                   fprintf( statfile, "# %s\n", myinfo );

                   fprintf( statfile, "%.lf -3000\n", 1000000 * thisDNSTime);
                   fprintf( statfile, "%.lf -2000\n", 1000000 * thisConnectTime);
                   fprintf( statfile, "%.lf -1000\n", 1000000 * thisFirstByte);

                   fprintf( statfile, "%s",                     data.statisticsTxtPtr );

                   fprintf( statfile, "%.lf %.lf\n",            thisTotalTime, urldata->withHeader ? (double)(thisDownloadSize + thisHeaderSize) : thisDownloadSize);
                   fprintf( statfile, "\n");

                   fflush(  statfile );
                   fclose(  statfile );
               }

               free( data.statisticsTxtPtr );
           }

       /* ------------------------------------ */
       /* the dump content task is to debug    */
       /* ------------------------------------ */
          if( aTask->mainURL.dumpContent ) {

              sprintf( filename, "%s/%s_%s_"SNARL_TASK_DUMP_FILE_SUFFIX, PERSISTENT_DIRECTORY, aTask->sitename, aTask->urlname );

              len = strlen( filename );

              for(i=sizeof(PERSISTENT_DIRECTORY); i<len; i++ ) {

                  if( (! isalnum(filename[i])) && (filename[i] != '.') ) {

                      filename[i] = '_';

                  }

              }




              strncpy( aTask->mainURL.dumpFileName, filename,   SNARL_TASK_MAX_FILENAME_LENGTH - 1 );
              len = min(strlen(filename), SNARL_TASK_MAX_FILENAME_LENGTH - 1 );
              aTask->mainURL.dumpFileName[len] = '\0';

              if( dumpfile = fopen( aTask->mainURL.dumpFileName, "a")) {

                  snarl_ctime( myinfo, &aTask->statusVars.next_run, SNARL_TASK_MAX_STATISTICS_FILE_LINE_LENGTH );

                  fprintf( dumpfile, "# %s\n", myinfo );

                  if( (data.start) && (data.currentSize)) {
                      fwrite(  data.start, data.currentSize, 1, dumpfile);
                  }
                  fprintf( dumpfile, "\n");

                  fflush(  dumpfile );
                  fclose(  dumpfile );
              }
          }


       /* ------------------------------------ */
       /* free and close the cookiefile if open*/
       /* ------------------------------------ */

            if( lockedCookieFile ) {

                if(cookiefile) fclose( cookiefile );
                pthread_mutex_unlock( siteVars->cookielock );
            }

       /* ------------------------------------ */
       /* return the data struct               */
       /* ------------------------------------ */

            snarlWebData->data = data.start;
            snarlWebData->size = data.currentSize;

       /* ------------------------------------ */
       /* this is a replacement for            */
       /* CURLINFO_TOTAL_TIME                  */
       /* ------------------------------------ */
           snarlWebData->totalTime = thisTotalTime / 1000000;

       /* ------------------------------------ */
       /* this is a replacement for            */
       /* CURLINFO_TOTAL_TIME                  */
       /* ------------------------------------ */


           return( result );

   	   } else {

       /* ------------------------------------ */
       /* memory allocation problem            */
       /* let's hope we'll survive this        */
       /* ------------------------------------ */

           return( -3 );
	   }
   }
}


/***************************************************************
*                                                              *
* !!! ---   PRIVATE  --- !!!                                   *
* !!! --- ASYNCHRONOUS HELPERFUNCTIONS ---!!!                  *
*                                                              *
* snarl_make_absolute_url                                      *
* parse dynamic url string and make an absolute url from it    *
*                                                              *
***************************************************************/


int snarl_make_absolute_url( snarltaskPtr aTask) {

  char           urlstring[SNARL_TASK_URL_MAX_LENGTH];
  char           protstring[SNARL_TASK_PROT_MAX_LENGTH];
  char           hoststring[SNARL_TASK_HOST_MAX_LENGTH];
  char           pathstring[SNARL_TASK_PATH_MAX_LENGTH];


  hoststring[0] = '\0';
  protstring[0] = '\0';
  pathstring[0] = '\0';


       /* ------------------------------------ */
       /* this is the strategy of this         */
       /* function to assemble an URL:         */
       /*                                      */
       /* first: check if the dynamic URL      */
       /* is setup correctly                   */
       /*  (this involves checking certain     */
       /*   things)                            */
       /*                                      */
       /* second: try to assemble the url from */
       /* the "normal" or "static" url field   */
       /* and the result of the dynamic match  */
       /* ------------------------------------ */


  if(    (2 == sscanf(aTask->dynamicURL, "%64[^:]://%M256[^\n]", protstring, pathstring))
      && strequal(protstring, "file")) {

       /* ------------------------------------ */
       /* we deal with file://<host>/<path>    */
       /* differently since it supports no     */
       /* hostname other than "localhost" and  */
       /* "127.0.0.1", which is unique among   */
       /* the URL protocols specified in RFC   */
       /* 1738                                 */
       /* ------------------------------------ */


    if( strnequal(pathstring, "localhost/", 10) ||
        strnequal(pathstring, "127.0.0.1/", 10)) {


       /* ------------------------------------ */
       /* If there's another host name than    */
       /* the one we support, <host>/ is       */
       /* quietly ommitted                     */
       /* ------------------------------------ */


      strncpy(hoststring, pathstring, 10);
      strcpy( pathstring, &pathstring[10]);
    }

       /* ------------------------------------ */
       /* store protocol string lowercase      */
       /* ------------------------------------ */


    strcpy(protstring, "file");

       /* ------------------------------------ */
       /* this was easy URL was setup correctly*/
       /* a n protocoll was file://            */
       /* ------------------------------------ */

  }
  else {

       /* ------------------------------------ */
       /* Now let's try to disassemble the dyn */
       /* amic match - if successful we're done*/
       /* ------------------------------------ */


    if (2 > sscanf(aTask->dynamicURL, "%64[^\n:]://%256[^\n/]/%256[^\n]",
                   protstring, hoststring, pathstring)) {


       /* ------------------------------------ */
       /* the dynmaic url was badly formed     */
       /* probably because it is in browser    */
       /* style omitting protocoll and host    */
       /* let's try to assemble the URL        */
       /* ------------------------------------ */


        if (2 > sscanf(aTask->mainURL.currenturl, "%64[^\n:]://%256[^\n/]/%256[^\n]",
                   protstring, hoststring, pathstring)) {


       /* ------------------------------------ */
       /* the static url is malformed we're in */
       /* deep trouble - let's resign          */
       /* ------------------------------------ */

           return( SNARL_TASK_URL_MALFORMAT );
        }

       /* ------------------------------------ */
       /* at this point we have protocoll      */
       /* and hostname from the static url     */
       /* we must assume that everything       */
       /* in the dynamic part is only path     */
       /* ------------------------------------ */


        if (1 > sscanf(aTask->dynamicURL, "%256[^\n]", pathstring)) {

       /* ------------------------------------ */
       /* We couldn't even get this format.    */
       /* ------------------------------------ */

            return SNARL_TASK_DYN_URL_MALFORMAT;
        }

     }
  }


       /* ------------------------------------ */
       /* now here we should have everything   */
       /* ------------------------------------ */

       sprintf(urlstring, "%s://%s%s%s", protstring, hoststring, pathstring[0] == '/' ? "" : "/", pathstring );
       if( strlen(urlstring) < (SNARL_TASK_URL_MAX_LENGTH - 1) ) {
           strcpy(aTask->dynamicURL, urlstring);
       }

   return( 0 );

}


/***************************************************************
*                                                              *
* !!! ---   PRIVATE  --- !!!                                   *
* !!! --- ASYNCHRONOUS HELPERFUNCTIONS ---!!!                  *
*                                                              *
* check novelty of incoming cookie                             *
*                                                              *
***************************************************************/

int new_cookie( FILE* cookiefile, char* cookie ) {

    char   inbuf[SNARL_TASK_MAX_COOKIEFILE_LINE_LENGTH];

    if( cookiefile ) {
		rewind( cookiefile );

		while( fgets( inbuf, sizeof(inbuf), cookiefile) ) {

			if( strstr( inbuf, cookie) ){
			   return( 0 );
			}
		}

		return( 1 );
	} else {
		return(0);
	}
}


/***************************************************************
*                                                              *
* !!! ---   PRIVATE  --- !!!                                   *
* !!! --- ASYNCHRONOUS HELPERFUNCTIONS ---!!!                  *
*                                                              *
* remove old cookies from cookiefile                           *
*                                                              *
***************************************************************/

void cleanup_cookiefile( char* cookiefilename) {


    FILE*     cookiefile;
    FILE*     tmpcookiefile;
    char      tmpcookiefilename[SNARL_TASK_MAX_FILENAME_LENGTH + 4];
    char      inbuf[SNARL_TASK_MAX_COOKIEFILE_LINE_LENGTH];
    int       closedcf = FALSE;
    char     *expiryPtr;
    char     *expiryDatePtr;
    time_t    expiryDate;
    time_t    now;

    if( cookiefile = fopen(cookiefilename, "r")) {

        sprintf( tmpcookiefilename, "%s.tmp", cookiefilename);

        if( tmpcookiefile = fopen(tmpcookiefilename, "w")) {

            while( fgets( inbuf, sizeof(inbuf), cookiefile) ) {

                if( expiryPtr = strstr( inbuf, "expires=") ) {

                    expiryDatePtr = (char*)&(expiryPtr[sizeof("expires=")-1]);

                    snarl_time(&now);
                    snarl_getdate( expiryDatePtr, &now, &expiryDate );

                    if( expiryDate > now ) {
                        fwrite(inbuf, strlen(inbuf), 1, tmpcookiefile );
                    }
                }
            }

            fclose( tmpcookiefile );
            fclose( cookiefile );
            closedcf = TRUE;

            rename( tmpcookiefilename, cookiefilename );
        }

        if( !closedcf ) fclose( cookiefile );
    }

}

/***************************************************************
*                                                              *
* !!! ---   PRIVATE  --- !!!                                   *
* !!! --- ASYNCHRONOUS CURL CALLBACKS ---!!!                   *
*                                                              *
* snarl_DataAcquisition                                        *
* fake an input stream for the CURL library                    *
* this function is supplied to the curl library as a callback  *
* for data reposition during data transfer                     *
*                                                              *
***************************************************************/

size_t snarlDataAcquisition( void* ptr, size_t size, size_t nmemb, void* data) {

       long             incomingSize, deltaSize, newSize;
       fakeStream      *thisStream;
       void*            utilityPtr;

       struct timeval   sometime;
       struct timezone  tz;
       char             myinfo[1024];

       /* ------------------------------------ */
       /* get microseconds timing data for log */
       /* download timing statistics           */
       /* ------------------------------------ */

       gettimeofday( &sometime, &tz);


       /* ------------------------------------ */
       /* possibly reallocate more memory      */
       /* ------------------------------------ */

       incomingSize = size * nmemb;
       thisStream   = (fakeStream*)data;


       if( thisStream->leftSize < incomingSize ) {

           newSize   =  thisStream->currentSize
                      + incomingSize;

           if( utilityPtr = realloc(thisStream->start, newSize ) ) {

               thisStream->leftSize   = incomingSize;
               thisStream->start      = utilityPtr;
               thisStream->currentPtr = utilityPtr + thisStream->currentSize;

           } else {

               thisStream->leftSize   = 0;
               thisStream->start      = NULL;
               thisStream->currentPtr = NULL;

               return( -1 );
           }
       }

       /* ------------------------------------ */
       /* save data chunk                      */
       /* ------------------------------------ */

       memcpy( thisStream->currentPtr, ptr, incomingSize);

       thisStream->currentSize     += incomingSize;
       thisStream->currentPtr      += incomingSize;
       thisStream->leftSize        -= incomingSize;


       /* ------------------------------------ */
       /* watch if maximum download size has   */
       /* been reached                         */
       /* ------------------------------------ */

       if( thisStream->maxSize && thisStream->maxSize < thisStream->currentSize ) {

               thisStream->leftSize   = 0;
               free( thisStream->start );
               thisStream->start      = NULL;
               thisStream->currentPtr = NULL;

           return(-1);
       }

       /* ------------------------------------ */
       /* create timing statistics in ascii    */
       /* ------------------------------------ */

       sprintf( myinfo, "%ld %ld\n", (sometime.tv_sec  - thisStream->firstInvocation.tv_sec ) * 1000000 +
                                      sometime.tv_usec - thisStream->firstInvocation.tv_usec,
                                      thisStream->currentSize );

       thisStream->statisticsTxtPtr = (char*)realloc( thisStream->statisticsTxtPtr, strlen( thisStream->statisticsTxtPtr ) + strlen( myinfo ) + 1 );

       if( thisStream->statisticsTxtPtr != NULL ) {
           memcpy(  &(thisStream->statisticsTxtPtr[strlen(thisStream->statisticsTxtPtr)]), myinfo, strlen(myinfo) + 1 );
       }

       return( incomingSize );
}

/***************************************************************
*                                                              *
* !!! ---   PRIVATE  --- !!!                                   *
* !!! --- ASYNCHRONOUS CURL CALLBACKS ---!!!                   *
*                                                              *
* snarl_filter_cookie_from_header                              *
* filter out cookies from headers and write them to file       *
*                                                              *
***************************************************************/

size_t snarl_filter_cookie_from_header( void* ptr, size_t size, size_t nmemb, void* stream) {


       size_t     actual_size;
       size_t     result;

       actual_size = size * nmemb;

       if( ! strncmp(ptr, "Set-Cookie:", min(actual_size, sizeof("Set-Cookie:")-1 )) ) {

           if( new_cookie( (FILE*)stream, (char*)ptr) ) {

               result = fwrite(ptr, size, nmemb, stream);

           } else {

               result = actual_size;
           }

       } else {

           result = actual_size;
       }

       return( result );
}


/***************************************************************
*                                                              *
* !!! ---   PRIVATE  --- !!!                                   *
* !!! --- ASYNCHRONOUS ---!!!                                  *
*                                                              *
* prioritizeTrap                                               *
*                                                              *
***************************************************************/

int prioritizeTrap( int a, int b) {

    int an, bn;

    an = (a > SNARLSNMP_TRAP_CRITICAL ? SNARLSNMP_TRAP_NORMAL : a);
    bn = (b > SNARLSNMP_TRAP_CRITICAL ? SNARLSNMP_TRAP_NORMAL : b);

    return(  max( an, bn)  );

}


/***************************************************************
*                                                              *
* !!! ---   PRIVATE  --- !!!                                   *
* !!! --- ASYNCHRONOUS ---!!!                                  *
*                                                              *
* create dynamic url data                                      *
*                                                              *
***************************************************************/


int     createDynamicURLData( snarltaskPtr aTask, char* urldata, int urldatamaxlength, char* subconfigline ) {

        int             success;
        int             subTokenID;
        int             index, index2, len;

        snarltaskPtr    currentTask;

        char           *configptr;
        char            inbuf[SNARL_TASK_MAX_LENGTH_SUBCONFIG_LINE];
        char            inbuf2[SNARL_TASK_MAX_LENGTH_SUBCONFIG_LINE];
        char            timestring[SNARL_TASK_MAX_URL_APPENDIX_LENGTH];



        configptr = subconfigline;
        success   = TRUE;

        while( configptr ) {

            configptr = copy_nword( configptr, inbuf, SNARL_TASK_MAX_LENGTH_SUBCONFIG_LINE);

            subTokenID = matchToken( inbuf );
            index      = 0;

            switch( subTokenID ) {

               case SNARL_SITE_SUBTOKEN_DYN_URL_COMP_MTIME:

                   snprintf( timestring, SNARL_TASK_MAX_URL_APPENDIX_LENGTH - 1, SNARL_TASK_URL_CRAWLERTIME_TOKEN"=%d", aTask->statusVars.current_run );

                   if( (strlen( urldata ) + strlen( timestring)) <= urldatamaxlength ) {
                       sprintf( &(urldata[strlen(urldata)]), "%s", timestring );
                   } else {
                       success = FALSE;
                   }
               break;

               case SNARL_SITE_SUBTOKEN_DYN_URL_COMP_TIMESTAMP:

                   snprintf( timestring, SNARL_TASK_MAX_URL_APPENDIX_LENGTH - 1, "%d", aTask->statusVars.current_run );

                   if( (strlen( urldata ) + strlen( timestring)) <= urldatamaxlength ) {
                       sprintf( &(urldata[strlen(urldata)]), "%s", timestring );
                   } else {
                       success = FALSE;
                   }
               break;

               case SNARL_SITE_SUBTOKEN_DYN_URL_COMP_CTIME:

                   strftime( timestring, SNARL_TASK_MAX_URL_APPENDIX_LENGTH - 1, "%H:%M:%S", localtime(&(aTask->statusVars.current_run)) );

                   if( (strlen( urldata ) + strlen( timestring)) <= urldatamaxlength ) {
                       sprintf( &(urldata[strlen(urldata)]), "%s", timestring );
                   } else {
                       success = FALSE;
                   }
               break;

               case SNARL_SITE_SUBTOKEN_DYN_URL_COMP_REGEX:

                     if( configptr ) {
                         configptr = copy_nword( configptr, inbuf, SNARL_TASK_MAX_LENGTH_SUBCONFIG_LINE );

                         if( configptr ) {
                             configptr = copy_nword( configptr, inbuf2, SNARL_TASK_MAX_LENGTH_SUBCONFIG_LINE );

       /* ------------------------------------ */
       /* first we interprete the name we get  */
       /* the regex result from and also the   */
       /* index we get the result from         */
       /*------------------------------------- */

                             len    = min((SNARL_TASK_MAX_SITENAME - 1), strlen(inbuf));
                             index2 = abs(atoi(inbuf2));

                             if( (index2 >= 0) && (index2 < SNARL_TASK_MAX_REGEX_MATCHES)) {

       /* ------------------------------------ */
       /* everything seems to be ok here       */
       /* let's store the result and copy      */
       /* the regex result                     */
       /* ------------------------------------ */

                                 strncpy( aTask->getRegexResultFrom, inbuf, len);
                                 aTask->getRegexResultFrom[len] = '\0';

                                 aTask->getRegexResultFromIndex = index2;

                                 index = 0;
                                 while( currentTask = find_snarltask( aTask->sitename, aTask->getRegexResultFrom, &index, globalTaskArray)) {

                                     pthread_mutex_lock(   currentTask->lock );

                                         if(   ((aTask->ignoreProxyGroup) || (!strcmp( aTask->mainURL.proxy, currentTask->mainURL.proxy))) ) {

                                             if( currentTask->hasData ) {

                                                 if( (strlen( urldata ) +
                                                      strlen( currentTask->regexMatchContent[aTask->getRegexResultFromIndex])) <= urldatamaxlength ) {

                                                      sprintf( &(urldata[strlen(urldata)]), "%s", currentTask->regexMatchContent[aTask->getRegexResultFromIndex] );
                                                 } else {
                                                     success = FALSE;
                                                 }
                                             } else {

       /* ------------------------------------ */
       /* we have found everything except that */
       /* our data supplying task doesn't have */
       /* data yet - so do not continue this   */
       /* task as we dont have data yet        */
       /* ------------------------------------ */
                                                 success = FALSE;
                                             }
                                         }
                                     pthread_mutex_unlock( currentTask->lock );
                                 }
                             }
                         }
                     }
                break;

                case SNARL_SITE_SUBTOKEN_DYN_URL_STATIC_TEXT:

                   if( configptr ) {

                       configptr = copy_nword( configptr, inbuf, SNARL_TASK_MAX_LENGTH_SUBCONFIG_LINE );

                       if( (strlen( urldata ) + strlen( inbuf )) <= urldatamaxlength) {

                          sprintf( &(urldata[strlen(urldata)]), "%s", inbuf );
                       }
                   }
                break;

                case SNARL_SITE_SUBTOKEN_DYN_URL_STATIC_AMPERSAND:

                   if( (strlen( urldata ) + 1) <= urldatamaxlength) {

                      sprintf( &(urldata[strlen(urldata)]), "&" );
                   }
                break;
           }
       }

       return( success );
}


/***************************************************************
*                                                              *
* initialize the task thread                                   *
*                                                              *
***************************************************************/

int   initSnarlTask( snarltaskPtr aTask) {


       if( aTask ) {

        /* ------------------------------------ */
        /* lock all memory and prevent paging   */
        /* during transfers that may skew       */
        /* timing data severely                 */
        /*                                      */
        /* !!! --- ATTENTION --- !!!            */
        /*                                      */
        /* calling mlock will end in a          */
        /* segmentation fault if too many       */
        /* threads are called at the same time  */
        /* as swap space is not available to    */
        /* this process after mlocking          */
        /* ------------------------------------ */

           if( getuid() == 0 ) {
               if( mlockall( MCL_FUTURE ) ) return ( FALSE );
           }

           pthread_mutex_lock( aTask->lock );

               if( aTask->isVirgin ) {

        /* ------------------------------------ */
        /* set all the necessary options        */
        /* for the transfer                     */
        /* ------------------------------------ */

        /* ------------------------------------ */
        /* we dont set the connecion data here  */
        /* because we might want to alter some  */
        /* url data at run time so we do setup  */
        /* the connection data for our main url */
        /* just prior to doing the transer      */
        /* ------------------------------------ */
                   strcpy( aTask->mainURL.currenturl,      aTask->mainURL.url      );
                   strcpy( aTask->mainURL.currentformData, aTask->mainURL.formData );

        //         snarl_setup_connection_data( &aTask->mainURL,
        //                                       aTask->sitename,
        //                                       aTask->urlname,
        //                                       SNARL_TASK_MAIN_URL );

                   if( aTask->task_contentMatchURL ) {

                       strcpy( aTask->contentMatchURL.currenturl,      aTask->contentMatchURL.url      );
                       strcpy( aTask->contentMatchURL.currentformData, aTask->contentMatchURL.formData );

                       snarl_setup_connection_data( &aTask->contentMatchURL,
                                                     aTask->sitename,
                                                     aTask->urlname,
                                                     SNARL_TASK_CONTENTMATCH_URL );
                   }

                   if( aTask->task_sizeMatchURL ) {

                       strcpy( aTask->sizeMatchURL.currenturl,      aTask->sizeMatchURL.url      );
                       strcpy( aTask->sizeMatchURL.currentformData, aTask->sizeMatchURL.formData );

                       snarl_setup_connection_data( &aTask->sizeMatchURL,
                                                     aTask->sitename,
                                                     aTask->urlname,
                                                     SNARL_TASK_SIZEMATCH_URL );
                   }
                   aTask->isVirgin = FALSE;
               }
           pthread_mutex_unlock( aTask->lock );
       }

       return( TRUE );
}

